{@abstract(Display format form)
@author(Nikolai Shokhirev <nikolai@shokhirev.com>)
@created(June 6, 2002)
@lastmod(August 8, 2003)
Nikolai V. Shokhirev, 2002-2003 }
{
Display format
  Width      - total length of a number for Real and Integer
  Decimals   - number of decimal digits for Real
  The numbers are displayed in the Scientific notation if Decimals < 0
  
  Default values for Width and Decimals can be set before call for ShowModal
  
  seW.Value := 14;
  seD.Value := -1;
  
  The modified value is saved in rVal (TFloat), iVal (TInt) or sVal (string).

  ArrayName, Idx1 and Idx2 are used only in the caption and may not be set at all

}

unit fDisplayVal;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, uMatTypes;

type

  TDisplayType = (dtReal, dtInt, dtStr);

  TFDisplVal = class(TForm)
    LabelArray: TLabel;
    EditValue: TEdit;
    LabelW: TLabel;
    LabelD: TLabel;
    seW: TSpinEdit;
    seD: TSpinEdit;
    procedure seWChange(Sender: TObject);
    procedure seDChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    dt: TDisplayType;
    frVal: TFloat;
    fiVal: TInt;
    fsVal: string;
    fArrayName: string;
    fIdx1: TInt;
    fIdx2: TInt;
    function GetDisplType: TDisplayType;
    function GetiVal: TInt;
    function GetrVal: TFloat;
    function GetsVal: string;
    procedure SetDisplType(const Value: TDisplayType);
    procedure SetiVal(const Value: TInt);
    procedure SetrVal(const Value: TFloat);
    procedure SetsVal(const Value: string);
    function GetIdx1: TInt;
    function GetIdx2: TInt;
    procedure SetIdx1(const Value: TInt);
    procedure SetIdx2(const Value: TInt);
    function GetArrayName: string;
    procedure SetArrayName(const Value: string);
  public
    { Public declarations }
    procedure Display;
    procedure Save;
    property ArrayName: string read GetArrayName write SetArrayName;
    property Idx1: TInt read GetIdx1 write SetIdx1;
    property Idx2: TInt read GetIdx2 write SetIdx2;
    property DisplType: TDisplayType read GetDisplType write SetDisplType;
    property rVal: TFloat read GetrVal write SetrVal;
    property iVal: TInt read GetiVal write SetiVal;
    property sVal: string read GetsVal write SetsVal;
  end;

var
  FDisplVal: TFDisplVal;

implementation

uses
  uDisplay;

{$R *.DFM}

procedure TFDisplVal.FormCreate(Sender: TObject);
begin
  fArrayName := '';
  fIdx1 := -1;
  fIdx2 := -1;
  dt := dtReal;
end;

procedure TFDisplVal.seWChange(Sender: TObject);
begin
  if seW.Value < 2 then seW.Value := 2;
  if seW.Value < seD.Value+2 then seD.Value := seW.Value-2;
  Display;
end;

procedure TFDisplVal.seDChange(Sender: TObject);
begin
  if seD.Value > seW.Value-2 then seW.Value := seD.Value+2;
  Display;
end;

procedure TFDisplVal.Display;
var
  s: string;
begin
  case dt of
    dtReal: EditValue.Text := RealToStr(rVal, seW.Value, seD.Value);
    dtInt:  EditValue.Text := IntgToStr(iVal, seW.Value);
    dtStr:  EditValue.Text := sVal;
  end;
  s := fArrayName;
  if Idx1 >= 0 then
  begin
    s := s+'['+IntToStr(Idx1);
    if Idx2 >= 0 then s := s+','+IntToStr(Idx2);
    s := s+']';
  end;
  LabelArray.Caption := s;
end;

procedure TFDisplVal.Save;
begin
  case dt of
    dtReal: rVal := StrToFloat(EditValue.Text);
    dtInt:  iVal := StrToInt(EditValue.Text);
    dtStr:  sVal := EditValue.Text;
  end;
end;

function TFDisplVal.GetDisplType: TDisplayType;
begin
  result := dt;
end;

function TFDisplVal.GetiVal: TInt;
begin
  result := fiVal;
end;

function TFDisplVal.GetrVal: TFloat;
begin
  result := frVal;
end;

function TFDisplVal.GetsVal: string;
begin
  result := fsVal;
end;

procedure TFDisplVal.SetDisplType(const Value: TDisplayType);
begin
  dt := Value;
end;

procedure TFDisplVal.SetiVal(const Value: TInt);
begin
  fiVal := Value;
end;

procedure TFDisplVal.SetrVal(const Value: TFloat);
begin
  frVal := Value;
end;

procedure TFDisplVal.SetsVal(const Value: string);
begin
  fsVal := Value;
end;

function TFDisplVal.GetIdx1: TInt;
begin
  result := fIdx1;
end;

function TFDisplVal.GetIdx2: TInt;
begin
  result := fIdx2;
end;

procedure TFDisplVal.SetIdx1(const Value: TInt);
begin
  fIdx1 := Value;
end;

procedure TFDisplVal.SetIdx2(const Value: TInt);
begin
  fIdx2 := Value;
end;

function TFDisplVal.GetArrayName: string;
begin
  result := fArrayName;
end;

procedure TFDisplVal.SetArrayName(const Value: string);
begin
  fArrayName := Value;
end;

procedure TFDisplVal.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Save;
end;

end.
