{: Advenced for the THeightField object.<p>

   Check the fxy sample first.<p>

   This sample shows a few more tricks : how to switch formulas at run-time,
   effects of base grid extents and resolution change as well as color and
   lighting options of the THeightField.<p>

   Note that maxed out grid size and minimum step (high resolution) will bring
   most of todays cards to their knees (if they do not just crash, that is).<p>

   Used formulas :<p>

   The Formula1 is of type Sin(d)/(1+d), with d=sqr(x)+sqr(y), you may note
   the interesting sampling-interference effect with big step values (low res)
   and remember your math teacher's warnings on graph-plotting :)<p>

   Formula2 is a more classic sin*cos mix<p>

   Dynamic is the third formula, if you pick it, a small ball will appear and
   move around, the plotted formula being the square distance to the ball.
}
unit fGLSurface;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls,
  ComCtrls, GLScene, GLGraph, GLObjects, GLMisc, ExtCtrls, GLTexture,
  GLCadencer, Geometry, GLWin32Viewer, Menus;

type

  TSingleFunction2D = function(x, y: Single): Single;

  TFormGLSurface = class(TForm)
    GLScene: TGLScene;
    GLCamera: TGLCamera;
    GLLightSource: TGLLightSource;
    HeightField: THeightField;
    XYGrid: TXYZGrid;
    XZGrid: TXYZGrid;
    YZGrid: TXYZGrid;
    PanelTop: TPanel;
    GLSceneViewer: TGLSceneViewer;
    PanelLeft: TPanel;
    TrackBarZ: TTrackBar;
    tbFocal: TTrackBar;
    PanelBottom: TPanel;
    TrackBarX: TTrackBar;
    LXextents: TLabel;
    LYextents: TLabel;
    LStep: TLabel;
    eS: TEdit;
    eYmax: TEdit;
    eXmax: TEdit;
    TrackBarY: TTrackBar;
    CheckBox2sided: TCheckBox;
    LColMode: TLabel;
    cbColorMode: TComboBox;
    CheckBoxLighting: TCheckBox;
    cbShowAxes: TCheckBox;
    LCamera: TLabel;
    cbXgrid: TCheckBox;
    cbYgrid: TCheckBox;
    cbZgrid: TCheckBox;
    eXmin: TEdit;
    eYmin: TEdit;
    eZmin: TEdit;
    LZextents: TLabel;
    eZmax: TEdit;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelZ: TLabel;
    cbStyle: TCheckBox;
    procedure GLSceneViewerMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure GLSceneViewerMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure CheckBox2sidedClick(Sender: TObject);
    procedure tbStepChange(Sender: TObject);
    procedure cbColorModeChange(Sender: TObject);
    procedure CheckBoxLightingClick(Sender: TObject);
    procedure cbShowAxesClick(Sender: TObject);
    procedure TrackBarZChange(Sender: TObject);
    procedure tbFocalChange(Sender: TObject);
    procedure TrackBarYChange(Sender: TObject);
    procedure TrackBarXChange(Sender: TObject);
    procedure cbZgridClick(Sender: TObject);
    procedure cbXgridClick(Sender: TObject);
    procedure cbYgridClick(Sender: TObject);
    procedure cbStyleClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Dclarations prives }
    fFormula : TSingleFunction2D;
    fZmin: single;
    fZmax: single;
    mx, my : Integer;
    fStep: single;
    procedure HeightFunction(const x, y: Single; var z: Single;
                             var color: TColorVector; var texPoint: TTexPoint);
    procedure SetFormula(const Value: TSingleFunction2D);
    function GetXmax: single;
    function GetXmin: single;
    function GetYmax: single;
    function GetYmin: single;
//    function GetZmax: single;
//    function GetZmin: single;
    procedure SetXmax(const Value: single);
    procedure SetXmin(const Value: single);
    procedure SetYmax(const Value: single);
    procedure SetYmin(const Value: single);
    procedure SetZmax(const Value: single);
    procedure SetZmin(const Value: single);
    procedure SetStep(const Value: single);
  public
    { Dclarations publiques }
    procedure SetLimits(const aXmin, aXmax, aYmin, aYmax, aZmin, aZmax,
                              aStep: single; UseFunction: boolean);
    property Formula : TSingleFunction2D read fFormula write SetFormula;
    property Xmin : single read GetXmin write SetXmin;
    property Xmax : single read GetXmax write SetXmax;
    property Ymin : single read GetYmin write SetYmin;
    property Ymax : single read GetYmax write SetYmax;
    property Zmin : single read fZmin write SetZmin;
    property Zmax : single read fZmax write SetZmax;
    property Step : single read fStep write SetStep;
  end;

var
  FormGLSurface: TFormGLSurface;
  zfactor: single;
implementation

{$R *.DFM}

uses
  math;

procedure TFormGLSurface.SetFormula(const Value: TSingleFunction2D);
begin
  fFormula := Value;
  if Assigned(fFormula) then
    HeightField.OnGetHeight := HeightFunction
  else
    HeightField.OnGetHeight := nil;
end;

procedure TFormGLSurface.HeightFunction(const x, y: Single; var z: Single;
                              var color: TColorVector; var texPoint: TTexPoint);
var
  t: single;
begin
   z:= fFormula(x,y);
   t := (z-Zmin)/(Zmax-Zmin);
   VectorLerp(clrBlue, clrRed, t, color);
end;


procedure TFormGLSurface.CheckBox2sidedClick(Sender: TObject);
begin
   // enable two sided surface
   if CheckBox2sided.Checked then
      HeightField.Options:=HeightField.Options+[hfoTwoSided]
   else HeightField.Options:=HeightField.Options-[hfoTwoSided];
end;

procedure TFormGLSurface.cbColorModeChange(Sender: TObject);
begin
   // change per vertex color mode
   case cbColorMode.ItemIndex of
      0 : HeightField.ColorMode:=hfcmEmission;
      1 : HeightField.ColorMode:=hfcmNone;
      2 : HeightField.ColorMode:=hfcmDiffuse;
      3 : HeightField.ColorMode:=hfcmAmbient;
      4 : HeightField.ColorMode:=hfcmAmbientAndDiffuse;
   end;
end;

procedure TFormGLSurface.CheckBoxLightingClick(Sender: TObject);
begin
   GLLightSource.Shining:=CheckBoxLighting.Checked;
end;

procedure TFormGLSurface.tbStepChange(Sender: TObject);
begin
end;

// following code takes care of camera movement, see camera & movement demos
// for explanations and more samples

procedure TFormGLSurface.GLSceneViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   mx:=x; my:=y;
end;

procedure TFormGLSurface.GLSceneViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   if Shift<>[] then begin
      GLCamera.MoveAroundTarget(my-y, mx-x);
      mx:=x; my:=y;
   end;
end;

procedure TFormGLSurface.cbShowAxesClick(Sender: TObject);
begin
  HeightField.ShowAxes := cbShowAxes.Checked;
end;

procedure TFormGLSurface.tbFocalChange(Sender: TObject);
begin
  GLCamera.FocalLength := tbFocal.Position;  
end;

procedure TFormGLSurface.TrackBarZChange(Sender: TObject);
var
  p: Single;
begin
  p := (Zmin-Zmax)*(TrackBarZ.Position/TrackBarZ.Max)+Zmax;
  XYGrid.ZSamplingScale.Origin := p;
  LabelZ.Caption := 'Z = '+FloatToStr(p);
end;

procedure TFormGLSurface.TrackBarYChange(Sender: TObject);
var
  p: Single;
begin
  p := (Ymax-Ymin)*(TrackBarY.Position/TrackBarY.Max)+Ymin;
  XZGrid.YSamplingScale.Origin := p;
  LabelY.Caption := 'Y = '+FloatToStr(p);
end;

procedure TFormGLSurface.TrackBarXChange(Sender: TObject);
var
  p: Single;
begin
  p := (Xmax-Xmin)*(TrackBarX.Position/TrackBarX.Max)+Xmin;
  YZGrid.XSamplingScale.Origin := p;
  LabelX.Caption := 'X = '+FloatToStr(p);
end;

procedure TFormGLSurface.cbZgridClick(Sender: TObject);
begin
  XYgrid.Visible := cbZgrid.Checked;
end;

procedure TFormGLSurface.cbXgridClick(Sender: TObject);
begin
  YZgrid.Visible := cbXgrid.Checked;
end;

procedure TFormGLSurface.cbYgridClick(Sender: TObject);
begin
  XZgrid.Visible := cbYgrid.Checked;
end;

procedure TFormGLSurface.cbStyleClick(Sender: TObject);
begin
  if cbStyle.Checked then
  begin
    GLCamera.CameraStyle := csPerspective;
  end else
  begin
    GLCamera.CameraStyle := csOrthogonal;
  end;
end;

function TFormGLSurface.GetXmax: single;
begin
  result := HeightField.XSamplingScale.Max
end;

function TFormGLSurface.GetXmin: single;
begin
  result := HeightField.XSamplingScale.Min;
end;

function TFormGLSurface.GetYmax: single;
begin
  result := HeightField.YSamplingScale.Max;
end;

function TFormGLSurface.GetYmin: single;
begin
  result := HeightField.YSamplingScale.Min;
end;

procedure TFormGLSurface.SetXmax(const Value: single);
begin
  HeightField.XSamplingScale.Max := Value;
  XZGrid.ZSamplingScale.Min := -Value;   //
  XYGrid.XSamplingScale.Max := Value;    //
end;

procedure TFormGLSurface.SetXmin(const Value: single);
begin
  HeightField.XSamplingScale.Min := Value;
  XZGrid.XSamplingScale.Max := -Value;   //
  XYGrid.XSamplingScale.Min := Value;    //
end;

procedure TFormGLSurface.SetYmax(const Value: single);
begin
  HeightField.YSamplingScale.Max := Value;
  XYGrid.YSamplingScale.Max := Value;
  YZGrid.YSamplingScale.Max := Value;
end;

procedure TFormGLSurface.SetYmin(const Value: single);
begin
  HeightField.YSamplingScale.Min := Value;
  XYGrid.YSamplingScale.Min := Value;
  YZGrid.YSamplingScale.Min := Value;
end;

procedure TFormGLSurface.SetZmax(const Value: single);
begin        
  fZmax := Value;
  YZGrid.ZSamplingScale.Max := Value; //0k
  XZGrid.XSamplingScale.Max := Value; //
end;

procedure TFormGLSurface.SetZmin(const Value: single);
begin
  fZmin := Value;
  YZGrid.ZSamplingScale.Min := Value;  //
  XZGrid.XSamplingScale.Min := Value;  //
end;

procedure TFormGLSurface.SetLimits(const aXmin, aXmax, aYmin, aYmax, aZmin,
  aZmax, aStep: single; UseFunction: boolean);
var
  x, y, z, mn, mx: single;
begin
  Xmin := aXmin;
  Xmax := aXmax;
  Ymin := aYmin;
  Ymax := aYmax;
  Zmin := aZmin;
  Zmax := aZmax;
  Step := aStep;
  if UseFunction then
    if Assigned(Formula) then
    begin
      mn :=  3.4e38; // max single
      mx := -3.4e38;
      x := Xmin;
      while x <= Xmax do
      begin
        y := Ymin;
        while y <= Ymax do
        begin
          z := Formula(x,y);
          mx := max(mx,z);
          mn := min(mn,z);
          y := y + Step;
        end;
        x := x + Step;
      end;
      Zmin := mn;
      Zmax := mx;
    end;
  GLCamera.Position.X := 3.0*max(abs(Xmin),abs(Xmax));
  GLCamera.Position.Y := 3.0*max(abs(Ymin),abs(Ymax));
  GLCamera.Position.Z := 3.0*max(abs(Zmin),abs(Zmax));
end;

procedure TFormGLSurface.SetStep(const Value: single);
begin
  fStep := Value;
// adjust grid steps (resolution)
  HeightField.XSamplingScale.Step:=fStep;
  HeightField.YSamplingScale.Step:=fStep;
  eS.Text := FloatToStr(fStep);
end;

procedure TFormGLSurface.FormShow(Sender: TObject);
begin
  TrackBarXChange(Self);
  TrackBarYChange(Self);
  TrackBarZChange(Self);
end;

procedure TFormGLSurface.FormResize(Sender: TObject);
begin
  LYextents.Left := (LXextents.Left+LZextents.Left) div 2;
  LabelY.Left := (LabelX.Left+LabelZ.Left) div 2;
  eYmin.Left := (eXmin.Left+eZmin.Left) div 2;
  eYmax.Left := (eXmax.Left+eZmax.Left) div 2;
  cbColorMode.ItemIndex := 0;
  cbColorModeChange(Self);
end;

end.
