{@abstract(uColorMap)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(June 06, 2002)
@lastmod(August 08, 2003)
Nikolai V. Shokhirev, 2002-2003 }
unit uColorMap;

interface

uses
  uMatTypes, Graphics;

type
  TRGB= (cR, cG, cB);

  TRGBSet = Set of TRGB;

  //  0 <= aNumb <=1
  function NumberToColor(aNumb: TFloat; RGBSet: TRGBSet): TColor;

  function ColorToNumber(aColor: TColor; RGBSet: TRGBSet): TFloat;

  { 0 <= x <= 320: 0-R, 64-G, 128-B, 192-R, 256-G, 320-B,  }
  { 0 <= x <= 160: one rainbow }
  function Rainbow(x: TFloat; n: TInt):TColor;

implementation

//  0 <= aNumb <=1
function NumberToColor(aNumb: TFloat; RGBSet: TRGBSet): TColor;
var
  i, n: TInt;
begin  // 16777215 = 255*65793 = 255*(1 + 256 + 65536)
  n := 0;
  if cR in RGBSet then n := n + 1;
  if cG in RGBSet then n := n + 256;
  if cB in RGBSet then n := n + 65536;
  if n = 0 then
    result :=  TColor(Round(16777215.0*aNumb))
  else
    result :=  TColor(n*Round(255.0*aNumb));
end;

function ColorToNumber(aColor: TColor; RGBSet: TRGBSet): TFloat;
var
  x: TFloat;
  i, n: TInt;
begin
  n := 0;
  if cR in RGBSet then n := n + 1;
  if cG in RGBSet then n := n + 256;
  if cB in RGBSet then n := n + 65536;
  if n = 0 then
  begin 
    i := integer(aColor);
    x := i;
    result := x/16777215.0;
  end else
  begin
    x := (255*n);
    result := integer(aColor)/x;
  end;
end;

function Rainbow(x: TFloat; n: TInt):TColor;
var
  c,i,j,k: TInt;

  function f(x, d: TFloat): TInt;
  begin
    case n of
    0:result := round(255.0*(1.0-sqr(x/d)));
    1:result := round(255.0*(1.0-abs(x/d)*sqr(x/d)));
    else
      result := round(255.0*(1.0-sqr(sqr(x/d))));
    end;
    if result < 0 then result := 0;
  end;

begin
    i := f(x,64.0)+f(x-192.0,64.0);
    j := f(x-64.0,64.0)+f(x-256.0,64.0);
    k := f(x-128.0,64.0)+f(x-320.0,64.0);;
    c := i + j*256 + k*65536;
  result := TColor(c);
end;

end.
