{@abstract(Display utilities)
@author(Nikolai V. Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(May 05, 2002)
@lastmod(June 15, 2004)
Nikolai V. Shokhirev, 2002-2004  }
{ 2007.07.07 - extended comments, other cleanup }
unit uDisplay;

interface

uses
  Math, uMatTypes, Sysutils, Classes, Grids, controls;

//  function BoolToStr(b: boolean): string; - moved to uMatTypes
//  function StrToBool(b: string): boolean; - moved to uMatTypes

  { result[1]       = s[i1]
    result[2]       = s[i1+1]
    . . . . . . . . . . .
    result[i2-i1+1] = s[i2]      }
  function CopyFromTo(const s: string; i1, i2: integer): string;

  // result := x:Width:Decimals
  function RealToStr(x: TFloat; Width, Decimals: TInt): string;

  function StrToFloatDef(const S: string; Default: TFloat = 0.0): TFloat;

  // fixed width IntToStr
  function IntgToStr(x, Width: TInt): string;

  {
    Truncate:                false             true
    Prefix:   XXXXXXXXX      XXXXXXXXX         XXXXXXXXX
    x:            DDDDD    DDDDDDDDDDD       DDDDDDDDDDD
    result:   XXXXDDDDD    DDDDDDDDDDD         DDDDDDDDD
  }
  function IntgToStrPrefix(x: TInt; Prefix: string; Truncate: boolean = false): string;

  { output format: 1.1 + i * 2.2 }
  function CplxToStr(x, y: TFloat; Width, Decimals: TInt): string;

  { input formats: 1, i, j, I, J, +1.1-i*2.2, 1.1+i*2.2, i-1, i 1 - 2 }
  procedure StrToCplx(const s: string; var x, y: TFloat);

  { minimal decimals:
    Minimal necessary number of decimals for labeling the interval [Xmin,Xmax] }
  function MinDec(Xmin, Xmax: TFloat): TInt;

  { Compact output of real numbers in scientific notation
    decimals = 0 example: 7e+12
    decimals = 1 example: 1.3e+0
    decimals = 2 example: 1.23e-1 }
  function CompactFloatToStr(z: TFloat; decimals: TInt): string;

//  function BoolToStr(Value: boolean): string;
//  function StrToBool(Value: string): boolean;

  // Alphabetic sort; does not move the first row
//  function SGSortA(List: TStringList; Index1, Index2: Integer): Integer;

  // Data sort; does not move the first row
//  function SGSortD(List: TStringList; Index1, Index2: Integer): Integer;

  // By Erwien Ekosaputra - erwieneko@bigfoot.com
//  procedure SortGrid (AGrid : TStringGrid; const SortIndex : integer;
//                                              Compare: TStringListSortCompare);
//  procedure FixStringGrid(const SG: TStringGrid; dh: TInt = 16; dw: TInt = 8);
//  procedure FixStringGrid2(const SG: TStringGrid);

  procedure CenterControl(Control: TControl);

implementation

uses
  StdCtrls, DateUtils;

function BoolToStr(b: boolean): string;
begin
  if b then result := 'True' else result := 'False';
end;

function StrToBool(b: string): boolean;
begin
  result := AnsiSameText(b,'true');
end;

function CopyFromTo(const s: string; i1, i2: integer): string;
begin       
  result := Copy(s, i1, i2-i1+1);
end;

function RealToStr(x: TFloat; Width, Decimals: TInt): string;
var
  s: string;
begin
  Str(x:Width:Decimals, s);
  result := s;
end;

function IntgToStr(x, Width: TInt): string;
var
  s: string;
begin
  Str(x:Width, s);
  result := s;
end;

function IntgToStrPrefix(x: TInt; Prefix: string; Truncate: boolean): string;
var
  s: string;
  L, LL: TInt;
begin
  LL := Length(Prefix);
  s := IntToStr(x);
  L := Length(s);
  if L < LL then
    result := Copy(Prefix,1,LL-L)+s
  else 
    if Truncate then
      result := Copy(s,LL-L+1,LL)
    else
      result := s;
end;

function CplxToStr(x, y: TFloat; Width, Decimals: TInt): string;
var
  s: string;
begin
  Str(x:Width:Decimals, s);
  result := s;
  if y <> 0.0 then
  begin
    Str(abs(y):Width:Decimals, s);
    s := Trim(s);
    if y > 0.0 then
      result := result + '+i*'+s
    else
      result := result + '-i*'+s;
  end;
end;

function StrToFloatDef(const S: string; Default: TFloat = 0.0): TFloat;
var
  Code: Integer;
  V: TFloat;
begin
  Val(S, V, Code);
  if Code = 0 then result := V else result := Default;
end;

{input formats: 1, i, j, I, J, +1.1-i*2.2, 1.1+i*2.2, i-1, i 1 - 2 }
procedure StrToCplx(const s: string; var x, y: TFloat);
var
  ss: string;
  i, n: TInt;
  sig: TFloat;

  function PosIm(const s: string): TInt;
  begin
    result := Pos('I', ss);
    if result = 0 then result := Pos('J', ss);
  end;

begin
  ss :=  UpperCase(Trim(s));
  ss :=  StringReplace(ss, ' ', '', [rfReplaceAll]);
  ss :=  StringReplace(ss, '*', '', [rfReplaceAll]);
  sig := 1.0;
  if ss[1] = '+' then
    ss := CopyFromTo(ss,2,255);

  if ss[1] = '-' then
  begin
    ss := CopyFromTo(ss,2,255);
    sig := -1.0;
  end;
  i := PosIm(ss);

  if i > 0 then
  begin // Im <> 0
    if (i=1) then
    begin  // Im first
      n := Pos('+',ss);
      if n = 0 then n := Pos('-',ss);
      if n > 0 then
      begin
        if n = i then
          y := sig
        else
          y := sig*StrToFloatDef(CopyFromTo(ss,2,n-1));
        x := StrToFloatDef(CopyFromTo(ss,n,255));
      end else
      begin
        if Length(ss) = 1 then
          y := sig
        else
          y := sig*StrToFloatDef(CopyFromTo(ss,2,255));
        x := 0.0;
      end;
    end else
    begin // Re first
      if ss[i-1] = '+' then
      begin
        x := sig*StrToFloatDef(CopyFromTo(ss,1,i-2));
        if Length(ss) = i then
          y := 1.0
        else
          y := StrToFloatDef(CopyFromTo(ss,i+1,255));
      end else
      if ss[i-1] = '-' then
      begin
        x := sig*StrToFloatDef(CopyFromTo(ss,1,i-2));
        if Length(ss) = i then
          y := -1.0
        else
          y := -StrToFloatDef(CopyFromTo(ss,i+1,255));
      end else
      begin
        x := sig*StrToFloatDef(CopyFromTo(ss,1,i-1));
        y := StrToFloatDef(CopyFromTo(ss,i+1,255));
      end;
    end;
  end else
  begin // Im = 0
    x := StrToFloatDef(ss);
    y := 0.0;
  end;
end;

// Minimal necessary number of decimals for labeling the interval [Xmin, Xmax]
function MinDec(Xmin, Xmax: TFloat): TInt;
begin
  result := 1 + round(-Log10(abs(Xmax-Xmin)/max(abs(Xmax),abs(Xmin))));
end;

// Compact output of real numbers
function CompactFloatToStr(z: TFloat; decimals: TInt): string;
const
  eps =1.0e-8;
var
  x: TFloat;
  exponent: TInt;
  s, ss: string;
begin
  if z < 0.0 then s := '-' else s := ' ';
  x := abs(z);
  if x < eps then
  begin
    result := s+'0.0';
  end else
  begin
    exponent := Floor(Log10(x));
    x := IntPower(10.0,-decimals)*round(x*IntPower(10.0,decimals-exponent));
    if x >9.999 then
    begin
      x := 0.1*x;
      inc(exponent);
    end;
    ss := IntToStr(exponent);
    if exponent < 0 then ss := 'e'+ss else ss := 'e+'+ss;
    result := s+FloatToStr(x)+ss;
  end;
end;

// Alphabetic sort; does not move the first row
function SGSortA(List: TStringList; Index1, Index2: Integer): Integer;
begin
  if Index1=Index2 then
    result := 0
  else if Index1 = 0 then
    result := -1
  else if Index2 = 0 then
    result := 1
  else
    result := AnsiCompareStr(List[Index1], List[Index2]);
end;

// Date sort; does not move the first row
function SGSortD(List: TStringList; Index1, Index2: Integer): Integer;
var
  date1, date2: TDateTime;
  day0, day1, day2: Word;
begin
//  date0 := Date;
  day0 := DayOfTheYear(Date);
  if Index1=Index2 then
    result := 0
  else if Index1 = 0 then
    result := -1
  else if Index2 = 0 then
    result := 1
  else
  begin
    date1 := StrToDatetime(List[Index1]);
    date2 := StrToDatetime(List[Index2]);
    day1 := DayOfTheYear(date1);
    day2 := DayOfTheYear(date2);
    if day1 < day0 then day1 := day1 + 365;
    if day2 < day0 then day2 := day2 + 365;
    result := day1 - day2;
  end;
end;

// By Erwien Ekosaputra - erwieneko@bigfoot.com
procedure SortGrid (AGrid : TStringGrid; const SortIndex : integer;
                                              Compare: TStringListSortCompare);
var
  SortStringList : TStringList;
  RowLoop : integer;
  RowValues : TStringList;
begin
  //Creating the string list to sort
  SortStringList := TStringList.Create;
  try
    //Iterating for all rows in a grid.
    for RowLoop := 0 to AGrid.RowCount-1 do
    begin
      //Creating a stringlist as a holder, reading a row from stringgrid and
      //add it into object property of StringList.
      RowValues := TStringList.Create;
      RowValues.Assign (AGrid.Rows [RowLoop]);
      SortStringList.AddObject (AGrid.Cells [SortIndex, RowLoop], RowValues);
    end;

    //Sort
    SortStringList.CustomSort(@Compare);
//    SortStringList.Sorted := True;

    //Reassigning sorted rows
    for RowLoop := 0 to SortStringList.Count-1 do
      AGrid.Rows [RowLoop].Assign (TStrings (SortStringList.Objects [RowLoop]));

  finally
    for RowLoop := 0 to SortStringList.Count-1 do
      TStringList (SortStringList.Objects [RowLoop]).Free;
    SortStringList.Free;
  end;
end;

procedure FixStringGrid(const SG: TStringGrid; dh: TInt = 16; dw: TInt = 8);
const
  d = 16;
var
  c, r, l, ll, lll : TInt;
  s: string;
begin
  lll := 0;
  with SG do
  begin
    DefaultRowHeight := -Font.Height+6;
    for c := 0 to ColCount-1 do
    begin
      ll := 0;
      for r := 0 to RowCount-1 do
      begin
        s := 'W'+Cells[c,r];
        l := Canvas.TextWidth(s);
        ll := max(ll, l);
      end;
      ColWidths[c] := ll;
      lll := lll + ll+1;
    end;
    SG.ClientWidth := lll;
    SG.ScrollBars := ssNone;
    if SG.Width > SG.Parent.ClientWidth-dw then
    begin
      SG.Width := SG.Parent.ClientWidth-dw;
      SG.ScrollBars := ssHorizontal;
    end;
    SG.ClientHeight := (DefaultRowHeight+1)*RowCount;
    if SG.Height > SG.Parent.ClientHeight-dh then
    begin
      SG.Height := SG.Parent.ClientHeight-dh;
      if SG.ScrollBars = ssHorizontal then
        SG.ScrollBars := ssBoth
      else
        SG.ScrollBars := ssVertical;
    end;
  end;
end;

procedure FixStringGrid2(const SG: TStringGrid);
const
  d = 16;
  dh = 16;
  dw = 8;
var
  c, r, l, ll, lll : TInt;
  s: string;
begin
  lll := 0;
  with SG do
  begin
    DefaultRowHeight := -Font.Height+6;
    for c := 0 to ColCount-1 do
    begin
      ll := ColWidths[c];
      for r := 0 to RowCount-1 do
      begin
        s := 'W'+Cells[c,r];
        l := Canvas.TextWidth(s);
        ll := max(ll, l);
      end;
      ColWidths[c] := ll;
      lll := lll + ll+1;
    end;
    SG.ClientWidth := lll;
    SG.ScrollBars := ssNone;
    if SG.Width > SG.Parent.ClientWidth-dw then
    begin
      SG.Width := SG.Parent.ClientWidth-dw;
      SG.ScrollBars := ssHorizontal;
    end;
    SG.ClientHeight := (DefaultRowHeight+1)*RowCount;
    if SG.Height > SG.Parent.ClientHeight-dh then
    begin
      SG.Height := SG.Parent.ClientHeight-dh;
      if SG.ScrollBars = ssHorizontal then
        SG.ScrollBars := ssBoth
      else
        SG.ScrollBars := ssVertical;
    end;
  end;
end;

procedure CenterControl(Control: TControl);
var
  pw, l: integer;
begin
  pw := Control.Parent.ClientWidth;
  l := (pw - Control.Width) div 2;
  if l < 0 then l := 0;
  Control.Left := l;
end;

end.
