{@abstract(uGrid2D)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(June 06, 2002)
@lastmod(August 08, 2003)
Nikolai V. Shokhirev, 2002-2003 }
unit uGrid2D;

interface

uses
  classes, extctrls, uMatTypes, controls, Graphics, Windows, uDynArrays,
  uColorMap;

type

  TGrid2D = class(TPaintBox)
  private
    fCellColor: array of array of TColor;
    fLineColor: TColor;
    fLo1: TInt;
    fLo2: TInt;
    fHi1: TInt;
    fHi2: TInt;
    Dim1: TInt;
    Dim2: TInt;
    fBgColor: TColor;
    t, b, l, r: TInt;
    function GetCellColor(idx1, idx2: integer): TColor; virtual;
    procedure SetCellColor(idx1, idx2: integer; const Value: TColor); virtual;
    procedure ClearColors;
    procedure DrawLines;
    procedure PaintCells;
  public
    constructor Create(aOwner: TComponent; aParent: TWinControl);  reintroduce;
    destructor Destroy; override;
    procedure SetRanges(aLo1, aHi1, aLo2, aHi2: TInt);
    procedure ClearGrid;
    procedure PaintGrid;
    procedure ScreenToCell(X, Y: integer; var i2, i1: TInt);
    property Lo1: TInt read fLo1;
    property Hi1: TInt read fHi1;
    property Lo2: TInt read fLo2;
    property Hi2: TInt read fHi2;
    property LineColor: TColor read fLineColor write fLineColor;
    property BgColor: TColor read fBgColor write fBgColor;
    property CellColor[idx1,idx2: integer]: TColor read GetCellColor write SetCellColor; Default;
  end;

  TGrid2DM = class(TGrid2D)
  private
    fMatrix: IFArr2D;
    fMmin, fMmax: TFloat;
    fRGBSet: TRGBSet;
    procedure SetMatrix(const Value: IFArr2D);
    function GetValue(i1, i2: TInt): TFloat;
    procedure SetValue(i1, i2: TInt; const Value: TFloat);
    procedure GetMinMax;
    function ColorToNumb(aColor: TColor): TFloat;
    function NumbToColor(aNumb: TFloat): TColor;
    function GetMatrix: IFArr2D;
  public
    procedure MatrixToColors;
    procedure ColorsToMatrix;
    property RGBSet: TRGBSet read fRGBSet write fRGBSet;
    property Mmin: TFloat read fMmin write fMmin;
    property Mmax: TFloat read fMmax write fMmax;
    property Matrix: IFArr2D read GetMatrix write SetMatrix;
    property Values[i1,i2: TInt]: TFloat read GetValue write SetValue;
  end;

implementation

uses
  Math;

{ TPainterLight }

constructor TGrid2D.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited Create(aOwner);
  Parent := aParent;
  Align := alClient;
end;

procedure TGrid2D.ClearColors;
var
  i1, i2: TInt;
begin
  for i1 := 0 to Dim1-1 do
    for i2 := 0 to Dim2-1 do
      fCellColor[i1,i2] := fBgColor;
end;

procedure TGrid2D.SetRanges(aLo1, aHi1, aLo2, aHi2: TInt);
begin
  fLo1 := aLo1;
  fHi1 := aHi1;
  fLo2 := aLo2;
  fHi2 := aHi2;
  Dim1 := fHi1-fLo1+1;
  Dim2 := fHi2-fLo2+1;
  SetLength(fCellColor,Dim1,Dim2);
  fLineColor := clDkGray;
  fBgColor := clLtGray;
  ClearColors;
end;

destructor TGrid2D.Destroy;
begin
  SetLength(fCellColor,0,0);
  inherited;
end;

procedure TGrid2D.DrawLines;
var
  i1, i2, y, x: TInt;
begin
  Canvas.Brush.Color := fBgColor;
  Canvas.FillRect(Parent.ClientRect);
  r := Parent.ClientRect.Right-4;
  l := Parent.ClientRect.Left;
  t := Parent.ClientRect.Top;
  b := Parent.ClientRect.Bottom-4;

  Canvas.Pen.Color := fLineColor;
  for i1 := 0 to Dim1 do
  begin
    y := t+((b-t)*i1) div Dim1;
    Canvas.MoveTo(l,y);
    Canvas.LineTo(r,y);
  end;
  for i2 := 0 to Dim2 do
  begin
    x := l+((r-l)*i2) div Dim2;
    Canvas.MoveTo(x,t);
    Canvas.LineTo(x,b);
  end;
end;

procedure TGrid2D.ClearGrid;
begin
  ClearColors;
  Canvas.Brush.Color := fBgColor;
  Canvas.FillRect(Parent.ClientRect);
  DrawLines;
end;

procedure TGrid2D.PaintCells;
var
  i1, i2: TInt;
  Rect: TRect;
begin
  Canvas.Pen.Color := fLineColor;
  for i1 := 0 to Dim1-1 do
  begin
    Rect.Top := t+((b-t)*i1) div Dim1 + 2;
    Rect.Bottom := t+((b-t)*(i1+1)) div Dim1 - 1;
    for i2 := 0 to Dim2-1 do
    begin
      Rect.Left := l+((r-l)*i2) div Dim2 + 2;
      Rect.Right := l+((r-l)*(i2+1)) div Dim2 - 1;
      Canvas.Brush.Color := fCellColor[i1,i2];
      Canvas.FillRect(Rect);
    end;
  end;
end;

procedure TGrid2D.PaintGrid;
begin
  DrawLines;
  PaintCells;
end;

function TGrid2D.GetCellColor(idx1, idx2: integer): TColor;
begin
  result := fCellColor[idx1-fLo1,idx2-fLo2];
end;

procedure TGrid2D.SetCellColor(idx1, idx2: integer; const Value: TColor);
begin
  fCellColor[idx1-fLo1,idx2-fLo2] := Value;
end;

procedure TGrid2D.ScreenToCell(X, Y: integer; var i2, i1: TInt);
begin
  i2 := ((X-l)*Dim2) div (r-l)+fLo2;
  i1 := ((Y-t)*Dim1) div (b-t)+fLo1;
end;

{ TGrid2DM }

function TGrid2DM.NumbToColor(aNumb: TFloat): TColor;
var
  i: TInt;
begin  // 16777215 = 255*65793 = 255*(1 + 256 + 65536)
//  i := Round(255.0*(aNumb-fMmin)/(fMmax-fMmin));
//  result :=  TColor(65793*i);
//  result :=  TColor(65537*i);
//  result :=  TColor(Round(16777215.0*(aNumb-fMmin)/(fMmax-fMmin)));
//  result := NumberToColor((aNumb-fMmin)/(fMmax-fMmin),[cR,cG,CB]);
  result := NumberToColor((aNumb-fMmin)/(fMmax-fMmin), fRGBSet);
end;

function TGrid2DM.ColorToNumb(aColor: TColor): TFloat;
var
  x: TFloat;
  i: TInt;
begin
  i := integer(aColor);
  x := i;
  result := fMmin+x*(fMmax-fMmin)/16777215.0;
end;

procedure TGrid2DM.GetMinMax;
var
  i1, i2: integer;
begin
  fMmin := MaxFloat;
  fMmax := -MaxFloat;
  for i1 := Lo1 to Hi1 do
    for i2 := Lo2 to Hi2 do
    begin
      fMmin := min(fMmin,fMatrix[i1,i2]);
      fMmax := max(fMmax,fMatrix[i1,i2]);
    end;
end;

function TGrid2DM.GetValue(i1, i2: TInt): TFloat;
begin
  result := fMatrix[i1,i2];
end;

procedure TGrid2DM.MatrixToColors;
var
  i1, i2: integer;
begin
  GetMinMax;
  for i1 := Lo1 to Hi1 do
    for i2 := Lo2 to Hi2 do
      CellColor[i1,i2] := NumbToColor(fMatrix[i1,i2]);
end;

procedure TGrid2DM.SetMatrix(const Value: IFArr2D);
begin
  fMatrix := Value;
  SetRanges(fMatrix.Lo1, fMatrix.Hi1, fMatrix.Lo2, fMatrix.Hi2);
  MatrixToColors;
end;

procedure TGrid2DM.SetValue(i1, i2: TInt; const Value: TFloat);
begin
  fMatrix[i1,i2] := Value;
end;

function TGrid2DM.GetMatrix: IFArr2D;
begin
  result := fMatrix;
end;

procedure TGrid2DM.ColorsToMatrix;
var
  i1, i2: integer;
begin
  for i1 := Lo1 to Hi1 do
    for i2 := Lo2 to Hi2 do
      fMatrix[i1,i2] := ColorToNumb(CellColor[i1,i2]);
end;

end.
