{@abstract(uMapBox)
@author(Eugene B. Krissinel <keb@ebi.ac.uk> <krissinel@fh.huji.ac.il>)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(June 06, 2002)
@lastmod(August 08, 2003)
Original algorithm: Eugene B. Krissinel, 1991
Windows version: Nikolai V. Shokhirev, 2002-2005 }
{2005.04.09 - addded DrawFrame }
unit uMapBox;

interface

uses
  uMatTypes, uGraph2D, extctrls, uDynArrays, classes, controls, Graphics;

type

  TMapBox = class(TPlotBox2D)
  private
    fX, fY: IFArr1D;
    fZ: IFArr2D;
    fNx, fNy: integer;
    fnFunc: TInt;
    Bzx, Bzy: IBArr2D;
    fLevels: IFArr1D;
    fMinZ, fMaxZ: TFloat;
    fMinX, fMaxX, fMinY, fMaxY: TFloat;
    fMapColor: TColor;
    fMonochrom: boolean;
  public
    constructor Create(aOwner: TComponent; aParent: TWinControl);
    procedure InitData(const aZ: IFArr2D; const aX, aY: IFArr1D;
                                          const SetOnPaint: boolean = true);
    procedure SetLevels(const Levels: IFArr1D); overload;
    procedure SetLevels(NLevels: TInt); overload;
    procedure DrawLevel(Height: TFloat);
    procedure DrawFrame;
    procedure DrawMap;
    procedure BoxDrawMap(Sender: TObject);
    property MapColor: TColor read fMapColor write fMapColor;
    property Monochrom: boolean read fMonochrom write fMonochrom;
    property nFunc: TInt read fnFunc write fnFunc;
  end;

implementation

uses
  sysutils, math, uColorMap;

{ TMapBox }

constructor TMapBox.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited;
//  inherited Create;
  fMonochrom := false;
  fMapColor := clBlack;
end;

procedure TMapBox.DrawFrame;
begin
  SetLimits(fMinX, fMaxX, fMinY, fMaxY);
  PlotFrame;
end;

procedure TMapBox.DrawMap;
var
  n: integer;
  c: TFloat;
begin
  for n := 1 to fLevels.Dim1 do
  begin
    c := (fLevels[n]-fLevels[fLevels.Dim1])/
         (fLevels[1]-fLevels[fLevels.Dim1]);
    if Monochrom then
      PenColor := MapColor
    else
      PenColor := Rainbow(160.0*c, nFunc);
    DrawLevel(fLevels[n]);
  end;
end;

procedure TMapBox.BoxDrawMap(Sender: TObject);
begin
  SetLimits(fMinX, fMaxX, fMinY, fMaxY);
  PlotFrame;
  DrawMap;
end;

procedure TMapBox.InitData(const aZ: IFArr2D; const aX, aY: IFArr1D;
                                              const SetOnPaint: boolean = true);
var
  ix, iy: TInt;
begin
  fNy := aZ.Dim1;
  fNx := aZ.Dim2;
  if (fNx = aX.Dim1) and (fNy = aY.Dim1) then
  begin
    fZ := TFArr2D.Create(fNy, fNx);
    fZ.Assign(aZ);
    fX := TFArr1D.Create(fNx);
    fX.Assign(aX);
    fY := TFArr1D.Create(fNy);
    fY.Assign(aY);
    Bzy := TBArr2D.Create(fNy-1,fNx);
    Bzy.Fill(true);
    Bzx := TBArr2D.Create(fNy,fNx-1);
    Bzx.Fill(true);
    fMinY :=  MaxFloat;
    fMaxY := -MaxFloat;
    fMinX :=  MaxFloat;
    fMaxX := -MaxFloat;
    for iy := 1 to fNy do
    begin
      fMinY := Min(fMinY, fY[iy]);
      fMaxY := Max(fMaxY, fY[iy]);
    end;
    for ix := 1 to fNx do
    begin
      fMinX := Min(fMinX, fX[ix]);
      fMaxX := Max(fMaxX, fX[ix]);
    end;

  end else
    raise Exception.CreateFmt('Z.Dim1=%d <> Y.Dim1=%d AND/OR Z.Dim2=%d <> X.Dim1=%d',
                              [fNy,aY.Dim1,fNx,aX.Dim1]);
  if SetOnPaint then
    OnPaint := BoxDrawMap;
end;

procedure TMapBox.SetLevels(const Levels: IFArr1D);
begin
  if Assigned(fLevels) then fLevels := nil;
  fLevels := TFArr1D.Create(Levels.Dim1);
  fLevels.Assign(Levels);
end;

procedure TMapBox.SetLevels(NLevels: TInt);
var
  dz, zz: TFloat;
  ix, iy: TInt;
begin
  if Assigned(fLevels) then fLevels := nil;
  fLevels := TFArr1D.Create(NLevels);
  fMinZ :=  MaxFloat;
  fMaxZ := -MaxFloat;
  for iy := 1 to fNy do
    for ix := 1 to fNx do
    begin
      fMinZ := Min(fMinZ, fZ[iy, ix]);
      fMaxZ := Max(fMaxZ, fZ[iy, ix]);
    end;
  dz := (fMaxZ - fMinZ)/NLevels;
  zz := fMinZ + 0.5*dz;
  for ix := 1 to NLevels do
  begin
    fLevels[ix] := zz;
    zz := zz + dz;
  end;
end;

procedure TMapBox.DrawLevel(Height: TFloat);
var
  i,j: integer;

  procedure PaintIsoline(Fx, Fy, Key: integer);
  var
    ix, iy, kxy: integer;
    u, v, u1, v1: TFloat;
    d11, d12, d21, d22: TFloat;
    Done: boolean;

    procedure CrossLine(var uu, vv: TFloat; HV: integer);
    begin
      if HV = 1 then  {  Horizontal line   }
      begin
        Done := not Bzx[iy,ix];
        Bzx[iy,ix] := false;
        vv := self.fY[iy];
        uu := self.fX[ix] + (Height-self.fZ[iy,ix])*(self.fX[ix+1]-self.fX[ix])
                                               /(self.fZ[iy,ix+1]-self.fZ[iy,ix]);
      end else  {  Vertical line     }
      begin
        Done := not Bzy[iy,ix];
        Bzy[iy,ix] := false;
        uu := self.fX[ix];
        vv := self.fY[iy] + (Height-self.fZ[iy,ix])*(self.fY[iy+1]-self.fY[iy])
                                               /(self.fZ[iy+1,ix]-self.fZ[iy,ix]);
      end;
    end;{ of CrossLine }

    procedure ComputeTops;
    begin
      d11 := fZ[iy,ix]     - Height;
      d21 := fZ[iy+1,ix]   - Height;
      d12 := fZ[iy,ix+1]   - Height;
      d22 := fZ[iy+1,ix+1] - Height;
    end;{ of ComputeTops }

  begin {of PaintIsoline }
   ix  := Fx;
   iy  := Fy;
   kxy := Key;

   CrossLine(u,v,kxy);
   u1 := u;
   v1 := v;
   repeat
     Done := true;
     case  kxy  of
     1:if iy < fNy  then  {  entering from the bottom  }
       begin
         ComputeTops;
         if d21*d22 < 0.0  then
         begin
           inc(iy);
           CrossLine(u1,v1,1);
         end
         else if d12*d22 < 0.0  then
         begin
           inc(ix);
           CrossLine(u1,v1,2);
           kxy := 2;
         end
         else if d21*d11 < 0.0  then
           begin
             CrossLine(u1,v1,2);
             kxy := 4;
           end
       end;
     2:if ix < fNx  then  {  entering from the left  }
       begin
         ComputeTops;
         if d12*d22 < 0.0  then
         begin
           inc(ix);
           CrossLine(u1,v1,2);
         end
         else if d12*d11 < 0.0  then
         begin
           CrossLine(u1,v1,1);
           kxy := 3;
         end
         else if d21*d22<0.0  then
         begin
           inc(iy);
           CrossLine(u1,v1,1);
           kxy := 1;
         end
       end;
     3:if iy > 1  then  {  entering from the top  }
       begin
         dec(iy);
         ComputeTops;
         if d11*d12 < 0.0  then
           CrossLine(u1,v1,1)
         else if d21*d11 < 0.0  then
           begin
             CrossLine(u1,v1,2);
             kxy := 4;
           end
         else if d12*d22 < 0.0  then
         begin
           inc(ix);
           CrossLine(u1,v1,2);
           kxy := 2;
         end
       end;
     4:if ix > 1  then  {  entering from the right  }
       begin
         dec(ix);
         ComputeTops;
         if d11*d21<0.0  then
           CrossLine(u1,v1,2)
         else if d21*d22 < 0.0  then
           begin
             inc(iy);
             CrossLine(u1,v1,1);
             kxy := 1;
           end
         else if d11*d12 < 0.0  then
           begin
             CrossLine(u1,v1,1);
             kxy := 3;
           end
       end;
     end;
     self.LineF(u,v,u1,v1);
     u := u1;
     v := v1;
   until  Done;

  end;{ of PaintIsoline }

begin{ of DrawLevel }

  for j:=1 to fNy  do
    for i:=1 to fNx-1  do
      Bzx[j,i] := true;

  for i:=1 to fNx  do
    for j:=1 to fNy-1  do
      Bzy[j,i] := true;

  for j:=1 to fNy  do
    for i:=1 to fNx  do
      if fZ[j,i]=Height  then
        fZ[j,i] := Height+1.0e-5;

  for j:=1 to fNy  do
    for i:=1 to fNx-1  do
      if Bzx[j,i] and
         (((fZ[j,i] >= Height) and (fZ[j,i+1] <= Height)) or
          ((fZ[j,i] <= Height) and (fZ[j,i+1] >= Height)))  then
        PaintIsoline (i,j,1);

  for i:=1 to fNx  do
    for j:=1 to fNy-1  do
      if Bzy[j,i] and
         (((fZ[j,i] >= Height) and (fZ[j+1,i] <= Height)) or
          ((fZ[j,i] <= Height) and (fZ[j+1,i] >= Height)))  then
        PaintIsoline (i,j,2);
end; { of DrawLevel }

end.
