{@abstract(TPainterLight and TPainter classes)
@author(Nikolai Shokhirev <nikolai@shokhirev.com>)
@created(June 6, 2002)
@lastmod(November 2, 2002)
Nikolai V. Shokhirev, 2002-2003 }
unit uPainter;

interface

uses
  classes, extctrls, uMatTypes, controls, Graphics, Windows;

type

  TPainterLight = class(TPaintBox)
  private
    fXmin: TFloat;
    fXmax: TFloat;
    fYmin: TFloat;
    fYmax: TFloat;
    fNXmin: TInt;
    fNXmax: TInt;
    fNYmin: TInt;
    fNYmax: TInt;
    fIWidth: TFloat;
    fIHeight: TFloat;
    fRWidth: TFloat;
    fRHeight: TFloat;
    fXYRatio: TFloat;
    fPPUX, fPPUY: TFloat;
    fBgColor: TColor; // used in Clear
    function GetXmin: TFloat;
    function GetXmax: TFloat;
    function GetYmin: TFloat;
    function GetYmax: TFloat;
    procedure SetXmin(const Value: TFloat);
    procedure SetXmax(const Value: TFloat);
    procedure SetYmin(const Value: TFloat);
    procedure SetYmax(const Value: TFloat);
    function GetXYRatio: TFloat;
    procedure SetXYRatio(const Value: TFloat);
    function GetPPUX: TFloat;
    function GetPPUY: TFloat;
    function GetBgColor: TColor;
    procedure SetBgColor(const Value: TColor);
    function GetBrushColor: TColor;
    function GetPenColor: TColor;
    procedure SetBrushColor(const Value: TColor);
    procedure SetPenColor(const Value: TColor);
    function GetPenWidth: integer;
    procedure SetPenWidth(const Value: integer);
  public
    constructor Create(aOwner: TComponent; aParent: TWinControl);  reintroduce;
    destructor Destroy; override;
    procedure SetRanges(aXmin, aXmax, aYmin, aYmax, aXYRatio: TFloat);
    procedure SetPPU;
    procedure Clear;
    function rRect2iRect(x1, x2, y1, y2: TFloat): TRect;
    function rx2ix(x: TFloat): TInt;
    function ry2iy(y: TFloat): TInt;
    function ix2rx(ix: TInt): TFloat;
    function iy2ry(iy: TInt): TFloat;
    procedure MoveTo(x, y: TFloat);
    procedure LineTo(x, y: TFloat);
    procedure Line(x1, y1, x2, y2: TFloat);
    procedure Circle(x, y, r: TFloat);
    property Xmin: TFloat read GetXmin write SetXmin;
    property Xmax: TFloat read GetXmax write SetXmax;
    property Ymin: TFloat read GetYmin write SetYmin;
    property Ymax: TFloat read GetYmax write SetYmax;
    property PPUX: TFloat read GetPPUX;
    property PPUY: TFloat read GetPPUY;
    property XYRatio: TFloat read GetXYRatio write SetXYRatio;
    property BgColor: TColor read GetBgColor write SetBgColor;
    property PenColor: TColor read GetPenColor write SetPenColor;
    property PenWidth: integer read GetPenWidth write SetPenWidth;
    property BrushColor: TColor read GetBrushColor write SetBrushColor;
  end;

  TPainter = class(TPainterLight)
  private
    fDelta: TFloat;
    fXOrigin: TFloat;  // Origin
    fYOrigin: TFloat;  // Origin
    fInitialAngle: TFloat;
    fCurrentAngle: TFloat;
    fMouseIsDown: boolean;
    function GetXOrigin: TFloat;
    function GetYOrigin: TFloat;
    procedure SetXOrigin(const Value: TFloat);
    procedure SetYOrigin(const Value: TFloat);
    function GetDelta: TFloat;
    procedure SetDelta(const Value: TFloat);
    function GetMouseIsDown: boolean;
    procedure SetMouseIsDown(const Value: boolean);
    function GetCurrentAngle: TFloat;
    function GetInitialAngle: TFloat;
    procedure SetCurrentAngle(const Value: TFloat);
    procedure SetInitialAngle(const Value: TFloat);
  public
    constructor Create(aOwner: TComponent; aParent: TWinControl); //override;
//    destructor Destroy; override;
    procedure SetRanges(aXmin, aXmax, aYmin, aYmax, aXYRatio: TFloat);
    procedure LineC(x1, y1, x2, y2: TFloat; c: TColor);
    procedure CircleC(x, y, r: TFloat; c: TColor);
    function Angle(x, y: TFloat): TFloat;
    function RelAngle(x2, y2, x1, y1: TFloat): TFloat;
    function HitLine(x1, y1, x2, y2: TFloat; ix, iy: TInt): boolean;
    function HitDot(rx, ry: TFloat; ix, iy: TInt): boolean;
    property Delta: TFloat read GetDelta write SetDelta;
    property XOrigin: TFloat read GetXOrigin write SetXOrigin;
    property YOrigin: TFloat read GetYOrigin write SetYOrigin;
    property InitialAngle: TFloat read GetInitialAngle write SetInitialAngle;
    property CurrentAngle: TFloat read GetCurrentAngle write SetCurrentAngle;
    property MouseIsDown: boolean read GetMouseIsDown write SetMouseIsDown;
  end;

implementation

uses
  Math;

{ TPainterLight }

constructor TPainterLight.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited Create(aOwner);
  Parent := aParent;
  Align := alClient;
end;

procedure TPainterLight.SetRanges(aXmin, aXmax, aYmin, aYmax, aXYRatio: TFloat);
begin
  fXmin := aXmin;
  fXmax := aXmax;
  fYmin := aYmin;
  fYmax := aYmax;
  fXYRatio := aXYRatio;
  SetPPU;
end;

destructor TPainterLight.Destroy;
begin
  inherited;
end;

procedure TPainterLight.SetPPU;
begin
  fIWidth := ClientWidth;
  fIHeight := ClientHeight;
  fRWidth := fXmax-fXmin;
  fRHeight := fYmax-fYmin;
  fNXmin := 0;
  fNXmax := ClientWidth-1;
  fNYmin := 0;
  fNYmax := ClientHeight-1;
  if fXYRatio > 0 then
    if(1.0 > fRWidth*fIHeight*fXYRatio/fRHeight/fIWidth) then
    begin
      fPPUY := fIHeight/fRHeight;
      fPPUX := fPPUY*fXYRatio;
      fNXmin := (ClientWidth-round(fPPUX*fRWidth)) div 2;
      fNXmax := ClientWidth - fNXmin - 1;
    end else
    begin
      fPPUX := fIWidth/fRWidth;
      fPPUY := fPPUX/fXYRatio;
      fNYmin := (ClientHeight-round(fPPUY*fRHeight)) div 2;
      fNYmax := ClientHeight - fNYmin - 1;
    end
  else
  begin
    fPPUX := fIWidth/fRWidth;
    fPPUY := fIHeight/fRHeight;
  end;
end;

function TPainterLight.GetXYRatio: TFloat;
begin
  result := fXYRatio;
end;

procedure TPainterLight.SetXYRatio(const Value: TFloat);
begin
  fXYRatio := Value;
  SetPPU;
end;

function TPainterLight.GetPPUX: TFloat;
begin
  result := fPPUX;
end;

function TPainterLight.GetPPUY: TFloat;
begin
  result := fPPUY;
end;

function TPainterLight.rRect2iRect(x1, x2, y1, y2: TFloat): TRect;
begin
  result := Rect(rx2ix(x1), ry2iy(y2), rx2ix(x2), ry2iy(y1));
end;

function TPainterLight.rx2ix(x: TFloat): TInt;
begin
  result := fNXmin + round(PPUX*(x-fXmin));
end;

function TPainterLight.ry2iy(y: TFloat): TInt;
begin
  result := fNYmax - round(PPUY*(y-fYmin));
end;

function TPainterLight.ix2rx(ix: TInt): TFloat;
begin
  result := fXmin + (ix-fNXmin)/fPPUX;
end;

function TPainterLight.iy2ry(iy: TInt): TFloat;
begin
  result := fYmin + (fNYmax-iy)/fPPUY;
end;

function TPainterLight.GetXmin: TFloat;
begin
  result := fXmin;
end;

function TPainterLight.GetXmax: TFloat;
begin
  result := fXmax;
end;

function TPainterLight.GetYmin: TFloat;
begin
  result := fYmin;
end;

function TPainterLight.GetYmax: TFloat;
begin
  result := fYmax;
end;

procedure TPainterLight.SetXmin(const Value: TFloat);
begin
  fXmin := Value;
  SetPPU;
end;

procedure TPainterLight.SetXmax(const Value: TFloat);
begin
  fXmax := Value;
  SetPPU;
end;

procedure TPainterLight.SetYmin(const Value: TFloat);
begin
  fYmin := Value;
  SetPPU;
end;

procedure TPainterLight.SetYmax(const Value: TFloat);
begin
  fYmax := Value;
  SetPPU;
end;

procedure TPainterLight.LineTo(x, y: TFloat);
begin
  with Canvas do
    LineTo(rx2ix(x),ry2iy(y));
end;

procedure TPainterLight.MoveTo(x, y: TFloat);
begin
  with Canvas do
    MoveTo(rx2ix(x),ry2iy(y));
end;

procedure TPainterLight.Line(x1, y1, x2, y2: TFloat);
begin
  with Canvas do
  begin
    MoveTo(rx2ix(x1),ry2iy(y1));
    LineTo(rx2ix(x2),ry2iy(y2));
  end;
end;

procedure TPainterLight.Circle(x, y, r: TFloat);
var
  ix, iy, irx, iry: TInt;
begin
  ix := rx2ix(x);
  iy := ry2iy(y);
  irx := round(r*PPUX);
  iry := round(r*PPUY);
  with Canvas do
  begin
    Ellipse(ix-irx,iy-iry,ix+irx,iy+iry);
  end;
end;

procedure TPainterLight.Clear;
begin
  with Canvas do
  begin
    Brush.Color := fBgColor;
    FillRect(Parent.ClientRect);
  end;
end;

function TPainterLight.GetBgColor: TColor;
begin
  result := fBgColor;
end;

procedure TPainterLight.SetBgColor(const Value: TColor);
begin
  fBgColor := Value
end;

function TPainterLight.GetBrushColor: TColor;
begin
  result := Canvas.Brush.Color;
end;

function TPainterLight.GetPenColor: TColor;
begin
  result := Canvas.Pen.Color;
end;

procedure TPainterLight.SetBrushColor(const Value: TColor);
begin
  Canvas.Brush.Color := Value;

end;

procedure TPainterLight.SetPenColor(const Value: TColor);
begin
  Canvas.Pen.Color := Value;
end;

function TPainterLight.GetPenWidth: integer;
begin
  result := Canvas.Pen.Width;
end;

procedure TPainterLight.SetPenWidth(const Value: integer);
begin
  Canvas.Pen.Width := Value;
end;

{ Painter }

constructor TPainter.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited Create( aOwner, aParent);
  InitialAngle := 0.0;
  CurrentAngle := 0.0;
  MouseIsDown := false;
end;

procedure TPainter.SetRanges(aXmin, aXmax, aYmin, aYmax, aXYRatio: TFloat);
begin
  inherited SetRanges(aXmin, aXmax, aYmin, aYmax, aXYRatio);
  fXOrigin := 0.5*(Xmax+Xmin);
  fYOrigin := 0.5*(Ymax+Ymin);
end;

(*
destructor TPainter.Destroy;
begin
  inherited;
end;
*)

function TPainter.GetXOrigin: TFloat;
begin
  result := fXOrigin;
end;

procedure TPainter.SetXOrigin(const Value: TFloat);
begin
  fXOrigin := Value;
end;

function TPainter.GetYOrigin: TFloat;
begin
  result := fYOrigin;
end;

procedure TPainter.SetYOrigin(const Value: TFloat);
begin
  fYOrigin := Value;
end;

procedure TPainter.LineC(x1, y1, x2, y2: TFloat; c: TColor);
var
  pc: TColor;
begin
  with Canvas do
  begin
    pc := Pen.Color;
    Pen.Color := c;
    MoveTo(rx2ix(x1),ry2iy(y1));
    LineTo(rx2ix(x2),ry2iy(y2));
    Pen.Color := pc;
  end;
end;

procedure TPainter.CircleC(x, y, r: TFloat; c: TColor);
var
  ix, iy, irx, iry: TInt;
  bc, pc: TColor;
begin
  ix := rx2ix(x);
  iy := ry2iy(y);
  irx := round(r*PPUX);
  iry := round(r*PPUY);
  with Canvas do
  begin
    bc := Brush.Color;
    pc := Pen.Color;
    Brush.Color := c;
    Pen.Color := c;
    Ellipse(ix-irx,iy-iry,ix+irx,iy+iry);
    Brush.Color := bc;
    Pen.Color := pc;
  end;
end;

function TPainter.Angle(x, y: TFloat): TFloat;
begin
  result := ArcTan2(y-YOrigin,x-XOrigin);
end;

function TPainter.RelAngle(x2, y2, x1, y1: TFloat): TFloat;
begin
  result := ArcTan2(-(x2-XOrigin)*(y1-YOrigin)+(y2-YOrigin)*(x1-XOrigin),
                     (x2-XOrigin)*(x1-XOrigin)+(y2-YOrigin)*(y1-YOrigin));
end;

function TPainter.GetDelta: TFloat;
begin
  result := fDelta;
end;

procedure TPainter.SetDelta(const Value: TFloat);
begin
  fDelta := Value;
end;

function TPainter.HitDot(rx, ry: TFloat; ix, iy: TInt): boolean;
begin
  result := (sqr(rx-ix2rx(ix))+sqr(ry-iy2ry(iy))) < sqr(fDelta);
end;

function TPainter.HitLine(x1, y1, x2, y2: TFloat; ix, iy: TInt): boolean;
var
  rr, t, x, y, x0, y0: TFloat;
begin
  x := x2-x1;
  y := y2-y1;
  x0 := ix2rx(ix)-x1;
  y0 := iy2ry(iy)-y1;
  rr := sqr(x)+sqr(y);
  t := (x*x0+y*y0)/rr;
  result := (t>0.0) and (t<1.0) and (sqr(x0)+sqr(y0)-sqr(t)*rr< sqr(fDelta));
end;

function TPainter.GetMouseIsDown: boolean;
begin
  Result := fMouseIsDown;
end;

procedure TPainter.SetMouseIsDown(const Value: boolean);
begin
  fMouseIsDown := Value;
end;

function TPainter.GetCurrentAngle: TFloat;
begin
  result := fCurrentAngle;
end;

procedure TPainter.SetCurrentAngle(const Value: TFloat);
begin
  fCurrentAngle := Value;
end;

function TPainter.GetInitialAngle: TFloat;
begin
  result := fInitialAngle;
end;

procedure TPainter.SetInitialAngle(const Value: TFloat);
begin
  fInitialAngle := Value;
end;

end.
