{*
 PasMatLibs Nikolai V. Shokhirev, 2001-2009
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2009.06.25 - created: convcersion from CppMatLib
}
unit uRainbow;

interface

const
  clR = 255;
  clG = 65280;
  clB = 16711680;

type
  TRGB2 = ( RG, RB, GR, GB, BR, BG);

  function rainbow(t: double): integer;
  function rainbow1(t: double): integer;

  function PositiveNegative(pair: TRGB2; t: double): integer;
  function Transition(pair: TRGB2; t: double): integer;

implementation

function rainbow(t: double): integer;
var
  i: integer;
  tt: double;
begin
  if( (t < 0.0) or (t > 1.0) ) then
  begin
    result := 0;
    exit;
  end;

  tt := t*5.0;
  if( tt < 1.0) then
  begin
    i := Trunc( 255*tt );
    result := (clR + (i shl 8) );
  end
  else if ( tt < 2.0) then
  begin
    i := Trunc( 255*(2.0-tt) );
    result := (clG + i );
  end
  else if ( tt < 3.0) then
  begin
    i := Trunc( 255*(tt-2.0) );
    result := (clG + (i shl 16) );
  end
  else if ( tt < 4.0) then
    begin i := Trunc( 255*(4.0-tt) );
    result := (clB + (i shl 8) );
  end
  else
  begin
    i := Trunc( 255*(tt-4.0) );
    result := (clB + i );
  end;
end;

function rainbow1(t: double): integer;
var
  i: integer;
  tt: double;
begin
  if( (t < 0.0) or (t > 1.0) ) then
  begin
    result := 0;
    exit;
  end;

  tt := t*6.0;
  if( tt < 1.0) then
  begin
    i := Trunc( 255*tt );
    result := (clR + (i shl 8) );
  end
  else if ( tt < 2.0) then
  begin
    i := Trunc( 255*(2.0-tt) );
    result := (clG + i );
  end
  else if ( tt < 3.0) then
  begin
    i := Trunc( 255*(tt-2.0) );
    result := (clG + (i shl 16) );
  end
  else if ( tt < 4.0) then
  begin
    i := Trunc( 255*(4.0-tt) );
    result := (clB + (i shl 8) );
  end
  else if ( tt < 5.0) then
  begin
    i := Trunc( 255*(tt-4.0) );
    result := (clB + i );
  end
  else
  begin
    i := Trunc( 255*(6.0-tt) );
    result := (clR + (i shl 16) );
  end;
end;

function PositiveNegative(pair: TRGB2; t: double): integer;
var
  s1, s2, s, c, cc: integer;
begin
  case pair of
    RG: begin s1 :=  0; s2 :=  8; s := 16; end;
    RB: begin s1 :=  0; s2 := 16; s :=  8; end;
    GR: begin s1 :=  8; s2 :=  0; s := 16; end;
    GB: begin s1 :=  8; s2 := 16; s :=  0; end;
    BR: begin s1 := 16; s2 :=  0; s :=  8; end;
    BG: begin s1 := 16; s2 :=  8; s :=  0; end;
    else
      s1 := 0; s2 :=  0; s :=  0;
  end;

  if(abs(t)>1.0) then
  begin // c = R
    c := Trunc(255.0/abs(t));
    cc := 0;
  end else
  begin // c = B
    c := 255;
    cc := Trunc(255.0*(1.0-abs(t)));
  end;
  
  if (t>0.0) then
    result := ( (c shl s1) + (cc shl s) )
  else
    result := ( (c shl s2) + (cc shl s) );
end;

function Transition(pair: TRGB2; t: double): integer;
var
  s1, s2, c, cc: integer;
begin

  case pair of
    RG: begin s1 :=  0; s2 :=  8; end;
    RB: begin s1 :=  0; s2 := 16; end;
    GR: begin s1 :=  8; s2 :=  0; end;
    GB: begin s1 :=  8; s2 := 16; end;
    BR: begin s1 := 16; s2 :=  0; end;
    BG: begin s1 := 16; s2 :=  8; end;
    else
      s1 := 0; s2 :=  0;
  end;

  if(abs(t)>1.0) then
  begin // c = R
    c := Trunc(255.0/abs(t));
    cc := 0;
  end else
  begin // c = B
    c := 255;
    cc := Trunc(255.0*(1.0-abs(t)));
  end;
  
  if (t>0.0) then
    result := ( (c shl s1) + (cc shl s2) )
  else
    result := ( (c shl s2) + (cc shl s1) );
end;

end.
