{*
 String grid routines and helpers
 Nikolai V. Shokhirev, 2001-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2006.10.09 - created
 @version 2007.08.14 - Added DelphiCodeToDoc comments
 }
unit uSGUtils;

interface

uses
  Classes, SysUtils, Grids, uMatTypes, Controls
  , uPanelData;

{*
  Compare function for alphabetic sort; does not move the first row
  @param Index1 Index1
  @param Index2 Index2
  @return AnsiCompareStr(List[Index1], List[Index2]);
}
function SGSortA(List: TStringList; Index1, Index2: Integer): Integer;

{*
  Numeric sort; does not move the first row
  @param Index1 Index1
  @param Index2 Index2
  @return sign(StrToFloatDef(List[Index1]) - StrToFloatDef(List[Index2]));
}
function SGSortN(List: TStringList; Index1, Index2: Integer): Integer;

{*
  Date sort; does not move the first row
  @param Index1 Index1
  @param Index2 Index2
  @return sign(Date(List[Index1]) - Date(List[Index2]));
}
function SGSortD(List: TStringList; Index1, Index2: Integer): Integer;

{*
// By Erwien Ekosaputra - erwieneko@bigfoot.com
}
procedure SortGrid (AGrid : TStringGrid; const SortIndex : integer;
                                              Compare: TStringListSortCompare);

{*
  Call to adjust dimensions
}
procedure FixStringGrid(const SG: TStringGrid; dh: TInt = 16; dw: TInt = 8);

{*
  Call to adjust dimensions
}
procedure FixStringGrid2(const SG: TStringGrid);

{*
// Loads comma-separeted table into a string grid
}
procedure LoadStringGrid(const SG: TStringGrid; const FileName: string;
                         var Comment: string);

{*
 Saves string grid into a comma-separeted table
}
procedure SaveStringGrid(const SG: TStringGrid; const FileName: string;
                         const Comment: string = '');

{*
  Clears cells

  @param MinRow Starting row
  @param MinCol Starting column
}
procedure ClearStringGrid(const SG: TStringGrid;
                                MinRow: integer = 1; MinCol: integer = 1);

{*
}
procedure StringGridAddCol(const SG: TStringGrid; Col: integer; const Title: string);

{*
}
procedure StringGridAddRow(const SG: TStringGrid; Row: integer; const Title: string);

{*
}
procedure StringGridDelCol(const SG: TStringGrid; Col: integer);

{*
}
procedure StringGridDelRow(const SG: TStringGrid; Row: integer);

{*
// for two-column grid
}
procedure StrGridToStrList(const SG: TStringGrid; const SL: TStringList);

{*
// for two-column grid
}
procedure StrListToStrGrid(const SG: TStringGrid; const SL: TStringList);

{*
procedure FixTwoColSG(const SG: TStringGrid; width: integer = -1;
}
procedure FixTwoColSG(const SG: TStringGrid; width: integer = -1;
                                             FixHeight: boolean = false);

///procedure PanelDataToStringGrid(const PD: IPanelData; const SG: TStringGrid);
procedure PanelDataToStringGrid(const PD: IPanelData; const SG: TStringGrid;
                                IsEditable: boolean = true);

///procedure StringGridToPanelData(const SG: TStringGrid; const PD: IPanelData);
procedure StringGridToPanelData(const SG: TStringGrid; const PD: IPanelData);

///procedure PanelDataToColumn(const PD: IPanelData; const SG: TStringGrid; Col: TInt);
procedure PanelDataToSGColumn(const PD: IPanelData; const SG: TStringGrid; Col: TInt);

///procedure SGColumnToPanelData(const PD: IPanelData; const SG: TStringGrid; Col: TInt);
procedure SGColumnToPanelData(const PD: IPanelData; const SG: TStringGrid; Col: TInt);
(*
{*
}
function AddSection(const SL: TStringList; const Section: string): boolean;

{*
}
function DelSection(const SL: TStringList; const Section: string): boolean;

{*
}
function AddKeyValue(const SL: TStringList;
                     const Section, Key, Value: string): boolean;

{*
}
function DelKey(const SL: TStringList; const Section, Key: string): boolean;

{*
}
function ModifyValue(const SL: TStringList;
                     const Section, Key, Value: string): boolean;
*)
type
  TStringGridHelper = class
  private
    //fPrecision: integer;
    //fDigits: integer;
    //fFormat: TFloatFormat;
    fSG: TStringGrid;
    //function FormatToStr(aFormat: TFloatFormat): string;
    //function StrToFormat(sFormat: string): TFloatFormat;
    function GetString(aRow, aCol: Integer): string;
    procedure SetString(aRow, aCol: Integer; const Value: string);
    function GetFloat(aRow, aCol: Integer): TFloat;
    function GetInt(aRow, aCol: Integer): TInt;
    procedure SetFloat(aRow, aCol: Integer; const Value: TFloat);
    procedure SetInt(aRow, aCol: Integer; const Value: TInt);
    function GetRowCount: integer;
    procedure SetRowCount(const Value: integer);
    function GetColCount: integer;
    procedure SetColCount(const Value: integer);
    function GetDateTime(aRow, aCol: Integer): TDateTime;
    procedure SetDateTime(aRow, aCol: Integer; const Value: TDateTime);
    //function GetFormat: TFloatFormat;
    //procedure SetFormat(const Value: TFloatFormat);
  public
    constructor Create(SG: TStringGrid);
    property AsString[aRow, aCol: Integer]: string read GetString write SetString;
                                                                        Default;
    property AsInt[aRow, aCol: Integer]: TInt read GetInt write SetInt;
    property AsFloat[aRow, aCol: Integer]: TFloat read GetFloat write SetFloat;
    property AsDateTime[aRow, aCol: Integer]: TDateTime read GetDateTime write SetDateTime;
    property RowCount: integer read GetRowCount write SetRowCount;
    property ColCount: integer read GetColCount write SetColCount;
//    property Format: TFloatFormat read GetFormat write SetFormat;
//    property Digits: integer read fDigits write fDigits;
//    property Precision: integer read fPrecision write fPrecision;
  end;
(*
  TStringGridX = class(TStringGrid)
  private
    fPrecision: integer;
    fDigits: integer;
    fFormat: TFloatFormat;
    function FormatToStr(aFormat: TFloatFormat): string;
    function StrToFormat(sFormat: string): TFloatFormat;
    function GetString(aRow, aCol: Integer): string;
    procedure SetString(aRow, aCol: Integer; const Value: string);
    function GetFloat(aRow, aCol: Integer): TFloat;
    function GetInt(aRow, aCol: Integer): TInt;
    procedure SetFloat(aRow, aCol: Integer; const Value: TFloat);
    procedure SetInt(aRow, aCol: Integer; const Value: TInt);
    function GetFormat: TFloatFormat;
    procedure SetFormat(const Value: TFloatFormat);
  public
    constructor Create(aOwner: TComponent; aParent: TWinControl);
    property AsString[aRow, aCol: Integer]: string read GetString write SetString;
    property AsInt[aRow, aCol: Integer]: TInt read GetInt write SetInt;
    property AsFloat[aRow, aCol: Integer]: TFloat read GetFloat write SetFloat;
    property Format: TFloatFormat read GetFormat write SetFormat;
    property Digits: integer read fDigits write fDigits;
    property Precision: integer read fPrecision write fPrecision;
  end;
*)
implementation

uses
  DateUtils, Math, StdCtrls, uDisplay;

const
   FixedCols = 1;
   FixedRows = 3;

/// Alphabetic sort; does not move the first row
function SGSortA(List: TStringList; Index1, Index2: Integer): Integer;
begin
  if Index1=Index2 then
    result := 0
  else if Index1 = 0 then
    result := -1
  else if Index2 = 0 then
    result := 1
  else
    result := AnsiCompareStr(List[Index1], List[Index2]);
end;

/// Numeric sort; does not move the first row
function SGSortN(List: TStringList; Index1, Index2: Integer): Integer;
begin
  if Index1=Index2 then
    result := 0
  else if Index1 = 0 then
    result := -1
  else if Index2 = 0 then
    result := 1
  else
    result := sign(StrToFloatDef(List[Index1]) - StrToFloatDef(List[Index2]));
end;

/// Date sort; does not move the first row
function SGSortD(List: TStringList; Index1, Index2: Integer): Integer;
var
  date1, date2: TDateTime;
  y, m, d, day0, day1, day2: Word;
begin
  if Index1=Index2 then
    result := 0
  else if Index1 = 0 then
    result := -1
  else if Index2 = 0 then
    result := 1
  else
  begin
    day0 := DayOfTheYear(Date);
    y := YearOf(Date);

    date1 := StrToDate(List[Index1]);
    m := MonthOf(date1);               //
    d := DayOf(date1);                 //
    date1 := EncodeDate(y, m, d);      //
    date2 := StrToDate(List[Index2]);
    m := MonthOf(date2);               //
    d := DayOf(date2);                 //
    date2 := EncodeDate(y, m, d);      //
    day1 := DayOfTheYear(date1);
    day2 := DayOfTheYear(date2);
    if day1 < day0 then day1 := day1 + DaysInAYear(y);
    if day2 < day0 then day2 := day2 + DaysInAYear(y);
    result := day1 - day2;
  end;
end;

/// By Erwien Ekosaputra - erwieneko@bigfoot.com
procedure SortGrid (AGrid : TStringGrid; const SortIndex : integer;
                                              Compare: TStringListSortCompare);
var
  SortStringList : TStringList;
  RowLoop : integer;
  RowValues : TStringList;
begin
  ///Creating the string list to sort
  SortStringList := TStringList.Create;
  try
    ///Iterating for all rows in a grid.
    for RowLoop := 0 to AGrid.RowCount-1 do
    begin
      //Creating a stringlist as a holder, reading a row from stringgrid and
      //add it into object property of StringList.
      RowValues := TStringList.Create;
      RowValues.Assign (AGrid.Rows [RowLoop]);
      SortStringList.AddObject (AGrid.Cells [SortIndex, RowLoop], RowValues);
    end;

    ///Sort
    SortStringList.CustomSort(@Compare);
//    SortStringList.Sorted := True;

    ///Reassigning sorted rows
    for RowLoop := 0 to SortStringList.Count-1 do
      AGrid.Rows [RowLoop].Assign (TStrings (SortStringList.Objects [RowLoop]));

  finally
    for RowLoop := 0 to SortStringList.Count-1 do
      TStringList (SortStringList.Objects [RowLoop]).Free;
    SortStringList.Free;
  end;
end;

///   Call to adjust dimensions
procedure FixStringGrid(const SG: TStringGrid; dh: TInt = 16; dw: TInt = 8);
const
  d = 16;
var
  c, r, l, ll, lll : TInt;
  s: string;
begin
  lll := 0;
  with SG do
  begin
    DefaultRowHeight := -Font.Height+6;
    for c := 0 to ColCount-1 do
    begin
      ll := 0;
      for r := 0 to RowCount-1 do
      begin
        s := 'W'+Cells[c,r];
        l := Canvas.TextWidth(s);
        ll := max(ll, l);
      end;
      ColWidths[c] := ll;
      lll := lll + ll+1;
    end;
    SG.ClientWidth := lll;
    SG.ScrollBars := ssNone;
    if SG.Width > SG.Parent.ClientWidth-dw then
    begin
      SG.Width := SG.Parent.ClientWidth-dw;
      SG.ScrollBars := ssHorizontal;
    end;
    SG.ClientHeight := (DefaultRowHeight+1)*RowCount;
    if SG.Height > SG.Parent.ClientHeight-dh then
    begin
      SG.Height := SG.Parent.ClientHeight-dh;
      if SG.ScrollBars = ssHorizontal then
        SG.ScrollBars := ssBoth
      else
        SG.ScrollBars := ssVertical;
    end;
  end;
end;

procedure FixStringGrid2(const SG: TStringGrid);
const
  d = 16;
  dh = 16;
  dw = 8;
var
  c, r, l, ll, lll : TInt;
  s: string;
begin
  lll := 0;
  with SG do
  begin
    DefaultRowHeight := -Font.Height+6;
    for c := 0 to ColCount-1 do
    begin
      ll := ColWidths[c];
      for r := 0 to RowCount-1 do
      begin
        s := 'W'+Cells[c,r];
        l := Canvas.TextWidth(s);
        ll := max(ll, l);
      end;
      ColWidths[c] := ll;
      lll := lll + ll+1;
    end;
    SG.ClientWidth := lll;
    SG.ScrollBars := ssNone;
    if SG.Width > SG.Parent.ClientWidth-dw then
    begin
      SG.Width := SG.Parent.ClientWidth-dw;
      SG.ScrollBars := ssHorizontal;
    end;
    SG.ClientHeight := (DefaultRowHeight+1)*RowCount;
    if SG.Height > SG.Parent.ClientHeight-dh then
    begin
      SG.Height := SG.Parent.ClientHeight-dh;
      if SG.ScrollBars = ssHorizontal then
        SG.ScrollBars := ssBoth
      else
        SG.ScrollBars := ssVertical;
    end;
  end;
end;

procedure LoadStringGrid(const SG: TStringGrid; const FileName: string;
                         var Comment: string);
var
  s : string;
  slData: TStringList;
  slRow: TStringList;
  r, c, n, rmin : integer;
  NCol, NRow: integer;
begin
  n := 0;
  slData := TStringList.Create;
  slRow := TStringList.Create;
  Comment := '';
  try
    slData.LoadFromFile(FileName);
    NRow := slData.Count;
    SG.RowCount := NRow;
    rmin := 0;
    for r := 0 to slData.Count-1 do
    begin
      s := Trim(slData[r]);
      if s = '' then
      begin
        dec(NRow);
        SG.RowCount := NRow;
      end else
      if s[1] = '#' then
      begin
        dec(NRow);
        inc(rmin);
        SG.RowCount := NRow;
        if Length(s) > 1 then
          Comment := Copy(s,2,255);
      end else
      begin
        slRow.CommaText := slData[r];
        n := slRow.Count;
        if r = rmin then
        begin
          NCol := n;
          SG.ColCount := NCol;
        end;

        if n = NCol then
        begin
          for c := 0 to NCol-1 do
          begin
            SG.Cells[c,r-rmin] := slRow[c];
          end;
        end else
          raise ERangeError.CreateFmt('Row %d: N Data = %d <> N Col = %d', [r, n, NCol]);
      end;
    end;
  finally
    slData.Free;
    slRow.Free;
  end;
end;

///   Call to adjust dimensions
procedure SaveStringGrid(const SG: TStringGrid; const FileName: string;
                         const Comment: string);
var
  slData: TStringList;
  slRow: TStringList;
  r, c : integer;
begin
  slData := TStringList.Create;
  if Comment <> '' then
    slData.Append('#'+Comment);
  slRow := TStringList.Create;
  for r := 0 to SG.RowCount-1 do
  begin
    slRow.Clear;
    for c := 0 to SG.ColCount-1 do
    begin
      slRow.Append(SG.Cells[c,r]);
    end;
    slData.Append(slRow.CommaText);
  end;

  try
    slData.SaveToFile(FileName);
  finally
    slData.Free;
    slRow.Free;
  end;
end;

procedure ClearStringGrid(const SG: TStringGrid;
                                MinRow: integer = 1; MinCol: integer = 1);
var
  r, c : integer;
begin
  for r := MinRow to SG.RowCount-1 do
    for c := MinCol to SG.ColCount-1 do
      SG.Cells[c,r] := '';
end;

procedure StringGridAddCol(const SG: TStringGrid; Col: integer; const Title: string);
var
  r, c : integer;
begin
  if Col < SG.FixedCols then Col := SG.FixedCols;
  if Col > SG.ColCount then Col := SG.ColCount;
  SG.ColCount := SG.ColCount+1;

  for c := SG.ColCount-1 downto Col+1 do
    for r := 0 to SG.RowCount-1 do
     SG.Cells[c,r] := SG.Cells[c-1,r];

  SG.Cells[Col,0] := Title;
  for r := 1 to SG.RowCount-1 do
   SG.Cells[Col,r] := '';
end;

procedure StringGridAddRow(const SG: TStringGrid; Row: integer; const Title: string);
var
  r, c : integer;
begin
  if Row < SG.FixedRows then Row := SG.FixedRows;
  if Row > SG.RowCount then Row := SG.RowCount;
  SG.RowCount := SG.RowCount+1;

  for r := SG.RowCount-1 downto Row+1 do
    for c := 0 to SG.ColCount-1 do
      SG.Cells[c,r] := SG.Cells[c,r-1];

  SG.Cells[0,Row] := Title;
  for c := 1 to SG.ColCount-1 do
   SG.Cells[c,Row] := '';
end;

procedure StringGridDelCol(const SG: TStringGrid; Col: integer);
var
  r, c : integer;
begin
  if Col < SG.FixedCols then Col := SG.FixedCols;
  if Col >= SG.ColCount then Col := SG.ColCount-1;

  for c := Col to SG.ColCount-2 do
    for r := 0 to SG.RowCount-1 do
     SG.Cells[c,r] := SG.Cells[c+1,r];

  SG.ColCount := SG.ColCount-1;
end;

procedure StringGridDelRow(const SG: TStringGrid; Row: integer);
var
  r, c : integer;
begin
  if Row < SG.FixedRows then Row := SG.FixedRows;
  if Row >= SG.RowCount then Row := SG.RowCount-1;

  for r := Row to SG.RowCount-2 do
    for c := 0 to SG.ColCount-1 do
      SG.Cells[c,r] := SG.Cells[c,r+1];

  SG.RowCount := SG.RowCount-1;
end;

// for two-column grid
procedure StrGridToStrList(const SG: TStringGrid; const SL: TStringList);
var
  r: integer;
begin
  SL.Clear;
  for r := SG.FixedRows to SG.RowCount-1 do
    SL.Values[SG.Cells[SG.FixedCols,r]] := SG.Cells[SG.FixedCols+1,r];
end;

// for two-column grid
procedure StrListToStrGrid(const SG: TStringGrid; const SL: TStringList);
var
  i: integer;
begin
  SG.RowCount := SL.Count+SG.FixedRows;

  for i := 0 to SL.Count-1 do
  begin
    SG.Cells[SG.FixedCols  ,i+SG.FixedRows] := SL.Names[i];
    SG.Cells[SG.FixedCols+1,i+SG.FixedRows] := SL.Values[SL.Names[i]];
  end;
end;

// for two-column grid
procedure FixTwoColSG(const SG: TStringGrid; width: integer; FixHeight: boolean);
var
  r, w : integer;
  s: string;
begin
  SG.DefaultRowHeight := -SG.Font.Height+6;
  if FixHeight then
    SG.ClientHeight := (SG.DefaultRowHeight+1)*SG.RowCount+1;
  w := 0;
  if width > 0 then
    SG.Width := width;
  for r := 0 to SG.RowCount-1 do
  begin
    s := 'W'+SG.Cells[0,r];
    w := max(w,SG.Canvas.TextWidth(s));
  end;
  SG.ColWidths[0] := w;
  SG.ColWidths[1] := SG.ClientWidth-w-2;
end;

procedure PanelDataToStringGrid(const PD: IPanelData; const SG: TStringGrid;
                                IsEditable: boolean);
var
  r, c, i, j : integer;
  s: string;
begin
  if IsEditable then
    SG.Options := SG.Options + [goEditing];
  SG.FixedCols := FixedCols;
  SG.FixedRows := FixedRows;
  SG.ColCount := PD.NCol+FixedCols;
  SG.RowCount := PD.NRow+FixedRows;

  for c := 1 to PD.NCol do
  begin
    i := c + FixedCols - 1;
    SG.Cells[i, 0] := IntToStr(c);
    s := PD.Titles[c];
    if s = '' then
      s := 'Column '+IntToStr(c);
    SG.Cells[i, 1] := s;
    SG.Cells[i, 2] := BasicTypeToStr(PD.Types[c]);
  end;

  for r := 1 to PD.NRow do
  begin
    j := r + FixedRows - 1;
    SG.Cells[0, j] := IntToStr(r);
    for c := 1 to PD.NCol do
    begin
      i := c + FixedCols - 1;
      SG.Cells[i, j] := PD[r,c];
    end;
  end;
  FixStringGrid(SG);
end;

procedure StringGridToPanelData(const SG: TStringGrid; const PD: IPanelData);
var
  r, c, i, j : integer;
begin
  PD.SetLengths(SG.RowCount - FixedRows, SG.ColCount - FixedCols);
  for c := 1 to PD.NCol do
  begin
    i := c + FixedCols - 1;
    PD.Titles[c] := SG.Cells[i, 1];
    PD.Types[c] := StrToBasicType(SG.Cells[i, 2]);
  end;

  for r := 1 to PD.NRow do
  begin
    j := r + FixedRows - 1;
    for c := 1 to PD.NCol do
    begin
      i := c + FixedCols - 1;
      PD[r,c] := SG.Cells[i, j];
    end;
  end;
end;

procedure PanelDataToSGColumn(const PD: IPanelData; const SG: TStringGrid; Col: TInt);
var
  r, i, j : integer;
begin

  for r := 1 to PD.NRow do
  begin
    j := r + FixedRows - 1;
    SG.Cells[0, j] := IntToStr(r);
      i := Col + FixedCols - 1;
      SG.Cells[i, j] := PD[r,Col];
  end;
  FixStringGrid(SG);
end;

procedure SGColumnToPanelData(const PD: IPanelData; const SG: TStringGrid; Col: TInt);
var
  r, i, j : integer;
begin

  for r := 1 to PD.NRow do
  begin
    j := r + FixedRows - 1;
    SG.Cells[0, j] := IntToStr(r);
      i := Col + FixedCols - 1;
      PD[r,Col] := SG.Cells[i, j];
  end;
  FixStringGrid(SG);
end;

(*
{ TStringGridX }

constructor TStringGridX.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited Create(aOwner);
  Parent := aParent;
  Align := alClient;

end;

function TStringGridX.FormatToStr(aFormat: TFloatFormat): string;
begin
  case aFormat of
    ffGeneral: result := 'General';
    ffExponent: result := 'Exponent';
    ffFixed: result := 'Fixed';
    ffNumber: result := 'Number';
    ffCurrency: result := 'Currency';
  else
    result := 'Unknown';
  end;
end;

function TStringGridX.StrToFormat(sFormat: string): TFloatFormat;
var
  c: char;
  s: string;
begin
  s := LowerCase(sFormat);
  c := s[1];
  case c of
  'g': result := ffGeneral;
  'e': result := ffExponent;
  'f': result := ffFixed;
  'n': result := ffNumber;
  'c': result := ffCurrency;
  else
    result := ffGeneral;
  end;
end;

function TStringGridX.GetFloat(aRow, aCol: Integer): TFloat;
var
  Code: Integer;
  x: TFloat;
begin
  Val(Cells[aCol, aRow], x, Code);
  if Code = 0 then
    result := x
  else
    result := NaN;
end;

function TStringGridX.GetInt(aRow, aCol: Integer): TInt;
var 

  I, Code: Integer;
begin
  Val(Cells[aCol, aRow], I, Code);
  if Code = 0 then
    result := I
  else
    result := iNaN;
end;

function TStringGridX.GetString(aRow, aCol: Integer): string;
begin
  result := Cells[aCol, aRow];
end;

procedure TStringGridX.SetFloat(aRow, aCol: Integer; const Value: TFloat);
begin
//  Cells[aCol, aRow] := FloatToStr(Value);
  Cells[aCol, aRow] := FloatToStrF(Value, fFormat, fPrecision, fDigits);
end;

procedure TStringGridX.SetInt(aRow, aCol: Integer; const Value: TInt);
begin
  Cells[aCol, aRow] := IntToStr(Value);
end;

procedure TStringGridX.SetString(aRow, aCol: Integer; const Value: string);
begin
  Cells[aCol, aRow] := Value;
end;

//function TStringGridX.StrToFormat(sFormat: string): TFloatFormat;

function TStringGridX.GetFormat: TFloatFormat;
begin
  result := FormatToStr(fFormat);
end;

procedure TStringGridX.SetFormat(const Value: string);
begin
  fFormat := StrToFormat(Value
end;
*)

{ TStringGridHelper }

constructor TStringGridHelper.Create(SG: TStringGrid);
begin
  inherited Create;
  fSG := SG;
end;

function TStringGridHelper.GetColCount: integer;
begin
  result := fSG.ColCount;
end;

function TStringGridHelper.GetDateTime(aRow, aCol: Integer): TDateTime;
begin
  result := StrToDateTime(fSG.Cells[aCol,aRow]);
end;

function TStringGridHelper.GetFloat(aRow, aCol: Integer): TFloat;
begin
  result := StrToFloat(fSG.Cells[aCol,aRow]);
end;

function TStringGridHelper.GetInt(aRow, aCol: Integer): TInt;
begin
  result := StrToInt(fSG.Cells[aCol,aRow]);
end;

function TStringGridHelper.GetRowCount: integer;
begin
  result := fSG.RowCount;
end;

function TStringGridHelper.GetString(aRow, aCol: Integer): string;
begin
  result := fSG.Cells[aCol,aRow];
end;

procedure TStringGridHelper.SetColCount(const Value: integer);
begin
  fSG.ColCount := Value;
end;

procedure TStringGridHelper.SetDateTime(aRow, aCol: Integer;
  const Value: TDateTime);
begin
  fSG.Cells[aCol,aRow] := DateTimeToStr(Value);
end;

procedure TStringGridHelper.SetFloat(aRow, aCol: Integer;
  const Value: TFloat);
begin
    fSG.Cells[aCol,aRow] := FloatToStr(Value);
end;

procedure TStringGridHelper.SetInt(aRow, aCol: Integer; const Value: TInt);
begin
  fSG.Cells[aCol,aRow] := IntToStr(Value);
end;

procedure TStringGridHelper.SetRowCount(const Value: integer);
begin
  fSG.RowCount := Value;
end;

procedure TStringGridHelper.SetString(aRow, aCol: Integer;
  const Value: string);
begin
  fSG.Cells[aCol,aRow] := Value;
end;

end.
