{@abstract(TStockChart class)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2006.04.04)
@version 2006.04.04 - created
@version 2007.09.27 - enhanced
Nikolai V. Shokhirev, 2002-2007 }

{ Basic notations and expressions                                   }
{                                                                   }
{                    LeftOffset          RightOffset                }
{               0                                   Width           }
{             0 +--------+--------------------+-------+--> iXscreen }
{               |                                     |             }
{               |      Yvalue                         |             }
{               |        |                            |             }
{  TopOffset    +  Ymax  +--------------------+       +             }
{               |        |                    |       |             }
{               |        |                    |       |             }
{               |        |                    |       |             }
{               |        |                    |       |             }
{               |        |                    |       |             }
{               |        |                    |       |             }
{               |        |                    |       |             }
{ BottomOffset  +  Ymin  +--------------------+-->    +             }
{               |                              IndexX |             }
{               |                                     |             }
{               |     IdxMin               IdxMax     |             }
{               |                                     |             }
{       Height  +--------+--------------------+-------+ YB          }
{               |                                                   }
{               |                                                   }
{                                                                   }
{           iYscreen                                                }
{                                                                   }
unit uSignalPlot;

interface

uses
  uMatTypes, extctrls, graphics, classes, controls, uDynArrays;

type

//  TYofX = function(const x: TFloat): TFloat;
  TYofX = function(const x: TFloat): TFloat;

  TSignalPlot = class
  private
    iWidth,
    iHeight,
    iRightOffset,
    iLeftOffset,
    iTopOffset,
    iBottomOffset: TInt;
    iLo,
    iMax,
    iMin,
    iStep,
    hLeft, hRight: TInt;
    fYmax: TFloat;
    fYmin: TFloat;
    fCanvas: TCanvas;
    procedure SetStep(const Value: TInt);
    procedure CheckMin(Lo: TInt);
  public
    constructor Create(aCanvas: TCanvas; aWidth, aHeight, aLeftOffset,
                        aRightOffset, aTopOffset, aBottomOffset: TInt);
    destructor Destroy; override;
    procedure ClearPlot;
    ///{ min index visible on the screen }
    function IdxMin(aMax: TInt): TInt;
    ///{ X screen coordinate corresponding to index i  }
    function iXscreen(i: TInt): TInt;
    ///{ index corresponding to iX screen coordinate   }
    function IndexX(iX: TInt): TInt;
    ///{ Y screen coordinate corresponding to valye Y  }
    function iYscreen(Y: TFloat): TInt;
    ///{ Y value corresponding to iY screen coordinate }
    function Yvalue(iY: TInt): TFloat;
    function GetVisiblePoints(): TInt;
    procedure MoveToIF(ix: TInt; y: TFloat);
    procedure LineToIF(ix: TInt; y: TFloat);
    procedure CircleIF(ix: TInt; y: TFloat; r: TInt);
    procedure LineIF(ix1: TInt; y1: TFloat; ix2: TInt; y2: TFloat);
    procedure DrawFrame(PenColor: TColor; NXTicks, NYTicks: TInt;
                        const XTitle, YTitle: string; automatic: boolean = true);
    {*
      draw line plot of array Y
    }
    procedure DrawPlot(const Y: IFArr1D; aMax: TInt; PenColor: TColor;
                             minIdx: TInt = MaxInt);
    {*
      draw bar plot of array Y
    }
    procedure DrawBars(const Y: IFArr1D; aMax: TInt; PenColor: TColor;
                             LeftLine, RightLine, FromZero: boolean);
    procedure DrawDots(const Y: IFArr1D; aMax, r: TInt;
                           PenColor, BrushColor: TColor);
    procedure SetDims(aWidth, aHeight: TInt);
    {*
    psSolid	      A solid line.
    psDash	      A line made up of a series of dashes.
    psDot	        A line made up of a series of dots.
    psDashDot	    A line made up of alternating dashes and dots.
    psDashDotDot	A line made up of a series of dash-dot-dot combinations.
    }
    procedure DrawHLine(Y: TFloat; PenColor: TColor; PenStyle: TPenStyle);
    procedure DrawVLine(Idx: TInt; aMax: TInt; PenColor: TColor;
                                               PenStyle: TPenStyle);
    property Ymin: TFloat read fYmin write fYmin;
    property Ymax: TFloat read fYmax write fYmax;
//    property IdxMax: TInt read iMax write SetIdxMax;
    property Step: TInt read iStep write SetStep;
//    property Canvas: TCanvas read fCanvas write fCanvas;
//    property Height: TInt read iHeight write iHeight;
//    property Width: TInt read iWidth write iWidth;
//    property TopOffset: TInt read iTopOffset write iTopOffset;
//    property BottomOffset: TInt read iBottomOffset write iBottomOffset;
//    property RightOffset: TInt read iRightOffset write iRightOffset;
//    property LeftOffset: TInt read iLeftOffset write iLeftOffset;
   end;

implementation

uses
  math, Sysutils, uDisplay, Types, Dialogs, Jpeg;

{ TSignalPlot }

constructor TSignalPlot.Create(aCanvas: TCanvas; aWidth, aHeight, aLeftOffset,
                                aRightOffset, aTopOffset, aBottomOffset: TInt);
begin
  inherited Create;
  fCanvas :=       aCanvas;
  iWidth :=        aWidth;
  iHeight :=       aHeight;
  iRightOffset :=  aRightOffset;
  iLeftOffset :=   aLeftOffset;
  iTopOffset :=    aTopOffset;
  iBottomOffset := aBottomOffset;
  iLo := 1;
  iStep := 1;
  iMax := 10;
  iMin := 1;
  hLeft := (iStep-1) div 2;
  hRight := iStep div 2;
end;

destructor TSignalPlot.Destroy;
begin
  //
  inherited;
end;

procedure TSignalPlot.SetDims(aWidth, aHeight: TInt);
begin
  iWidth := aWidth;
  iHeight := aHeight;
  CheckMin(iLo);
end;

function TSignalPlot.IdxMin(aMax: TInt): TInt;
begin
  result := aMax - ( (iWidth - iRightOffset - iLeftOffset) div iStep );
  if result < iLo then
    result := iLo;
end;

procedure TSignalPlot.CheckMin(Lo: TInt);
begin
  iMin := iMax - ( (iWidth - iRightOffset - iLeftOffset) div iStep );
  if iMin < Lo then
    iMin := Lo;
end;

procedure TSignalPlot.SetStep(const Value: TInt);
begin
  iStep := Value;
  hLeft := (iStep-1) div 2;
  hRight := iStep div 2;
  CheckMin(iLo);
end;

procedure TSignalPlot.ClearPlot;
begin
  with fCanvas do
  begin
    Brush.Color := clWhite;
    FillRect(Rect(0,0,iWidth-1,iHeight-1));
  end;
end;

procedure TSignalPlot.DrawFrame(PenColor: TColor; NXTicks, NYTicks: TInt;
                       const XTitle, YTitle: string; automatic: boolean = true);
const
  decimals = 3;
var
  OldColor: TColor;
  tH, tW, lW, n, minIdx, maxIdx, idx, i: integer;
  y, factor: TFloat;
begin
  OldColor := fCanvas.Pen.Color;
  fCanvas.Pen.Color := PenColor;
  ClearPlot;
  n := round(Log10(fYmax - fYmin));
  factor := IntPower(10.0, n)/2.5;
  minIdx := round(fYmin/factor);
  maxIdx := round(fYmax/factor);

  with fCanvas do
  begin
    tH := TextHeight(XTitle+YTitle);
    tW := TextWidth(XTitle);
    lW := TextWidth('X');
    if automatic then
    begin
      i := lW;
      for idx  := minIdx to maxIdx do
          i := max(i, TextWidth(CompactFloatToStr(idx*factor,decimals)));
      iLeftOffset := i +lW;
      iTopOffset := tH*3 div 2;
      iBottomOffset := iTopOffset;
      iRightOffset := lW;
    end;

    MoveTo(iLeftOffset,iTopOffset);
    LineTo(iLeftOffset,iHeight-iBottomOffset);
    LineTo(iWidth-iRightOffset,iHeight-iBottomOffset);
    LineTo(iWidth-iRightOffset,iTopOffset);
    LineTo(iLeftOffset,iTopOffset);
    TextOut(iWidth-tW-lW, iHeight - tH - (tH div 4), XTitle);
    TextOut(lW, (tH div 4), YTitle);
    for idx  := minIdx to maxIdx do
    begin
      y := idx*factor;
      if (y > fYmin) and (y < fYmax) then
      begin
        i := iYscreen(y);
        MoveTo(iLeftOffset, i);
        LineTo(iLeftOffset+5, i);
        TextOut(lW div 2, i - (tH div 2), CompactFloatToStr(y,decimals));
      end;
    end;
  end;
{
  with fCanvas do
  begin
    MoveTo(1,1);
    LineTo(1,iHeight-2);
    LineTo(iWidth-2,iHeight-2);
    LineTo(iWidth-2,1);
    LineTo(1,1);
  end;
}
  fCanvas.Pen.Color := OldColor;
end;

procedure TSignalPlot.LineIF(ix1: TInt; y1: TFloat; ix2: TInt; y2: TFloat);
begin
  fCanvas.MoveTo(ix1, iYscreen(y1));
  fCanvas.LineTo(ix2, iYscreen(y2));
end;

procedure TSignalPlot.LineToIF(ix: TInt; y: TFloat);
begin
  fCanvas.LineTo(ix, iYscreen(y));
end;

procedure TSignalPlot.MoveToIF(ix: TInt; y: TFloat);
begin
  fCanvas.MoveTo(ix, iYscreen(y));
end;

procedure TSignalPlot.CircleIF(ix: TInt; y: TFloat; r: TInt);
var
  iy: TInt;
begin
  iy := iYscreen(y);
  fCanvas.Ellipse(ix-r,iy-r,ix+r,iy+r);
end;

function TSignalPlot.IndexX(iX: TInt): TInt;
begin
  result := iMax - ( (iWidth - iRightOffset - iX) div iStep );
end;

function TSignalPlot.iXscreen(i: TInt): TInt;
begin
  result := (i-iMax)*iStep + iWidth - iRightOffset;
end;

function TSignalPlot.iYscreen(Y: TFloat): TInt;
var
  dY: TFloat;
begin
  dY := (iHeight - iTopOffset - iBottomOffset);
  result := iTopOffset + round(dY*(fYmax - Y)/(fYmax - fYmin));
end;

function TSignalPlot.Yvalue(iY: TInt): TFloat;
var
  dY: TFloat;
begin
  dY := (iHeight - iTopOffset - iBottomOffset);
  result := fYmax - (iY - iTopOffset)*(fYmax - fYmin)/dY ;
end;

procedure TSignalPlot.DrawBars(const Y: IFArr1D; aMax: TInt; PenColor: TColor;
                                     LeftLine, RightLine, FromZero: boolean);
var
  i, iX, h, iL, iR: TInt;
  Y1, Y2: TFloat;
  LeftColor, RightColor, OldColor: TColor;
begin
  LeftColor := clWhite;
  RightColor := clBlack;
  iMax := aMax;
  CheckMin(Y.Lo1);

  OldColor := fCanvas.Pen.Color;
  if LeftLine and (hLeft > 0) then
    iL := hLeft - 1
  else
    iL := hLeft;
  if RightLine and (hRight > 0) then
    iR := hRight - 1
  else
    iR := hRight;
  for i := iMin to iMax do
  begin
    Y1 := Y[i];
    iX := iXscreen(i);
    if FromZero then
      Y2  := 0.0
    else
      Y2  := Ymin;
    fCanvas.Pen.Color := PenColor;
    for h := -iL to iR do
    begin
      LineIF(iX+h,Y1, iX+h,Y2);
    end;
    if hLeft > iL then
    begin
      fCanvas.Pen.Color := LeftColor;
      LineIF(iX-hLeft,Y1, iX-hLeft,Y2);
    end;
    if hRight > iR then
    begin
      fCanvas.Pen.Color := RightColor;
      LineIF(iX+hRight,Y1, iX+hRight,Y2);
    end;
  end;
  fCanvas.Pen.Color := OldColor;
end;

procedure TSignalPlot.DrawPlot(const Y: IFArr1D; aMax: TInt; PenColor: TColor;
                                      minIdx: TInt = MaxInt);
var
  i, iX: TInt;
  OldColor: TColor;
begin

  iMax := aMax;
//  CheckMin(Y.Lo1);
  CheckMin(minIdx);

  OldColor := fCanvas.Pen.Color;
  fCanvas.Pen.Color := PenColor;
  i := iMin;
  iX := iXscreen(i);
  MoveToIF(iX,Y[i]);
  for i := iMin+1 to iMax do
  begin
    iX := iXscreen(i);
    LineToIF(iX,Y[i]);
  end;
  fCanvas.Pen.Color := OldColor;
end;

procedure TSignalPlot.DrawDots(const Y: IFArr1D; aMax, r: TInt;
                                     PenColor, BrushColor: TColor);
var
  i, iX: TInt;
  OldPenColor, OldBrushColor: TColor;
begin
  iMax := aMax;
  CheckMin(Y.Lo1);

  OldPenColor := fCanvas.Pen.Color;
  OldBrushColor := fCanvas.Brush.Color;
  fCanvas.Pen.Color := PenColor;
  fCanvas.Brush.Color := BrushColor;

  for i := iMin to iMax do
  begin
    iX := iXscreen(i);
    CircleIF(iX, y[i], r);
  end;
  fCanvas.Brush.Color := OldBrushColor;
  fCanvas.Pen.Color := OldPenColor;
end;

procedure TSignalPlot.DrawVLine(Idx: TInt; aMax: TInt; PenColor: TColor;
                                PenStyle: TPenStyle);
var
  iX: TInt;
  OldColor: TColor;
  OldPenStyle: TPenStyle;
begin
  iMax := aMax;

  OldColor := fCanvas.Pen.Color;
  fCanvas.Pen.Color := PenColor;
  OldPenStyle := fCanvas.Pen.Style;
  fCanvas.Pen.Style := PenStyle;

  iX := iXscreen(Idx);
  fCanvas.MoveTo(iX,iTopOffset);
  fCanvas.LineTo(iX,iHeight - iBottomOffset);

  fCanvas.Pen.Color := OldColor;
  fCanvas.Pen.Style := OldPenStyle;
end;

procedure TSignalPlot.DrawHLine(Y: TFloat; PenColor: TColor; PenStyle: TPenStyle);
var
  OldColor: TColor;
  OldPenStyle: TPenStyle;
begin
  OldColor := fCanvas.Pen.Color;
  fCanvas.Pen.Color := PenColor;
  OldPenStyle := fCanvas.Pen.Style;
  fCanvas.Pen.Style := PenStyle;

  LineIF(iLeftOffset,Y,iWidth - iRightOffset,Y);

  fCanvas.Pen.Color := OldColor;
  fCanvas.Pen.Style := OldPenStyle;
end;

function TSignalPlot.GetVisiblePoints: TInt;
begin
  result := (iWidth - iRightOffset - iLeftOffset) div iStep;
end;

end.
