{@abstract(TPlot2D and TPlotBox2D classes)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2002.06.06)
@lastmod(2005.04.14)
Nikolai V. Shokhirev, 2002-2005  }
{2005.05.15 Addedd SaveToBMP and SaveToJPEG
 2005.06.08 Added TPlotBox2D.TextOutF
 2005.10.04 in TPlotBox2D Added: PlotArray, PlotArrays, PlotFunc
 }

{ The unit for simple 2D graphics }
{                                                       }
{ Basic notations and expressions                       }
{                                                       }
{        0       LdW                iW-RdW    iW    iX  }
{                                                       }
{      0 +--------+--------------------+-------+-- YT   }
{        |                                     |        }
{        |        Y                            |        }
{        |        |                            |        }
{   TdH  +  Ymax  +--------------------+       +        }
{        |        |                    |       |        }
{        |        |                    |       |        }
{        |        |                    |       |        }
{        |        |                    |       |        }
{        |        |                    |       |        }
{        |        |                    |       |        }
{        |        |                    |       |        }
{ iH-BdH +  Ymin  +--------------------+-- X   +        }
{        |                                     |        }
{        |       Xmin                 Xmax     |        }
{        |                                     |        }
{   iH   +--------+--------------------+-------+ YB     }
{        |                                              }
{        |                                              }
{   iY                                                  }
{ TdH - Plot Top offset (pixels)                        }
{ BdH - Plot Bottom offset (pixels)                     }
{       Ymin    <=  Y <= Ymax                           }
{ iH > iH - BdH >= iY >= TdH >= 0                       }
{                                                       }
{ LdW - Plot Left offset (pixels)                       }
{ RdW - Plot Right offset (pixels)                      }
{      Xmin <=  X <= Xmax                               }
{ 0 <= rLdW <= iX <= rW-rRdW < rW                       }
{
(ix-LdW)/(iW-RdW-LdW)=(x-Xmin)/(Xmax-Xmin)
(iH-BdH-iy)/(iH-BdH-TdH)=(y-Ymin)/(Ymax-Ymin)

(ix-LdW)*(Xmax-Xmin)=(iW-RdW-LdW)*(x-Xmin)
(iH-BdH-iy)*(Ymax-Ymin)=(iH-BdH-TdH)*(y-Ymin)

ix-LdW = (iW-RdW-LdW)*(x-Xmin)/(Xmax-Xmin)
iH-BdH-iy = (iH-BdH-TdH)*(y-Ymin)/(Ymax-Ymin)

x-Xmin = (ix-LdW)*(Xmax-Xmin)/(iW-RdW-LdW)
y-Ymin = (iH-BdH-iy)*(Ymax-Ymin)/(iH-BdH-TdH)

ix = iW - RdW
Xmax = Xmin + (iW-RdW-LdW)*(Xmax-Xmin)/(iW-RdW-LdW)

iy = TdH
Ymax = Ymin + (iH-BdH-TdH)*(Ymax-Ymin)/(iH-BdH-TdH)

ix = 0
XL = Xmin - LdW*(Xmax-Xmin)/(iW-RdW-LdW)

ix = iW
XR = Xmin + (iW-LdW)*(Xmax-Xmin)/(iW-RdW-LdW)
   = Xmax - (iW-RdW-LdW)*(Xmax-Xmin)/(iW-RdW-LdW)
          + (iW-LdW)*(Xmax-Xmin)/(iW-RdW-LdW)
   = Xmax + RdW*(Xmax-Xmin)/(iW-RdW-LdW)

iy = 0
YT = Ymin + (iH-BdH)*(Ymax-Ymin)/(iH-BdH-TdH)
   = Ymax - (iH-BdH-TdH)*(Ymax-Ymin)/(iH-BdH-TdH)
          + (iH-BdH)*(Ymax-Ymin)/(iH-BdH-TdH)
   = Ymax + TdH*(Ymax-Ymin)/(iH-BdH-TdH)

iy = iH
YB = Ymin - BdH*(Ymax-Ymin)/(iH-BdH-TdH)
}
{                                                    }
{       0      LdW                 W-RdW   iW    iX  }
{                                                    }
{     0 +-------+--------------------+------+-- YT   }
{       |                                   |        }
{       |    YTitle                         |        }
{       |                                   |        }
{  TdH  +  5e-1 +--------------------+      +        }
{       |       |                    |      |        }
{       |       |                    |      |        }
{       |       |                    |      |        }
{       |  0e+0 +                    |      |        }
{       |       |                    |      |        }
{       |       |                    |      |        }
{       |       |                    |      |        }
{ H-BdH + -5e-1 +---------+----------+      +        }
{       |      1.1e1     1.2e2      1.3e3   |        }
{       |                                   |        }
{       |                         XTitle    |        }
{       |                                   |        }
{   iH  +-------+--------------------+------+ YB     }
{       |                                            }
{       |                                            }
{   iY                                               }
unit uStockChart;

interface

uses
  uMatTypes, extctrls, graphics, classes, controls, uDynArrays;

type

  TYofX = function(const x: TFloat): TFloat;


  TStockChart = class(TPaintBox)
  private
    fXLeft: TInt;
    fXRight: TInt;
    fYTop: TInt;
    fYBottom: TInt;
    fXstep: TInt;
    fdX: TInt;

    fIdxMin: TInt;
    fIdxMax: IInt;

    
    fXmin: TFloat;
    fXmax: TFloat;
    fYmin: TFloat;
    fYmax: TFloat;
    // Canvas Dimensions [pixels]
    // Plot Offsets [pixels]
    TdH, BdH, LdW, RdW: TInt;
    XDec: TInt; // X-Label decimals
    YDec: TInt; // Y-Label decimals
    rH, rTdH, rBdH: TFloat;
    rW, rLdW, rRdW: TFloat;
    fXCaption: boolean;
    fYCaption: boolean;
    fXTitle: string;
    fYTitle: string;
    fNYTicks: TInt;
    fNXTicks: TInt;
    fXYRatio: TFloat;
    fFrameColor: TColor;
    fFrameWidth: TInt;
    function GetXmin: TFloat;
    procedure SetXmin(const Value: TFloat);
    function GetXmax: TFloat;
    procedure SetXmax(const Value: TFloat);
    function GetYmin: TFloat;
    procedure SetYmin(const Value: TFloat);
    function GetYmax: TFloat;
    procedure SetYmax(const Value: TFloat);
    function GetPenWidth: TInt;
    procedure SetPenWidth(const Value: TInt);
    function GetPenColor: TColor;
    procedure SetPenColor(const Value: TColor);
    function GetBrushColor: TColor;
    procedure SetBrushColor(const Value: TColor);
  public
    constructor Create(aOwner: TComponent; aParent: TWinControl);
    destructor Destroy; override;
    function Xi2r(iX: TInt): TFloat; { Xi2r(rLdW) = Xmin,  Xi2r(iW-rRdW) = Xmax}
    function Yi2r(iY: TInt): TFloat; { Yi2r(iH - BdH) = Ymin,  Yi2r(TdH) = Ymax}
    function Yr2i(Y: TFloat): TInt; { Yr2i(Ymin) = iH - BdH,  Yr2i(Ymax) = TdH }
    function Xr2i(X: TFloat): TInt; { Xr2i(Xmin) = rLdW,  Xr2i(Xmax) = iW-rRdW }

    procedure PlotPrice(const Open, High, Low, Close: IFArr1D; IdxMin, IdxMax: TInt; Color: TColor);
//    procedure PlotStock(const Open, High, Low, Close, Volume: IFArr1D; Color: TColor);


    procedure PlotArray(const YArr: IFArr1D; Color: TColor); overload;
    procedure PlotArray(const YArr: IFArr1D; Color: TColor; Xmin, Xmax: TFloat); overload;
    procedure PlotArrays(const XArr, YArr: IFArr1D; Color: TColor);
    procedure PlotFunction(YFunc: TFloatFunc1D; Xmin, Xmax: TFloat; Nx: TInt; Color: TColor);
    procedure SaveToBMP(const FileName: string);
    procedure SaveToJPEG(const FileName: string);
    procedure SetLimits(Xmin, Xmax, Ymin, Ymax: TFloat);
    procedure Circle(x, y: TFloat; r: TInt);
    procedure MoveToF(x, y: TFloat);
    procedure LineToF(x, y: TFloat);
    procedure LineF(x1, y1, x2, y2: TFloat);
    procedure TextOutF(X, Y: TFloat; const Text: string; ix, iy: TInt);
    procedure PlotFrame;
    property Xmin: TFloat read GetXmin write SetXmin;
    property Xmax: TFloat read GetXmax write SetXmax;
    property Ymin: TFloat read GetYmin write SetYmin;
    property Ymax: TFloat read GetYmax write SetYmax;
    property BrushColor: TColor read GetBrushColor write SetBrushColor;
    property PenColor: TColor read GetPenColor write SetPenColor;
    property PenWidth: TInt read GetPenWidth write SetPenWidth;
    property FrameWidth: TInt read fFrameWidth write fFrameWidth;
    property XCaption: boolean read fXCaption write fXCaption;
    property YCaption: boolean read fYCaption write fYCaption;
    property NXTicks: TInt read fNXTicks write fNXTicks;
    property NYTicks: TInt read fNYTicks write fNYTicks;
    property XTitle: string read fXTitle write fXTitle;
    property YTitle: string read fYTitle write fYTitle;
    property XYRatio: TFloat read fXYRatio write fXYRatio;
    property FrameColor: TColor read fFrameColor write fFrameColor;
   end;
{
  TFuncBox2D = class(TStockChart)
  private

  end;
}
implementation

uses
  math, Sysutils, uDisplay, Types, Jpeg;

{ TStockChart }

constructor TStockChart.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited Create(aOwner);
  Parent := aParent;
  Align := alClient;
  FrameColor := clBlack;
  FrameWidth := 1;
  XCaption := true;
  YCaption := true;
  NXTicks := 3;
  NYTicks := 3;
  XTitle := 'X Axis';
  YTitle := 'Y Axis';
  XYRatio := -1.0;

  fXoffset := 10;
  fYoffset := 0;
  fXstep := 3;
  fdX := 1;    // "half" Xstep

end;

procedure TStockChart.SetLimits(Xmin, Xmax, Ymin, Ymax: TFloat);
begin
  fXmin := Xmin;
  fXmax := Xmax;
  fYmin := Ymin;
  fYmax := Ymax;
  XDec := MinDec(Xmin, Xmax);
  YDec := MinDec(Ymin, Ymax);
end;

destructor TStockChart.Destroy;
begin
  //
  inherited;
end;

procedure TStockChart.PlotFrame;
var
  // FrameOffsets
  FTdH, FBdH, FLdW, FRdW : TInt;
  H,    // Text height
  HalfH,
  W,    // All other spaces
  WY,   // Y-Label width
  WX    // X-Label Half-width
        : TInt;
  iX, iY, iHm, iWm, i, d: TInt;
  x,  y, PPUx, PPUy: TFloat;
  s: string;
  OldColor: TColor;
  OldWidth: TInt;
begin
  OldColor := PenColor;
  OldWidth := PenWidth;
  PenColor := FrameColor;
  PenWidth := FrameWidth;

  with Canvas do
  begin
    Brush.Color := clWhite;
    FillRect(Rect(0,0,Width-1,Height-1));
    rH := Height;
    rW := Width;

    H := TextHeight('H');
    HalfH := H div 2;
    W := TextWidth('H');

    Pen.Color := FrameColor;
    WX := 0; // Half X Label Width
    BdH := W+W;
    if XCaption then
    begin
      for i := 1 to NXTicks do
      begin
        x := Xmin +(i-1)*(Xmax-Xmin)/(NXTicks-1);
        s := CompactFloatToStr(x,XDec);
        WX := max(WX,TextWidth(s) div 2);
      end;
      BdH := BdH + H+H + W+W+W;
    end;

    WY := 0; // Y Label width
    TdH := W+W;
    LdW := W+W;
    if YCaption then
    begin
      for i := 1 to NYTicks do
      begin
        y := Ymin +(i-1)*(Ymax-Ymin)/(NYTicks-1);
        s := CompactFloatToStr(y,YDec);
        WY := max(WY,TextWidth(s));
      end;
      TdH := TdH + H+W;
      LdW := LdW + WY + W+W;
    end;
    // Adjustment
    RdW := max(W+W,WX+W);
    LdW := max(LdW,WX+W);
    // Adjustment for X/Y ratio
    if XYRatio > 0.0 then
    begin
      iWm := Width - LdW - RdW;
      PPUx := iWm;
      PPUx := PPUx/(Xmax - Xmin); // Pixes Per Unit
      iHm := Height - TdH - BdH;
      PPUy := iHm;
      PPUy := PPUy/(Ymax - Ymin); // Pixes Per Unit
      if PPUx > PPUy*XYRatio then
      begin
        PPUx := PPUy*XYRatio;
        d := (iWm - Round(PPUx*(Xmax-Xmin))) div 2;
        LdW := LdW + d;
        RdW := RdW + d;
      end else
      begin
        PPUy := PPUx/XYRatio;
        d := (iHm - Round(PPUy*(Ymax-Ymin))) div 2;
        TdH := TdH + d;
        BdH := BdH + d;
      end;
    end;
    // Frame Offsets
    FTdH := TdH - W;
    FBdH := BdH - W;
    FLdW := LdW - W;
    FRdW := RdW - W;
    rTdH := TdH;
    rBdH := BdH;
    rLdW := LdW;
    rRdW := RdW;
    // Draw Frame
    MoveTo(   FLdW,    FTdH);
    LineTo(Width-FRdW,    FTdH); // Top
    LineTo(Width-FRdW, Height-FBdH); // Right
    LineTo(   FLdW, Height-FBdH); // Bottom
    LineTo(   FLdW,    FTdH); // Left

    if YCaption then
    begin
      d := TextWidth(YTitle) div 2;
      TextOut(max(W, FLdW-d), max(W,FTdH-W-H), YTitle);

      for i := 1 to NYTicks do
      begin
        y := Ymin +(i-1)*(Ymax-Ymin)/(NYTicks-1);
        iY := Yr2i(y);
        s := CompactFloatToStr(y,YDec);
        TextOut(FLdW-W-W-WY,iY-halfH, s);
      end;
      for i := 1 to NYTicks do
      begin
        y := Ymin +(i-1)*(Ymax-Ymin)/(NYTicks-1);
        iY := Yr2i(y);
        MoveTo(FLdW,iY);   // Y-Ticks
        LineTo(FLdW-W,iY);
      end;
    end;

    if XCaption then
    begin
      d := TextWidth(XTitle) div 2;
      i := min(Width-FRdW+d,Width-H);
      TextOut(max(i-d-d,W), Height-FBdH+W+W+W+H, XTitle);
      for i := 1 to NXTicks do
      begin
        x := Xmin +(i-1)*(Xmax-Xmin)/(NXTicks-1);
        iX := Xr2i(x);
        s := CompactFloatToStr(x,XDec);
        d := TextWidth(s) div 2;
        TextOut(iX-d,Height-FBdH+W+W, s);
        MoveTo(iX,Height-FBdH);   // X-Ticks
        LineTo(iX,Height-FBdH+W);
      end;
    end;
  end;
  PenColor := OldColor;
  PenWidth := OldWidth;
end;

function TStockChart.GetXmax: TFloat;
begin
  result := fXmax;
end;

function TStockChart.GetXmin: TFloat;
begin
  result := fXmin;
end;

function TStockChart.GetYmax: TFloat;
begin
  result := fYmax;
end;

function TStockChart.GetYmin: TFloat;
begin
  result := fYmin;
end;

procedure TStockChart.SetXmax(const Value: TFloat);
begin
  fXmax := Value;
  XDec := MinDec(Xmin, Xmax);
end;

procedure TStockChart.SetXmin(const Value: TFloat);
begin
  fXmin := Value;
  XDec := MinDec(Xmin, Xmax);
end;

procedure TStockChart.SetYmax(const Value: TFloat);
begin
  fYmax := Value;
  YDec := MinDec(Ymin, Ymax);
end;

procedure TStockChart.SetYmin(const Value: TFloat);
begin
  fYmin := Value;
  YDec := MinDec(Ymin, Ymax);
end;

function TStockChart.Xi2r(iX: TInt): TFloat;
var
  X: TFloat;
begin
  X := iX;
  result := (X*(Xmax-Xmin)- rLdW*Xmax-rRdW*Xmin+ rW*Xmin)/(rW-rLdW-rRdW);
end;

function TStockChart.Xr2i(X: TFloat): TInt;
begin
  result := round(((rW-rLdW-rRdW)*X + rLdW*Xmax+rRdW*Xmin- rW*Xmin)/(Xmax-Xmin));
end;

function TStockChart.Yi2r(iY: TInt): TFloat;
var
  Y: TFloat;
begin
  Y := iY;
  result := (Y*(Ymax - Ymin)-rH*Ymax+rBdH*Ymax+rTdH*Ymin)/(rBdH+rTdH-rH);
end;

function TStockChart.Yr2i(Y: TFloat): TInt;
begin
  result := round((Y*(rBdH+rTdH-rH)+rH*Ymax-rBdH*Ymax-rTdH*Ymin)/(Ymax-Ymin));
end;

procedure TStockChart.LineF(x1, y1, x2, y2: TFloat);
begin
  Canvas.MoveTo(Xr2i(x1), Yr2i(y1));
  Canvas.LineTo(Xr2i(x2), Yr2i(y2));
end;

procedure TStockChart.LineToF(x, y: TFloat);
begin
  Canvas.LineTo(Xr2i(x), Yr2i(y));
end;

procedure TStockChart.MoveToF(x, y: TFloat);
begin
  Canvas.MoveTo(Xr2i(x), Yr2i(y));
end;

procedure TStockChart.TextOutF(X, Y: TFloat; const Text: string; ix, iy: TInt);
begin
  Canvas.TextOut(Xr2i(X)+ix, Yr2i(Y)+iy, Text);
end;

function TStockChart.GetPenWidth: TInt;
begin
  result := Canvas.Pen.Width;
end;

procedure TStockChart.SetPenWidth(const Value: TInt);
begin
  Canvas.Pen.Width := Value;
end;

function TStockChart.GetPenColor: TColor;
begin
  result := Canvas.Pen.Color;
end;

procedure TStockChart.SetPenColor(const Value: TColor);
begin
  Canvas.Pen.Color := Value;
end;

function TStockChart.GetBrushColor: TColor;
begin
  result := Canvas.Brush.Color;
end;

procedure TStockChart.SetBrushColor(const Value: TColor);
begin
  Canvas.Brush.Color := Value;
end;

procedure TStockChart.Circle(x, y: TFloat; r: TInt);
begin
  Canvas.Ellipse(Xr2i(x)-r, Yr2i(y)-r, Xr2i(x)+r, Yr2i(y)+r);
end;

procedure TStockChart.SaveToBMP(const FileName: string);
var
  bmp: TBitmap;
  rect: TRect;
begin
  bmp := TBitmap.Create;
  bmp.Width := self.Width;
  bmp.Height := self.Height;
  rect := Types.Rect(0, 0, bmp.Width-1, bmp.Height-1);
  bmp.Canvas.CopyRect(rect,self.Canvas, rect);
  try
    bmp.SaveToFile(FileName);
  finally
    bmp.FreeImage;
    bmp.Free;
  end;
end;

procedure TStockChart.SaveToJPEG(const FileName: string);
const
 CompressionQuality = 100;
var
  Bitmap: TBitmap;
  rect: TRect;
  JPEGImage: TJPEGImage;
begin      
  Bitmap := TBitmap.Create;
  Bitmap.Width := self.Width;
  Bitmap.Height := self.Height;
  rect := Types.Rect(0, 0, Bitmap.Width-1, Bitmap.Height-1);
  Bitmap.Canvas.CopyRect(rect,self.Canvas, rect);
  try
    JPEGImage:=TJPEGImage.Create;
    JPEGImage.CompressionQuality := CompressionQuality;
    JPEGImage.Assign(Bitmap);
    JPEGImage.SaveToFile(FileName);
  finally
    Bitmap.FreeImage;
    Bitmap.Free;
    JPEGImage.Free;
  end;
end;

procedure TStockChart.PlotArray(const YArr: IFArr1D; Color: TColor);
var
  i: TInt;
  OldColor: TColor;
begin
  OldColor := PenColor;
  PenColor := Color;
  i := YArr.Lo1;
  MoveToF(i,YArr[i]);
  for i := YArr.Lo1+1 to YArr.Hi1 do
    LineToF(i,YArr[i]);

  PenColor := OldColor;
end;

procedure TStockChart.PlotArray(const YArr: IFArr1D; Color: TColor;
                                     Xmin, Xmax: TFloat);
var
  i: TInt;
  OldColor: TColor;
  X, dX: TFloat;
begin
  OldColor := PenColor;
  PenColor := Color;
  dX := (Xmax - Xmin)/(YArr.Dim1-1);
  i := YArr.Lo1;
  X := Xmin;
  MoveToF(X,YArr[i]);
  for i := YArr.Lo1+1 to YArr.Hi1 do
  begin
    X := X + dX;
    LineToF(X,YArr[i]);
  end;
  PenColor := OldColor;
end;

procedure TStockChart.PlotArrays(const XArr, YArr: IFArr1D; Color: TColor);
var
  i: TInt;
  OldColor: TColor;
begin
  OldColor := PenColor;
  PenColor := Color;
  i := XArr.Lo1;
  MoveToF(XArr[i],YArr[i]);
  for i := YArr.Lo1+1 to YArr.Hi1 do
    LineToF(XArr[i],YArr[i]);

  PenColor := OldColor;
end;

procedure TStockChart.PlotFunction(YFunc: TFloatFunc1D; Xmin,
  Xmax: TFloat; Nx: TInt; Color: TColor);
var
  i: TInt;
  OldColor: TColor;
  X, dX, Y: TFloat;
begin
  OldColor := PenColor;
  PenColor := Color;
  dX := (Xmax - Xmin)/(Nx-1);
  X := Xmin;
  Y := YFunc(X);
  MoveToF(X,Y);
  for i := 2 to Nx do
  begin
    X := X + dX;
    Y := YFunc(X);
    LineToF(X,Y);
  end;
  PenColor := OldColor;
end;

procedure TStockChart.PlotPrice(const Open, High, Low, Close: IFArr1D;
  IdxMin, IdxMax: TInt; Color: TColor);
begin

end;

end.
