{@abstract( TSurfase definition)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(January 01, 2001)
@lastmod(may 05, 2003)
Nikolai V. Shokhirev, 2001-2003
}
unit uSurface;
interface

uses
  uMatTypes, extctrls, classes, controls, Graphics;

const
  code: array[1..8,1..3] of TInt =
        ((0,0,0),(1,0,0),(0,1,0),(1,1,0),(0,0,1),(1,0,1),(0,1,1),(1,1,1));

type

  TZofXY = function(const x, y: TFloat): TFloat;

  TSurface = class(TPaintBox)
  private
    { Private declarations }
    Border, BorderMin, BorderMax: array of TInt;
    fLx, fLy, fLz, fL: TFloat;
    LL: array[1..3] of TFloat;
    cube: array[1..8,1..3] of TFloat;
    axes: array[1..6,1..3] of TFloat;
    CubeOrder: array[1..8] of TInt;
    AxisOrder: array[1..6] of TInt;
    BaseOrder: array[1..4] of TInt;
    fWidth, fHeight: TFloat;
    rWidth, rHeight: TFloat; // = Width, Height
    fAngle, fViewAngle: TFloat;  // rad
    fxMin, fxMax, fyMin, fyMax, fzMin, fzMax: TFloat;
    fNInterval: TInt;
    fStep: TFloat;
    fZFunc: TZofXY;
    fLineColor: TColor;
    fFillColor: TColor;
    function GetAngle: TFloat;
    procedure SetAngle(const Value: TFloat);
    function GetNInterval: TInt;
    procedure SetNInterval(const Value: TInt);
    procedure RefToScreen(xx, yy, zz: TFloat; var iX, iY: TInt);
    procedure RefToInternal(xx, yy, zz: TFloat; var x, y, z: TFloat);
    procedure InternalToRef(x, y, z: TFloat; var xx, yy, zz: TFloat);
    function GetBrushColor: TColor;
    procedure SetBrushColor(const Value: TColor);
    function GetPenColor: TColor;
    procedure SetPenColor(const Value: TColor);
  public
    { Public declarations }
    constructor Create(aOwner: TComponent; aParent: TWinControl);
    destructor Destroy; override;
    procedure SetMinMax(xMin, xMax, yMin, yMax, zMin, zMax: TFloat);
    procedure SetDimensions(margin: TInt; ViewAngle: TFloat);
    procedure SetFunction(ZFunc: TZofXY);
    procedure GetOrder;
    procedure Clear;
    procedure Line(ix1,iy1,ix2,iy2: TInt);
    procedure SetFramePoints;
    procedure DrawSurface;
    procedure DrawFrame;
    property BrushColor: TColor read GetBrushColor write SetBrushColor;
    property PenColor: TColor read GetPenColor write SetPenColor;
    property LineColor: TColor read fLineColor write fLineColor;
    property FillColor: TColor read fFillColor write fFillColor;
    property Angle: TFloat read GetAngle write SetAngle;  // deg
    property NInterval: TInt read GetNInterval write SetNInterval;
  end;

implementation

{          Y
   +-------|-------+ Ly/2
   |       |       |
   |       |       |
   +-------+-------+- X
   |       |       |
   |       |       |
   +-------+-------+ -Ly/2
 -Lx/2            Lx/2

Internal real (natural) coordinates
xMin < x < xMax, yMin < y < yMax, zMin < z < zMax
  x = xMin + (xx/Lx+1/2)*(xMax-xMin)
  y = yMin + (yy/Lx+1/2)*(yMax-yMin)
  z = zMin + (zz/Lz+1/2)*(zMax-zMin)
Internal 'integer' (symmetric) coordinates
-Lx/2 < xx < Lx/2, -Ly/2 < yy < Ly/2, -Lz/2 < zz < Lz/2
  xx = Lx*(x-xMin)/(xMax-xMin) - Lx/2
  yy = Lx*(y-yMin)/(yMax-yMin) - Ly/2
  zz = Lz*(z-zMin)/(zMax-zMin) - Lz/2
Coordinates in Reference (non-rotated) frame
  xx' = xx*Cos(Angle)-yy*Sin(Angle)
  yy' = xx*Sin(Angle)+yy*Cos(Angle)
  zz' = zz
  xx =  xx'*Cos(Angle)+yy'*Sin(Angle)
  yy = -xx'*Sin(Angle)+yy'*Cos(Angle)
  zz =  zz'
External 'integer' coordinates. The origin is in the bottom-left corner.
The X-axis goes to the right, Z - up, Y - into the screen.
  X = Width/2 + xx'
  Y = yy'
  Z = Height/2 + zz'
Screen coordinates. The origin is in the top-left corner.
The iX-axis goes to the right, iY - down.
  iX = round(X)
  iY = round(Height - ( Z + Y*Sin(ViewAngle) ) )

  iX = round(Width/2 + xx')
  iY = round(Height/2 - ( zz' + yy'*Sin(ViewAngle) ) )

  xx =  xx'*Cos(Angle)+yy'*Sin(Angle)
  yy = -xx'*Sin(Angle)+yy'*Cos(Angle)
  x = xMin + (xx/Lx+1/2)*(xMax-xMin)
  y = yMin + (yy/Lx+1/2)*(yMax-yMin)
  z = f(x,y)

  zz' = Lz*(z-zMin)/(zMax-zMin) - Lz/2
  iX = round(Width/2 + xx')
  iY = round(Height/2 - ( zz' + yy'*Sin(ViewAngle) ) )
}
uses
  Sysutils, math;


{ TSurface }

constructor TSurface.Create(aOwner: TComponent; aParent: TWinControl);
begin
  inherited Create(aOwner);
  Parent := aParent;
  Align := alClient;
  fNinterval := 50;
end;

destructor TSurface.Destroy;
begin
  Border := nil;
  BorderMin := nil;
  BorderMax := nil;
  inherited;
end;

procedure TSurface.Clear;
begin
  Canvas.Brush.Color := fFillColor;
  Canvas.FillRect(Parent.ClientRect);
end;

function TSurface.GetAngle: TFloat;
begin
  result := 180.0*fAngle/Pi;
end;

procedure TSurface.RefToScreen(xx, yy, zz: TFloat; var iX, iY: TInt);
begin
  iX := round(rWidth/2.0 + xx);
  iY := round(rHeight/2.0 - ( zz + yy*Sin(fViewAngle) ) );
end;

procedure TSurface.RefToInternal(xx, yy, zz: TFloat; var x, y, z: TFloat);
var
  x1, y1: TFloat;
begin
  x1 :=  xx*Cos(fAngle)+yy*Sin(fAngle);
  y1 := -xx*Sin(fAngle)+yy*Cos(fAngle);
//  z1 :=  zz;
  x := fxMin + (x1/fLx+1/2.0)*(fxMax-fxMin);
  y := fyMin + (y1/fLx+1/2.0)*(fyMax-fyMin);
  z := fzMin + (zz /fLz+1/2.0)*(fzMax-fzMin);
end;

procedure TSurface.InternalToRef(x, y, z: TFloat; var xx, yy, zz: TFloat);
var
  x1, y1: TFloat;
begin
  x1 := fLx*(x-fxMin)/(fxMax-fxMin) - fLx/2.0;
  y1 := fLx*(y-fyMin)/(fyMax-fyMin) - fLy/2.0;
  zz := fLz*(z-fzMin)/(fzMax-fzMin) - fLz/2.0;
  xx := x1*Cos(fAngle)-y1*Sin(fAngle);
  yy := x1*Sin(fAngle)+y1*Cos(fAngle);
//  zz := z1
end;

procedure TSurface.GetOrder;
var
  x: TFloat;
  i, j, k: TInt;
  t: array[1..8] of TFloat;

function Projection(n: TInt): TFloat;
begin
  result := -cube[n,2]*Cos(fViewAngle)+cube[n,3]*Sin(fViewAngle);
end;

function AxisProjection(n: TInt): TFloat;
begin
  result := -axes[n,2]*Cos(fViewAngle)+axes[n,3]*Sin(fViewAngle);
end;

function Across(n, m: TInt): boolean;
begin
  result := (code[n,1]+code[m,1]+code[n,2]+code[m,2]+code[n,3]+code[m,3])=2;
end;

begin
  for j := 1 to 8 do
  begin
    t[j] := Projection(j);
    CubeOrder[j] := j;
  end;
  for j := 1 to 7 do
  begin
    for i := j+1 to 8 do
    begin
      if (t[i] < t[j]) then
      begin
        x := t[i]; t[i] := t[j]; t[j] := x;
        k := CubeOrder[i];
        CubeOrder[i] := CubeOrder[j];
        CubeOrder[j] := k;
      end;
    end;
  end;

  for j := 1 to 4 do
  begin
    t[j] := Projection(j);
    BaseOrder[j] := j;
  end;
  for j := 1 to 3 do
  begin
    for i := j+1 to 4 do
    begin
      if ( t[i] < t[j]) then
      begin
        x := t[i]; t[i] := t[j]; t[j] := x;
        k := BaseOrder[i];
        BaseOrder[i] := BaseOrder[j];
        BaseOrder[j] := k;
      end;
    end;
  end;
  if not Across(BaseOrder[4],BaseOrder[1]) then
  begin
    k := BaseOrder[1];
    BaseOrder[1] := BaseOrder[2];
    BaseOrder[2] := k;
  end;

  for j := 1 to 6 do
  begin
    t[j] := AxisProjection(j);
    AxisOrder[j] := j;
  end;
  for j := 1 to 5 do
  begin
    for i := j+1 to 6 do
    begin
      if ( t[i] < t[j]) then
      begin
        x := t[i]; t[i] := t[j]; t[j] := x;
        k := AxisOrder[i];
        AxisOrder[i] := AxisOrder[j];
        AxisOrder[j] := k;
      end;
    end;
  end;
end;

procedure TSurface.Line(ix1, iy1, ix2, iy2: TInt);
begin
  Canvas.MoveTo(ix1, iy1);
  Canvas.LineTo(ix2, iy2);
end;

procedure TSurface.SetAngle(const Value: TFloat);
begin
  fAngle := Value*Pi/180.0;
end;

procedure TSurface.SetMinMax(xMin, xMax, yMin, yMax, zMin, zMax: TFloat);
begin
  fxMin := xMin;
  fxMax := xMax;
  fyMin := yMin;
  fyMax := yMax;
  fzMin := zMin;
  fzMax := zMax;
end;

procedure TSurface.SetDimensions(margin: TInt; ViewAngle: TFloat);
var
  r: TFloat;
begin
  SetLength(Border,ClientWidth);
  SetLength(BorderMin,ClientWidth);
  SetLength(BorderMax,ClientWidth);
  fViewAngle := ViewAngle*Pi/180.0;
  rWidth := ClientWidth;
  fWidth := (ClientWidth - margin*2);
  fLx := fxMax - fxMin;
  fLy := fyMax - fyMin;
  fL  := sqrt(sqr(fLx)+sqr(fLy));
  r  := fWidth/fL;
  fLx := r*fLx;
  fLy := r*fLy;
  fL  := r*fL;
  rHeight := ClientHeight;
  fHeight := (ClientHeight - margin*4);
  fHeight := fHeight - abs(Sin(fViewAngle)*fL);
  fLz := fzMax - fzMin;
  r  := fHeight/fLz;
  fLz := r*fLz;
  LL[1] := fLx;
  LL[2] := fLy;
  LL[3] := fLz;
end;

procedure TSurface.SetFramePoints;
var
  n, k: TInt;
  x: TFloat;

  function factor(n, k: TInt): TFloat;
  begin
    if code[n,k] = 1 then result := 0.5 else result := -0.5;
  end;

begin
  for n := 1 to 8 do
    for k := 1 to 3 do
      cube[n,k] := LL[k]*factor(n, k);

  for n := 1 to 6 do
    for k := 1 to 3 do
      axes[n,k] := 0.0;

  axes[1,1] := -fL/2.0;
  axes[2,1] :=  fL/2.0;
  axes[3,2] := -fL/2.0;
  axes[4,2] :=  fL/2.0;
  axes[5,3] := -fLz/1.8;
  axes[6,3] :=  fLz/1.8;

  for n := 1 to 8 do
  begin
    x := cube[n,1];
    cube[n,1] := x*cos(fAngle) - cube[n,2]*sin(fAngle);
    cube[n,2] := x*sin(fAngle) + cube[n,2]*cos(fAngle);
  end;

  for n := 1 to 6 do
  begin
    x := axes[n,1];
    axes[n,1] := x*cos(fAngle) - axes[n,2]*sin(fAngle);
    axes[n,2] := x*sin(fAngle) + axes[n,2]*cos(fAngle);
  end;
end;

procedure TSurface.DrawSurface;
var
  n, m, ix1, iy1, ix2, iy2, i, k, kk: TInt;
  x0, y0, z0, x1, y1, z1, x2, y2, z2: TFloat;
  debug: boolean;

  procedure VertexToRef(n: TInt; var x, y, z: TFloat);
  begin
    x := cube[n,1];
    y := cube[n,2];
    z := cube[n,3];
  end;

  procedure OnePoint(n, m: TInt; var ix, iy: TInt);
  var
    p, q: TFloat;
    x, y, z: TFloat;     // internal coordinates
    xx, yy, zz: TFloat;   // reference coordinates
    xxx, yyy: TFloat; // rotated coordinates
  begin
    q := fStep*n;
    p := fStep*m;
    // reference <- (p,q)
    xx := x0 + p*(x1-x0) + q*(x2-x0);
    yy := y0 + p*(y1-y0) + q*(y2-y0);
    // zz := z0 + p*(z1-z0) + q*(z2-z0); we do not need z
    // rotated <- reference
    xxx :=  xx*Cos(fAngle)+yy*Sin(fAngle);
    yyy := -xx*Sin(fAngle)+yy*Cos(fAngle);
    //  zzz :=  zz;
    //  internal <- rotated
    x := fxMin + (xxx/fLx+1/2.0)*(fxMax-fxMin);
    y := fyMin + (yyy/fLx+1/2.0)*(fyMax-fyMin);
    z := fZFunc(x, y);
    // reference <- (rotated) <- internal
    zz := fLz*(z-fzMin)/(fzMax-fzMin) - fLz/2.0;
    // xx, yy remain the same, use them
    RefToScreen(xx, yy, zz, ix, iy);
  end;

  function CanDraw(ix, iy: TInt): boolean;
  begin
    result := (iy >= BorderMax[ix]) or (iy <= BorderMin[ix]);
  end;

  procedure DrawLine(n1, m1, n2, m2: TInt);
  var
    dx, dy, adx, ady, step: TInt;
    d, ix, iy, k: TInt;
    factor: TFloat;
  begin
    OnePoint(n1, m1, ix1, iy1);
    OnePoint(n2, m2, ix2, iy2);
    dx := ix2-ix1;
    dy := iy2-iy1;
    adx := abs(dx);
    ady := abs(dy);
    if (adx >= ady) then
    begin
      factor := dy;
      if dx = 0 then factor := 0 else factor :=  factor/dx;
      if ix2 > ix1 then step := 1 else step := -1;
      if CanDraw(ix1,iy1) then
      begin
        d := 0;
        for k := 0 to adx do
        begin
          ix := ix1 + d;
          iy := iy1 + round(factor*d);
          if not CanDraw(ix,iy) then break;
          Border[ix] := iy;
          d := d + step;
        end;
        Line(ix1,iy1,ix,iy);
      end else
      begin
        if CanDraw(ix2,iy2) then
        begin
          d := 0;
          for k := 0 to adx do
          begin
            ix := ix2 + d;
            iy := iy2 + round(factor*d);
            if not CanDraw(ix,iy) then break;
            Border[ix] := iy;
            d := d - step;
          end;
          Line(ix,iy,ix2,iy2);
        end
      end;
    end else
    begin
      factor := dx;
      if dy = 0 then factor := 0 else factor :=  factor/dy;
      if iy2 > iy1 then step := 1 else step := -1;
      if CanDraw(ix1,iy1) then
      begin
        d := 0;
        for k := 0 to ady do
        begin
          iy := iy1 + d;
          ix := ix1 + round(factor*d);
          if not CanDraw(ix,iy) then break;
            Border[ix] := iy;
          d := d + step;
        end;
        Line(ix1,iy1,ix,iy);
      end else
      begin
        if CanDraw(ix2,iy2) then
        begin
          d := 0;
          for k := 0 to ady do
          begin
            iy := iy2 + d;
            ix := ix2 + round(factor*d);
            if not CanDraw(ix,iy) then break;
            Border[ix] := iy;
            d := d - step;
          end;
          Line(ix,iy,ix2,iy2);
        end
      end;
    end;
  end;

begin
  debug := false;

  if not Assigned(fZFunc) then exit;
  // these values are used in OnePoint
  VertexToRef(BaseOrder[4], x0, y0, z0);
  VertexToRef(BaseOrder[3], x1, y1, z1);
  VertexToRef(BaseOrder[2], x2, y2, z2);
  PenColor := LineColor;

  // Border Min/Max init
  for n := Low(BorderMin) to High(BorderMin) do
  begin
    BorderMin[n] := ClientHeight-1;
    BorderMax[n] := 0;
  end;
  for i := 1 to NInterval do
  begin
    for k := Low(BorderMin) to High(BorderMin) do
      Border[k] := -1;
    n := i-1;
    for m := 1 to NInterval do
    begin
      DrawLine(n+1, m, n, m);
      DrawLine(n, m-1, n, m);
    end;
    if i = NInterval then
      for m := 1 to NInterval do
        DrawLine(i, m-1, i, m);

    m := i-1;
    if i = 1 then
    for n := NInterval downto 1 do
    begin
      DrawLine(n, i-1, n, i);
      DrawLine(n, i-1, n-1, i-1);
    end;

    for k := Low(BorderMin) to High(BorderMin) do
    begin
      kk := Border[k];
      if kk >= 0 then
      begin
        BorderMax[k] := max(BorderMax[k],kk);
        BorderMin[k] := min(BorderMin[k],kk);
      end;
    end;
  end;

  // Draw borders - debug
  if debug then
  begin
    Canvas.Pen.Width := 2;
    Canvas.Pen.Color := clBlue;
    for n := Low(BorderMin)+1 to High(BorderMin) do
    begin
      Line(n-1,BorderMin[n-1],n,BorderMin[n]);
    end;
    Canvas.Pen.Color := clGreen;
    for n := Low(BorderMin)+1 to High(BorderMin) do
    begin
      Line(n-1,BorderMax[n-1],n,BorderMax[n]);
    end;
    Canvas.Pen.Width := 1;
  end;
end;

procedure TSurface.DrawFrame;
var
  n, k, m, ix1, iy1, ix2, iy2, ix0, iy0,
  ixx: TInt;

  function SameEdge(n,m: TInt): boolean;
  begin
    result := (abs(code[n,1]-code[m,1])+abs(code[n,2]-code[m,2])+abs(code[n,3]-code[m,3]))=1;
  end;

  function SameFace(n,m: TInt): boolean;
  begin
    result := ((code[n,1]=code[m,1]) or (code[n,2]=code[m,2]) or (code[n,3]=code[m,3]));
  end;

  procedure VertexToScreen(n: TInt; var ix, iy: TInt);
  var
    x, y, z: TFloat;
  begin
    x := cube[n,1];
    y := cube[n,2];
    z := cube[n,3];
    RefToScreen(x, y, z, ix, iy);
  end;

begin
  // Draw the rear part of the frame
  Canvas.Pen.Color := clLtGray;
  n := CubeOrder[1];
  VertexToScreen(n, ix1, iy1);
  for m := 1 to 8 do
    if SameEdge(n,m) then
    begin
      VertexToScreen(m, ix2, iy2);
      Line(ix1, iy1, ix2, iy2);
//      Canvas.TextOut(ix1+2,iy1,IntToStr(n)+'(1)');  // debug
    end;

  // Draw positive axes
  Canvas.Pen.Color := clBlack;
  RefToScreen(0.0, 0.0, 0.0, ix0, iy0);
  RefToScreen(axes[2,1], axes[2,2], axes[2,3], ix1, iy1);
  Line(ix1, iy1, ix0, iy0);
  if abs(Angle) > 90.0 then ixx := ix1-10 else ixx := ix1;
  Canvas.TextOut(ixx,iy1+2,'x');
  RefToScreen(axes[4,1], axes[4,2], axes[4,3], ix1, iy1);
  Line(ix1, iy1, ix0, iy0);
  if abs(Angle) < 0.0 then ixx := ix1-10 else ixx := ix1;
  Canvas.TextOut(ixx,iy1,'y');
  RefToScreen(axes[6,1], axes[6,2], axes[6,3], ix1, iy1);
  Line(ix1, iy1, ix0, iy0);
  Canvas.TextOut(ix1+2,iy1,'z');

  DrawSurface;

  // Draw the front part of frame
  Canvas.Pen.Color := clDkGray;
  n := CubeOrder[8];
  VertexToScreen(n, ix0, iy0);
//  Canvas.TextOut(ix0+2,iy0,IntToStr(n)+'(8)'); // debug
  for k := 1 to 8 do
  begin
    if SameEdge(n,k) then
    begin
      VertexToScreen(k, ix1, iy1);
      Line(ix1, iy1, ix0, iy0);
//      Canvas.TextOut(ix1+2,iy1,IntToStr(k));   // debug
      for m := 1 to 8 do
      begin
        if SameEdge(k,m) and SameFace(m,n) then
        begin
          VertexToScreen(m, ix2, iy2);
          Line(ix1, iy1, ix2, iy2);
//          Canvas.TextOut(ix2+2,iy2,IntToStr(m));  // debug
        end;
      end;
    end;
  end;

end;

function TSurface.GetNInterval: TInt;
begin
 result := fNInterval;
end;

procedure TSurface.SetNInterval(const Value: TInt);
begin
  fNInterval := Value;
  fStep := 1.0/fNInterval;
end;

procedure TSurface.SetFunction(ZFunc: TZofXY);
begin
  fZFunc := ZFunc;
end;

function TSurface.GetBrushColor: TColor;
begin
  result := Canvas.Brush.Color;
end;

procedure TSurface.SetBrushColor(const Value: TColor);
begin
  Canvas.Brush.Color := Value;
end;

function TSurface.GetPenColor: TColor;
begin
  result := Canvas.Pen.Color;
end;

procedure TSurface.SetPenColor(const Value: TColor);
begin
  Canvas.Pen.Color := Value;
end;
{
procedure TSurface.Paint(Sender: TObject);
begin
  //
end;
}
end.
