{*
 Polynom Libraty
 Nikolai V. Shokhirev, 2004-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2004.12.12 - created
 @version 2005.01.22 - modified
 @version 2005.05.29 Added TPolynom2D and TPolySystem2D
 @version 2007.08.08 - Added DelphiCodeToDoc comments
 }
unit uPolynom;

interface

uses
  Classes, uMatTypes, uDynArrays, uDynData;

const
  Version = '1.0';
  sPolySystem = 'PolySystem';
  sPolySystem2D = 'PolySystem2D';
  offset = 3; // extra lines

type

  TCoeff = array of TFloat;
  TCoeff2D = array of array of TFloat;
  IndexType = (None, Index1, Index2, Both);

  IPolynom = interface
    ['{C358C758-C98B-47EA-9007-BA654DE8EB55}']
    function GetCoeff(Index: TInt): TFloat;
    function GetDegree: TInt;
    procedure SetCoeff(Index: TInt; const Value: TFloat);
    function GetCoefficients: string;
    procedure SetCoefficients(const Value: string);
    function Add(const P: IPolynom; c:TFloat=1.0): IPolynom;
    function Sub(const P: IPolynom; c:TFloat=1.0): IPolynom;
    function Mul(const P: IPolynom): IPolynom;
    procedure Combine(const P: IPolynom; c:TFloat=1.0);
    procedure Assign(const P: IPolynom);
    procedure Times(C: TFloat);
    function Eval(x: TFloat): TFloat;
    function Integral(x1, x2: TFloat): TFloat;
    function Integrate: IPolynom;
    function Diff: IPolynom;
    property Degree: TInt read GetDegree;
    property Coeff[Index: TInt]: TFloat read GetCoeff write SetCoeff; default;
    property Coefficients: string read GetCoefficients write SetCoefficients;
  end;

  IPolySystem = interface(IComment)
  ['{8BB59D6B-2986-45B8-B651-7ECC0526085C}']
    function GetMaxDegree: TInt;
    function GetPolynom(Index: TInt): IPolynom;
    procedure SetPolynom(Index: TInt; const Value: IPolynom);
    procedure SaveToFile(const FileName: string);
    procedure LoadFromFile(const FileName: string);
    procedure Add(const aPolynom: IPolynom);
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function Combine(const Factors: IFArr1D): IPolynom;
    property MaxDegree: TInt read GetMaxDegree;
    property Polynom[Index: TInt]: IPolynom read GetPolynom write SetPolynom; default;
  end;

  IPolynom2D = interface
    ['{226CB747-75C6-4029-94A2-09929547EAD3}']
    function GetCoeff(Index1, Index2: TInt): TFloat;
    function GetDegree1: TInt;
    function GetDegree2: TInt;
    procedure SetCoeff(Index1, Index2: TInt; const Value: TFloat);
    function GetCoefficients: string;
    procedure SetCoefficients(const Value: string);
    function Add(const P: IPolynom2D; c:TFloat=1.0): IPolynom2D;
    function Sub(const P: IPolynom2D; c:TFloat=1.0): IPolynom2D;
    function Mul(const P: IPolynom2D): IPolynom2D;
    procedure Combine(const P: IPolynom2D; c:TFloat=1.0);
    procedure Assign(const P: IPolynom2D);
    procedure Times(C: TFloat);
    function Eval(x1, x2: TFloat): TFloat;
    function Polynom1(x2: TFloat): IPolynom;
    function Polynom2(x1: TFloat): IPolynom;
    function Integral(x1min, x1max, x2min, x2max: TFloat): TFloat;
    function Integrate(i: IndexType): IPolynom2D;
    function Diff(i: IndexType): IPolynom2D;
    property Degree1: TInt read GetDegree1;
    property Degree2: TInt read GetDegree2;
    property Coeff[Index1, Index2: TInt]: TFloat read GetCoeff write SetCoeff; default;
    property Coefficients: string read GetCoefficients write SetCoefficients;
  end;

  IPolySystem2D = interface(IComment)
  ['{944610C0-CC75-11D9-99A1-0001023EA792}']
    function GetCount: TInt;
    function GetPolynom(Index: TInt): IPolynom2D;
    procedure SetPolynom(Index: TInt; const Value: IPolynom2D);
    function GetMaxDegree1: TInt;
    function GetMaxDegree2: TInt;
    procedure SaveToFile(const FileName: string);
    procedure LoadFromFile(const FileName: string);
    procedure Add(const aPolynom: IPolynom2D);
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function Combine(const Factors: IFArr1D): IPolynom2D;
    property Count: TInt read GetCount;
    property MaxDegree1: TInt read GetMaxDegree1;
    property MaxDegree2: TInt read GetMaxDegree2;
    property Polynom[Index: TInt]: IPolynom2D read GetPolynom write SetPolynom; default;
  end;

  TPolynom = class(TInterfacedObject, IPolynom)
  private
    { Private declarations }
    cf: TCoeff;
    function GetCoeff(Index: TInt): TFloat;
    function GetDegree: TInt;
    procedure SetCoeff(Index: TInt; const Value: TFloat);
    function GetCoefficients: string;
    procedure SetCoefficients(const Value: string);
  public
    { Public declarations }
    constructor Create; overload;
    constructor Create(Deg: TInt); overload;
    constructor Create(const P: IPolynom); overload;
    constructor Create(const Coeffs: string); overload;
    destructor Destroy; override;
    function Add(const P: IPolynom; c:TFloat=1.0): IPolynom;
    function Sub(const P: IPolynom; c:TFloat=1.0): IPolynom;
    function Mul(const P: IPolynom): IPolynom;
    procedure Combine(const P: IPolynom; c:TFloat=1.0);
    procedure Assign(const P: IPolynom);
    procedure Times(C: TFloat);
    function Eval(x: TFloat): TFloat;
    function Integral(x1, x2: TFloat): TFloat;
    function Integrate: IPolynom;
    function Diff: IPolynom;
    property Degree: TInt read GetDegree;
    property Coeff[Index: TInt]: TFloat read GetCoeff write SetCoeff; default;
    property Coefficients: string read GetCoefficients write SetCoefficients;
  end;

  TPolySystem = class(TComment, IPolySystem)
  private
    fList: TInterfaceList;
    function GetMaxDegree: TInt;
    function GetPolynom(Index: TInt): IPolynom;
    procedure SetPolynom(Index: TInt; const Value: IPolynom);
  public
    constructor Create; overload;
    constructor Create(const aMaxDegree: Integer); overload;
    constructor Create(const FileName: string); overload;
    destructor Destroy; override;
    procedure SaveToFile(const FileName: string);
    procedure LoadFromFile(const FileName: string);
    procedure Add(const aPolynom: IPolynom);
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function Combine(const Factors: IFArr1D): IPolynom;
    property MaxDegree: TInt read GetMaxDegree;
    property Polynom[Index: TInt]: IPolynom read GetPolynom write SetPolynom; default;
  end;

  TPolynom2D = class(TInterfacedObject, IPolynom2D)
  private
    { Private declarations }
    cf: TCoeff2D;
    function GetCoeff(Index1, Index2: TInt): TFloat;
    function GetDegree1: TInt;
    function GetDegree2: TInt;
    procedure SetCoeff(Index1, Index2: TInt; const Value: TFloat);
    function GetCoefficients: string;
    procedure SetCoefficients(const Value: string);
  public
    { Public declarations }
    constructor Create; overload;
    constructor Create(Deg1, Deg2: TInt); overload;
    constructor Create(const P: IPolynom2D); overload;
    constructor Create(const Coeffs: string); overload;
    destructor Destroy; override;
    function Add(const P: IPolynom2D; c:TFloat=1.0): IPolynom2D;
    function Sub(const P: IPolynom2D; c:TFloat=1.0): IPolynom2D;
    function Mul(const P: IPolynom2D): IPolynom2D;
    procedure Combine(const P: IPolynom2D; c:TFloat=1.0);
    procedure Assign(const P: IPolynom2D);
    procedure Times(C: TFloat);
    function Eval(x1, x2: TFloat): TFloat;
    function Polynom1(x2: TFloat): IPolynom;
    function Polynom2(x1: TFloat): IPolynom;
    function Integral(x1min, x1max, x2min, x2max: TFloat): TFloat;
    function Integrate(i: IndexType): IPolynom2D;
    function Diff(i: IndexType): IPolynom2D;
    property Degree1: TInt read GetDegree1;
    property Degree2: TInt read GetDegree2;
    property Coeff[Index1, Index2: TInt]: TFloat read GetCoeff write SetCoeff; default;
    property Coefficients: string read GetCoefficients write SetCoefficients;
  end;

  TPolySystem2D = class(TComment, IPolySystem2D)
  private
    fList: TInterfaceList;
    function GetCount: TInt;
    function GetPolynom(Index: TInt): IPolynom2D;
    procedure SetPolynom(Index: TInt; const Value: IPolynom2D);
    function GetMaxDegree1: TInt;
    function GetMaxDegree2: TInt;
  public
    constructor Create;  overload;
    constructor Create(const FileName: string); overload;
    destructor Destroy; override;
    procedure SaveToFile(const FileName: string);
    procedure LoadFromFile(const FileName: string);
    procedure Add(const aPolynom: IPolynom2D);
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function Combine(const Factors: IFArr1D): IPolynom2D;
    property MaxDegree1: TInt read GetMaxDegree1;
    property MaxDegree2: TInt read GetMaxDegree2;
    property Count: TInt read GetCount;
    property Polynom[Index: TInt]: IPolynom2D read GetPolynom write SetPolynom; default;
  end;

{ Polynom routines }

  { result := ( |P1-P2| < eps )
    if CheckDegree and (P1.Degree <> P1.Degree) then  result := false  }
  function SamePolynom(const P1, P2: IPolynom; eps: TFloat;
                             CheckDegree: boolean = false): boolean;

  { result := P1 x P2  }
  function PxP(const P1, P2: IPolynom): IPolynom;

  { result := x**n  }
  function Monomial(const n: TInt; MaxDeg: TInt = -1): IPolynom;

  { result = Sum(i = X.Lo1 to X.Hi1, P1(X[i])*P2(X[i]) ) }
  function DotProdPoly(const P1, P2: IPolynom; const X: IFArr1D): TFloat;

  { result = Sum(i = X.Lo1 to X.Hi1, P1(X[i])^2 ) }
  function Norm2Poly(const P: IPolynom; const X: IFArr1D): TFloat;

  { P = P/sqrt(Norm2Poly(P)); result = Norm2Poly(P) > MinFloat }
  function NormalizePoly(const P: IPolynom; const X: IFArr1D): boolean;

  { Coeff = +/- Variation/2 - for tests }
  procedure RandPolynom(const P: IPolynom; Variation: TFloat=9.9);

{ Polynom2D routines }

  { result := ( |P1-P2| < eps ) }
  function SamePolynom2D(const P1, P2: IPolynom2D; eps: TFloat;
                               CheckDegree: boolean = false): boolean;

  { result := P1 x P2  }
  function P2DxP2D(const P1, P2: IPolynom2D): IPolynom2D;

  { result := x1**n1*x2**n2  }
  function Monomial2D(const n1, n2: TInt; MaxDeg1: TInt = -1;
                                          MaxDeg2: TInt = -1): IPolynom2D;

  { result = Sum(i = R.Lo1 to R.Hi1, P1(R.x1[i],R.x2[i])*P2(R.x1[i],R.x2[i]) ) }
  function DotProdPoly2D(const P1, P2: IPolynom2D; const R: IFData1D): TFloat;

  { result = Sum(i = R.Lo1 to R.Hi1, P1(R.x1[i],R.x2[i])^2 ) }
  function Norm2Poly2D(const P: IPolynom2D; const R: IFData1D): TFloat;

  { P = P/sqrt(Norm2Poly2D(P)); result = Norm2Poly2D(P) > MinFloat }
  function NormalizePoly2D(const P: IPolynom2D; const R: IFData1D): boolean;

  { Coeff = +/- Variation/2 - for tests }
  procedure RandPolynom2D(const P: IPolynom2D; Variation: TFloat = 9.9);

  function Poly2DToStr(const Polynom: IPolynom2D): string;

{ PolySystem routines }

  { P(x) = x^k, k = 0,..,MaxDegree }
  function GenPolySys(const MaxDegree: TInt;
                                         FullSize: boolean = true): IPolySystem;

{ PolySystem2D routines }

  { P(x,y) = x^k*y^(m-k), m = 0,..,MaxDegree }
  function GenPolySys2D(const MaxDegree: TInt;
                                       FullSize: boolean = true): IPolySystem2D;

  { Gramm-Schmidt orthonormalization }
  procedure GrammSchmidt2D(const PolySys: IPolySystem2D; const R: IFData1D);

  { P(X,i, Yi) = Zi }
function PolyFitSurface(const XYZ: IFArr2D; MaxDegree: TInt;
                        var Variance: TFloat; tol: TFloat=cMachEps): IPolynom2D;
implementation

uses
  math, Sysutils, uDynLinAlg;

function SamePolynom(const P1, P2: IPolynom; eps: TFloat; CheckDegree: boolean): boolean;
var
  i: TInt;
  d: TFloat;
begin
  if CheckDegree then
  begin
    result := (P1.Degree=P2.Degree);
    if not result then exit;
  end;

  d := 0.0;
  if (P1.Degree >= P2.Degree) then
    for i := min(P1.Degree,P2.Degree)+1 to P1.Degree do
      d := max(d,abs(P1[i]))
  else
    for i := min(P1.Degree,P2.Degree)+1 to P2.Degree do
      d := max(d,abs(P2[i]));
  for i := 0 to min(P1.Degree,P2.Degree) do
    d := max(d,abs(P1[i]-P2[i]));
  result := (d < abs(eps));
end;

function PxP(const P1, P2: IPolynom): IPolynom;
var
  k, i, n: TInt;
  d: TFloat;
  P: IPolynom;
begin
  n := P1.Degree + P2.Degree;
  P := TPolynom.Create(n);
  for k := 0 to n do
  begin
    d := 0.0;
    for i := max(0,k-P2.Degree) to min(P1.Degree, k) do
      d := d + P1[i]*P2[k-i];
    P[k] := d;
  end;
  result := P;
end;

function Monomial(const n: TInt; MaxDeg: TInt): IPolynom;
var
  P: IPolynom;
begin
  P := TPolynom.Create(max(n,MaxDeg));
  P[n] := 1.0;
  result := P;
end;

{ result = Sum(i = X.Lo1 to X.Hi1, P1(X[i])*P2(X[i]) ) }
function DotProdPoly(const P1, P2: IPolynom; const X: IFArr1D): TFloat;
var
  i: TInt;
  sum: TFloat;
begin
  sum := 0.0;
  for i := X.Lo1 to X.Hi1 do
    sum := sum + P1.Eval(X[i])*P2.Eval(X[i]);
  result := sum;
end;

{ result = Sum(i = X.Lo1 to X.Hi1, P1(X[i])^2 ) }
function Norm2Poly(const P: IPolynom; const X: IFArr1D): TFloat;
var
  i: TInt;
  sum: TFloat;
begin
  sum := 0.0;
  for i := X.Lo1 to X.Hi1 do
    sum := sum + sqr(P.Eval(X[i]));
  result := sum;
end;

{ P = P/sqrt(Norm2Poly(P)) }
function NormalizePoly(const P: IPolynom; const X: IFArr1D): boolean;
var
  sum: TFloat;
begin
  sum := Norm2Poly(P,X);
  if sum > MinFloat then
  begin
    P.Times(1.0/sqrt(sum));
    result := true;
  end else
    result := false;
end;

procedure RandPolynom(const P: IPolynom; Variation: TFloat);
var
  k: TInt;
begin
  for k := 0 to P.Degree do
    P[k] := Variation*(random-0.5);
end;

{ Polynom2D routines }

function SamePolynom2D(const P1, P2: IPolynom2D; eps: TFloat; CheckDegree: boolean): boolean;
var
  i1, i2, max1, max2: TInt;
  d: TFloat;
begin
  if CheckDegree then
  begin
    result := (P1.Degree1=P2.Degree1) and (P1.Degree2=P2.Degree2);
    if not result then exit;
  end;

  d := 0.0;
  max1 := max(P1.Degree1,P2.Degree1);
  max2 := max(P1.Degree2,P2.Degree2);
  for i1 := 0 to max1 do
    for i2 := 0 to max2 do
      d := max(d,abs(P1[i1,i2]-P2[i1,i2]));
  result := (d < abs(eps));
end;

function P2DxP2D(const P1, P2: IPolynom2D): IPolynom2D;
var
  k1, k2, i1, i2, n1, n2: TInt;
  d: TFloat;
  P: IPolynom2D;
begin
  n1 := P1.Degree1 + P2.Degree1;
  n2 := P1.Degree2 + P2.Degree2;
  P := TPolynom2D.Create(n1,n2);
  for k1 := 0 to n1 do
  begin
    for k2 := 0 to n2 do
    begin
      d := 0.0;
      for i1 := max(0,k1-P2.Degree1) to min(P1.Degree1, k1) do
       for i2 := max(0,k1-P2.Degree2) to min(P1.Degree2, k2) do
        d := d + P1[i1,i2]*P2[k1-i1,k2-i2];
      P[k1,k2] := d;
    end;
  end;
  result := P;
end;

function Monomial2D(const n1, n2: TInt; MaxDeg1: TInt; MaxDeg2: TInt): IPolynom2D;
var
  P: IPolynom2D;
begin
  P := TPolynom2D.Create(max(n1,MaxDeg1),max(n2,MaxDeg2));
  P[n1,n2] := 1.0;
  result := P;
end;

{ result = Sum(i = R.Lo1 to R.Hi1, P1(R.x1[i],R.x2[i])*P2(R.x1[i],R.x2[i]) ) }
function DotProdPoly2D(const P1, P2: IPolynom2D; const R: IFData1D): TFloat;
var
  i: TInt;
  sum: TFloat;
begin
  sum := 0.0;
  for i := R.Lo1 to R.Hi1 do
    sum := sum + P1.Eval(R.x1[i],R.x2[i])*P2.Eval(R.x1[i],R.x2[i]);
  result := sum;
end;

{ result = Sum(i = R.Lo1 to R.Hi1, P1(R.x1[i],R.x2[i])^2 ) }
function Norm2Poly2D(const P: IPolynom2D; const R: IFData1D): TFloat;
var
  i: TInt;
  sum: TFloat;
begin
  sum := 0.0;
  for i := R.Lo1 to R.Hi1 do
    sum := sum + sqr(P.Eval(R.x1[i],R.x2[i]));
  result := sum;
end;

{ P = P/sqrt(Norm2Poly2D(P)) }
function NormalizePoly2D(const P: IPolynom2D; const R: IFData1D): boolean;
var
  sum: TFloat;
begin
  sum := Norm2Poly2D(P,R);
  if sum > MinFloat then
  begin
    P.Times(1.0/sqrt(sum));
    result := true;
  end else
    result := false;
end;

procedure RandPolynom2D(const P: IPolynom2D; Variation: TFloat);
var
  k1, k2: TInt;
begin
  for k1 := 0 to P.Degree1 do
    for k2 := 0 to P.Degree2 do
      P[k1,k2] := Variation*(random-0.5);
end;

function Poly2DToStr(const Polynom: IPolynom2D): string;
var
  n, n1, n2, m1, m2: TInt;
  s, m, ss: string;
  c: TFloat;

  function Monom1(n: TInt; x: string):string;
  begin
    if n = 0 then
      result := ''
    else if n = 1 then
      result := x
    else
      result := x+'^'+IntToStr(n);
  end;

  function Monom2(n1, n2:TInt):string;
  var
    m: string;
  begin
    if (n2 > 0) and (n1 > 0) then m := ' * ' else m := '';
    result := Monom1(n1,'x')+m+Monom1(n2,'y');
//    result := Monom1(n1,'x1')+m+Monom1(n2,'x2');
  end;

begin
  ss := '';
  m1 := Polynom.Degree1;
  m2 := Polynom.Degree2;
  for n := 0 to (m1+m2) do
    for n1 := 0 to max(m1,m2) do
    begin
      n2 := n-n1;
      if (n1 <= m1) and (n2 >= 0) and (n2 <= m2) then
      begin
        if (n2 > 0) or (n1 > 0) then m := ' * ' else m := '';
        c := Polynom[n1,n2];
        if (c > 0) and  ( (n1 > 0) or (n2 > 0) ) then s := ' +' else s := ' ';
        ss := ss + s+FloatToStrF(c,ffGeneral,8,2)+m+Monom2(n1,n2)+#13#10;
      end;
    end;
  result := ss;
end;

{ PolySystem routines }

function GenPolySys(const MaxDegree: TInt; FullSize: boolean): IPolySystem;
var
  S: IPolySystem;
  k: TInt;
begin
  S := TPolySystem.Create;
  for k := 0 to MaxDegree do
    if FullSize then
      S.Add(Monomial(k,MaxDegree))
    else
      S.Add(Monomial(k));
  result := S;
end;

{ PolySystem2D routines }

function GenPolySys2D(const MaxDegree: TInt; FullSize: boolean): IPolySystem2D;
var
  S: IPolySystem2D;
  m, k: TInt;
begin
  S := TPolySystem2D.Create;
  for m := 0 to MaxDegree do
    for k := 0 to m do
      if FullSize then
        S.Add(Monomial2D(k,m-k,MaxDegree,MaxDegree))
      else
        S.Add(Monomial2D(k,m-k));
  result := S;
end;

procedure GrammSchmidt2D(const PolySys: IPolySystem2D; const R: IFData1D);
var
  m, k, kmax, i: TInt;
  n, nmax: TFloat;
begin
  m := 0;
  // initial check and normalization
  while m < PolySys.Count do
    if not NormalizePoly2D(PolySys[m], R) then
      PolySys.Delete(m)
    else
      inc(m);

  for m := 0 to PolySys.Count-1 do
  begin
    if m > 0 then // no need for the first time
    begin
      // find polynomial with the largest norm
      kmax := m;
      nmax := Norm2Poly2D(PolySys[m],R);
      for k := m+1 to PolySys.Count-1 do
      begin
        n := Norm2Poly2D(PolySys[k],R);
        if n > nmax then
        begin
          nmax := n;
          kmax := k;
        end;
      end;
      // put it in the m-th position
      if (kmax <> m) then PolySys.Exchange(kmax,m);

      // remove small polynomials
      if not NormalizePoly2D(PolySys[m], R) then
      begin
        for i := PolySys.Count-1 downto m+1 do PolySys.Delete(i);
        exit;
      end;
    end;

    // subtract the m-th component from the rest
    for k := m+1 to PolySys.Count-1 do
    begin
      n := DotProdPoly2D(PolySys[k],PolySys[m],R);
      PolySys[k].Combine(PolySys[m],-n)
    end;
  end;
end;

function PolyFitSurface(const XYZ: IFArr2D; MaxDegree: TInt;
                                 var Variance: TFloat; tol: TFloat): IPolynom2D;
var
  R: IFData1D;
  PolySys: IPolySystem2D;
  P: IPolynom2D;
  m, k: TInt;
  sum: TFloat;
begin
  R := TFData1D.Create(XYZ.Lo1,XYZ.Hi1);
  for m := XYZ.Lo1 to XYZ.Hi1 do
  begin
    R.x1[m] := XYZ[m,1];
    R.x2[m] := XYZ[m,2];
  end;

  PolySys := GenPolySys2D(MaxDegree);
  GrammSchmidt2D(PolySys, R);

  P := TPolynom2D.Create(MaxDegree,MaxDegree);
  for m := 0 to PolySys.Count-1 do
  begin
    sum := 0.0;
    for k := XYZ.Lo1 to XYZ.Hi1 do
      sum := sum + PolySys[m].Eval(R.x1[k],R.x2[k])*XYZ[k,3];
    if abs(sum) > tol then
      P.Combine(PolySys[m],sum);
  end;
  sum := 0.0;
  for k := XYZ.Lo1 to XYZ.Hi1 do
    sum := sum + sqr(P.Eval(R.x1[k],R.x2[k])-XYZ[k,3]);
  Variance := sum/(XYZ.Dim1-1);
  result := P;
end;

{ TPolynom }

constructor TPolynom.Create;
begin
  inherited;

end;

constructor TPolynom.Create(Deg: TInt);
var
  k: TInt;
begin
  Create;
  SetLength(cf, Deg+1);
  for k := 0 to Deg do cf[k] := 0.0;
end;

constructor TPolynom.Create(const P: IPolynom);
begin
  Create;
  Self.Assign(P);
end;

constructor TPolynom.Create(const Coeffs: string);
begin
  Create;
  Coefficients := Coeffs;
end;

destructor TPolynom.Destroy;
begin
  SetLength(cf,0);
  inherited;
end;

function TPolynom.Diff: IPolynom;
var
  Q: IPolynom;
  m: TInt;
  a: TFloat;
begin
  Q := TPolynom.Create(Degree-1);
  a := 1.0;
  for m := 1 to Degree do
  begin
    Q[m-1] := Self[m]*a;
    a := a+1.0;
  end;
  result := Q;
end;

function TPolynom.Add(const P: IPolynom; c: TFloat): IPolynom;
var
  Q: IPolynom;
  m, N: TInt;
begin
  N := max(Degree, P.Degree);
  Q := TPolynom.Create(N);
  for m := 0 to N do
    Q[m] := Self[m]+ c*P[m];
  result := Q;
end;

function TPolynom.Eval(x: TFloat): TFloat;
var
  m: TInt;
  s: TFloat;
begin
  s := 0.0;
  for m := Degree downto 0 do
    s := Coeff[m] + x*s;
  result := s
end;

function TPolynom.GetCoeff(Index: TInt): TFloat;
begin
  if (Index >= 0) and (Index <= Degree) then
    result := cf[Index]
  else
    result := 0.0;
end;

function TPolynom.GetDegree: TInt;
begin
  result := High(cf);
end;

function TPolynom.Integral(x1, x2: TFloat): TFloat;
var
  Q: IPolynom;
begin
  Q := Integrate;
  result := Q.Eval(x2)-Q.Eval(x1);
end;

function TPolynom.Integrate: IPolynom;
var
  Q: IPolynom;
  m: TInt;
  a: TFloat;
begin
  Q := TPolynom.Create(Degree+1);
  Q[0] := 0.0;
  a := 0.0;
  for m := 0 to Degree do
  begin
    a := a + 1.0;
    Q[m+1] := Self[m]/a;
  end;
  result := Q;
end;

function TPolynom.Mul(const P: IPolynom): IPolynom;
var
  Q: IPolynom;
  k, m, N: TInt;
  s: TFloat;
begin
  N := Degree + P.Degree;
  Q := TPolynom.Create(N);
  for m := 0 to N do
  begin
    s := 0.0;
    for k := max(0,m-P.Degree) to min(m,Degree) do
    begin
      s := s + Self[k]*P[m-k];
    end;
    Q[m] := s;
  end;
  result := Q;
end;

procedure TPolynom.SetCoeff(Index: TInt; const Value: TFloat);
begin
  if (Index >= 0) and (Index <= Degree) then
   cf[Index] := Value;
end;

function TPolynom.Sub(const P: IPolynom; c: TFloat): IPolynom;
var
  Q: IPolynom;
  m, N: TInt;
begin
  N := max(Degree, P.Degree);
  Q := TPolynom.Create(N);
  for m := 0 to N do
    Q[m] := Self[m]- c*P[m];
  result := Q;
end;

function TPolynom.GetCoefficients: string;
var
  m: TInt;
  s, delim: string;
begin
  s := '';
  delim := '';
  for m := 0 to Degree do
  begin
    s := s + delim + FloatToStr(cf[m]);
    delim := ',';
  end;
  result := s
end;

procedure TPolynom.SetCoefficients(const Value: string);
var
  m: TInt;
  sl: TStringList;
begin
  sl := TStringList.Create;
  try
    sl.CommaText := Value;
// Need check for Count = 0 !!!
    if sl.Count <> (Degree+1) then
      SetLength(cf,sl.Count);
    for m := 0 to Degree do
    begin
      cf[m] := StrToFloat(sl[m]);
    end;
  finally
    sl.free;
  end;
end;

procedure TPolynom.Times(C: TFloat);
var
  m: TInt;
begin
  for m := 0 to Degree do
  begin
    cf[m] := cf[m]*C;
  end;
end;

procedure TPolynom.Assign(const P: IPolynom);
var
  m: TInt;
begin
  if P.Degree <> Degree then
    SetLength(cf,P.Degree+1);
  for m := 0 to Degree do
  begin
    Self[m] := P[m];
  end;
end;

procedure TPolynom.Combine(const P: IPolynom; c: TFloat);
var
  m1: TInt;
begin
  Assert( Degree >= P.Degree, 'Dim mismatch');
  for m1 := 0 to P.Degree do
      Self[m1] := Self[m1]+ c*P[m1];
end;

{ TPolySystem }

constructor TPolySystem.Create;
begin
  inherited Create;
  fList := TInterfaceList.Create;
end;

constructor TPolySystem.Create(const aMaxDegree: Integer);
begin
  Create;
  if aMaxDegree >= 0 then
    fList.Count := aMaxDegree+1;
end;

constructor TPolySystem.Create(const FileName: string);
begin
  Create;
  LoadFromFile(FileName);
end;

procedure TPolySystem.Add(const aPolynom: IPolynom);
begin
  fList.Add(aPolynom);
end;

destructor TPolySystem.Destroy;
var
  idx, c: TInt;
begin
  c := fList.Count;
  for idx := c-1 downto 0 do
    fList.Delete(idx);
  fList.Free;
  fList := nil;
  inherited;
end;

function TPolySystem.GetMaxDegree: TInt;
begin
  result := fList.Count-1;
end;

function TPolySystem.GetPolynom(Index: TInt): IPolynom;
begin
  if (Index >= 0) and (Index <= MaxDegree) then
    result := (fList.Items[Index] as IPolynom)
  else
    result := nil;
end;

procedure TPolySystem.LoadFromFile(const FileName: string);
var
  idx: TInt;
  slPS: TStringList;
begin
  slPS := TStringList.Create;
  try
    slPS.LoadFromFile(FileName);
    // slPS[0] = Version
    // slPS[1] =sPolySystem
    Comment := slPS[2];
    for idx := offset to slPS.Count-1 do
    begin
      fList.Add(TPolynom.Create(slPS[idx]));
    end;
  finally
    slPS.Free;
  end;
end;

procedure TPolySystem.SaveToFile(const FileName: string);
var
  idx: TInt;
  slPS: TStringList;
begin
  slPS := TStringList.Create;
  slPS.Append(Version);
  slPS.Append(sPolySystem);
  slPS.Append(Comment);

  for idx := 0 to MaxDegree do
  begin
    slPS.Append((fList.Items[idx] as IPolynom).Coefficients);
  end;
  try
    slPS.SaveToFile(FileName);
  finally
    slPS.Free;
  end;
end;

procedure TPolySystem.SetPolynom(Index: TInt; const Value: IPolynom);
begin
  if (Index >= 0) and (Index <= MaxDegree) then
    fList.Items[Index] := Value;
end;

procedure TPolySystem.Delete(Index: Integer);
begin
  fList.Delete(Index);
end;

procedure TPolySystem.Exchange(Index1, Index2: Integer);
var
  temp: IInterface;
begin
  temp := fList.Items[Index1];
  fList.Items[Index1] := fList.Items[Index2];
  fList.Items[Index2] := temp;
end;

function TPolySystem.Combine(const Factors: IFArr1D): IPolynom;
var
  P: IPolynom;
  m, k: TInt;
begin
  Assert((MaxDegree+1) = Factors.Dim1, 'Dim mismatch');
  P := TPolynom.Create(MaxDegree);
  k := Factors.Lo1;
  for m := 0 to MaxDegree do
  begin
    P.Combine(Polynom[m],Factors[k]);
    inc(k);
  end;
  result := P;
end;

{ TPolynom2D }

constructor TPolynom2D.Create;
begin
  inherited;
end;

constructor TPolynom2D.Create(const P: IPolynom2D);
begin
  Create;
  Self.Assign(P);
end;

constructor TPolynom2D.Create(Deg1, Deg2: TInt);
var
  k1, k2: TInt;
begin
  Create;
  SetLength(cf, Deg1+1, Deg2+1);
  for k1 := 0 to Deg1 do
    for k2 := 0 to Deg2 do
      cf[k1,k2] := 0.0;
end;

constructor TPolynom2D.Create(const Coeffs: string);
begin
  Create;
  Coefficients := Coeffs;
end;

destructor TPolynom2D.Destroy;
begin
  SetLength(cf, 0, 0);
  inherited;
end;

function TPolynom2D.Add(const P: IPolynom2D; c: TFloat): IPolynom2D;
var
  Q: IPolynom2D;
  m1, m2, N1, N2: TInt;
begin
  N1 := max(Degree1, P.Degree1);
  N2 := max(Degree2, P.Degree2);
  Q := TPolynom2D.Create(N1,N2);
  for m1 := 0 to N1 do
    for m2 := 0 to N2 do
      Q[m1,m2] := Self[m1,m2]+ c*P[m1,m2];
  result := Q;
end;

procedure TPolynom2D.Assign(const P: IPolynom2D);
var
  m1, m2: TInt;
begin
  if (P.Degree1 <> Degree1) or (P.Degree1 <> Degree1) then
    SetLength(cf,P.Degree1+1,P.Degree2+1);
  for m1 := 0 to Degree1 do
    for m2 := 0 to Degree2 do
      Self[m1,m2] := P[m1,m2];
end;

procedure TPolynom2D.Combine(const P: IPolynom2D; c: TFloat);
var
  m1, m2: TInt;
begin
  Assert( (Degree1 >= P.Degree1) and (Degree2 >= P.Degree2), 'Dim mismatch');
  for m1 := 0 to P.Degree1 do
    for m2 := 0 to P.Degree2 do
      Self[m1,m2] := Self[m1,m2]+ c*P[m1,m2];
end;

function TPolynom2D.Diff(i: IndexType): IPolynom2D;
var
  k1, k2: TInt;
  Q: IPolynom2D;
begin
  case i of
  None:  begin
           Q := TPolynom2D.Create(Self);
         end;
  Index1:begin
           Q := TPolynom2D.Create(Degree1-1, Degree2);
           for k2 := 0 to Degree2 do
             for k1 := 1 to Degree1 do
             begin
               Q[k1-1,k2] := Self[k1,k2]*(k1);
             end;
         end;
  Index2:begin
           Q := TPolynom2D.Create(Degree1, Degree2-1);
           for k1 := 0 to Degree1 do
             for k2 := 1 to Degree2 do
               Q[k1,k2-1] := Self[k1,k2]*(k2);
         end;
  Both:  begin
           Q := TPolynom2D.Create(Degree1-1, Degree2-1);
           for k1 := 1 to Degree1 do
             for k2 := 1 to Degree2 do
               Q[k1-1,k2-1] := Self[k1,k2]*(k1)*(k2);
         end;
  else
    //
  end;
  result := Q;
end;

function TPolynom2D.Eval(x1, x2: TFloat): TFloat;
var
  m1, m2: TInt;
  s1, s2: TFloat;
begin
  s1 := 0.0;
//  s2 := 0.0;
  for m1 := Degree1 downto 0 do
  begin
    s2 := 0.0;
    for m2 := Degree2 downto 0 do
      s2 := cf[m1,m2] + x2*s2;
    s1 := s2 + x1*s1;
  end;
  result := s1;
end;

function TPolynom2D.GetCoeff(Index1, Index2: TInt): TFloat;
begin
  if (Index1 >= 0) and (Index1 <= Degree1) and
     (Index2 >= 0) and (Index2 <= Degree2) then
    result := cf[Index1, Index2]
  else
    result := 0.0;
end;

function TPolynom2D.GetCoefficients: string;
var
  m1, m2: TInt;
  s1, s2, delim1, delim2: string;
begin
  s2 := '';
  delim2 := '';
  for m2 := 0 to Degree2 do
  begin
    s1 := '';
    delim1 := '';
    for m1 := 0 to Degree1 do
    begin
      s1 := s1 + delim1 + FloatToStrF(cf[m1,m2],ffGeneral,15,3);
      delim1 := ',';
    end;
    s2 := s2 + delim2 + s1;
    delim2 := ';';
  end;
  result := s2
end;

function TPolynom2D.GetDegree1: TInt;
begin
  if Assigned(cf) then
    result := High(cf)
  else
    result := 0;
end;

function TPolynom2D.GetDegree2: TInt;
begin
  if Assigned(cf) then
    result := High(cf[0])
  else
    result := 0;
end;

function TPolynom2D.Integral(x1min, x1max, x2min, x2max: TFloat): TFloat;
var
//  k1, k2: TInt;
  Q: IPolynom2D;
begin
  Q := Self.Integrate(Both);
  result := Q.Eval(x1max, x2max)-Q.Eval(x1min, x2max)
           -Q.Eval(x1max, x2min)+Q.Eval(x1min, x2min);
end;

function TPolynom2D.Integrate(i: IndexType): IPolynom2D;
var
  k1, k2: TInt;
  Q: IPolynom2D;
begin
  case i of
  None:  begin
           Q := TPolynom2D.Create(Self);
         end;
  Index1:begin
           Q := TPolynom2D.Create(Degree1+1, Degree2);
           for k2 := 0 to Degree2 do
           begin
             for k1 := 0 to Degree1 do
               Q[k1+1,k2] := Self[k1,k2]/(k1+1.0);
           end;
//           Q[0,k2] := 0.0;
         end;
  Index2:begin
           Q := TPolynom2D.Create(Degree1, Degree2+1);
           for k1 := 0 to Degree1 do
           begin
             for k2 := 0 to Degree2 do
               Q[k1,k2+1] := Self[k1,k2]/(k2+1.0);
//             Q[k1,0] := 0.0;
           end;
         end;
  Both:  begin
           Q := TPolynom2D.Create(Degree1+1, Degree2+1);
           for k1 := 0 to Degree1 do
           begin
             for k2 := 0 to Degree2 do
               Q[k1+1,k2+1] := Self[k1,k2]/(k1+1.0)/(k2+1.0);
//             Q[k1,0] := 0.0;
           end;
//             for k2 := 0 to Degree2 do Q[0,k2] := 0.0;
         end;
  else
    //
  end;
  result := Q;
end;

function TPolynom2D.Mul(const P: IPolynom2D): IPolynom2D;
var
  k1, k2, i1, i2, n1, n2: TInt;
  d: TFloat;
  Q: IPolynom2D;
begin
  n1 := Self.Degree1 + P.Degree1;
  n2 := Self.Degree2 + P.Degree2;
  Q := TPolynom2D.Create(n1,n2);
  for k1 := 0 to n1 do
  begin
    for k2 := 0 to n2 do
    begin
      d := 0.0;
      for i1 := max(0,k1-P.Degree1) to min(Self.Degree1, k1) do
       for i2 := max(0,k1-P.Degree2) to min(Self.Degree2, k2) do
        d := d + Self[i1,i2]*P[k1-i1,k2-i2];
      Q[k1,k2] := d;
    end;
  end;
  result := Q;
end;

function TPolynom2D.Polynom1(x2: TFloat): IPolynom;
var
  m1, m2: TInt;
  s2: TFloat;
  Q: IPolynom;
begin
  Q := Tpolynom.Create(Degree1);
  for m1 := Degree1 downto 0 do
  begin
    s2 := 0.0;
    for m2 := Degree2 downto 0 do
      s2 := Coeff[m1,m2] + x2*s2;
    Q[m1] := s2;
  end;
  result := Q;
end;

function TPolynom2D.Polynom2(x1: TFloat): IPolynom;
var
  m1, m2: TInt;
  s1: TFloat;
  Q: IPolynom;
begin
  Q := Tpolynom.Create(Degree2);
  for m2 := Degree2 downto 0 do
  begin
    s1 := 0.0;
    for m1 := Degree1 downto 0 do
      s1 := Coeff[m1,m2] + x1*s1;
    Q[m2] := s1;
  end;
  result := Q;
end;

procedure TPolynom2D.SetCoeff(Index1, Index2: TInt; const Value: TFloat);
begin
  if (Index1 >= 0) and (Index1 <= Degree1) and
     (Index2 >= 0) and (Index2 <= Degree2) then
    cf[Index1, Index2] := Value;
end;

procedure TPolynom2D.SetCoefficients(const Value: string);
var
  m1, m2, c1, c2: TInt;
  sl1, sl2: TStringList;
begin
  sl1 := TStringList.Create;
  sl2 := TStringList.Create;
  sl2.Delimiter := ';';
  try
    sl2.DelimitedText := Value;
    c2 := sl2.Count;
    sl1.CommaText := sl2[0];
    c1 := sl1.Count;
// Need check for Count = 0 !!!
//    if (c1 <> (Degree1+1)) or (c2 <> (Degree2+1)) then
      SetLength(cf, c1, c2);
    for m2 := 0 to Degree2 do
    begin
      sl1.CommaText := sl2[m2];
// Need check for Count  !!!
      for m1 := 0 to Degree1 do
      begin
        cf[m1,m2] := StrToFloat(sl1[m1]);
      end;
    end;
  finally
    sl1.free;
    sl2.free;
  end;

end;

function TPolynom2D.Sub(const P: IPolynom2D; c: TFloat): IPolynom2D;
var
  Q: IPolynom2D;
  m1, m2, N1, N2: TInt;
begin
  N1 := max(Degree1, P.Degree1);
  N2 := max(Degree2, P.Degree2);
  Q := TPolynom2D.Create(N1,N2);
  for m1 := 0 to N1 do
    for m2 := 0 to N2 do
      Q[m1,m2] := Self[m1,m2] - c*P[m1,m2];
  result := Q;
end;

procedure TPolynom2D.Times(C: TFloat);
var
  m1, m2: TInt;
begin
  for m1 := 0 to Degree1 do
    for m2 := 0 to Degree2 do
      cf[m1,m2] := C*cf[m1,m2];
end;

{ TPolySystem2D }

constructor TPolySystem2D.Create;
begin
  inherited Create;
  fList := TInterfaceList.Create;
end;

constructor TPolySystem2D.Create(const FileName: string);
begin
  Create;
  LoadFromFile(FileName);
end;

procedure TPolySystem2D.Add(const aPolynom: IPolynom2D);
begin
  fList.Add(aPolynom);
end;

procedure TPolySystem2D.Delete(Index: Integer);
begin
  fList.Delete(Index);
end;

destructor TPolySystem2D.Destroy;
var
  idx, c: TInt;
begin
  c := fList.Count;
  for idx := c-1 downto 0 do
    fList.Delete(idx);
  fList.Free;
  fList := nil;
  inherited;
end;

function TPolySystem2D.GetCount: TInt;
begin
  result := fList.Count;
end;

function TPolySystem2D.GetPolynom(Index: TInt): IPolynom2D;
begin
  if (Index >= 0) and (Index < Count) then
    result := (fList.Items[Index] as IPolynom2D)
  else
    result := nil;
end;

procedure TPolySystem2D.LoadFromFile(const FileName: string);
var
  idx: TInt;
  slPS: TStringList;
begin
  slPS := TStringList.Create;
  try
    slPS.LoadFromFile(FileName);
    // slPS[0] = Version
    // slPS[1] =sPolySystem
    Comment := slPS[2];
    for idx := offset to slPS.Count-1 do
    begin
      fList.Add(TPolynom2D.Create(slPS[idx]));
    end;
  finally
    slPS.Free;
  end;
end;

procedure TPolySystem2D.SaveToFile(const FileName: string);
var
  idx: TInt;
  slPS: TStringList;
begin
  slPS := TStringList.Create;
  slPS.Append(Version);
  slPS.Append(sPolySystem2D);
  slPS.Append(Comment);

  for idx := 0 to Count-1 do
  begin
    slPS.Append((fList.Items[idx] as IPolynom2D).Coefficients);
  end;
  try
    slPS.SaveToFile(FileName);
  finally
    slPS.Free;
  end;
end;

procedure TPolySystem2D.SetPolynom(Index: TInt; const Value: IPolynom2D);
begin
  if (Index >= 0) and (Index < Count) then
    fList.Items[Index] := Value;
end;

procedure TPolySystem2D.Exchange(Index1, Index2: Integer);
var
  temp: IInterface;
begin
  temp := fList.Items[Index1];
  fList.Items[Index1] := fList.Items[Index2];
  fList.Items[Index2] := temp;
end;

function TPolySystem2D.GetMaxDegree1: TInt;
var
  m, n, idx: TInt;
begin
  m := 0;
  for idx := 0 to Count-1 do
  begin
    n := (fList.Items[idx] as IPolynom2D).Degree1;
    m := max(m,n);
  end;
  result := m;
end;

function TPolySystem2D.GetMaxDegree2: TInt;
var
  m, n, idx: TInt;
begin
  m := 0;
  for idx := 0 to Count-1 do
  begin
    n := (fList.Items[idx] as IPolynom2D).Degree2;
    m := max(m,n);
  end;
  result := m;
end;

// PolySys.Count = Factors.Dim1
function TPolySystem2D.Combine(const Factors: IFArr1D): IPolynom2D;
var
  P: IPolynom2D;
  m, k: TInt;
begin
  Assert(Count = Factors.Dim1, 'Dim mismatch');
  P := TPolynom2D.Create(MaxDegree1,MaxDegree2);
  k := Factors.Lo1;
  for m := 0 to Count-1 do
  begin
    P.Combine(Polynom[m],Factors[k]);
    inc(k);
  end;
  result := P;
end;

end.
