{*
 EBK&NVS Pascal-Delphi Math Library: Real Special functions
 @author(Eugene B. Krissinel <keb@ebi.ac.uk> <krissinel@fh.huji.ac.il>)
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 1991.09.09 - created original Turbo Pascal version
 @version 2002.02.02 - included in Delphi PasMatLib
 @version 2003.02.02 - modified
 @version 2008.02.02 - Added new Erf, CumNorm, InvCumNorm
 @version 2009.03.22 - Added InvNormCDF, BivarCumNorm
 Nikolai V. Shokhirev, 2001-2009 Delphi version
 }
unit uSpecFunc;

interface

uses
  uMatTypes;

const
  ln10     = 2.302585092994046; // 2.3025850929940456840179915
  Gamma_Eu = 0.577215664901532860606512;

{ obsolete }
function ArcSin(x: TFloat): TFloat;
function ArcCos(X: TFloat): TFloat;
{ Hyperbolic Functions }
function ch(x: TFloat)    : TFloat;
function sh(x: TFloat)    : TFloat;
function th    (x: TFloat): TFloat;
function arch  (x: TFloat): TFloat;
function arsh  (x: TFloat): TFloat;
function arth  (x: TFloat): TFloat;

function NormPDF(x: TFloat): TFloat;

function NormDist(x, m, sig: TFloat): TFloat;

{ Special functions for the Real Calculations }

{ er(x) = exp(x**2)*erfc(x)    }
function er(x: TFloat)        : TFloat;

{ error function  error < 8.2e-11 }
function erf_(x: TFloat)       : TFloat;

{ EVALUATION OF THE REAL ERROR FUNCTION
  Based upon a Fortran 66 routine in the Naval Surface Warfare Center's
  Mathematics Library (1993 version).
  Adapted by Alan.Miller @ vic.cmis.csiro.au
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 2.3e-16 for |x| < 6  }
function erf(x: TFloat)       : TFloat;

{ complementary error function 1 - erf}
function erfc(x: TFloat)      : TFloat;
{ Modified Bessel function     }
function I0_(x: TFloat)       : TFloat;
{ Modified Bessel function     }
function I1_(x: TFloat)       : TFloat;
{ Modified Bessel function     }
function K0_(x: TFloat)       : TFloat;
{ Modified Bessel function     }
function K1_(x: TFloat)       : TFloat;
{ fk0(x)=K0(x)+ln(x/2)*I0(x)   }
function fk0_(x: TFloat)      : TFloat;
{ fk1(x)=K1(x)-ln(x/2)*I1(x)   }
function fk1_(x: TFloat)      : TFloat;

{ Gamma-function  error < 5 e-7  }
function Gamma_(x: TFloat)    : TFloat;
{ Gamma-function   error < 5 e-16 , x <> 0,-1,-2, .. }
function Gamma(X: TFloat): TFloat;
{ incomplete Gamma-function   error < 5 e-16
 GIM = Gamma(A,X)  int(X < t < inf )
 GIN = gamma(A,X)  int(0 < t < X )
 GIP = P(A,X), gamma(A,X) = P(A,X)*Gamma(A)
 Gamma(A) = GIN + GIM }
procedure INCOG(A, X: TFloat; var GIN, GIM, GIP: TFloat);
{ incomplete gamma-function   error < 5 e-10  }
function gammaInc(a,x: TFloat): TFloat;
{ incomplete gamma-function   error < 5 e-10 gammaX(a,-x) = -gammaX(a,x) }
function  gammaX(a,x: TFloat): TFloat;
{ PolyGamma(n,z) = d[n+1]ln(Gamma(z))/dz[n+1] ,  z > 10, 0 <= n <= 3  }
function  PolyGamma(n: TInt; z: TFloat): TFloat;

{ Inverse Cumulative Normal Distribution Fuction.
  Abromowitz and Stegun, Handbook of Mathematical Functions, Dover, NY.
  see also http://www.johndcook.com/normal_cdf_inverse.html
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 4.5e-4  }
function InvNormCDF(p: TFloat): TFloat;

{ Delphi implementation byGreg McCormick (unit icnorm.pas )
  Water Operational Research Centre, Department of Systems Engineering,
  Brunel University, Uxbridge, UK.
  Rel error < 1.1510-9 for invCumNorm > -38, see more in implementation.
  function obtained from P. J. Acklam,
  http://home.online.no/~pjacklam/notes/invnorm/index.html          }
function invCumNorm(const p: double): double;

{ Cumulative Normal Distribution Fuction. Algorithm 26.2.17 from
  Abromowitz and Stegun, Handbook of Mathematical Functions, Dover, NY.
  http://www.sitmo.com/doc/Calculating_the_Cumulative_Normal_Distribution
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 7.5e-8 ( error < 7.406e-8 for |x| < 8 ) }
function CumNormDF(x: TFloat): TFloat;

{ Better Approximations to Cumulative Normal Fuction by Graeme West,
  http://www4.ncsu.edu/unity/users/p/pfackler/www/ECG790C/accuratecumnorm.pdf
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 2.23e-16 for |x| < 8                                   }
function CumNorm(x: TFloat): TFloat;

{ Approximations to Bivariate Cumulative Normal Fuctiona by Graeme West,
  http://www.finmod.co.za/accuratecumnorm.pdf
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 1.2e-16 for |x| < 8  ???                                 }
Function BivarCumNorm(a, b, r: TFloat): TFloat;

(*
{ integral exp-function        }
function E1_(x: TFloat)       : TFloat;
{ Modified Bessel function     }
function I_nu(nu, x: TFloat)  : TFloat;

{ Undocumented Functions }
//function Gamma(x: TFloat): TFloat;
function  BJn   (n,x: TFloat): TFloat;
function  BIn   (n,x: TFloat): TFloat;
function  BKn   (n,x: TFloat): TFloat;
function  Sn(n: TInt; z: TFloat): TFloat;
function  Un(n: TInt; z,a: TFloat): TFloat;
*)
{ ===================================================================  }

implementation

uses
  math;

function  ArcSin (x: TFloat): TFloat;
var
  R,S,y    : TFloat;
  l        : TInt;
begin
  if  x>=1.0    then    ArcSin := Pi/2.0
  else if x<=-1.0 then  ArcSin := -Pi/2.0
  else
    begin
      if abs(x)<0.71  then  y := x
                      else  y := sqrt(1.0-sqr(x));
      S := 0.0;
      R := y;
      l := 1;
      repeat
        S := S+R;
        R := R*sqr(y*l)/((l+1) * (l+2));
        l := l+2;
      until (abs(R)<1.0e-35) or
            (abs(R)<=1.0e-16*abs(S));
      S := S+R;
      if abs(x-y)>1.0e-12  then
        begin
          if x>0.0  then   S := Pi/2.0-S
                    else   S := -Pi/2.0+S;
        end;
      ArcSin := S;
    end;
end;

function  ArcCos(X: TFloat): TFloat;
begin
  ArcCos := Pi*0.5 - ArcSin(X);
end;

{ Hyperbolic Functions }

function ch(x: TFloat): TFloat;
begin
  ch := 0.5*(exp1(x)+exp1(-x));
end;

function sh(x: TFloat): TFloat;
begin
  sh := 0.5*(exp1(x)-exp1(-x));
end;

function th(x: TFloat): TFloat;
begin
  th := (1.0-exp1(-2.0*x))/(1.0+exp1(-2.0*x));
end;

function arch(x: TFloat): TFloat;
begin
  arch := ln(x + sqrt(sqr(x)-1.0) );
end;

function arsh(x: TFloat): TFloat;
begin
  arsh := ln(x + sqrt(sqr(x)+1.0) );
end;

function arth(x: TFloat): TFloat;
begin
  arth := 0.5 * ln((1.0+x)/(1.0-x));
end;

function NormPDF(x: TFloat): TFloat;
begin
  result := exp(-x*x/2.0)/sqrt(2.0*Pi);
end;

function NormDist(x, m, sig: TFloat): TFloat;
begin
  result := NormPDF((x-m)/sig)/sig;
end;

{ Special functions for the Real Calculations }

function er(x: TFloat): TFloat;
const
  epsilon = 1.0E-14;
  sp = 1.7724538509055160;{ sqrt(pi) }
  a1 = 0.4613135;    b1 = 0.1901635;
  a2 = 0.09999216;   b2 = 1.7844927;
  a3 = 0.002883894;  b3 = 5.5253437;
var
  s, t, m, xx, xx2           : TFloat;
begin
  xx := x*x;
  if abs(x) < 3.0
  then {  series expansion of er(x)   }
  begin
    m := 1.0; t   := 1.0;
    s := 0.0; xx2 := xx*2.0;
    repeat
      s := s + t; m := m + 2.0;
      t := t*xx2; t := t/m;
    until (abs(t) < epsilon);
    er := exp1(xx) - s*x*2.0/sp;
  end
  else{  asymptotics from Abramowitz }
    if x > 0.0 then
      er := x*(a1/(b1+xx)+a2/(b2+xx)+a3/(b3+xx))
    else
      er := exp1(xx)*2.0 +
            x*(a1/(b1+xx)+a2/(b2+xx)+a3/(b3+xx));
end;{ of er }

//-----------------------------------------------------------------------
//             EVALUATION OF THE REAL ERROR FUNCTION
// Based upon a Fortran 66 routine in the Naval Surface Warfare Center's
// Mathematics Library (1993 version).
// Adapted by Alan.Miller @ vic.cmis.csiro.au
// Translated to Pascal by Nikolai Shokhirev Nikolai@Shokhirev.com
//-----------------------------------------------------------------------
function erf(x: TFloat): TFloat;
const
  c = 0.564189583547756;
//  zero := 0.0;
  a: array[1..5] of TFloat = ( 0.771058495001320E-04, -0.133733772997339E-02,
                               0.323076579225834E-01, 0.479137145607681E-01,
                               0.128379167095513E+00);
  b: array[1..3] of TFloat = ( 0.301048631703895E-02, 0.538971687740286E-01,
                               0.375795757275549E+00);
  p: array[1..8] of TFloat = (-1.36864857382717E-07, 5.64195517478974E-01,
                               7.21175825088309E+00, 4.31622272220567E+01,
                               1.52989285046940E+02, 3.39320816734344E+02,
                               4.51918953711873E+02, 3.00459261020162E+02);
  q: array[1..8] of TFloat = (1.00000000000000E+00, 1.27827273196294E+01,
                              7.70001529352295E+01, 2.77585444743988E+02,
                              6.38980264465631E+02, 9.31354094850610E+02,
                              7.90950925327898E+02, 3.00459260956983E+02);
  r: array[1..5] of TFloat = (2.10144126479064E+00, 2.62370141675169E+01,
                              2.13688200555087E+01, 4.65807828718470E+00,
                              2.82094791773523E-01);
  s: array[1..4] of TFloat = (9.41537750555460E+01, 1.87114811799590E+02,
                              9.90191814623914E+01, 1.80124575948747E+01);
var
  ax, bot, t, top, x2: TFloat;
begin
  //-------------------------
  ax := ABS(x);
  if (ax <= 0.5) then
  begin
    t := x*x;
    top := ((((a[1]*t + a[2])*t + a[3])*t + a[4])*t + a[5]) + 1.0;
    bot := ((b[1]*t + b[2])*t + b[3])*t + 1.0;
    result := x*(top/bot);
  end
  else if (ax <= 4.0) then
  begin
    top := ((((((p[1]*ax + p[2])*ax + p[3])*ax + p[4])*ax + p[5])*ax
               + p[6])*ax + p[7])*ax + p[8];
    bot := ((((((q[1]*ax + q[2])*ax + q[3])*ax + q[4])*ax + q[5])*ax
               + q[6])*ax + q[7])*ax + q[8];
    result := 0.5 + (0.5 - exp(-x*x)*top/bot);
    if (x < 0.0) then
      result := -result;
  end
  else if (ax < 5.8) then
  begin
    x2 := x*x;
    t := 1.0 / x2;
    top := (((r[1]*t + r[2])*t + r[3])*t + r[4])*t + r[5];
    bot := (((s[1]*t + s[2])*t + s[3])*t + s[4])*t + 1.0;
    result := (c - top/(x2*bot)) / ax;
    result := 0.5 + (0.5 - exp(-x2)*result);
    if (x < 0.0) then
      result := -result;
  end
  else
    result := sign1(x);   /// Asymptotics?
end;

function erf_(x: TFloat): TFloat;
const
  epsilon = 1.0E-14;
  sp = 1.7724538509055160;{ sqrt(pi) }
  a1 = 0.4613135;    b1 = 0.1901635;
  a2 = 0.09999216;   b2 = 1.7844927;
  a3 = 0.002883894;  b3 = 5.5253437;
var
  s, t, m, xx, xx2           : TFloat;
begin
  xx := x*x;
  if abs(x) < 3.0
  then {  series expansion of erf(x)   }
  begin
    m := 1.0; t   := 1.0;
    s := 0.0; xx2 := xx*2.0;
    repeat
      s := s + t;  m := m + 2.0;  t := t*xx2/m;
    until (abs(t)<epsilon);
    result := exp1(-xx)*s*x*2.0/sp;
  end
  else{  asimptotics from Abramowitz }
    result := sign1(x) - exp1(-xx)*x*(a1/(b1+xx)+a2/(b2+xx)+a3/(b3+xx));
end;

function erfc(x: TFloat): TFloat;
begin
  erfc := 1.0 - erf(x);
end;

function I0_(x: TFloat): TFloat;
var
  r               : TFloat;
begin
  if x < 3.75 then
  begin
    r  := sqr(x/3.75);
    I0_:= 1.0+r*(3.5156229+r*(3.0899424+r*(1.2067492
             +r*(0.2659732+r*(0.0360768+r*0.0045813)))));
  end else
  begin
    r  := 3.75/x;
    I0_:=exp1(x)*(0.39894228+r*(0.01328592+r*(0.00225319
              +r*(-0.00157565+r*(0.00916281+r*(-0.02057706
              +r*(0.02635537+r*(-0.01647633+r*0.00392377))))))))/sqrt(x);
  end;
end;

function I1_(x: TFloat): TFloat;
var
  r               : TFloat;
begin
  if x < 3.75 then
  begin
    r  := sqr(x/3.75);
    I1_:= (0.5+r*(0.87890594+r*(0.51498869+r*(0.15084934
              +r*(0.02658733+r*(0.00301532+r*0.00032411))))))*x;
  end else
  begin
    r  := 3.75/x;
    I1_:=exp1(x)*(0.39894228+r*(-0.03988024+r*(-0.00362018
              +r*(0.00163801+r*(-0.01031555+r*(0.02282967
              +r*(-0.02895312+r*(0.01787654-r*0.00420059))))))))/sqrt(x);
  end;
end;

function fk0_(x: TFloat): TFloat;
var
  r                : TFloat;
begin
    r   := sqr(x/2.0);
    fk0_:= -0.57721566+r*(0.42278420+r*(0.23069756+r*(0.03488590
        +r*(0.00262698+r*(0.0001075+r*0.0000074)))));
end;

function K0_(x: TFloat): TFloat;
var
  r               : TFloat;
begin
  if x < 2.0
  then
    K0_:= -ln(x/2.0)*I0_(x) + fk0_(x)
  else
  begin
    r := 2.0/x;
    K0_:=exp1(-x)*(1.25331414+r*(-0.07832358
              +r*(0.02189568+r*(-0.01062446+r*(0.00587872
              +r*(-0.00251540+r*0.00053208))))))/sqrt(x);
  end;
end;

function fk1_(x: TFloat): TFloat;
var
  r: TFloat;
begin
  r  := sqr(x/2.0);
  fk1_:= (1.0+r*(0.15443144-r*(0.67278579+r*(0.18156897
            +r*(0.01919402+r*(0.00110404+r*0.00004686))))))/x;
end;

function K1_(x: TFloat): TFloat;
var
  r: TFloat;
begin
  if x < 2.0
  then
    K1_:= ln(x/2.0)*I1_(x)+fk1_(x)
  else
  begin
    r  := 2.0/x;
    K1_:=exp1(-x)*(1.25331414+r*(0.23498619
               +r*(-0.03655620+r*(0.01504268+r*(-0.00780353
               +r*(0.00325614-r*0.00068245))))))/sqrt(x)
  end;
end;

{ Gamma-function  error < 5 e-7    }
function Gamma_(x: TFloat): TFloat;
const{  for expansion from Abramowitz }
  b1 = -0.577191652;    b2 = 0.988205891;
  b3 = -0.897056937;    b4 = 0.918206857;
  b5 = -0.756704078;    b6 = 0.482199394;
  b7 = -0.193527818;    b8 = 0.035868343;
var
  t: TFloat;
begin
  t := 1.0;
  while x >= 2.0 do begin x := x - 1.0; t := t*x; end;
  while x < 1.0  do begin t := t/x; x := x + 1.0; end;
  x := x - 1.0;
  result := t*((((((((b8*x+b7)*x+b6)*x+b5)*x+b4)*x+b3)*x+b2)*x+b1)*x+1.0);
end;

function GammaInc(a, x :TFloat): TFloat;
const
  epsilon = 1.0E-8;
  Xmax    = 10.0;
var
  s, t, m: TFloat;
begin
  if x < Xmax then {  series expansion  }
  begin
    m := 0.0;  t := exp1(a*ln(x))/a;  s := t;
    repeat
      m := m + 1.0;  t := -t*x*a/(a+1.0)/m;
      s := s + t;    a := a + 1.0;
    until (abs(t)<epsilon);
    result := s;
  end
  else{  asimptotics from Abramowitz }
  begin
    m := a - 1.0;  t := exp1(m*ln(x)-x);  s := t;
    repeat
      t := t*m/x; s := s + t;  m := m - 1.0;
    until (abs(t)<epsilon) or (abs(a) > x);
    result := Gamma_(a) - s;
  end;
end;

{Gamma function: Gamma(N+1) = N! x is not equal to 0,-1,-2, ..
 Translated from FORTRAN:
 Shanjie Zhang and Jianming Jin, COMPUTATION OF SPECIAL FUNCTIONS,
 John Wiley & Sons, Inc, 1996.}
function Gamma(X: TFloat): TFloat;
const
  G: array[1..26] of TFloat = (1.0, 0.5772156649015329, -0.6558780715202538,
              -0.420026350340952E-1,0.1665386113822915, -0.421977345555443E-1,
              -0.96219715278770E-2, 0.72189432466630E-2,-0.11651675918591E-2,
              -0.2152416741149E-3,  0.1280502823882E-3, -0.201348547807E-4,
              -0.12504934821E-5,    0.11330272320E-5,   -0.2056338417E-6,
               0.61160950E-8,       0.50020075E-8,      -0.11812746E-8,
               0.1043427E-9,        0.77823E-11,        -0.36968E-11,
               0.51E-12,           -0.206E-13,          -0.54E-14,
               0.14E-14, 0.1E-15);
var
  M1, M, K: TInt;
  GR, R, Z, GA: TFloat;
begin
  if ( abs(X-INT(X)) <= abs(MachEps*4.0*X)) then 
  begin
    if (X > 0.0) then
    begin
      GA := 1.0;
      M1 := round(X)-1;
      for K :=2 to M1 do
        GA := GA*K;
    end else
      GA := MaxFloat;
  end else
  begin
    if (abs(X) > 1.0) then
    begin
      Z := abs(X);
      M := trunc(Z);
      R := 1.0;
      for K := 1 to M do
      begin
        Z := Z - 1.0;
        R := R*Z;
      end;
    end else
      Z:=X;

    GR:=G[26];
    for K := 25 downto 1 do
      GR :=GR*Z+G[K];

    GA :=1.0/(GR*Z);
    if (abs(X) > 1.0) then
    begin
      GA := GA*R;
      if (X < 0.0) then
        GA:=-Pi/(X*GA*sin(PI*X));
    end;
  end;
  result := GA;
end;

procedure INCOG(A, X: TFloat; var GIN, GIM, GIP: TFloat);
var
  K: TInt;
  T0, XAM, GA, S, R: TFloat;
begin
  XAM := -X + A*ln(X);  //???
  if (XAM > 700.0) or (A > 170.0) then
  begin
//     WRITE(*,*)'a and/or x too large';
//     STOP;
  end;
  if (X = 0.0) then
  begin
     GIN := 0.0;
     GA := Gamma(A);
     GIM := GA;
     GIP := 0.0;
  end else
  if (X<=1.0+A) then
  begin
    S := 1.0/A;
    R := S;
    for K := 1 to 60 do
    begin
      R := R*X/(A+K);
      S := S+R;
      if (ABS(R/S)<1.0E-15) then
        break;
    end;
    GIN := EXP(XAM)*S;
    GA := Gamma(A);
    GIP := GIN/GA;
    GIM := GA-GIN;
  end else
  if (X > 1.0+A) then
  begin
    T0 := 0.0;
    for K := 60 downto 1 do
    begin
       T0 := (K-A)/(1.0+K/(X+T0));
    end;
    GIM := EXP(XAM)/(X+T0);
    GA := Gamma(A);
    GIN := GA - GIM;
    GIP := 1.0 - GIM/GA;
  end;
end;

function E1_(x: TFloat)    : TFloat;
const
  epsilon = 1.0E-9;
  Xmax    = 10.0;
  Gamma   = 0.57721566490153286;{ Euler's constant }
var
  s, t, m           : TFloat;
begin
  if x < Xmax then {  series expansion  }
  begin
    m := 1.0;  t := x;  s := t - Gamma - ln(x);
    repeat
      t := -t*x*m/sqr(m+1.0);
      s := s + t;    m := m + 1.0;
    until (abs(t)<epsilon);
    E1_ := s;
  end
  else{  asymptotics from Abramowitz }
  begin
    m := 1.0;  t := exp1(-x)/x;  s := t;
    repeat
      t := -t*m/x; s := s + t;  m := m + 1.0;
    until (abs(t)<epsilon) or (m > x);
    E1_ := s;
  end;
end;

function I_nu(nu, x: TFloat): TFloat;
const
  epsilon = 1.0E-7;
  Xmax    = 10.0;
  s2pi    = 2.506628274631;{ sqrt(2*pi) }
var
  s, t, m, f, n, x8: TFloat;
begin
  if x < Xmax then {  series expansion  }
  begin
    x  := x/2.0;     t := exp1(nu*ln(x));  x := sqr(x);
    nu := nu + 1.0;  t := t/Gamma_(nu);    m := 1.0;
    s  := t;
    repeat
      t  := t*x/m/nu;  s := s + t;
      nu := nu +1.0;   m := m + 1.0;
    until (abs(t)<epsilon);
    I_nu := s;
  end
  else{  asimptotics from Abramowitz }
  begin
    nu := 4.0*sqr(nu);  m := 1.0;  n := 1.0;
    x8 := 8.0*x;        t := 1.0;  s := t;
    repeat
      f := (nu-sqr(m))/n/x8;  t := -t*f;
      s := s + t;
      m := m + 2.0;  n := n + 1.0;
    until (abs(t)<epsilon) or (abs(f) > 1.0);
    I_nu := s*exp1(x)/s2pi/sqrt(x);
  end;
end;

{ Undocumented Functions }
(*
function Gamma(x: TFloat): TFloat;
{    x <> 0.0, -TInt   }
const
  Ck: array [2..26] of TFloat =
  ( 0.5772156649015329, -0.6558780715202538, -0.0420026350340952,
      0.1665386113822915, -0.0421977345555443, -0.0096219715278770,
      0.0072189432466630, -0.0011651675918591, -0.0002152416741149,
      0.0001280502823882, -0.0000201348547807, -0.0000012504934821,
      0.0000011330272320, -0.0000002056338417,  0.0000000061160950,
      0.0000000050020075, -0.0000000011812746,  0.0000000001043427,
      0.0000000000077823, -0.0000000000036968,  0.0000000000005100,
     -0.0000000000000206, -0.0000000000000054,  0.0000000000000014,
      0.0000000000000001 );
{
const
  a1 = 1.0/12.0;          a2 = 1.0/288.0;
  a3 = 139.0/51840.0;     a4 = 571.0/2488320.0;
}
var
  B,D,F,BB,T    : TFloat;
  i             : byte;
begin
  B := abs(x);
  D := 1.0;
  while B>1.0  do
  begin
    D := D*B;  B := B-1.0;
  end;
  F  := 1.0;
  BB := B;
  T  := BB;
  for i:=2 to 26  do
    begin
      F := F + Ck[i]*T;     T := T*BB;
    end;
  if x>0.0  then  Gamma := D/x/F
            else  Gamma := F*Pi/sin(Pi*x)/D;
{
  F := exp1(-B+ln(B)*(B-0.5))*sqrt(2.0*Pi) *
  (1.0 + (a1 + (a2 - (a3 + a4/B)/B)/B)/B );
  if x>0.0  then  Gamma := F
            else  Gamma := Pi/sin(Pi*x)/x;
}
end;
*)
function  PolyGamma(n: TInt; z: TFloat): TFloat;
const
  BernN = 32;
  Bernoulli: array [1..BernN] of TFloat =
   ( 1.0,                    -0.5,                      1.0/6.0,
    -1.0/30.0,                1.0/42.0,                -1.0/30.0,
     5.0/66.0,               -691.0/2730.0,             7.0/6.0,
    -3617.0/510.0,            43867.0/798.0,           -174611.0/330.0,
     854513.0/138.0,         -236364091.0/2730.0,       8553103.0/6.0,
    -23749461029.0/870.0,     8615841276005.0/14322.0, -7709321041217.0/510.0,
     2577687858367.0/6.0,    -1.371165521e+13,          4.883323190e+14,
    -1.929657934e+16,         8.416930476e+17,         -4.033807185e+19,
     2.115074864e+21,        -1.208662652e+23,          7.500866746e+24,
    -5.038778101e+26,         3.652877648e+28,         -2.849876930e+30,
     2.386542750e+32,        -2.139994926e+34 );

 function PGam_Assc(n: TInt; z: TFloat): TFloat;
 var
   S,A,A1,zz,Item,N1fac,Nom,Denom  : TFloat;
   i                               : TInt;
 begin   
   zz  := sqr(z);      S     := 0.0;
   Nom := n;           N1fac := 1.0;
   for i:=1 to n-1  do
     N1fac := N1fac*i;    {  (n-1)!  }
   Item := N1fac;      A     := MaxFloat;
   i    := 3;          Denom := 2.0;
   repeat
     A1 := A;
     if n>0  then
     begin
       Item := Item*Nom/Denom*(Nom+1.0)/(Denom-1.0)/zz;
       Nom  := Nom+2.0;
       A    := Bernoulli[i]*Item;
     end   else
     begin
       Item := Item/zz;
       A    := Bernoulli[i]*Item/Denom;
     end;
     Denom := Denom+2.0;
     inc(i);
     S := S+A;
   until (S=S+A) or (abs(A)>abs(A1)) or (i>BernN);
     
   if n=0  then  PGam_Assc := ln(z) - 0.5/z - S
   else
   begin
     S := S + N1fac*(1.0+n/(2.0*z));
     for i:=1 to n  do
       S := S/z;
     if Odd(n)  then  PGam_Assc := S
                else  PGam_Assc := -S;
   end;
end;
 
const
  AssLim = 30.0;
var
  S,Nfac,A,zz,PG,Sz,Cz : TFloat;
  i,j,k                : TInt;
begin
  zz := abs(z);
  if z<0.0  then  zz := z+1.0;
  if zz>=AssLim  then  PG := PGam_Assc(n,zz )
  else
  begin
    k := trunc(AssLim-zz) + 1;
    S := 0.0;
    for i:=k-1 downto 0  do
      begin
        A := 1.0;
        for j:=1 to n+1  do
          A := A/(zz+i);
        S := S+A;
      end;
    Nfac := 1.0;
    if n>0  then
      for i:=1 to n  do
        Nfac := Nfac*(-i);
    PG := PGam_Assc(n,k+zz) - Nfac*S;
  end;
  if z>=0.0  then
    PolyGamma := PG
  else     
  begin
    Sz := sin(Pi*zz);     Cz := cos(Pi*zz);
    S  := Pi/Sz;
    case  n  of
      0: PolyGamma := PG + Cz*S;
      1: PolyGamma := -PG - sqr(S);
      2: PolyGamma := PG + 2.0*Cz*S*sqr(S);
      3: PolyGamma := -PG - 2.0*Pi*S*sqr(S)*(1.0+2.0*sqr(Cz));
       else
         PolyGamma := PG;  {  wrong, of course  }
    end;
  end;
end;
 
{  ----------------------------------------------------------  }

function GammaX(a,x: TFloat): TFloat;
  
 function GammaAbsX(x: TFloat): TFloat;
 var
   n,S,T,dS : TFloat;
 begin
   if x=0.0  then   GammaAbsX := 0.0
   else if x<=12.0  then
     begin
       T := 1.0;        dS := T/a;
       S := 0.0;        n  := 1.0;
       repeat
         S  := S+dS;
         T  := T/n;
         T  := T*(-x);
         dS := T/(a+n);
         n  := n+1.0;
       until ((S>0.0) and (S+dS=S)) or (n>64000.0);
       if a<=3.0  then  GammaAbsX := exp1(a*ln(x))*S
                  else  GammaAbsX := exp1(a*ln(x)+ln(S));
     end
   else
     begin
       T := 1.0;    S := 0.0;
       n := 1.0;
       repeat
         S  := S + T;
         dS := T;
         T  := T/x;
         T  := T*(a-n);
         n  := n+1.0;
       until (S+T=S) or ((dS<0.0) and (abs(dS)<abs(T)));
       GammaAbsX := Gamma(a) - exp1((a-1.0)*ln(x)-x+ln(S));
     end;
 end;

begin
  if x>=0.0  then  GammaX := GammaAbsX(x )
             else  GammaX := -GammaAbsX(-x );
end;

{ Inverse Cumulative Normal Distribution Fuction.
  Abromowitz and Stegun, Handbook of Mathematical Functions, Dover, NY.
  see also http://www.johndcook.com/normal_cdf_inverse.html
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 4.5e-4  |x| <= 7 }
function InvNormCDF(p: TFloat): TFloat;
var
  t: TFloat;

  function X(t: TFloat): TFloat;
  const
     c: array [0..2] of TFloat = (2.515517, 0.802853, 0.010328);
     d: array [0..2] of TFloat = (1.432788, 0.189269, 0.001308);
  begin
     result := t - ((c[2]*t + c[1])*t + c[0]) /
                   (((d[2]*t + d[1])*t + d[0])*t + 1.0);
  end;

begin
  assert((p > 0.0) and (p < 1.0), 'Probability outsidr (0,1)');
  if(p < 0.5) then
  begin
    t := sqrt(-2.0*ln(p));
    result := -X(t);
  end else
  begin
    t := sqrt(-2.0*ln(1.0-p));
    result := X(t);
  end
end;

{ Delphi implementation byGreg McCormick (unit icnorm.pas )
  Water Operational Research Centre, Department of Systems Engineering,
  Brunel University, Uxbridge, UK.
  Rel error < 1.1510-9 for invCumNorm > -38, see more in implementation.
  function obtained from P. J. Acklam,
  http://home.online.no/~pjacklam/notes/invnorm/index.html

  Actual accuracy in comparison with Mathematica:

 Y (Mathematica X=n)      X = invCumNorm     abs         rel
6.106226635438361e-16,  -8.00229181445019, -2.291e-03,  2.864e-04
1.2798095916366492e-12, -7.00000031597657, -3.159E-07,  4.513E-08
9.865876449133282e-10,  -6.00000000028878, -2.887E-10,  4.813E-11
2.8665157186802404e-7,  -5.00000000561963, -5.619E-09,  1.122E-09
3.167124183311998e-5,   -3.99999999882021,  1.179E-09, -2.949E-10
0.0013498980316301035,  -2.99999999957808,  4.219E-10, -1.406E-10
0.02275013194817921,    -2.00000000045769, -4.576E-10,  2.288E-10
0.15865525393145702,    -1.00000000092742, -9.274E-10,  9.274E-10
}
function invCumNorm(const p: TFloat): TFloat;
const
  a: array[1..6] of TFloat = (-3.969683028665376e+01,  2.209460984245205e+02,
                             -2.759285104469687e+02,  1.383577518672690e+02,
                             -3.066479806614716e+01,   2.506628277459239e+00);

  b: array[1..5] of TFloat = (-5.447609879822406e+01, 1.615858368580409e+02,
                              -1.556989798598866e+02, 6.680131188771972e+01,
                              -1.328068155288572e+01);

  c: array[1..6] of TFloat = (-7.784894002430293e-03, -3.223964580411365e-01,
                              -2.400758277161838e+00, -2.549732539343734e+00,
                               4.374664141464968e+00,  2.938163982698783e+00);

  d: array[1..4] of TFloat = ( 7.784695709041462e-03, 3.224671290700398e-01,
                               2.445134137142996e+00, 3.754408661907416e+00 );
//Define break-points.
  p_low = 0.02425;
  p_high = 1 - p_low ;

//const infinity=1.7E308; // maximum size of double

var
  q, r: extended;

begin
  result := 0.0;
  if ((p <= 0.0) or (p >= 1.0)) then
    raise Einvalidargument.create('Inverse Cum Norm argument must be in range 0<p<1');

  //Rational approximation for lower region.
  if ((0.0 < p) and  (p < p_low)) then
  begin
    q := sqrt(-2.0*ln(p));
    result := (((((c[1]*q + c[2])*q + c[3])*q + c[4])*q + c[5])*q + c[6]) /
              ((((d[1]*q + d[2])*q + d[3])*q + d[4])*q + 1.0);
  end;

  //Rational approximation for central region.
  if (p_low <= p) and (p <= p_high ) then
  begin
    q := p - 0.5 ;
    r := q*q ;
    result := (((((a[1]*r + a[2])*r + a[3])*r + a[4])*r + a[5])*r + a[6])*q /
              (((((b[1]*r + b[2])*r + b[3])*r + b[4])*r + b[5])*r + 1.0) ;
  end;

  //Rational approximation for upper region.
  if ((p_high < p) and (p < 1))  then
  begin
    q := sqrt(-2*ln(1-p));
    result := -(((((c[1]*q + c[2])*q + c[3])*q + c[4])*q + c[5])*q + c[6]) /
               ((((d[1]*q + d[2])*q + d[3])*q + d[4])*q + 1.0)  ;
  end;
end;

{ Cumulative Normal Distribution Fuction. Algorithm 26.2.17 from
  Abromowitz and Stegun, Handbook of Mathematical Functions, Dover, NY.
  http://www.sitmo.com/doc/Calculating_the_Cumulative_Normal_Distribution
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 7.5e-8 ( error < 7.406e-8 for |x| < 8 ) }
function CumNormDF(x: TFloat): TFloat;
const
  b1: TFloat =  0.319381530;
  b2: TFloat = -0.356563782;
  b3: TFloat =  1.781477937;
  b4: TFloat = -1.821255978;
  b5: TFloat =  1.330274429;
  p : TFloat =  0.2316419;
  c : TFloat =  0.3989422804014327; { 1/sqrt(2*Pi) }
var
  t: TFloat;
begin
  if(x >= 0.0) then
  begin
    t := 1.0 / ( 1.0 + p * x );
    result := (1.0 - c * exp( -x * x / 2.0 ) * t *
              ( t *( t * ( t * ( t * b5 + b4 ) + b3 ) + b2 ) + b1 ));
  end else
  begin
    t := 1.0 / ( 1.0 - p * x );
    result := ( c * exp( -x * x / 2.0 ) * t *
              ( t *( t * ( t * ( t * b5 + b4 ) + b3 ) + b2 ) + b1 ));
  end
end;

{ Better Approximations to Cumulative Normal Fuctiona by Graeme West,
  http://www.finmod.co.za/accuratecumnorm.pdf
  Based on J. Hart, Computer Approximations, Wiley, 1968.
  Algorithm 5666 for the error function.
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 2.23e-16 for |x| < 8                                   }
function CumNorm(x: TFloat): TFloat;
var
  build, XAbs, Exponential: TFloat;
begin
  XAbs := Abs(x);
  If XAbs > 37.0 Then
    result := 0.0
  Else
  begin
    Exponential := Exp(-x*x / 2.0);
    If XAbs < 7.07106781186547 Then
    begin
      build := 3.52624965998911E-02 * XAbs + 0.700383064443688;
      build := build * XAbs + 6.37396220353165;
      build := build * XAbs + 33.912866078383;
      build := build * XAbs + 112.079291497871;
      build := build * XAbs + 221.213596169931;
      build := build * XAbs + 220.206867912376;
      result := Exponential * build;
      build := 8.83883476483184E-02 * XAbs + 1.75566716318264;
      build := build * XAbs + 16.064177579207;
      build := build * XAbs + 86.7807322029461;
      build := build * XAbs + 296.564248779674;
      build := build * XAbs + 637.333633378831;
      build := build * XAbs + 793.826512519948;
      build := build * XAbs + 440.413735824752;
      result := result / build;
    end Else
    begin
      build := XAbs + 0.65;
      build := XAbs + 4.0 / build;
      build := XAbs + 3.0 / build;
      build := XAbs + 2.0 / build;
      build := XAbs + 1.0 / build;
      result := Exponential / build / 2.506628274631;
    end;
  End;
  If x > 0.0 Then
    result := 1.0 - result;
end;

{ Approximations to Bivariate Cumulative Normal Fuctiona by Graeme West,
  http://www.finmod.co.za/accuratecumnorm.pdf
  Translated to Pascal by Nikolai Shokhirev 2008.02.02
  Nikolai@Shokhirev.com, http://www.shokhirev.com/nikolai.html
  error < 1.2e-16 for |x| < 8  ???                                 }
Function BivarCumNorm(a, b, r: TFloat): TFloat;
const
  x: array[1..5] of TFloat = (0.04691008, 0.23076534, 0.5,
                              0.76923466, 0.95308992);
  W: array[1..5] of TFloat = (0.018854042, 0.038088059, 0.0452707394,
                              0.038088059, 0.018854042);
var
  i: Integer;
  h1, h2, LH, h12, h3, h5, h6, h7, r1, r2, r3, rr, AA, ab: TFloat;
begin
  h1 := a;
  h2 := b;
  h12 := (h1*h1 + h2*h2)/2.0;
  If Abs(r) >= 0.7 Then
  begin
    r2 := 1.0 - r*r;
    r3 := Sqrt(r2);
    If r < 0.0 Then
      h2 := -h2;
    h3 := h1*h2;
    h7 := Exp(-h3/2.0);
    If Abs(r) < 1.0 Then
    begin
      h6 := Abs(h1 - h2);
      h5 := h6*h6/2.0;
      h6 := h6/r3;
      AA := 0.5 - h3/8.0;
      ab := 3.0 - 2.0*AA*h5;
      LH := 0.13298076*h6*ab*(1.0 - Cumnorm(h6)) - Exp(-h5/r2)*(ab + AA*r2)*0.053051647;
      For i := 1 To 5 do
      begin
        r1 := r3*x[i];
        rr := r1*r1;
        r2 := Sqrt(1.0 - rr);
        LH := LH - W[i]*Exp(-h5/rr)*(Exp(-h3/(1.0 + r2))/r2/h7 - 1.0 - AA*rr);
      end;
    End;
    result := LH*r3*h7 + Cumnorm(Min(h1, h2));
    If r < 0.0 Then
      result := Cumnorm(h1) - result;
  end
  Else
  begin
    h3 := h1*h2;
    If r <> 0.0 Then
    begin
      For i := 1 To 5 do
      begin
        r1 := r*x[i];
        r2 := 1.0 - r1*r1;
        LH := LH + W[i]*Exp((r1*h3 - h12)/r2)/Sqrt(r2);
      end;
    End;
    result := Cumnorm(h1)*Cumnorm(h2) + r*LH;
  End;
End;

{  ----------------------------------------------------------  }

function IJn(n,x,W: TFloat): TFloat;
var
  F,R,i  : TFloat;
begin
  if  x=0.0  then
    begin
      if n=0.0  then  IJn := 1.0
                else  IJn := 0.0;
      Exit;
    end;
  if n<>0.0  then   F := 1.0/Gamma(n)/n
             else   F := 1.0;
  R := F;
  i := 0.0;
  repeat
    i := i + 1.0;
    R := R*W;
    R := R/i/(n + i);
    F := F + R;
  until (F-R=F) or (i>64000.0);
  if  W<=0.0  then   IJn := exp1(n*ln(x/2.0))*F
              else   IJn := exp1(n*ln(x/2.0) + ln(F));
end;

function BJn(n,x: TFloat): TFloat;
const
  Xmax = 23.0;
var
  k                        : byte;
  s,f,nu,x8,Sn,Cs,Ck,m,p   : TFloat;
begin
  if x<=Xmax  then
    BJn := IJn(n,x,-sqr(x)/4.0 )
  else
    begin
      s  := x-Pi*(n + 0.5)/2.0;
      Sn := sin(s);         Cs := cos(s);
      Ck := 1.0;            s  := 0.0;
      nu := 4.0*sqr(n);     x8 := 8.0*x;
      k  := 0;              m  := 1.0;
      p  := 1.0;
      repeat
        case  k  of
          0: s := s + Ck*Cs;
          1: s := s - Ck*Sn;
          2: s := s - Ck*Cs;
          3: s := s + Ck*Sn;
        end;
        k  := (k+1) mod 4;
        f  := (nu-sqr(m))/p/x8;
        Ck := Ck*f;
        m  := m + 2.0;
        p  := p + 1.0;
      until ((s+Ck=s) and (p>=n)) or (f<=-1.0);
      BJn := sqrt(2.0/(Pi*x))*s;
    end;
end;

function BIn(n,x: TFloat): TFloat;
const
  Xmax    = 35.0;
  s2pi    = 2.506628274631;  { sqrt(2*pi) }
var
  s,t,m,f,nu,x8,p    : TFloat;
begin
  if x<Xmax  then   BIn := IJn(n,x,+sqr(x)/4.0 )
  else
    begin
      nu := 4.0*sqr(n);   m := 1.0;  p := 1.0;
      x8 := 8.0*x;        t := 1.0;  s := t;
      repeat
        f := (nu-sqr(m))/p/x8;
        t := -t*f;
        s := s + t;
        m := m + 2.0;
        p := p + 1.0;
      until (s-t=s) or (f<=-1.0);
      BIn := s*exp1(x)/s2pi/sqrt(x);
    end;
end;

function BKn(n,x: TFloat): TFloat;
const
  Dmin = 1.0e-4;
var
  dn: TFloat;
 function BK(nu: TFloat): TFloat;
 begin
   BK := Pi/2.0*(BIn(-nu,x)-BIn(nu,x))/sin(nu*Pi);
 end;
begin
  dn := frac(n);
  if abs(dn)>=Dmin  then BKn := BK(n)
  else
    BKn := BK(round(n)+Dmin)*((Dmin+dn)/(2.0*Dmin)) +
           BK(round(n)-Dmin)*((Dmin-dn)/(2.0*Dmin));
end;
 
{  ----------------------------------------------------------  }

const
  Z0   = 20.0;
  Nmax = 20;
//  S_Coef_Done: boolean = false;
var
  S_Coef_Done: boolean = false;
  S_Coef     : array [1..Nmax+1] of TFloat;

function  Sn( n: TInt; z: TFloat): TFloat;

var
  i          : TInt;
  lnZ,lniZ,S : TFloat;
  l          : TInt;

 function  S_n(x: TFloat; n: TInt): TFloat;
 var
   S,Item,k,A  : TFloat;
   i           : TInt;
 begin
   if n=0  then
   begin
     S_n := exp1(-x)-1.0;
     Exit;
   end;
   S := -x;   Item := S;    k := 1.0;
   repeat
     k    := k+1.0;
     Item := Item/k;
     Item := Item*(-x);
     A    := 1.0;
     for i:=1 to n  do  A := A*k;
     A    := Item/A;
     S := S+A;
   until  S=S+A;
   S_n := S;
 end;

 procedure  InitSn;
 var
   lnZ0,Ci,lnjZ0: TFloat;
   i,j, l          : TInt;
 begin
   lnZ0 := ln(Z0);
   for i:=0 to Nmax  do
     S_Coef[i+1] := S_n(Z0,i);
   for i:=0 to Nmax  do
   begin
     Ci    := -S_Coef[i+1];
     l     := 1;
     lnjZ0 := 1.0;
     for j:=1 to i  do
     begin
       l     := l*j;
       lnjZ0 := lnjZ0*lnZ0;
       Ci    := Ci - S_Coef[i-j+1]*lnjZ0/l;
     end;
     S_Coef[i+1] := Ci;
   end;
 end;

begin
  if  (n=0) or (z<=Z0) or (n>Nmax)  then   Sn := S_n(z,n)
  else
    begin
      if not S_Coef_Done  then
        begin
          InitSn;  S_Coef_Done := true;
        end;
      l    := 1;
      lnZ  := ln(Z);
      lniZ := 1.0;
      S    := -S_Coef[n+1];
      for i:=1 to n  do
        begin
          l    := l*i;
          lniZ := lniZ*lnZ;
          S    := S-S_Coef[n-i+1]*lniZ/l;
        end;
      Sn := S;
    end;
end;
 
{  ----------------------------------------------------------  }

function  Un( n: TInt; z,a: TFloat): TFloat;

 function  U_n(x,a: TFloat; n: TInt): TFloat;
 {    Direct Sum calculations   }
 var
   S,Item,k,B  : TFloat;
   i           : TInt;
 begin
   if n=0  then
     begin
       U_n := exp1(-x)-1.0;
       Exit;
     end;
   Item := 1.0;   k := 0.0;   S := 0.0;   B := MaxFloat;
   repeat                                              
     k    := k+1.0;
     Item := Item/k;
     Item := Item*(-x);
     B    := Item;
     for i:=1 to n  do  B := B/(k+a);
     S := S+B;
   until  S=S+B;
   U_n := S;
 end; 

 function  Un_Ass(n: TInt; z,a: TFloat): TFloat;
 var
   S,G1,PG,G2,P,S1,S2,A1,A2,MachLimit : TFloat;
   ZF                                 : boolean;
   k                                  : TInt;
 begin
   {    Asymptotics for large  z,   n=2,3     }
   S := ln(z);
   G1 := exp1(-a*S)*Gamma(a);
   PG := PolyGamma(0,a)-S;
   if n=2  then  G1 := G1*PG
           else  G1 := G1*(sqr(PG)+PolyGamma(1,a));
           
  {    questionable exectness off  GammaX    }
   S := 0.0;    k  := 1;     A1 := MaxFloat;
   P := 1.0;    S1 := 0.0;   S2 := 0.0;
   MachLimit := 32*sqrt(MachEps);
   ZF := false;
   repeat
     A2 := A1;
     PG := a-k;
     if abs(PG)>MachLimit  then
     begin
       P  := P*PG;  S1 := S1+1.0/PG;
       S2 := S2+1.0/sqr(PG);
     end else
       ZF := true;
     P := P/z;
     if n=2  then
     begin
       if ZF  then  A1 := P
              else  A1 := P*S1;
     end else
       if k>1  then
       begin
         if ZF  then  A1 := P*S1
                else  A1 := P*(sqr(S1)-S2);
         S := S + A1;
       end;
     inc(k);
   until (S+A1=S) or ((A2<0.0) and (abs(A2)<abs(A1))) or (k>512);
   G2 := exp1(-z)/z*S;                         
  (*   else  G2 := 0.0     *)
   if n=2  then  Un_Ass := -(G1-G2+1.0/sqr(a))
           else  Un_Ass := (G1-G2-2.0/sqr(a)/a)/2.0;
 end;
 
const
  ZLim = 20.0;
    
begin
  if (a=0.0) or (n=0)  then   Un := Sn(n,z)
  else if (z<=ZLim) or (n>3)  then  Un := U_n(z,a,n)
  else if n=1  then              
    {    exact formulae for arbitrary z>0.0   }
    Un := exp1(-a*ln(z))*GammaX(a,z) - 1.0/a
  else Un := Un_Ass(n,z,a);
end;


end.
