{@abstract(Dynamic arrays examples)
@author(Nikolai Shokhirev <nikolai@shokhirev.com>)
@created(February 02, 2001)
@lastmod(June 06, 2004)
Nikolai V. Shokhirev, 2001-2006 }
unit fExamples;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    ButtonSort: TButton;
    ButtonGramSchmidt: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure ButtonSortClick(Sender: TObject);
    procedure ButtonGramSchmidtClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
uses
  uMatTypes,
  uDynArrays,
  uDisplay,
  uDynArrUtils;

// Simple sort of row-vectors
procedure TForm1.ButtonSortClick(Sender: TObject);
var
  i, j: TInt;
  M: IFArr2D;  // Declared as interface
  s: string;
begin
  M := TFArr2D.Create(4,3);         // One-based 2D array 4 x 3
  RandArr2D(M,-1.0,1.0,0);          // Fill with random numbers r: -1 < r < 1
  for i := M.Lo1 to M.Hi1-1 do
    for j := i+1 to M.Hi1 do
      if M.Norm2(i,_) > M.Norm2(j,_) then  //  M.Row[i].Norm > |M.Row[j]|
        M.Swap(i, j, _Row);
  s := '';
  for i := M.Lo1 to M.Hi1 do
    s := s + FloatToStr(M.Norm2(i,_))+' ';
  Label1.Caption := s;
end;

// Gram-Schmidt Orthonormalization
procedure TForm1.ButtonGramSchmidtClick(Sender: TObject);
var
  i, j, m: TInt;
  t: TFloat;
  D: IFArr2D;
begin
  D := TFArr2D.Create(3,4);
  m := D.Dim2;
  RandArr2D(D,-1.0,1.0);     // Fill with random numbers [-1 < r < 1]
  // The algorithm itself
  for i := 1 to D.Dim2 do    // For arbitrary limits: for i := D.Lo2 to D.Hi2 do
  begin
    t := D.Norm2(_,i);            // = |D.Col[i]|
    if t > MinFloat then
    begin
      CxCol(D, i, 1.0/t);         // = D.Col[i]/t
      for j := i+1 to D.Dim2 do
      begin
        t := ColxCol(D, i, j);   // = <D.Col[i]|D.Col[j]>
        ColAddCol(D, j, i, -t);  // = D.Col[j] - t*D.Col[i])
      end;
    end else
    begin
      m := i-1;               // m is the number of linear independent vectors
      break;
    end;                      // no need to destroy D!
  end;
  Label2.Caption := 'm='+IntToStr(m);
end;

end.
