{*
 Test/Demo Display unit
 Nikolai V. Shokhirev, 2001-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2002.06.06 - created
 @version 2003.08.08 - modified
 @version 2007.08.14 - Added DelphiCodeToDoc comments
 }
unit fTestDisplay;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, fDisplay, fDisplayVal, fDisplayCVal,
  uMatTypes, uDynArrays, uDynObjAlg;

type
  TFormC = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    bCalc: TButton;
    bMult: TButton;
    LabelSignal: TLabel;
    ButtonCalcA: TButton;
    Panel3: TPanel;
    ButtonDiagA: TButton;
    Button3: TButton;
    Button4: TButton;
    Panel4: TPanel;
    rgShowMatrixVector: TRadioGroup;
    bMatrix4x3: TButton;
    bVector4: TButton;
    Label1: TLabel;
    eI: TEdit;
    Label2: TLabel;
    eJ: TEdit;
    bMIJ: TButton;
    bComplex: TButton;
    EditC: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure bCalcClick(Sender: TObject);
    procedure bMatrix4x3Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bVector4Click(Sender: TObject);
    procedure bMIJClick(Sender: TObject);
    procedure bMultClick(Sender: TObject);
    procedure ButtonCalcAClick(Sender: TObject);
    procedure ButtonDiagAClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure bComplexClick(Sender: TObject);
  private
    { Private declarations }
    Matrix4x3: IFArr2D;
    Vector3, Vector4: IFArr1D;
    FDisp: TFormDisplay;
    dim: TInt;
    N: ISArr1D;
    A: IFArr2D;
    ES: IEigenSys;
  public
    { Public declarations }
  end;

var
  FormC: TFormC;

implementation

uses
  {uILinAlg,} uDisplay;

{$R *.DFM}

procedure TFormC.FormCreate(Sender: TObject);
begin
  Matrix4x3 := TFArr2D.Create(4,3);
  Vector3 := TFArr1D.Create(3);
  Vector4 := TFArr1D.Create(4);
  FDisp := TFormDisplay.Create(self);
// for Eigen test
  dim := 4;
  N := TSArr1D.Create(dim);
  A := TFArr2D.Create(dim,dim);
//  ES := TEigensystem.Create(dim,dim);  // should not create, it will be assigned later
end;

procedure TFormC.bCalcClick(Sender: TObject);
var
  i1, i2: TInt;
begin
  for i1 := 1 to Matrix4x3.Dim1 do
    for i2 := 1 to Matrix4x3.Dim2 do
      Matrix4x3[i1,i2] := i1*i2;
  for i2 := 1 to Matrix4x3.Dim2 do
    Vector3[i2] := round(4-i2);
  bMatrix4x3.Enabled := true;
  bMIJ.Enabled := true;
end;

procedure TFormC.bMatrix4x3Click(Sender: TObject);
begin
// diMatrix,diNames,diRowNum,diColNum,diValues,diVector,diHeaders
  with FDisp do
    try
      Editing := true;
      Vector := Vector3;
      Matrix := Matrix4x3;
      case rgShowMatrixVector.ItemIndex of
      0:begin
          DisplSet := [diMatrix,diRowNum,diColNum];
          Caption := ' Matrix';
        end;
      1:begin
          DisplSet := [diVector,diRowNum];
          Caption := ' Vector';
          Width := 300;
          Height := 200;
        end;
      end;
      Display;
      ShowModal;
    except
      on E:Exception do
        ShowMessage(Format('Error occurred displaying array: %s',[E.Message]));
    end;
end;

procedure TFormC.FormDestroy(Sender: TObject);
begin
  FDisp.Free;
end;

procedure TFormC.bVector4Click(Sender: TObject);
var
  FD: TFormDisplay;
begin
  try
    FD := TFormDisplay.Create(self);
    FD.Values := Vector4;
    FD.DisplSet := [diColNum,diValues];
    FD.Display;
    FD.ShowModal;
  finally
    FD.Free;
  end;
end;

procedure TFormC.bMIJClick(Sender: TObject);
var
  FDV: TFDisplVal;
  i, j: TInt;
begin
  i := StrToInt(eI.Text);
  j := StrToInt(eJ.Text);
  try
    FDV := TFDisplVal.Create(self);
    FDV.ArrayName := ' Matrix element';
    FDV.Idx1 := i;
    FDV.Idx2 := j;
    FDV.rVal := Matrix4x3[i,j];
    FDV.DisplType := dtReal;
    FDV.Display;
    FDV.seW.Value := 14;
    FDV.seD.Value := -1;
    FDV.ShowModal;
    Matrix4x3[i,j] := FDV.rVal;
  finally
    FDV.Release;
  end;
end;

procedure TFormC.bMultClick(Sender: TObject);
var
  signal: TInt;
begin
  Vector4 := MtxVt(Matrix4x3, Vector3);
  LabelSignal.Caption := 'Signal='+IntToStr(signal);
  bVector4.Enabled := true;
end;

procedure TFormC.ButtonCalcAClick(Sender: TObject);
var
  signal, i, j : TInt;
  dev: TFloat;
begin
  for i := 1 to dim do
  begin
    N[i] := 'Func'+IntToStr(i);
    for j := 1 to dim do
    begin
    A[i,j] := 4.0-abs(i-j);
    end;
  end;
  button3.Enabled := true;
  ButtonDiagA.Enabled := true;
end;

procedure TFormC.ButtonDiagAClick(Sender: TObject);
var
  signal: TInt;
begin
  ES := DiagMt(A, signal);
//  ES.Comment := 'Eigensystem Test';
  ES.Names := N;
  Button4.Enabled := true;
end;

procedure TFormC.Button3Click(Sender: TObject);
var
  FormDisplay: TFormDisplay;
begin
  try
    FormDisplay := TFormDisplay.Create(nil);
    FormDisplay.Caption := 'Test matrix';
    FormDisplay.Matrix := A;
    FormDisplay.Names := N;
    FormDisplay.DisplSet := [diMatrix,diNames,diRowNum,diColNum];
    FormDisplay.Editing := true;
    FormDisplay.Display;
    FormDisplay.ShowModal;
  finally
    FormDisplay.Release;
  end;
end;

procedure TFormC.Button4Click(Sender: TObject);
var
  FormDisplay: TFormDisplay;
begin
  if Assigned(ES) then
  try
    FormDisplay := TFormDisplay.Create(nil);
    FormDisplay.Caption := 'Eigensystem Test';
    FormDisplay.Matrix := ES.Vectors;
    FormDisplay.Names := ES.Names;
    FormDisplay.Values := ES.Values;
    FormDisplay.DisplSet := [diMatrix,diNames,diRowNum,diColNum,diValues];
    FormDisplay.Editing := false;
    FormDisplay.Display;
    FormDisplay.ShowModal;
  finally
    FormDisplay.Release;
  end;
end;

procedure TFormC.bComplexClick(Sender: TObject);
var
  FDV: TFDisplCVal;
  i, r: TFloat;
begin
  try
    FDV := TFDisplCVal.Create(self);
    FDV.Caption := ' Complex Test';
    StrToCplx(EditC.Text, r, i);
    FDV.ReVal := r;
    FDV.ImVal := i;
    FDV.Display;
    FDV.seW.Value := 11;
    FDV.seD.Value := 4;
    FDV.ShowModal;
    EditC.Text := CplxToStr(FDV.ReVal,FDV.ImVal, 11, 4);
  finally
    FDV.Release;
  end;
end;

end.
