{@abstract(Test/Demo for Isolin)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(June 06, 2002)
@lastmod(August 08, 2003)
Nikolai V. Shokhirev, 2002-2003
}
unit fIsoline;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, uMatTypes, fMapBox, uDynArrays, uMapBox, ComCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    PanelMap: TPanel;
    TrackBarE: TTrackBar;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure TrackBarEChange(Sender: TObject);
  private
    { Private declarations }
    X, Y, Levels: IFArr1D;
    Z: IFArr2D;
    Nx, Ny: integer;
    NLevels: integer;
    MapBox: TMapBox;
    procedure Calculate;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Calculate;
var
  i, j: integer;
  e: TFloat;
  dz, zz: TFloat;
begin

  e := TrackBarE.Position;
  e := e*0.004;
  for i:=1 to Nx  do
    for j:=1 to Ny  do
    begin
      Z[j,i] := 9.0*exp(-(sqr(X[i]- 5.0)+sqr(Y[j]- 5.0))*e)
              +9.0*exp(-(sqr(X[i]-15.0)+sqr(Y[j]-15.0))*e)
              -3.0*exp(-(sqr(X[i]- 5.0)+sqr(Y[j]-15.0))*e)
              -3.0*exp(-(sqr(X[i]-15.0)+sqr(Y[j]- 5.0))*e) ;
    end;

  dz := 1.0;
  zz := 0.5;
  for i := 1 to Nlevels do
  begin
    Levels[i] := zz;
    zz := zz + dz;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Calculate;
  MapBox.InitData(Z, X, Y);
  MapBox.XCaption := true;
  MapBox.YCaption := true;
  MapBox.NXTicks := 5;
  MapBox.NYTicks := 5;
  MapBox.XTitle := 'X Axis';
  MapBox.YTitle := 'Y Axis';
  MapBox.XYRatio := 1.0;
  MapBox.SetLevels(NLevels);
  MapBox.MapColor := clBlue;
  MapBox.BoxDrawMap(self);  // MapBox.DrawMap(self);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: integer;
  x1, y1, x2,y2  : TFloat;
  dx, dy: TFloat;
begin
  MapBox := TMapBox.Create(self, PanelMap);
  Nx := 81;
  Ny := 81;
  NLevels := 19;
  X := TFArr1D.Create(Nx);
  Y := TFArr1D.Create(Ny);
  Z := TFArr2D.Create(Ny, Nx);
  Levels := TFArr1D.Create(NLevels);

  x1 :=  0.0;
  y1 :=  0.0;
  x2 := 20.0;
  y2 := 20.0;

  dx := (x2-x1)/(Nx-1);
  dy := (y2-y1)/(Ny-1);

  for i:=1 to Nx  do
    X[i] := x1 + dx*(i-1);

  for i:=1 to Ny  do
    Y[i] := y1 + dy*(i-1);

  Button1.Click;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  FormMap: TFormMap;
begin
  try
    Calculate;
    FormMap := TFormMap.Create(nil);
    FormMap.MapBox.InitData(Z, X, Y);
    FormMap.MapBox.XCaption := true;
    FormMap.MapBox.YCaption := true;
    FormMap.MapBox.NXTicks := 5;
    FormMap.MapBox.NYTicks := 5;
    FormMap.MapBox.XTitle := 'X Axis';
    FormMap.MapBox.YTitle := 'Y Axis';
    FormMap.MapBox.XYRatio := 1.0;
    FormMap.MapBox.SetLevels(NLevels);
    FormMap.MapBox.MapColor := clBlue;
    FormMap.ShowModal;
  finally
    FormMap.Release;
  end;
//
end;

procedure TForm1.TrackBarEChange(Sender: TObject);
begin
 // Button1.Click;
end;

end.
