{@abstract(Test pseudo-random number generation)
@author(Nikolai V. Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(09.09.2002)
@lastmod(11.11.2002)
Nikolai V. Shokhirev, 2002
}
unit fTestRandom;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Spin, Menus
  , uRand13, uIsaac
  , mt19937
  ;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    PaintBox2: TPaintBox;
    Panel3: TPanel;
    PaintBox1: TPaintBox;
    Panel4: TPanel;
    bStart: TButton;
    Button2: TButton;
    Button3: TButton;
    Label3: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    EditTerms: TSpinEdit;
    EditDelay: TSpinEdit;
    rgType: TRadioGroup;
    Timer1: TTimer;
    rgRNG: TRadioGroup;
    Label5: TLabel;
    seN: TSpinEdit;
    seCoord: TSpinEdit;
    Label4: TLabel;
    Label6: TLabel;
    seSmooth: TSpinEdit;
    MainMenu1: TMainMenu;
    Help1: TMenuItem;
    Help2: TMenuItem;
    About1: TMenuItem;
    procedure bStartClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Help2Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    { Private declarations }
    ncount, factor, nterms, ArrPos : integer;
    arr, arrsum, arrav: array of integer;
    Bitmap1: TBitmap;
    Bitmap2: TBitmap;
    Q: Rand13Point;
    x : TIsaac;
    mt: Tmt19937;
    function IntRand(N: integer): integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  Math, uMatTypes
  , fAbout;

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  Bitmap1 := TBitmap.Create;
  Bitmap1.Canvas.Pen.Color := clBlue;
  Bitmap2 := TBitmap.Create;
  Bitmap2.Canvas.Pen.Color := clBlue;
  Randomize;
  Randomize13;
  x := TIsaac.Create;
  ncount := 0;
  factor := 1;
  mt := Tmt19937.Create(19650218);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Bitmap1.Free;
  Bitmap2.Free;
  x.Free;
  mt.Free;
end;

procedure TForm1.bStartClick(Sender: TObject);
var
  i: integer;
begin
  Timer1.Enabled := true;
  SetLength(arr, PaintBox1.ClientWidth);
  SetLength(arrsum, PaintBox2.ClientHeight);
  SetLength(arrav, PaintBox2.ClientHeight);
  ArrPos := Low(arr)-1;
  for i := Low(arr) to High(arr) do
    arr[i] := 0;
  Bitmap1.Width := PaintBox1.ClientWidth;
  Bitmap1.Height := PaintBox1.ClientHeight;
  Bitmap2.Width := PaintBox2.ClientWidth;
  Bitmap2.Height := PaintBox2.ClientHeight;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  m, l, j, k, i, n, maxsum, halfFactor: integer;
begin
  Timer1.Interval := EditDelay.Value;
  inc(ncount);
  Nterms := Editterms.Value;
  halfFactor := Factor div 2;
  n := High(arr);
  if (ArrPos < n) then
  begin
    inc(ArrPos);
  end else
  begin
    ArrPos := High(arr);
    for i := Low(arr) to ArrPos-1 do
      arr[i] := arr[i+1];
  end;

  n := 0;
  for i := 1 to NTerms do
    n := n + IntRand(PaintBox1.ClientHeight div NTerms);

  n := n + (NTerms div 2);
  arr[ArrPos] := n;
  arrsum[n] := arrsum[n]+1;

  Bitmap1.Canvas.FillRect(PaintBox1.ClientRect);
    Bitmap1.Canvas.MoveTo(0,PaintBox1.ClientHeight div 2 -1);
  for i := Low(arr) to ArrPos do
  begin
    case rgType.ItemIndex of
    0:begin
        Bitmap1.Canvas.MoveTo(i,PaintBox1.ClientHeight-1);
        Bitmap1.Canvas.LineTo(i,PaintBox1.ClientHeight-arr[i]-1);
     end;
    1: Bitmap1.Canvas.LineTo(i,PaintBox1.ClientHeight-arr[i]-1);
    2: Bitmap1.Canvas.Pixels[i,PaintBox1.ClientHeight-arr[i]-1] := clBlue;
    end;
  end;

  Bitmap2.Canvas.FillRect(PaintBox2.ClientRect);

  i := Low(arrsum);
  k := seSmooth.Value;
  repeat
    l := min(k, High(arrsum) - i);
    m := 0;
    for j := i to i + l -1 do
      m := m + arrsum[j];

    m := m div l;
    for j := i to i + l -1 do
      arrav[j] := m;

    inc(i, l);
  until i >= High(arrsum);

  maxsum := 0;
  for i := Low(arrsum) to High(arrsum) do
  begin
    maxsum := max(maxsum,arrav[i]);
    Bitmap2.Canvas.MoveTo(0,PaintBox2.ClientHeight-i-1);
    Bitmap2.Canvas.LineTo((arrav[i]+halfFactor) div factor ,PaintBox2.ClientHeight-i-1);
  end;

  if (maxsum+halfFactor) div factor > (PaintBox2.Clientwidth-6) then
    factor := factor + factor;

  PaintBox1.Canvas.Draw(0,0,Bitmap1);
  PaintBox2.Canvas.Draw(0,0,Bitmap2);
  Label1.Caption :='Count = '+IntToStr(ncount)+'  Factor = '+IntToStr(factor);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Timer1.Enabled := false;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  i: integer;
begin
  ncount := 0;
  factor := 1;
  ArrPos := Low(arr)-1;
  for i := Low(arr) to High(arr) do
    arr[i] := 0;
  for i := Low(arrsum) to High(arrsum) do
    arrsum[i] := 0;
  PaintBox2.Canvas.FillRect(PaintBox2.ClientRect);
  PaintBox1.Canvas.FillRect(PaintBox1.ClientRect);
end;

function TForm1.IntRand(N: integer): integer;
var
  t, s, nn: TFloat;
  i: TInt;
begin
  case rgRNG.ItemIndex of
  0: result := random(N);
  1: result := x.IntRand(N);
  2: begin
       result := mt.int(N);
     end;
  3: begin
       t := N;
       Rand13(13, Q);
       result := Trunc(t*Q[seCoord.Value]);
     end;
  4: begin
       t := N;
       nn := seN.Value;
       Rand13(13, Q);
       s := 0.0;
       for i := 1 to seN.Value do
         s := s + Q[i];
       result := Trunc(t*s/nn);
     end;
  end;
end;

procedure TForm1.Help2Click(Sender: TObject);
begin
  ShowMessage('Help is coming');
end;

procedure TForm1.About1Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

end.
