{*
 PasMatLib - Basic Math Types, Interfaces, Constants and Math utilities
 Nikolai V. Shokhirev, 2001-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2006.04.04 - created
 @version 2007.09.27 - enhanced
 }
unit fSignalPlot;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls
  , uSignalPlot, StdCtrls, uMatTypes, uDynArrays;

type
  TFormEditType = class(TForm)
    PanelLeft: TPanel;
    PanelChart: TPanel;
    Button1: TButton;
    PB: TPaintBox;
    Panel1: TPanel;
    ScrollBar1: TScrollBar;
    RadioGroupType: TRadioGroup;
    CheckBoxLeftLine: TCheckBox;
    CheckBoxRightLine: TCheckBox;
    CheckBoxFromZero: TCheckBox;
    EditStep: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    EditR: TEdit;
    CheckBoxFilled: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PBPaint(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    aMax: TInt;
    iRightOffset,
    iLeftOffset,
    iTopOffset,
    iBottomOffset: TInt;
    Y: IFArr1D;
    SP: TSignalPlot;
  public
    { Public declarations }
  end;

var
  FormEditType: TFormEditType;

implementation

{$R *.dfm}

uses
  uDynArrUtils;
  
procedure TFormEditType.Button1Click(Sender: TObject);
var
  i: TInt;
  Ymin, Ymax, x: TFloat;
begin
  Y := TFArr1D.Create(200);
  x := 1.0;
  for i := Y.Lo1 to Y.Hi1 do
  begin
    Y[i] := x-100.0+50.0*exp(-0.005*x)*Cos(0.2*x);
    x := x + 1.0;
  end;
  ScrollBar1.Max := Y.Hi1;
  ScrollBar1.Position := ScrollBar1.Max;
  MinMaxVec(Y, Ymin, Ymax, true);
  SP.Ymin := Ymin;
  SP.Ymax := Ymax;
  PB.OnPaint := PBPaint;
  PBPaint(Self);
end;

procedure TFormEditType.PBPaint(Sender: TObject);
var
  BrushColor: TColor;
//  iMin0: TInt;
begin
  SP.Step := StrToInt(EditStep.Text);
  SP.SetDims(PB.ClientWidth, PB.ClientHeight);
  SP.DrawFrame(clDkGray,5,5,'X-Axis','Y-Axis');

//  iMin0 := IdxMin(Y.Hi1);
//  ScrollBar1.Min := Y.Hi1 - iMin0 + 1;
  aMax := ScrollBar1.Position;

  if CheckBoxFilled.Checked then
    BrushColor := clBlue
  else
    BrushColor := clWhite;

  case RadioGroupType.ItemIndex of
  0: SP.DrawPlot(Y, aMax, clBlue);
  1: SP.DrawBars(Y, aMax, clBlue, CheckBoxLeftLine.Checked,
                  CheckBoxRightLine.Checked, CheckBoxFromZero.Checked);
  2: SP.DrawDots(Y, aMax, StrToInt(EditR.Text), clBlue, BrushColor);
  end;
  SP.DrawHLine(0.0, clGreen, psDot);
  SP.DrawVLine(100, aMax, clRed, psDash);
end;

procedure TFormEditType.FormCreate(Sender: TObject);
begin
  iRightOffset := 10;
  iLeftOffset := 50;
  iTopOffset := 20;
  iBottomOffset := 20;

  SP := TSignalPlot.Create(PB.Canvas,PB.ClientWidth, PB.ClientHeight,
                 iLeftOffset, iRightOffset, iTopOffset, iBottomOffset);
  SP.Step := 5;
{
  iMin := IdxMin(iMax, Xstep, iWidth, iRightOffset, iLeftOffset);
}
end;

procedure TFormEditType.FormDestroy(Sender: TObject);
begin
  SP.Free;
end;

procedure TFormEditType.ScrollBar1Change(Sender: TObject);
begin
  PBPaint(Self);
end;

procedure TFormEditType.FormResize(Sender: TObject);
begin
  ScrollBar1.Min := SP.GetVisiblePoints+1;
end;

end.



