{*
@abstract(Tests for Data Utils)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2003.01.01)
@lastmod(2005.04.14)
@version 2007.07.07 - removed compiler warnings, extended comments, other cleanup
@version 2007.09.30 - extensed StatMoments test
Nikolai V. Shokhirev, 2003-2005
}
unit Test_uDataUtils;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays,
  uDFT;

type

Check_FD = class(TTestCase)
private
  eps, y, y0, x1,x2,x3,x4,y1,y2,y3,y4: TFloat;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFD3d1;
  procedure VerifyFD3d2;
  procedure VerifyFD4d1;
  procedure VerifyFD4d2;
  procedure VerifyFD4d3;
end;

Check_StatMoments = class(TTestCase)
private
  F1: IFArr1D;
  N: integer;
  eps: TFloat;
  msg0: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyRectangle;
  procedure VerifyDelta;
  procedure VerifyTriangle;
  procedure VerifyLine;
  procedure VerifyLineM;
  procedure VerifyPlane;
  procedure VerifyEllipse;
end;

Check_Average = class(TTestCase)
private
  eps: TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyDerivpm;
  procedure VerifyDeriv;
  procedure VerifyAvg5Arr;
  procedure VerifyAvg5Arrd1;
  procedure VerifyAvg5Arrd2;
  procedure VerifyDerivData;
  procedure VerifyAvg3n;
  procedure VerifyAvg2n;
end;

Check_Spline = class(TTestCase)
private
  eps: TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifySpline;
  procedure VerifySplint;
  procedure VerifySplie2;
  procedure VerifySplin2;
  procedure VerifySplin2Exact;
  procedure Verifyindex;
end;

Check_LinSpline = class(TTestCase)
private
  eps: TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyLinSpline1D;
  procedure VerifyLinSpline2D;
  procedure VerifyLinSpline1Ds;
  procedure VerifyLinSpline2Ds;
  procedure VerifyLinSpline1Da;
  procedure VerifyLinSpline2Da;
end;

function Suite : ITestSuite;

implementation

uses
  math,
  uDynArrUtils,
  uDynArrUtilsX,
  uDynDataUtils,
  uStatUtils,
  uDynData,
  uDynArrIO,
  uSpline;
  
function Suite : ITestSuite;
begin
  result := TTestSuite.Create('Data Utils Tests');

  result.addTest(Check_FD.Suite);
  result.addTest(Check_StatMoments.Suite);
  result.addTest(Check_Average.Suite);
  result.addTest(Check_Spline.Suite);
  result.addTest(Check_LinSpline.Suite);
end;

{ Check_StatMoments }

procedure Check_StatMoments.setUp;
begin
  inherited;
  eps := 4.0e-12;
  Randomize;
end;

procedure Check_StatMoments.tearDown;
begin
  inherited;

end;

procedure Check_StatMoments.VerifyTriangle;
var
  average, variance, stdDev, deviation, skewness, kurtosis, minValue, maxValue: TFloat;
  average0, variance0, stdDev0, deviation0, skewness0, kurtosis0, minValue0, maxValue0: TFloat;
  k : TInt;
begin
  msg0 := 'Fail VerifyTriangle: ';
  N := 1000;
//  L := N;
  F1 := TFArr1D.Create(N);
  for k := F1.Lo1 to F1.Hi1 do
    F1[k] := k;
  StatMoments(F1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  StatMoments0(F1, average0, variance0, stdDev0, skewness0, kurtosis0, deviation0, minValue0, maxValue0);
  CheckEquals(average, average, eps,'average');
  CheckEquals(variance0, variance, eps,'variance');
  CheckEquals(stdDev0, stdDev, eps,'stdDev');
  CheckEquals(skewness0, skewness, eps,'skewness');
  CheckEquals(kurtosis0, kurtosis, eps,'kurtosis');
  CheckEquals(deviation0, deviation, eps,'deviation');
  CheckEquals(minValue0, minValue, eps,'minValue');
  CheckEquals(maxValue0, maxValue, eps,'maxValue');
end;

procedure Check_StatMoments.VerifyDelta;
var
  c, average, variance, stdDev, deviation, skewness, kurtosis, minValue, maxValue: TFloat;
begin
  msg0 := 'Fail VerifyDelta: ';
  N := 1000;

  F1 := TFArr1D.Create(N);
  c := sqrt(2.0);
  F1.Fill(c);
  StatMoments(F1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(c, average, eps,'average');
  CheckEquals(0.0, variance, eps,'variance');
  CheckEquals(0.0, stdDev, eps,'stdDev');
  CheckEquals(0.0, skewness, eps,'skewness');
  CheckEquals(0.0, kurtosis, eps,'kurtosis');
  CheckEquals(0.0, deviation, eps,'deviation');

  StatMoments0(F1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(c, average, eps,'average0');
  CheckEquals(0.0, variance, eps,'variance0');
  CheckEquals(0.0, stdDev, eps,'stdDev0');
  CheckEquals(0.0, skewness, eps,'skewness0');
  CheckEquals(0.0, kurtosis, eps,'kurtosis0');
  CheckEquals(0.0, deviation, eps,'deviation');
end;

procedure Check_StatMoments.VerifyRectangle;
var
  average, variance, stdDev, deviation, skewness, kurtosis, minValue, maxValue: TFloat;
  average0, variance0, stdDev0, deviation0, skewness0, kurtosis0: TFloat;
begin
  msg0 := 'Fail VerifyRectangle: ';
  N := 1000;
//  L := N;
  F1 := TFArr1D.Create(N);
  RandArr1D(F1,0.0,10.0);
  StatMoments(F1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  StatMoments0(F1, average0, variance0, stdDev0, skewness0, kurtosis0, deviation0, minValue, maxValue);
  CheckEquals(average, average, eps,'average');
  CheckEquals(variance0, variance, eps,'variance');
  CheckEquals(stdDev0, stdDev, eps,'deviation');
  CheckEquals(skewness0, skewness, eps,'skewness');
  CheckEquals(kurtosis0, kurtosis, eps,'kurtosis');
  CheckEquals(deviation0, deviation, eps,'deviation');
end;


procedure Check_StatMoments.VerifyLine;
var
  x1x2: IFData1D;
  x1av, x2av, var1, var2, cov, corr, cov0: TFloat;
  k : TInt;
  average, variance, stdDev, deviation, skewness, kurtosis, L, minValue, maxValue: TFloat;
begin
  msg0 := 'Fail VerifyRectangle: ';
  N := 1001;
  L := N;
  x1x2 := TFData1D.Create(N);
  for k := x1x2.Lo1 to x1x2.Hi1 do
  begin
    x1x2.x1[k] := k;
    x1x2.x2[k] := k;
  end;
  covariance(x1x2, x1av, x2av, var1, var2, cov, corr);
  StatMoments(x1x2.x1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x1av, eps,'x1av');
  CheckEquals(variance, var1, eps,'var1');

  StatMoments(x1x2.x2, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x2av, eps,'x2av');
  CheckEquals(variance, var2, eps,'var2');

  cov0 := 0.0;
  for k := x1x2.Lo1 to x1x2.Hi1 do
    cov0 := cov0 + (x1x2.x1[k]-x1av)*(x1x2.x2[k]-x2av);
  CheckEquals(cov0/L, cov, eps,'cov');

  CheckEquals(1.0, corr, eps,'corr');
end;

procedure Check_StatMoments.VerifyLineM;
var
  x1x2: IFData1D;
  x1av, x2av, var1, var2, cov, corr, cov0, minValue, maxValue: TFloat;
  k : TInt;
  average, variance, stdDev, deviation, skewness, kurtosis, L: TFloat;
begin
  msg0 := 'Fail VerifyRectangle: ';
  N := 1001;
  L := N;
  x1x2 := TFData1D.Create(N);
  for k := x1x2.Lo1 to x1x2.Hi1 do
  begin
    x1x2.x1[k] := k;
    x1x2.x2[k] := -k;
  end;
  covariance(x1x2, x1av, x2av, var1, var2, cov, corr);
  StatMoments(x1x2.x1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x1av, eps,'x1av');
  CheckEquals(variance, var1, eps,'var1');

  StatMoments(x1x2.x2, average, variance, deviation, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x2av, eps,'x2av');
  CheckEquals(variance, var2, eps,'var2');

  cov0 := 0.0;
  for k := x1x2.Lo1 to x1x2.Hi1 do
    cov0 := cov0 + (x1x2.x1[k]-x1av)*(x1x2.x2[k]-x2av);
  CheckEquals(cov0/L, cov, eps,'cov');

  CheckEquals(-1.0, corr, eps,'corr');
end;

procedure Check_StatMoments.VerifyPlane;
var
  x1x2: IFData1D;
  x1av, x2av, var1, var2, cov, corr, d, minValue, maxValue: TFloat;
  k : TInt;
  average, variance, stdDev, deviation, skewness, kurtosis, L: TFloat;
begin
  msg0 := 'Fail VerifyPlane: ';
  N := 10000;
  L := N;
  d := 2.0/sqrt(L);
  x1x2 := TFData1D.Create(N);
  for k := x1x2.Lo1 to x1x2.Hi1 do
  begin
    x1x2.x1[k] := random;
    x1x2.x2[k] := random;
  end;
  covariance(x1x2, x1av, x2av, var1, var2, cov, corr);
  StatMoments(x1x2.x1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x1av, eps,'x1av');
  CheckEquals(variance, var1, eps,'var1');

  StatMoments(x1x2.x2, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x2av, eps,'x2av');
  CheckEquals(variance, var2, eps,'var2');

  CheckEquals(0.0, cov, d,'cov');
  CheckEquals(0.0, corr, d,'corr');
end;

procedure Check_StatMoments.VerifyEllipse;
var
  x1x2: IFData1D;
  x1av, x2av, var1, var2, cov, corr, d, r, a, cov0, x, y: TFloat;
  k : TInt;
  average, variance, stdDev, deviation, skewness, kurtosis, L, minValue, maxValue: TFloat;
begin
  msg0 := 'Fail VerifyEllipse: ';
  N := 10000;
  L := N;
  d := 1.0/sqrt(L);
  x1x2 := TFData1D.Create(N);
  for k := x1x2.Lo1 to x1x2.Hi1 do
  begin
    r := random;
    a := 2.0*Pi*random;
    x := r*cos(a-Pi/4.0);
    y := 0.1*r*sin(a-Pi/4.0);
    x1x2.x1[k] := x+y;
    x1x2.x2[k] := x-y;
  end;
  covariance(x1x2, x1av, x2av, var1, var2, cov, corr);
  StatMoments(x1x2.x1, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x1av, eps,'x1av');
  CheckEquals(variance, var1, eps,'var1');

  StatMoments(x1x2.x2, average, variance, stdDev, skewness, kurtosis, deviation, minValue, maxValue);
  CheckEquals(average, x2av, eps,'x2av');
  CheckEquals(variance, var2, eps,'var2');

  cov0 := 0.0;
  for k := x1x2.Lo1 to x1x2.Hi1 do
    cov0 := cov0 + (x1x2.x1[k]-x1av)*(x1x2.x2[k]-x2av);
  CheckEquals(cov0/L, cov, eps,'cov');

  CheckEquals(0.98, corr, d,'corr');
end;

{ Check_Average }

procedure Check_Average.setUp;
begin
  inherited;
  eps := 1.0e-12;
end;

procedure Check_Average.tearDown;
begin
  inherited;
end;

procedure Check_Average.VerifyDerivpm;
var
  Ain: IFSignal1D;
  V, D1Aout: IFArr1D;
  k : TInt;
  msg: string;
begin
  Ain := TFSignal1D.Create(2,11);
  D1Aout := TFArr1D.Create(Ain);
  V := TFArr1D.Create(Ain);
  V.Fill(1.0);
  Ain.Start1 := 0.0;
  Ain.Step1 := 0.1;
  for k := Ain.Lo1 to Ain.Hi1 do
    Ain[k] := 1.0 + k*Ain.Step1;
  deriv1f(Ain, D1Aout);
  Check(SameArr(V, D1Aout, msg,eps),'Fail CVerifyDeriv2p:'+msg);
  deriv1b(Ain, D1Aout);
  Check(SameArr(V, D1Aout, msg,eps),'Fail CVerifyDeriv2m:'+msg);
end;

procedure Check_Average.VerifyDeriv;
var
  Ain: IFSignal1D;
  V, D1Aout: IFArr1D;
  k : TInt;
  t: TFloat;
begin
  Ain := TFSignal1D.Create(2,11);
  D1Aout := TFArr1D.Create(Ain);
  V := TFArr1D.Create(Ain);
  Ain.Start1 := 1.0;
  Ain.Step1 := 0.1;

  for k := Ain.Lo1 to Ain.Hi1 do
  begin
    t := k*Ain.Step1;
    Ain[k] := 3.0 + 2.0*t + t*t;
    V[k] := 2.0 + 2.0*t;
  end;
  deriv1c(Ain, D1Aout);
  Check(SameArr(V, D1Aout, msg,eps),'Fail VerifyDeriv:'+msg);

  for k := Ain.Lo1 to Ain.Hi1 do
  begin
    t := k*Ain.Step1;
    Ain[k] := 3.0 + 2.0*t + t*t;
    V[k] := 2.0;
  end;
  deriv2c(Ain, D1Aout);
  Check(SameArr(V, D1Aout, msg,eps),'Fail VerifyDeriv2:'+msg);
end;

procedure Check_Average.VerifyAvg5Arr;
var
  Ain: IFSignal1D;
  V, Aout: IFArr1D;
  k, Lo, Hi : TInt;
  t: TFloat;
begin
  Lo := 2; Hi := 11;
  Ain := TFSignal1D.Create(Lo,Hi);
  Aout := TFArr1D.Create(Ain);
  V := TFArr1D.Create(Ain);
  Ain.Start1 := 1.0;
  Ain.Step1 := 0.1;

  for k := Lo to Hi do
  begin
    t := k*Ain.Step1;
    Ain[k] := 3.0 + 2.0*t + t*t;
    V[k] := 3.0 + 2.0*t + t*t;
  end;
  Avg5Arr(Ain, Aout);
  Check(SameArr(V, Aout, msg,eps),'Fail VerifyAvg5Arr:'+msg);

  Aout[Lo]   := Avg5(Ain[Lo],Ain[Lo+1],Ain[Lo+2],Ain[Lo+3],Ain[Lo+4],1);
  Aout[Lo+1] := Avg5(Ain[Lo],Ain[Lo+1],Ain[Lo+2],Ain[Lo+3],Ain[Lo+4],2);
  for k := Lo+2 to Hi-2 do
    Aout[k] := Avg5(Ain[k-2],Ain[k-1],Ain[k],Ain[k+1],Ain[k+2], 3);
  Aout[Hi-1] := Avg5(Ain[Hi-4],Ain[Hi-3],Ain[Hi-2],Ain[Hi-1],Ain[Hi],4);
  Aout[Hi]   := Avg5(Ain[Hi-4],Ain[Hi-3],Ain[Hi-2],Ain[Hi-1],Ain[Hi],5);
  Check(SameArr(V, Aout, msg,eps),'Fail VerifyAvg5:'+msg);
end;

procedure Check_Average.VerifyAvg5Arrd1;
var
  Ain: IFSignal1D;
  V, Aout: IFArr1D;
  k, Lo, Hi : TInt;
  t, h: TFloat;
begin
  Lo := 2; Hi := 11;
  Ain := TFSignal1D.Create(Lo,Hi);
  Aout := TFArr1D.Create(Ain);
  V := TFArr1D.Create(Ain);
  Ain.Start1 := 1.0;
  Ain.Step1 := 0.1;
  h := Ain.Step1;
  for k := Ain.Lo1 to Ain.Hi1 do
  begin
    t := k*h;
    Ain[k] := 3.0 + 2.0*t + t*t;
    V[k] := 2.0 + 2.0*t;
  end;
  Avg5Arrd1(Ain, Aout);
  Check(SameArr(V, Aout, msg,eps),'Fail VerifyAvg5Arrd1:'+msg);

  Aout[Lo]   := Avg5d1(Ain[Lo],Ain[Lo+1],Ain[Lo+2],Ain[Lo+3],Ain[Lo+4],h,1);
  Aout[Lo+1] := Avg5d1(Ain[Lo],Ain[Lo+1],Ain[Lo+2],Ain[Lo+3],Ain[Lo+4],h,2);
  for k := Lo+2 to Hi-2 do
    Aout[k] := Avg5d1(Ain[k-2],Ain[k-1],Ain[k],Ain[k+1],Ain[k+2],h,3);
  Aout[Hi-1] := Avg5d1(Ain[Hi-4],Ain[Hi-3],Ain[Hi-2],Ain[Hi-1],Ain[Hi],h,4);
  Aout[Hi]   := Avg5d1(Ain[Hi-4],Ain[Hi-3],Ain[Hi-2],Ain[Hi-1],Ain[Hi],h,5);
  Check(SameArr(V, Aout, msg,eps),'Fail VerifyAvg5d1:'+msg);
end;

procedure Check_Average.VerifyAvg5Arrd2;
var
  Ain: IFSignal1D;
  V, Aout: IFArr1D;
  k, Lo, Hi : TInt;
//  s1, s2: string;
  t, h: TFloat;
begin
  Lo := 2; Hi := 11;
  Ain := TFSignal1D.Create(Lo,Hi);
  Aout := TFArr1D.Create(Ain);
  V := TFArr1D.Create(Ain);
  Ain.Start1 := 1.0;
  Ain.Step1 := 0.1;
  h := Ain.Step1;
  for k := Ain.Lo1 to Ain.Hi1 do
  begin
    t := k*h;
    Ain[k] := 3.0 + 2.0*t + t*t;
    V[k] := 2.0;
  end;

  Avg5Arrd2(Ain, Aout);
  Check(SameArr(V, Aout, msg,eps),'Fail VerifyAvg5Arrd2:'+msg);

  Aout[Lo]   := Avg5d2(Ain[Lo],Ain[Lo+1],Ain[Lo+2],Ain[Lo+3],Ain[Lo+4],h,1);
  Aout[Lo+1] := Avg5d2(Ain[Lo],Ain[Lo+1],Ain[Lo+2],Ain[Lo+3],Ain[Lo+4],h,2);
  for k := Lo+2 to Hi-2 do
    Aout[k] := Avg5d2(Ain[k-2],Ain[k-1],Ain[k],Ain[k+1],Ain[k+2],h, 3);
  Aout[Hi-1] := Avg5d2(Ain[Hi-4],Ain[Hi-3],Ain[Hi-2],Ain[Hi-1],Ain[Hi],h,4);
  Aout[Hi]   := Avg5d2(Ain[Hi-4],Ain[Hi-3],Ain[Hi-2],Ain[Hi-1],Ain[Hi],h,5);
  Check(SameArr(V, Aout, msg,eps),'Fail VerifyAvg5d2:'+msg);
end;

procedure Check_Average.VerifyDerivData;
var
  Ain, Aout: IFData1D;
  V: IFArr1D;
  k : TInt;
  t: TFloat;
begin
  Ain := TFData1D.Create(2,11);
  Aout := TFData1D.Create(2,11);
  V := TFArr1D.Create(2,11);

  for k := Ain.Lo1 to Ain.Hi1 do
  begin
    t := k; t := 0.1*t;
    Ain.x1[k] := t;
    Ain.x2[k] := 3.0+t*(2.0+t);
    V[k] := 2.0 + 2.0*t;
  end;
  DerivData1D(Ain, Aout);
  VectorToStr(V, msg);
  VectorToStr(Aout.x2, msg);
  Check(SameArr(V, Aout.x2, msg,eps),'Fail VerifyDerivData:'+msg);

  for k := Ain.Lo1 to Ain.Hi1 do
  begin
    t := k; t := 0.1*t;
    Ain.x1[k] := t;
    Ain.x2[k] := 3.0+t*(2.0+t);
    V[k] := 2.0;
  end;
  Deriv2Data1D(Ain, Aout);
  Check(SameArr(V, Aout.x2, msg,eps),'Fail VerifyDeriv2Data:'+msg);
end;

procedure Check_Average.VerifyAvg3n;
var
  c: IFArr2D;
  y, yy, y1: IFArr1D;
  i: TInt;
  t: TFloat;
begin
  c := GenSmoothCoeff(2,0);
  y := TFArr1D.Create(5);
  yy := TFArr1D.Create(5);
  for i := 1 to 5 do
  begin
    t := i;
    y[i] := 1.0+t+t*t+t*t*t;
  end;

  for i := 1 to 5 do
    yy[i] := Avg(y, 3, i-3, c);
  Check(SameArr(y, yy, msg, eps),'Fail VerifyAvg3n, Order = 0 :'+msg);

  c := GenSmoothCoeff(2,1);
  y1 := TFArr1D.Create(5);
  for i := 1 to 5 do
  begin
    t := i;
    y1[i] := 1.0+2.0*t+3.0*t*t;
  end;
  for i := 1 to 5 do
    yy[i] := Avg(y, 3, i-3, c);
  Check(SameArr(y1, yy, msg, eps),'Fail VerifyAvg3n, Order = 1 :'+msg);

  c := GenSmoothCoeff(2,2);
  y1 := TFArr1D.Create(5);
  for i := 1 to 5 do
  begin
    t := i;
    y1[i] := 2.0+6.0*t;
  end;
  for i := 1 to 5 do
    yy[i] := Avg(y, 3, i-3, c);
  Check(SameArr(y1, yy, msg, eps),'Fail VerifyAvg3n, Order = 1 :'+msg);
end;

procedure Check_Average.VerifyAvg2n;
var
  c: IFArr2D;
  y, yy, y1: IFArr1D;
  i: TInt;
  t: TFloat;
begin
  c := GenSmoothCoeff(2,0,2);
  y := TFArr1D.Create(5);
  yy := TFArr1D.Create(5);
  for i := 1 to 5 do
  begin
    t := i;
    y[i] := 1.0+t+t*t;
  end;

  for i := 1 to 5 do
    yy[i] := Avg(y, 3, i-3, c);
  Check(SameArr(y, yy, msg, eps),'Fail VerifyAvg2n, Order = 0 :'+msg);

  c := GenSmoothCoeff(2,1,2);
  y1 := TFArr1D.Create(5);
  for i := 1 to 5 do
  begin
    t := i;
    y1[i] := 1.0+2.0*t;
  end;
  for i := 1 to 5 do
    yy[i] := Avg(y, 3, i-3, c);
  Check(SameArr(y1, yy, msg, eps),'Fail VerifyAvg2n, Order = 1 :'+msg);

  c := GenSmoothCoeff(2,2,2);
  y1 := TFArr1D.Create(5);
  for i := 1 to 5 do
  begin
//    t := i;
    y1[i] := 2.0;
  end;
  for i := 1 to 5 do
    yy[i] := Avg(y, 3, i-3, c);
  Check(SameArr(y1, yy, msg, eps),'Fail VerifyAvg2n, Order = 1 :'+msg);
end;

{ Check_Spline }

procedure Check_Spline.setUp;
begin
  inherited;
  eps := 1.0e-12;
end;

procedure Check_Spline.tearDown;
begin
  inherited;

end;

procedure Check_Spline.VerifySpline;
var
  i, n, k : integer;
  yp1, ypn, t : TFloat;
  x, y, y2, yy2: IFArr1D;
  s1, s2: string;

  function f(x: TFloat; n:TInt): TFloat;
  begin
    case n of
    0: result := 1.0;
    1: result := x;
    2: result := x*x;
    else
      result := 0.0;
    end;
  end;

  function f1(x: TFloat; n:TInt): TFloat;
  begin
    case n of
    0: result := 0.0;
    1: result := 1.0;
    2: result := 2.0*x;
    else
      result := 0.0;
    end;
  end;

  function f2(x: TFloat; n:TInt): TFloat;
  begin
    case n of
    0: result := 0.0;
    1: result := 0.0;
    2: result := 2.0;
    else
      result := 0.0;
    end;
  end;

begin
  n := 50;
  x := TFArr1D.Create(n);
  y := TFArr1D.Create(n);
  y2 := TFArr1D.Create(n);
  yy2 := TFArr1D.Create(n);
  // second-derivatives for x**k
  for k := 0 to 2 do
  begin
    // generate array for interpolation
    for i := 1 to n do
    begin
      t := i;
      x[i] := t/n;
      y[i] := f(x[i], k);
      yy2[i] := f2(x[i], k);
    end;
    // calculate 2nd derivative with spline
    yp1 := f1(x[1],k);
    ypn := f1(x[n],k);
    VectorToStr(x,s1);
    VectorToStr(y,s2);
    spline(x, y, y2, yp1, ypn);
    // test result
    Check(SameArr(y2, yy2, msg, eps),'Fail VerifySpline x**'+IntToStr(k)+':'+msg);
  end;
  // second-derivatives for sin(x) from 0 to pi
  // generate array for interpolation
  for i := 1 to n do
  begin
    x[i] := i*pi/n;
    y[i] := sin(x[i]);
    yy2[i] := -sin(x[i]);
  end;
  // calculate 2nd derivative with spline
  yp1 := cos(x[1]);
  ypn := cos(x[n]);
  spline(x, y, y2, yp1, ypn);
//  spline(x, y, y2);
  VectorToStr(y2,s1);
  VectorToStr(yy2,s2);
  // test result
  eps := 0.0004;
  Check(SameArr(y2, yy2, msg, eps),'Fail VerifySpline Sin(x):'+msg);
end;

procedure Check_Spline.VerifySplint;
var
  i, n, k : integer;
  yp1, ypn, t, tt, dt : TFloat;
  x, y, y2, yy2, x0, y0: IFArr1D;
  s1, s2: string;

  function f(x: TFloat; n:TInt): TFloat;
  begin
    case n of
    0: result := sin(Pi*x);
    1: result := exp(-x);
    2: result := exp(-x*x);
    else
      result := 0.0;
    end;
  end;

  function f1(x: TFloat; n:TInt): TFloat;
  begin
    case n of
    0: result := Pi*cos(Pi*x);
    1: result := -exp(-x);
    2: result := -2.0*x*exp(-x*x);
    else
      result := 0.0;
    end;
  end;

  function f2(x: TFloat; n:TInt): TFloat;
  begin
    case n of
    0: result := -sqr(Pi)*sin(Pi*x);
    1: result := exp(-x);
    2: result := (4.0*x*x-2.0)*exp(-x*x);
    else
      result := 0.0;
    end;
  end;

begin
  n := 20;
  dt := n;
  dt := 1.0/dt;
  x := TFArr1D.Create(n);
  y := TFArr1D.Create(n);
  y2 := TFArr1D.Create(n);
  yy2 := TFArr1D.Create(n);
  x0 := TFArr1D.Create(n);
  y0 := TFArr1D.Create(n);
  // second-derivatives for x**k
  for k := 0 to 2 do
  begin
    // generate array for interpolation
    t := dt;
    for i := 1 to n do
    begin
      x[i] := t;
      y[i] := f(x[i], k);
      yy2[i] := f2(x[i], k);
      t := t + dt;
    end;
    // calculate 2nd derivative with spline
    yp1 := f1(x[1],k);
    ypn := f1(x[n],k);
    VectorToStr(x,s1);
    VectorToStr(y,s2);
    spline(x, y, y2, yp1, ypn);
    // test result
    t := dt;
    for i := 1 to n do
    begin
      x0[i] := t;
      y0[i] := f(t, k);
      splint(x, y, y2, t, tt);
      yy2[i] := tt;
      t := t + dt;
    end;
    eps := 1.0e-12;
    Check(SameArr(y0, yy2, msg, eps),'Fail VerifySplinT d=0 f_'+IntToStr(k)+' : '+msg);
    t := dt/2.0;
    for i := 1 to n do
    begin
      x0[i] := t;
      y0[i] := f(t, k);
      splint(x, y, y2, t, tt);
      yy2[i] := tt;
      t := t + dt;
    end;
    eps := 3.0e-6;
    Check(SameArr(y0, yy2, msg, eps),'Fail VerifySplinT  d<>0 f_'+IntToStr(k)+' : '+msg);
  end;
end;

procedure Check_Spline.VerifySplie2;
var
  i, n, m, j : integer;
  x1x2 : TFloat;
  x1, x2: IFArr1D;
  y, y2, yy2: IFArr2D;
//  s1, s2: string;
begin
  n := 20;
  m := n;
  x1 := TFArr1D.Create(m);
  x2 := TFArr1D.Create(n);
  y := TFArr2D.Create(m,n);
  y2 := TFArr2D.Create(m,n);
  yy2 := TFArr2D.Create(m,n);

  for i := 1 to n do
    x2[i] := 0.2*i;

  for i := 1 to m do
  begin
    x1[i] := 0.2*i;
    for j := 1 to n do
    begin
      x1x2 := x1[i]*x2[j];
      y[i,j] := sqr(x1x2);
    end;
  end;

  splie2(x1,x2,y,y2);

   for i := 1 to m do
     for j := 1 to n do
       yy2[i,j] := 2.0*sqr(x1[i]);

  eps := 1.1e-11;
  Check(SameArr(y2, yy2, msg, eps),'Fail VerifySplie2:'+msg);
end;

procedure Check_Spline.VerifySplin2;
var
  i, n, m, j, k, l : integer;
  x1x2, f, xx1, xx2, h : TFloat;
  x1, x2: IFArr1D;
  y, y0, y2, yy2: IFArr2D;
begin
  n := 30;  k := 2;
  m := n;   l := 3;
  x1  := TFArr1D.Create(k,m+k-1);
  x2  := TFArr1D.Create(l,n+l-1);
  y   := TFArr2D.Create(k,m+k-1,l,n+l-1);
  y0  := TFArr2D.Create(k,m+k-1,l,n+l-1);
  y2  := TFArr2D.Create(k,m+k-1,l,n+l-1);
  yy2 := TFArr2D.Create(k,m+k-1,l,n+l-1);

  h := 2.0/n;
  for i := x2.Lo1 to x2.Hi1 do
    x2[i] := h*i;

  for i := y.Lo1 to y.Hi1 do
  begin
    x1[i] := h*i;
    for j := y.Lo2 to y.Hi2 do
    begin
      x1x2 := x1[i]*x2[j];
      y[i,j] := x1x2*exp(-x1x2);
    end;
  end;
  splie2(x1,x2,y,y2);

  for i := yy2.Lo1 to yy2.Hi1 do
  begin
    for j := yy2.Lo2 to yy2.Hi2 do
    begin
      splin2(x1,x2,y,y2,x1[i],x2[j],f);
      yy2[i,j] := f;
    end;
  end;

  eps := 1.0e-12;
  Check(SameArr(y, yy2, msg, eps),'Fail VerifySplin2 d=0 : '+msg);

  h := 1.8/n;
  for i := yy2.Lo1 to yy2.Hi1 do
  begin
    xx1 := h*i;
    for j := yy2.Lo2 to yy2.Hi2 do
    begin
      xx2 := h*j;
      x1x2 := xx2*xx1;
      y0[i,j] := x1x2*exp(-x1x2);;
      splin2(x1,x2,y,y2,xx1,xx2,f);
      yy2[i,j] := f;
    end;
  end;

  eps := 2.0e-4;
  Check(SameArr(y0, yy2, msg, eps),'Fail VerifySplin2 d<>0 : '+msg);

end;

procedure Check_Spline.VerifySplin2Exact;
var
  i, n, m, j : integer;
//  x1x2,
  f, xx1, xx2, h : TFloat;
  x1, x2: IFArr1D;
  y, y0, y2, yy2: IFArr2D;
begin
  n := 20;
  m := n;
  x1 := TFArr1D.Create(m);
  x2 := TFArr1D.Create(n);
  y := TFArr2D.Create(m,n);
  y0 := TFArr2D.Create(m,n);
  y2 := TFArr2D.Create(m,n);
  yy2 := TFArr2D.Create(m,n);
  h := 2.0/n;
  for i := 1 to n do
    x2[i] := h*i;

  for i := 1 to m do
  begin
    x1[i] := h*i;
    for j := 1 to n do
    begin
      y[i,j] :=(3.0+x1[i]*(2.0+x1[i]))*(3.0+x2[j]*(2.0+x2[j]));
    end;
  end;

  splie2(x1,x2,y,y2);

  h := 1.8/n;
  for i := 1 to n do
  begin
    xx1 := h*i;
    for j := 1 to n do
    begin
      xx2 := h*j;
      splin2(x1,x2,y,y2,xx1,xx2,f);
      yy2[i,j] := f;
      y0[i,j] := (3.0+xx1*(2.0+xx1))*(3.0+xx2*(2.0+xx2));
    end;
  end;

  eps := 2.0e-12;
  Check(SameArr(y0, yy2, msg, eps),'Fail VerifySplin2Exact d<>0 : '+msg);

end;

procedure Check_Spline.Verifyindex;
var
  xa: IFArr1D;

  procedure CreateFill(n: TInt);
  var
    i : TInt;
    x : TFloat;
  begin
    xa := TFArr1D.Create(n);
    x := 0.0;
    for i := xa.Lo1 to xa.Hi1 do
    begin
      x := x + 1.0;
      xa[i] := x;
    end;
  end;

begin
  CreateFill(1);
  CheckEquals(1,Index2(0.5,xa),'Fail n = 1 Index2');

  CheckEquals(1,Index0(0.5,xa),'Fail n = 1 Index0');

  CreateFill(2);
  CheckEquals(1,Index2(0.5,xa),'Fail n = 2 Index2 x = 0.5');
  CheckEquals(1,Index2(1.5,xa),'Fail n = 2 Index2 x = 1.5');
  CheckEquals(1,Index2(2.5,xa),'Fail n = 2 Index2 x = 2.5');

  CheckEquals(2,Index0(0.5,xa),'Fail n = 2 Index0 x = 0.5');
  CheckEquals(2,Index0(1.5,xa),'Fail n = 2 Index0 x = 1.5');
  CheckEquals(2,Index0(2.5,xa),'Fail n = 2 Index0 x = 2.5');

  CreateFill(3);
  CheckEquals(1,Index2(0.5,xa),'Fail n = 3 Index2 x = 0.5');
  CheckEquals(1,Index2(1.5,xa),'Fail n = 3 Index2 x = 1.5');
  CheckEquals(2,Index2(2.5,xa),'Fail n = 3 Index2 x = 2.5');
  CheckEquals(2,Index2(3.5,xa),'Fail n = 3 Index2 x = 3.5');

  CheckEquals(2,Index0(0.5,xa),'Fail n = 3 Index0 x = 0.5');
  CheckEquals(2,Index0(1.5,xa),'Fail n = 3 Index0 x = 1.5');
  CheckEquals(2,Index0(2.5,xa),'Fail n = 3 Index0 x = 2.5');
  CheckEquals(2,Index0(3.5,xa),'Fail n = 3 Index0 x = 3.5');

  CreateFill(4);
  CheckEquals(2,Index0(0.5,xa),'Fail n = 4 Index0 x = 0.5');
  CheckEquals(2,Index0(1.5,xa),'Fail n = 4 Index0 x = 1.5');
  CheckEquals(2,Index0(2.4,xa),'Fail n = 4 Index0 x = 2.4');
  CheckEquals(3,Index0(2.6,xa),'Fail n = 4 Index0 x = 2.6');
  CheckEquals(3,Index0(3.5,xa),'Fail n = 4 Index0 x = 3.5');
  CheckEquals(3,Index0(4.5,xa),'Fail n = 4 Index0 x = 4.5');

  CreateFill(5);
  CheckEquals(2,Index0(0.5,xa),'Fail n = 5 Index0 x = 0.5');
  CheckEquals(2,Index0(1.5,xa),'Fail n = 5 Index0 x = 1.5');
  CheckEquals(2,Index0(2.4,xa),'Fail n = 5 Index0 x = 2.4');
  CheckEquals(3,Index0(2.6,xa),'Fail n = 5 Index0 x = 2.6');
  CheckEquals(3,Index0(3.4,xa),'Fail n = 5 Index0 x = 3.4');
  CheckEquals(4,Index0(3.5,xa),'Fail n = 5 Index0 x = 3.5');
  CheckEquals(4,Index0(3.6,xa),'Fail n = 5 Index0 x = 3.6');
  CheckEquals(4,Index0(4.5,xa),'Fail n = 5 Index0 x = 4.5');
  CheckEquals(4,Index0(5.5,xa),'Fail n = 5 Index0 x = 5.5');
end;

{ Check_FD }

procedure SetFD(var x1,x2,x3,x4,y1,y2,y3,y4: TFloat; n: TInt);
begin
  x1 := 0.5;  x2 := 1.0;  x3 := 1.5;  x4 := 2.0;
  y1 := IntPower(x1,n);
  y2 := IntPower(x2,n);
  y3 := IntPower(x3,n);
  y4 := IntPower(x4,n);
end;

procedure Check_FD.setUp;
begin
  inherited;
  eps := 1.0e-12;
end;

procedure Check_FD.tearDown;
begin
  inherited;
end;

procedure Check_FD.VerifyFD3d1;
var
  n:TInt;
begin
  for n := 0 to 2 do
  begin
    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);
    y := FD3d1(x1,x2,x3,y1,y2,y3,1);
    y0 := dnPower(x1,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD3d1 n='+IntToStr(n)+' point 1');

    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);
    y := FD3d1(x1,x2,x3,y1,y2,y3,2);
    y0 := dnPower(x2,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD3d1 n='+IntToStr(n)+' point 2');

    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);
    y := FD3d1(x1,x2,x3,y1,y2,y3,3);
    y0 := dnPower(x3,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD3d1 n='+IntToStr(n)+' point 3');
  end;
end;

procedure Check_FD.VerifyFD3d2;
var
  n:TInt;
begin
  for n := 0 to 2 do
  begin
    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);
    y := FD3d2(x1,x2,x3,y1,y2,y3,1);
    y0 := dnPower(x1,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD3d1 n='+IntToStr(n)+' point 1');

    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);
    y := FD3d2(x1,x2,x3,y1,y2,y3,2);
    y0 := dnPower(x2,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD3d1 n='+IntToStr(n)+' point 2');

    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);
    y := FD3d2(x1,x2,x3,y1,y2,y3,3);
    y0 := dnPower(x3,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD3d1 n='+IntToStr(n)+' point 3');
  end;
end;

procedure Check_FD.VerifyFD4d1;
var
  n:TInt;
begin
  for n := 0 to 3 do
  begin
    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);

    y := FD4d1(x1,x2,x3,x4,y1,y2,y3,y4,1);
    y0 := dnPower(x1,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d1 n='+IntToStr(n)+' point 1');

    y := FD4d1(x1,x2,x3,x4,y1,y2,y3,y4,2);
    y0 := dnPower(x2,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d1 n='+IntToStr(n)+' point 2');

    y := FD4d1(x1,x2,x3,x4,y1,y2,y3,y4,3);
    y0 := dnPower(x3,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d1 n='+IntToStr(n)+' point 3');

    y := FD4d1(x1,x2,x3,x4,y1,y2,y3,y4,4);
    y0 := dnPower(x4,n,1);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d1 n='+IntToStr(n)+' point 4');
  end;
end;

procedure Check_FD.VerifyFD4d2;
var
  n:TInt;
begin
  for n := 0 to 3 do
  begin
    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);

    y := FD4d2(x1,x2,x3,x4,y1,y2,y3,y4,1);
    y0 := dnPower(x1,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d2 n='+IntToStr(n)+' point 1');

    y := FD4d2(x1,x2,x3,x4,y1,y2,y3,y4,2);
    y0 := dnPower(x2,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d2 n='+IntToStr(n)+' point 2');

    y := FD4d2(x1,x2,x3,x4,y1,y2,y3,y4,3);
    y0 := dnPower(x3,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d2 n='+IntToStr(n)+' point 3');

    y := FD4d2(x1,x2,x3,x4,y1,y2,y3,y4,4);
    y0 := dnPower(x4,n,2);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d2 n='+IntToStr(n)+' point 4');
  end;
end;

procedure Check_FD.VerifyFD4d3;
var
  n:TInt;
begin
  for n := 0 to 3 do
  begin
    SetFD(x1,x2,x3,x4,y1,y2,y3,y4,n);

    y := FD4d3(x1,x2,x3,x4,y1,y2,y3,y4,1);
    y0 := dnPower(x1,n,3);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d3 n='+IntToStr(n)+' point 1');

    y := FD4d3(x1,x2,x3,x4,y1,y2,y3,y4,2);
    y0 := dnPower(x2,n,3);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d3 n='+IntToStr(n)+' point 2');

    y := FD4d3(x1,x2,x3,x4,y1,y2,y3,y4,3);
    y0 := dnPower(x3,n,3);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d3 n='+IntToStr(n)+' point 3');

    y := FD4d3(x1,x2,x3,x4,y1,y2,y3,y4,4);
    y0 := dnPower(x4,n,3);
    Check(abs(y0-y)<eps,'Fail VerifyFD4d3 n='+IntToStr(n)+' point 4');
  end;
end;

{ Check_LinSpline }

procedure Check_LinSpline.setUp;
begin
  inherited;
  eps := 1.0e-14;
end;

procedure Check_LinSpline.tearDown;
begin
  inherited;

end;

procedure Check_LinSpline.VerifyLinSpline1D;
var
  Dat1D: IFData1D;
  y, yy: IFArr1D;
  i: TInt;
  x:TFloat;
begin
  Dat1D := TFData1D.Create(0,4);
  y := TFArr1D.Create(0,12);
  yy := TFArr1D.Create(0,12);
  for i := 0 to 4 do
  begin
    x := i;
    Dat1D.x1[i] := x/4.0;
    Dat1D.x2[i] := 1.0+x;
  end;
  for i := 0 to 12 do
  begin
    x := i;
    y[i] := 1.0+x/3.0;
    yy[i] := LinSpline1D(x/12.0, Dat1D);
  end;
  Check(SameArr(y, yy, msg, eps),'Fail VerifyLinSpline1D :'+msg);
end;

procedure Check_LinSpline.VerifyLinSpline1Da;
var
  Dat1D: IFArr1D;
  y, yy: IFArr1D;
  i: TInt;
  x:TFloat;
  Start, Step: TFloat;
begin
  Dat1D := TFArr1D.Create(0,4);
  Step := 0.25;
  Start := 0.0;
  y := TFArr1D.Create(0,12);
  yy := TFArr1D.Create(0,12);
  for i := 0 to 4 do
  begin
    x := i;
    Dat1D[i] := 1.0+x;
  end;
  for i := 0 to 12 do
  begin
    x := i;
    y[i] := 1.0+x/3.0;
    yy[i] := LinSpline1D(x/12.0, Dat1D, Start, Step);
  end;
  Check(SameArr(y, yy, msg, eps),'Fail VerifyLinSpline1D :'+msg);
end;

procedure Check_LinSpline.VerifyLinSpline1Ds;
var
  Dat1D: IFSignal1D;
  y, yy: IFArr1D;
  i: TInt;
  x:TFloat;
begin
  Dat1D := TFSignal1D.Create(0,4);
  Dat1D.Step1 := 0.25;
  Dat1D.Start1 := 0.0;
  y := TFArr1D.Create(0,12);
  yy := TFArr1D.Create(0,12);
  for i := 0 to 4 do
  begin
    x := i;
    Dat1D[i] := 1.0+x;
  end;
  for i := 0 to 12 do
  begin
    x := i;
    y[i] := 1.0+x/3.0;
    yy[i] := LinSpline1D(x/12.0, Dat1D);
  end;
  Check(SameArr(y, yy, msg, eps),'Fail VerifyLinSpline1D :'+msg);
end;

procedure Check_LinSpline.VerifyLinSpline2D;
var
  Dat2D: IFData2D;
  z, zz: IFArr2D;
  i,j: TInt;
  x, y:TFloat;
begin
  Dat2D := TFData2D.Create(0,4,0,4);
  z := TFArr2D.Create(0,12,0,12);
  zz := TFArr2D.Create(0,12,0,12);

  for j := 0 to 4 do
  begin
    y := j;
    Dat2D.x2[j] := y/4.0;
  end;

  for i := 0 to 4 do
  begin
    x := i;
    Dat2D.x1[i] := x/4.0;
    for j := 0 to 4 do
    begin
      y := j;
      Dat2D.z[i,j] := 1.0+x*y;
    end;
  end;

  for i := 0 to 12 do
  begin
    x := i;
    for j := 0 to 12 do
    begin
      y := j;
      z[i,j] := 1.0+x*y/9.0;
      zz[i,j] := LinSpline2D(x/12.0, y/12, Dat2D);
    end;
  end;
  Check(SameArr(z, zz, msg, eps),'Fail VerifyLinSpline2D :'+msg);
end;

procedure Check_LinSpline.VerifyLinSpline2Da;
var
  Dat2D: IFArr2D;
  z, zz: IFArr2D;
  i,j: TInt;
  x, y:TFloat;
  Start1, Step1, Start2, Step2: TFloat;
begin
  Dat2D := TFArr2D.Create(0,4,0,4);
  z := TFArr2D.Create(0,12,0,12);
  zz := TFArr2D.Create(0,12,0,12);
  Step1 := 0.25;
  Start1 := 0.0;
  Step2 := 0.25;
  Start2 := 0.0;

  for i := 0 to 4 do
  begin
    x := i;
    for j := 0 to 4 do
    begin
      y := j;
      Dat2D[i,j] := 1.0+x*y;
    end;
  end;

  for i := 0 to 12 do
  begin
    x := i;
    for j := 0 to 12 do
    begin
      y := j;
      z[i,j] := 1.0+x*y/9.0;
      zz[i,j] := LinSpline2D(x/12.0, y/12, Dat2D,Start1, Step1, Start2, Step2);
    end;
  end;
  Check(SameArr(z, zz, msg, eps),'Fail VerifyLinSpline2D :'+msg);
end;

procedure Check_LinSpline.VerifyLinSpline2Ds;
var
  Dat2D: IFSignal2D;
  z, zz: IFArr2D;
  i,j: TInt;
  x, y:TFloat;
begin
  Dat2D := TFSignal2D.Create(0,4,0,4);
  z := TFArr2D.Create(0,12,0,12);
  zz := TFArr2D.Create(0,12,0,12);
  Dat2D.Step1 := 0.25;
  Dat2D.Start1 := 0.0;
  Dat2D.Step2 := 0.25;
  Dat2D.Start2 := 0.0;

  for i := 0 to 4 do
  begin
    x := i;
    for j := 0 to 4 do
    begin
      y := j;
      Dat2D[i,j] := 1.0+x*y;
    end;
  end;

  for i := 0 to 12 do
  begin
    x := i;
    for j := 0 to 12 do
    begin
      y := j;
      z[i,j] := 1.0+x*y/9.0;
      zz[i,j] := LinSpline2D(x/12.0, y/12, Dat2D);
    end;
  end;
  Check(SameArr(z, zz, msg, eps),'Fail VerifyLinSpline2D :'+msg);
end;

initialization

  RegisterTest('uDataUtils Tests',Suite);

end.

