{@abstract(Tests for Data I/O)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2005.04.12)
@lastmod(2005.04.13)
Nikolai V. Shokhirev, 2005-2005 }
unit Test_uDynDataIO;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays,
  uDynData,
  uDynDataIO;

type

Check_TFSignal1D = class(TTestCase)
private
  A1, A2: IFSignal1D;
  d1, l1, h1: integer;
  eps :TFloat;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyGetSetComment;
   procedure VerifyStepStart;
end;

Check_TFSignal2D = class(TTestCase)
private
  A1, A2: IFSignal2D;
  d1, d2, l1, h1, l2, h2: integer;
  eps :TFloat;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyGetSetComment;
   procedure VerifyStepStart;
end;

function Suite : ITestSuite;

implementation

uses
  uDynArrUtils,
  uDynArrUtilsX;
//  uDynDataUtilsX;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDynDataIO Tests');
  result.addTest(Check_TFSignal1D.Suite);
  result.addTest(Check_TFSignal2D.Suite);
end;

{ Check_TFSignal1D }

procedure Check_TFSignal1D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  A1 := TFSignal1D.Create(l1,h1);
  A1.Comment := 'Test '+sFSignal1D;
  A1.Step1 := 0.1;
  A1.Start1 := 1.1;
  RandArr1D(A1,-9,9);
  FSignal1DSave(A1, FileName);
end;

procedure Check_TFSignal1D.setUp;
begin
  inherited;
  d1 := 5;
  eps := 1.0e-12;
  Randomize;
  FileName := 'test.f1d';
end;

procedure Check_TFSignal1D.tearDown;
begin
  inherited;

end;

procedure Check_TFSignal1D.VerifyCreateSave;
begin
  CreateSave;
  A2 := FSignal1DCreate(FileName);
  Check(SameArr(A1, A2, msg, eps), 'Fail VerifyCreateSaveF1D:'+msg);
end;

procedure Check_TFSignal1D.VerifyGetSetComment;
begin
  CreateSave;
  A2 := FSignal1DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentF1D');
end;

procedure Check_TFSignal1D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TFSignal1D.Create(l1,h1);
  FSignal1DRead(A2, FileName);
end;

procedure Check_TFSignal1D.VerifyStepStart;
begin
  CreateSave;
  A2 := FSignal1DCreate(FileName);
  Check(IsEqual(A1.Step1, A2.Step1, msg, eps), 'Fail step1:'+msg);
  Check(IsEqual(A1.Start1, A2.Start1, msg, eps), 'Fail start1:'+msg);
end;

{ Check_TFSignal2D }

procedure Check_TFSignal2D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  A1 := TFSignal2D.Create(l1,h1,l2,h2);
  A1.Comment := 'Test '+sFSignal2D;
  A1.Step1 := 0.1;
  A1.Start1 := 1.1;
  A1.Step2 := 0.2;
  A1.Start2 := 2.2;
  RandArr2D(A1,-9.0,9.0);
  FSignal2DSave(A1, FileName);
end;

procedure Check_TFSignal2D.setUp;
begin
  inherited;
  d1 := 5;
  d2 := 4;
  eps := 1.0e-12;
  randomize;
  FileName := 'test.f2d';
end;

procedure Check_TFSignal2D.tearDown;
begin
  inherited;

end;

procedure Check_TFSignal2D.VerifyCreateSave;
begin
  CreateSave;
  A2 := FSignal2DCreate(FileName);
  Check(SameArr(A1, A2, msg, eps), 'Fail VerifyCreateSaveF1D:'+msg);
end;

procedure Check_TFSignal2D.VerifyGetSetComment;
begin
  CreateSave;
  A2 := FSignal2DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentF3D');
end;

procedure Check_TFSignal2D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TFSignal2D.Create(l1,h1,l2,h2);
  FSignal2DRead(A2, FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyReadFromFileF1D:'+msg);
end;

procedure Check_TFSignal2D.VerifyStepStart;
begin
  CreateSave;
  A2 := FSignal2DCreate(FileName);
  Check(IsEqual(A1.Step1, A2.Step1, msg, eps), 'Fail step1:'+msg);
  Check(IsEqual(A1.Start1, A2.Start1, msg, eps), 'Fail start1:'+msg);
  Check(IsEqual(A1.Step2, A2.Step2, msg, eps), 'Fail step2:'+msg);
  Check(IsEqual(A1.Start2, A2.Start2, msg, eps), 'Fail start2:'+msg);
end;

initialization

  RegisterTest('Dyn Data IO Tests', Suite);
end.
