{@abstract(Tests for Dynamic Array I/O)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(January 1, 2003)
@lastmod(November 11, 2004)
Nikolai V. Shokhirev, 2003-2004
2005.08.11 added VerifyWriteReadTable }
unit Test_uDynArrIO;

interface

uses
  TestFramework,
  uMatTypes,
  SysUtils,
  uDynArrays,
  uDynArrIO;

type

Check_TIArray1D = class(TTestCase)
private
  A1, A2: IIArr1D;
  d1, l1, h1: integer;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyGetSetComment;
end;

Check_TIArray2D = class(TTestCase)
private
  A1, A2: IIArr2D;
  d1, d2, l1, h1, h2, l2: integer;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyGetSetComment;
end;

Check_TFArray1D = class(TTestCase)
private
  A1, A2: IFArr1D;
  d1, l1, h1: integer;
  eps :TFloat;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyGetSetComment;
end;

Check_TFArray2D = class(TTestCase)
private
  A1, A2: IFArr2D;
  d1, d2, l1, h1, l2, h2: integer;
  eps :TFloat;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyWriteReadTable;
   procedure VerifyGetSetComment;
end;

Check_TFArray3D = class(TTestCase)
private
  A1, A2: IFArr3D;
  d1, d2, d3, l1, h1, l2, h2, l3, h3: integer;
  eps :TFloat;
  FileName, msg: string;
  procedure CreateSave;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyReadFromFile;
   procedure VerifyCreateSave;
   procedure VerifyGetSetComment;
end;

function Suite : ITestSuite;

implementation

uses
  uDynArrUtils,
  uDynArrUtilsX;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uFileArrays Tests');
  result.addTest(Check_TIArray1D.Suite);
  result.addTest(Check_TIArray2D.Suite);
  result.addTest(Check_TFArray1D.Suite);
  result.addTest(Check_TFArray2D.Suite);
  result.addTest(Check_TFArray3D.Suite);
end;

{ Check_TIArray1D }
procedure Check_TIArray1D.setUp;
begin
  inherited;
  d1 := 5;
  Randomize;
  FileName := 'test.i1d';
end;

procedure Check_TIArray1D.tearDown;
begin
  inherited;
end;

procedure Check_TIArray1D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TIArr1D.Create(l1,h1);
  IArr1DRead(A2, FileName);
  Check(SameArr(A1,A2,msg), 'Fail VerifyReadFromFileI1D:'+msg);
end;

procedure Check_TIArray1D.VerifyCreateSave;
begin
  CreateSave;
  A2 := IArr1DCreate(FileName);
  Check(SameArr(A1,A2,msg), 'Fail VerifyCreateSaveI1D:'+msg);
end;

procedure Check_TIArray1D.VerifyGetSetComment;
begin
//   fail('Test Not Implemented Yet');
  CreateSave;
  A2 := IArr1DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentI1D');
end;

procedure Check_TIArray1D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  A1 := TIArr1D.Create(l1,h1);
  A1.Comment := 'Test '+sIArr1D;
  RandArr1D(A1,-9,9);
  IArr1DSave(A1, FileName);
end;

{ Check_TIArray2D }

procedure Check_TIArray2D.setUp;
begin
  inherited;
  d1 := 4;
  d2 := 3;
  randomize;
  FileName := 'test.i2d';
end;

procedure Check_TIArray2D.tearDown;
begin
end;

procedure Check_TIArray2D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TIArr2D.Create(l1,h1,l2,h2);
  IArr2DRead(A2, FileName);
  Check(SameArr(A1,A2,msg), 'Fail VerifyReadFromFileI2D:'+msg);
end;

procedure Check_TIArray2D.VerifyCreateSave;
begin
  CreateSave;
  A2 := IArr2DCreate(FileName);
  Check(SameArr(A1,A2,msg), 'Fail VerifyCreateSaveI2D:'+msg);
end;

procedure Check_TIArray2D.VerifyGetSetComment;
begin
//  fail('Test Not Implemented Yet');
  CreateSave;
  A2 := IArr2DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentI2D');
end;

procedure Check_TFArray1D.setUp;
begin
  inherited;
  d1 := 5;
  eps := 1.0e-12;
  randomize;
  FileName := 'test.f1d';
end;

procedure Check_TIArray2D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  A1 := TIArr2D.Create(l1,h1,l2,h2);
  A1.Comment := 'Test '+sIArr2D;
  RandArr2D(A1,-9,9);
  IArr2DSave(A1, FileName);
end;

{ Check_TFArray1D }

procedure Check_TFArray1D.tearDown;
begin
  inherited;
end;

procedure Check_TFArray1D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  A1 := TFArr1D.Create(l1,h1);
  A1.Comment := 'Test '+sFArr1D;
  RandArr1D(A1,-9.0,9.0);
  FArr1DSave(A1, FileName);
end;

procedure Check_TFArray1D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TFArr1D.Create(l1,h1);
  FArr1DRead(A2, FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyReadFromFileF1D:'+msg);
end;

procedure Check_TFArray1D.VerifyCreateSave;
begin
  CreateSave;
  A2 := FArr1DCreate(FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyCreateSaveF1D:'+msg);
end;

procedure Check_TFArray1D.VerifyGetSetComment;
begin
  CreateSave;
  A2 := FArr1DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentF13D');
end;

{ Check_TFArray2D }

procedure Check_TFArray2D.setUp;
begin
  inherited;
  d1 := 5;
  d2 := 4;
  eps := 1.0e-12;
  randomize;
  FileName := 'test.f2d';
end;

procedure Check_TFArray2D.tearDown;
begin
  inherited;
end;

procedure Check_TFArray2D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  A1 := TFArr2D.Create(l1,h1,l2,h2);
  A1.Comment := 'Test '+sFArr2D;
  RandArr2D(A1,-9.0,9.0);
  FArr2DSave(A1, FileName);
end;

procedure Check_TFArray2D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TFArr2D.Create(l1,h1,l2,h2);
  FArr2DRead(A2, FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyReadFromFileF1D:'+msg);
end;

procedure Check_TFArray2D.VerifyWriteReadTable;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  A1 := TFArr2D.Create(l1,h1,l2,h2);
  A1.Comment := 'Test '+sFArr2D;
  RandArr2D(A1,-9.0,9.0);
  FArr2DSave(A1, 'test.tbl',true);
  A2 := TFArr2D.Create(l1,h1,l2,h2);
//  FArr2DRead(A2, 'test.tbl',true);
  FArr2DRead(A2, 'test.tbl');
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyWriteReadTable:'+msg);
end;

procedure Check_TFArray2D.VerifyCreateSave;
begin
  CreateSave;
  A2 := FArr2DCreate(FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyCreateSaveF2D:'+msg);
end;

procedure Check_TFArray2D.VerifyGetSetComment;
begin
  CreateSave;
  A2 := FArr2DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentF3D');
end;

{ Check_TFArray3D }

procedure Check_TFArray3D.setUp;
begin
  inherited;
  d1 := 3;
  d2 := 5;
  d3 := 4;
  eps := 1.0e-12;
  randomize;
  FileName := 'test.f3d';
end;

procedure Check_TFArray3D.tearDown;
begin
  inherited;
end;

procedure Check_TFArray3D.CreateSave;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  A1 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  A1.Comment := 'Test '+sFArr3D;
  RandArr3D(A1,-9.0,9.0);
  FArr3DSave(A1, FileName);
end;

procedure Check_TFArray3D.VerifyCreateSave;
begin
  CreateSave;
  A2 := FArr3DCreate(FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyCreateSaveF3D:'+msg);
end;

procedure Check_TFArray3D.VerifyGetSetComment;
begin
  CreateSave;
  A2 := FArr3DCreate(FileName);
  Check(A1.Comment=A2.Comment,'Fail VerifyGetSetCommentF3D');
end;

procedure Check_TFArray3D.VerifyReadFromFile;
begin
  CreateSave;
  A2 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  FArr3DRead(A2, FileName);
  Check(SameArr(A1,A2,msg,eps), 'Fail VerifyReadFromFileF3D:'+msg);
end;

initialization

  RegisterTest('Dyn Arr IO Tests', Suite);

end.
