{@abstract(Tests for Dynamic Array Utilities)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2003.01.01)
@lastmod(2007.07.07)
Nikolai V. Shokhirev, 2003-2007 }
{ 2005.01.22 - added test for SortI1D }
{ 2007.07.07 - added tests for AssignVector, ResizeF1D andResizeF2D }
unit Test_uDynArrUtils;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays;

type

Check_Sort = class(TTestCase)
private
  d1, l1, h1, l2, h2: integer;
  b: boolean;
  A, V: IFArr1D;
  AA: IFArr2D;
  Ascending: boolean;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyBubleSortI1D;
  procedure VerifyInsertSortI1D;
  procedure VerifyNRQuickSortI1D;
  procedure VerifyQuickSortI1D;
  procedure VerifyQSortI1D;
  procedure VerifyQSortF1D;
  procedure VerifyQSortF2D;
end;

Check_Utils = class(TTestCase)
private
  l1, h1, l2, h2, l3, h3: integer;
  V1, V2, V3, V4: IFArr1D;
  M1, M2, M3, M4: IFArr2D;
  x, eps: TFloat;
  d1, d2, d3, b1, b2: integer;
  msg: string;
public
  procedure setUp; override;
  procedure tearDown; override;
published
  procedure VerifyMtT;
  procedure VerifyMT_MT;
  procedure VerifyMxDxMT;
  procedure VerifyMTxDxM;
  procedure VerifyM1xDxM2T;
  procedure VerifyM1TxDxM2;
  procedure VerifySumArr2D;
  procedure VerifySumArr1D;
//  procedure VerifyMxDxMT;
end;

Check_CUtils = class(TTestCase)
private
  l1, h1, l2, h2, l3, h3: integer;
  V1r, V2r, V3r: IFArr1D;
  V1i, V2i, V3i: IFArr1D;
  Mr, M1r, M2r, M3r, M4r: IFArr2D;
  Mi, M1i, M2i, M3i, M4i: IFArr2D;
  x: TFloat;
  d1, d2, d3, b1, b2: integer;
  msg: string;
public
  procedure setUp; override;
  procedure tearDown; override;
published
  procedure VerifyCMxV;
  procedure VerifyCMHxDxCM;
  procedure VerifyCMxDxCMH;
  procedure VerifyCMxCMH;
  procedure VerifyCMHxCM;
end;
(*
procedure ResizeF1D(A: IFArr1D; NewLo1, NewHi1: TInt; CopyData: boolean = true); overload;
procedure ResizeF1D(A: IFArr1D; NewLim: ILim1D; CopyData: boolean = true); overload;
procedure ResizeF1D(A: IFArr1D; NewDim1: TInt; CopyData: boolean = true); overload;

{ OldA -> NewA
                                              If CopyData :
                                 |* * * * * *|              |0 0 0 0 0 0|
            |x x x x x x|        |* * * * * *|              |0 0|x x x x|. .|
 OldA =     |x x x x x x| NewA = |* * * * * *|       NewA = |0 0|x x x x|. .|
            |x x x x x x|        |* * * * * *|              |0 0|x x x x|. .|
                                 |* * * * * *|              |0 0 0 0 0 0|      }
procedure ResizeF2D(A: IFArr2D; NewLo1, NewHi1, NewLo2, NewHi2: TInt;
                                          CopyData: boolean = true); overload;
procedure ResizeF2D(A: IFArr2D; NewDim1, NewDim2: TInt;
                                          CopyData: boolean = true); overload;
procedure ResizeF2D(A: IFArr2D; NewLim: ILim2D; CopyData: boolean = true); overload;   *)
Check_Resize = class(TTestCase)
private
  V, NewV: IFArr1D;
  M, NewM: IFArr2D;
  x: TFloat;
  msg: string;
public
  procedure setUp; override;
  procedure tearDown; override;
published
  procedure VerifyResizeF1D;
  procedure VerifyResizeF2D;
end;

function Suite : ITestSuite;

implementation

uses
  uDynArrUtils,
  uDynArrUtilsX, // for tests
  uDynArrIO;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDynArrUtils Tests 1');
  result.addTest(Check_Sort.Suite);
  result.addTest(Check_Utils.Suite);
  result.addTest(Check_CUtils.Suite);
  result.addTest(Check_Resize.Suite);
end;

{ Check_Sort }

procedure Check_Sort.setUp;
begin
  d1 := 10;
  Randomize;
end;

procedure Check_Sort.tearDown;
begin
  inherited;
end;

procedure Check_Sort.VerifyBubleSortI1D;
var
  i: integer;
  IA: IIArr1D;
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  IA := TIArr1D.Create(l1,h1);
  RandArr1D(IA,-9,9);
  BubleSortI1D(IA);
  b := true;
  for i := IA.Lo1 to IA.Hi1-1 do
    if IA[i+1] < IA[i] then b := false;
  Check( b, 'Fail Ascending VerifyBubleSortI1D');
end;

procedure Check_Sort.VerifyInsertSortI1D;
var
  i: integer;
  IA: IIArr1D;
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  IA := TIArr1D.Create(l1,h1);
  RandArr1D(IA,-9,9);
  InsertSortI1D(IA);
  b := true;
  for i := IA.Lo1 to IA.Hi1-1 do
    if IA[i+1] < IA[i] then b := false;
  Check( b, 'Fail Ascending VerifyInsertSortI1D');
end;

procedure Check_Sort.VerifyNRQuickSortI1D;
var
  i: integer;
  IA: IIArr1D;
  s: string; // debug
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  IA := TIArr1D.Create(l1,h1);
  RandArr1D(IA,-9,9);
  IVectorToStr(IA, s); // debug
  NRQuickSortI1D(IA);
  IVectorToStr(IA, s); // debug
  b := true;
  for i := IA.Lo1 to IA.Hi1-1 do
    if IA[i+1] < IA[i] then b := false;
  Check( b, 'Fail Ascending VerifyNRQuickSortI1D');

end;

procedure Check_Sort.VerifyQuickSortI1D;
var
  i: integer;
  IA: IIArr1D;
  s: string; // debug
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  IA := TIArr1D.Create(l1,h1);
  RandArr1D(IA,-9,9);
  IVectorToStr(IA, s); // debug
  QuickSortI1D(IA);
  IVectorToStr(IA, s); // debug
  b := true;
  for i := IA.Lo1 to IA.Hi1-1 do
    if IA[i+1] < IA[i] then b := false;
  Check( b, 'Fail Ascending VerifyQuickSortI1D');

end;

procedure Check_Sort.VerifyQSortI1D;
var
  i: integer;
  IA: IIArr1D;
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  IA := TIArr1D.Create(l1,h1);
  RandArr1D(IA,-9,9);

  Ascending := true;
  QSortI1D(IA,Ascending);
  b := true;
  for i := IA.Lo1 to IA.Hi1-1 do
    if IA[i+1] < IA[i] then b := false;
  Check( b, 'Fail Ascending ISort');

  Ascending := false;
  QSortI1D(IA,Ascending);
  b := true;
  for i := IA.Lo1 to IA.Hi1-1 do
    if IA[i+1] > IA[i] then b := false;
  Check( b, 'Fail descending ISort');
end;

procedure Check_Sort.VerifyQSortF1D;
var
  i: integer;
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  A := TFArr1D.Create(l1,h1);
  RandArr1D(A,-9.0,9.0);

  Ascending := true;
  QSortF1D(A,Ascending);
  b := true;
  for i := A.Lo1 to A.Hi1-1 do
    if A[i+1] < A[i] then b := false;
  Check( b, 'Fail Ascending Sort');

  Ascending := false;
  QSortF1D(A,Ascending);
  b := true;
  for i := A.Lo1 to A.Hi1-1 do
    if A[i+1] > A[i] then b := false;
  Check( b, 'Fail descending Sort');
end;

procedure Check_Sort.VerifyQSortF2D;
var
  i: integer;
  var s: string;
begin
  l1 := RndNum(-5,5); h1 := l1+d1-1;
  l2 := l1;           h2 := h1;
  A := TFArr1D.Create(l1,h1);
  V := TFArr1D.Create(l1,h1);
  RandArr1D(A,-9.0,9.0);
  AA := TFArr2D.Create(l1,h1,l2,h2);

  for i := A.Lo1 to A.Hi1 do
  begin
    Fill(V,A[i]);
    CopyArr1D(AA, V, i, _Row);
  end;
  MatrixToStr(AA, s);
  Ascending := true;
  QSortF2D(A,AA,Ascending, _Row);
  MatrixToStr(AA, s);
  b := true;
  for i := A.Lo1 to A.Hi1-1 do
    if AA[i+1,l1] < AA[i,l1] then b := false;
  Check( b, 'Fail Ascending Sort Rows');

  Ascending := false;
  QSortF2D(A,AA,Ascending, _Row);
  b := true;
  for i := A.Lo1 to A.Hi1-1 do
    if AA[i+1,l1] > AA[i,l1] then b := false;
  Check( b, 'Fail descending Sort Rows');

  RandArr1D(A,-9.0,9.0);
  for i := A.Lo1 to A.Hi1 do
  begin
    Fill(V,A[i]);
    CopyArr1D(AA, V, i, _Col);
  end;
  MatrixToStr(AA, s);
  Ascending := true;
  QSortF2D(A,AA,Ascending, _Col);
  MatrixToStr(AA, s);
  b := true;
  for i := A.Lo1 to A.Hi1-1 do
    if AA[l1,i+1] < AA[l1,i] then b := false;
  Check( b, 'Fail Ascending Sort Cols');

  Ascending := false;
  QSortF2D(A,AA,Ascending, _Col);
  b := true;
  for i := A.Lo1 to A.Hi1-1 do
    if AA[l1, i+1] > AA[l1,i] then b := false;
  Check( b, 'Fail descending Sort Cols');
end;

{ Check_Utils }

procedure Check_Utils.setUp;
begin
  inherited;
  d1 := 5;
  d2 := 4;
  d3 := 6;
  b1 := -1;
  b2 := 2;
  x := d1*d2;
end;

procedure Check_Utils.tearDown;
begin
  inherited;

end;

procedure Check_Utils.VerifyMtT;
var
  i, j: TInt;
//  x: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l2,h2,l1,h1);
  RandArr2D(M1,-1.0,1.0,0);
  M3 := TFArr2D.Create(M1);
  for i := M2.Lo1 to M2.Hi1 do
    for j := M2.Lo2 to M2.Hi2 do
      M2[i,j] := M1[j,i];
  MtT(M2,M3);
  Check(SameArr(M3, M1, msg, eps), 'Fail VerifyMtT: '+ msg);
end;

{ Check: MxM, MT_MT }
procedure Check_Utils.VerifyMT_MT;
var
  M1T, M2T: IFArr2D;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l3,h3,l1,h1);
  M3 := TFArr2D.Create(l2,h2,l3,h3);
  M1T := TFArr2D.Create(l2,h2,l1,h1);
  M2T := TFArr2D.Create(l1,h1,l3,h3);
  M4 := TFArr2D.Create(M3);
  RandArr2D(M1,-1.0,1.0,0);
  RandArr2D(M2,-1.0,1.0,0);
  MtT(M1,M1T);
  MtT(M2,M2T);
  MxM(M1T,M2T,M4);
  MT_MT(M1,M2,M3);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail VerifyMT_MT: '+msg);
end;

{ check: MxD, MxMT, MxDxMT }
procedure Check_Utils.VerifyMxDxMT;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(M1);
  V1 := TFArr1D.Create(l2,h2);
  M3 := TFArr2D.Create(l1,h1,l1,h1);
  M4 := TFArr2D.Create(M3);
  RandArr2D(M1,-1.0,1.0,0);
  RandArr1D(V1,-1.0,1.0);
  MxD(M1,V1,M2);
  MxMT(M2,M1,M3);
  MxDxMT(M1,V1,M4);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail MxDxMT: '+ msg);
end;

procedure Check_Utils.VerifyMTxDxM;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l2,h2,l1,h1);
  V1 := TFArr1D.Create(l1,h1);
  M3 := TFArr2D.Create(l2,h2,l2,h2);
  M4 := TFArr2D.Create(M3);
  RandArr2D(M1,-1.0,1.0,0);
  RandArr1D(V1,-1.0,1.0);
  MTrxD(M1,V1,M2);
  MxM(M2,M1,M3);
  MTxDxM(M1,V1,M4);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail MxDxMT: '+ msg);
end;

{ check: MxD, MxMT, M1xDxM2T }
procedure Check_Utils.VerifyM1xDxM2T;
var
  M22: IFArr2D;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  V1 := TFArr1D.Create(l2,h2);
  M2 := TFArr2D.Create(l3,h3,l2,h2);
  M22 := TFArr2D.Create(M1);
  M3 := TFArr2D.Create(l1,h1,l3,h3);
  M4 := TFArr2D.Create(M3);
  RandArr2D(M1,-1.0,1.0,0);
  RandArr1D(V1,-1.0,1.0);
  RandArr2D(M2,-1.0,1.0,0);
  MxD(M1,V1,M22);
  MxMT(M22,M2,M3);
  M1xDxM2T(M1,M2,V1,M4);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail M1xDxM2T: '+ msg);
end;

{ check: DxM, MTxM, M1TxDxM2 }
procedure Check_Utils.VerifyM1TxDxM2;
var
  M22: IFArr2D;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1 := TFArr2D.Create(l2,h2,l1,h1);
  V1 := TFArr1D.Create(l2,h2);
  M2 := TFArr2D.Create(l2,h2,l3,h3);
  M22 := TFArr2D.Create(M2);
  M3 := TFArr2D.Create(l1,h1,l3,h3);
  M4 := TFArr2D.Create(M3);
  RandArr2D(M1,-1.0,1.0,0);
  RandArr1D(V1,-1.0,1.0);
  RandArr2D(M2,-1.0,1.0,0);
  DxM(V1,M2,M22);
  MTxM(M1,M22,M3);
  M1TxDxM2(M1,M2,V1,M4);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail M1TxDxM2: '+ msg);
end;

{ check: MAddM, SumArr2D }
procedure Check_Utils.VerifySumArr2D;
var
  c1, c2, c3: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(M1);
  M3 := TFArr2D.Create(M1);
  M4 := TFArr2D.Create(M1);
  RandArr2D(M1,-1.0,1.0,0);  c1 := RndFloat(-2.0,2.0);
  RandArr2D(M2,-1.0,1.0,0);  c2 := RndFloat(-2.0,2.0);
  RandArr2D(M3,-1.0,1.0,0);  c3 := RndFloat(-2.0,2.0);
  MAddM(M4, M1, c1);
  MAddM(M4, M2, c2);
  MAddM(M4, M3, c3);
  SumArr2D([c1,c2,c3], [M1,M2,M3], M3);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail SumArr1D: '+ msg);
end;

{ check: VAddV, SumArr1D }
procedure Check_Utils.VerifySumArr1D;
var
  c1, c2, c3: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  V2 := TFArr1D.Create(V1);
  V3 := TFArr1D.Create(V1);
  V4 := TFArr1D.Create(V1);
  RandArr1D(V1,-1.0,1.0);  c1 := RndFloat(-2.0,2.0);
  RandArr1D(V2,-1.0,1.0);  c2 := RndFloat(-2.0,2.0);
  RandArr1D(V3,-1.0,1.0);  c3 := RndFloat(-2.0,2.0);
  VAddV(V4, V1, c1);
  VAddV(V4, V2, c2);
  VAddV(V4, V3, c3);
  SumArr1D([c1,c2,c3], [V1,V2,V3], V3);
  Check(SameArr(V3, V4, msg, x*MachEps), 'Fail SumArr1D: '+ msg);
end;

{ Check_CUtils }

procedure Check_CUtils.setUp;
begin
  inherited;
  d1 := 5;
  d2 := 4;
  d3 := 6;
  b1 := -1;
  b2 := 2;
  x := d1*d2;
end;

procedure Check_CUtils.tearDown;
begin
  inherited;

end;

{ check: MxD, MxMT, MxDxMT }
procedure Check_CUtils.VerifyCMxV;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1r := TFArr2D.Create(l1,h1,l2,h2);
  M1i := TFArr2D.Create(M1r);
  V1r := TFArr1D.Create(l2,h2);
  V1i := TFArr1D.Create(V1r);
  V2r := TFArr1D.Create(l1,h1);
  V2i := TFArr1D.Create(V2r);
  RandArr1D(V1r,-1.0,1.0);
  RandArr1D(V1r,-1.0,1.0);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M1r,-1.0,1.0);
  CMxCV(M1r, M1i, V1r, V1i, V2r, V2i);

  M2r := TFArr2D.Create(l2,h2,l1,h1);
  M2i := TFArr2D.Create(M2r);
  V3r := TFArr1D.Create(l1,h1);
  V3i := TFArr1D.Create(V3r);
  CMT(M1r, M1i, M2r, M2i);
  CVxCM(V1r, V1i, M2r, M2i, V3r, V3i);

  Check(SameArr(V3r, V2r, msg, x*MachEps), 'Fail VerifyCMxCMH Re: '+ msg);
  Check(SameArr(V3i, V2i, msg, x*MachEps), 'Fail VerifyCMxCMH Im: '+ msg);
end;

procedure Check_CUtils.VerifyCMHxDxCM;
var
  i,j:TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1r := TFArr2D.Create(l1,h1,l2,h2);
  M1i := TFArr2D.Create(M1r);
  V1r := TFArr1D.Create(l1,h1);
  M3r := TFArr2D.Create(l2,h2,l2,h2);
  M3i := TFArr2D.Create(M3r);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M1r,-1.0,1.0);
  RandArr1D(V1r,-1.0,1.0);
  CMHxDxCM(M1r, M1i, V1r, M3r, M3i);

  M2r := TFArr2D.Create(M1r);
  M2i := TFArr2D.Create(M1r);
  for i := M2r.Lo1 to M2r.Hi1 do
    for j := M2r.Lo2 to M2r.Hi2 do
      M2r[i,j] := V1r[i]*M1r[i,j];

  M4r := TFArr2D.Create(M3r);
  M4i := TFArr2D.Create(M3r);
  CMHxCM(M2r, M2i, M1r, M1i, M4r, M4i);
  Check(SameArr(M3r, M4r, msg, x*MachEps), 'Fail VerifyCMxCMH Re: '+ msg);
  Check(SameArr(M3i, M4i, msg, x*MachEps), 'Fail VerifyCMxCMH Im: '+ msg);
end;

{ check: MxD, MxMT, M1xDxM2T }
procedure Check_CUtils.VerifyCMxDxCMH;
var
  i,j:TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1r := TFArr2D.Create(l1,h1,l2,h2);
  M1i := TFArr2D.Create(M1r);
  V1r := TFArr1D.Create(l2,h2);
  M3r := TFArr2D.Create(l1,h1,l1,h1);
  M3i := TFArr2D.Create(M3r);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M1r,-1.0,1.0);
  RandArr1D(V1r,-1.0,1.0);
  CMxDxCMH(M1r, M1i, V1r, M3r, M3i);

  M2r := TFArr2D.Create(M1r);
  M2i := TFArr2D.Create(M1r);
  for i := M2r.Lo1 to M2r.Hi1 do
    for j := M2r.Lo2 to M2r.Hi2 do
      M2r[i,j] := M1r[i,j]*V1r[j];

  M4r := TFArr2D.Create(M3r);
  M4i := TFArr2D.Create(M3r);
  CMxCMH(M2r, M2i, M1r, M1i, M4r, M4i);
  Check(SameArr(M3r, M4r, msg, x*MachEps), 'Fail VerifyCMxCMH Re: '+ msg);
  Check(SameArr(M3i, M4i, msg, x*MachEps), 'Fail VerifyCMxCMH Im: '+ msg);
end;

{ check: DxM, MTxM, M1TxDxM2 }
procedure Check_CUtils.VerifyCMxCMH;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1r := TFArr2D.Create(l1,h1,l2,h2);
  M1i := TFArr2D.Create(M1r);
  M2r := TFArr2D.Create(l3,h3,l2,h2);
  M2i := TFArr2D.Create(M2r);
  M3r := TFArr2D.Create(l1,h1,l3,h3);
  M3i := TFArr2D.Create(M3r);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M2r,-1.0,1.0);
  RandArr2D(M2r,-1.0,1.0);
  CMxCMH(M1r, M1i, M2r, M2i, M3r, M3i);

  Mr := TFArr2D.Create(l2,h2,l3,h3);
  Mi := TFArr2D.Create(Mr);
  CMH(M2r, M2i, Mr, Mi);
  M4r := TFArr2D.Create(M3r);
  M4i := TFArr2D.Create(M3r);
  CMxCM(M1r, M1i, Mr, Mi, M4r, M4i);
  Check(SameArr(M3r, M4r, msg, x*MachEps), 'Fail VerifyCMxCMH Re: '+ msg);
  Check(SameArr(M3i, M4i, msg, x*MachEps), 'Fail VerifyCMxCMH Im: '+ msg);
end;

{ check: MAddM, SumArr2D }
procedure Check_CUtils.VerifyCMHxCM;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1r := TFArr2D.Create(l1,h1,l2,h2);
  M1i := TFArr2D.Create(M1r);
  M2r := TFArr2D.Create(l1,h1,l3,h3);
  M2i := TFArr2D.Create(M2r);
  M3r := TFArr2D.Create(l2,h2,l3,h3);
  M3i := TFArr2D.Create(M3r);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M1r,-1.0,1.0);
  RandArr2D(M2r,-1.0,1.0);
  RandArr2D(M2r,-1.0,1.0);
  CMHxCM(M1r, M1i, M2r, M2i, M3r, M3i);

  Mr := TFArr2D.Create(l2,h2,l1,h1);
  Mi := TFArr2D.Create(Mr);
  CMH(M1r, M1i, Mr, Mi);
  M4r := TFArr2D.Create(M3r);
  M4i := TFArr2D.Create(M3r);
  CMxCM(Mr, Mi, M2r, M2i, M4r, M4i);
  Check(SameArr(M3r, M4r, msg, x*MachEps), 'Fail VerifyCMxCMH Re: '+ msg);
  Check(SameArr(M3i, M4i, msg, x*MachEps), 'Fail VerifyCMxCMH Im: '+ msg);
end;

{ Check_Resize }

procedure Check_Resize.setUp;
begin
  inherited;
  x := 12.0;
  msg := '';
end;

procedure Check_Resize.tearDown;
begin
  inherited;
end;

procedure Check_Resize.VerifyResizeF1D;
begin
  msg := '#1';
  V := TFArr1D.Create(1,5);
  NewV := TFArr1D.Create(0,4);
  RandArr1D(V,-1.0,1.0);
  AssignVector(NewV, V, false);  // MatchLim = false
  ResizeF1D(V, NewV); // CopyData = true
  Check(SameArr(V, NewV, msg, x*MachEps), 'Fail ResizeF1D: '+ msg);

  msg := '#2';
  V := TFArr1D.Create(1,5);
  NewV := TFArr1D.Create(2,7);
  RandArr1D(V,-1.0,1.0);
  AssignVector(NewV, V, false);  // MatchLim = false
  ResizeF1D(V, NewV); // CopyData = true
  Check(SameArr(V, NewV, msg, x*MachEps), 'Fail ResizeF1D: '+ msg);
end;

procedure Check_Resize.VerifyResizeF2D;
begin
  msg := '#1';
  M := TFArr2D.Create(1,4,1,3);
  NewM := TFArr2D.Create(0,3,2,4);
  RandArr2D(M,-1.0,1.0);
  AssignMatrix(NewM, M, false); // MatchLim = false
  ResizeF2D(M, NewM); // CopyData = true
  Check(SameArr(M, NewM, msg, x*MachEps), 'Fail ResizeF2D: '+ msg);

  msg := '#2';
  M := TFArr2D.Create(1,4,1,3);
  NewM := TFArr2D.Create(2,6,1,4);
  RandArr2D(M,-1.0,1.0);
  AssignMatrix(NewM, M, false); // MatchLim = false
  ResizeF2D(M, NewM); // CopyData = true
  Check(SameArr(M, NewM, msg, x*MachEps), 'Fail ResizeF2D: '+ msg);
end;

initialization

  RegisterTest('Dyn Arr Tests', Suite);

end.
