{@abstract(Tests for Dynamic Arrays)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2003.01.01)
@lastmod(2004.12.12)
Nikolai V. Shokhirev, 2003-2004 }
{ 2007.07.07 - Modified for Slice property }
unit Test_uDynArrays;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays;

type

CRACK_TBArr1D = class(TBArr1D);
CRACK_TBArr2D = class(TBArr2D);

CRACK_TIArr1D = class(TIArr1D);
CRACK_TIArr2D = class(TIArr2D);
CRACK_TIArr3D = class(TIArr3D);

CRACK_TFArr1D = class(TFArr1D);
CRACK_TFArr2D = class(TFArr2D);
CRACK_TFArr3D = class(TFArr3D);

Check_TBArr1D = class(TTestCase)
private
  V1, V2: IBArr1D;
  d1, l1, h1: integer;
  i, j: integer;
  b, n, m: boolean;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifySetGetValue;
  procedure VerifyGetDimLo;
end;

Check_TBArr2D = class(TTestCase)
private
  i1, i2: integer;
  V1, V2, V3, V4: IBArr1D;
  M1, M2: IBArr2D;
  i, j, d1, d2, l1, h1, l2, h2: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifySetGetValue;
  procedure VerifyGetDimLo;
  procedure VerfyGetVector;
  procedure VerfySetVector;
end;

Check_TIArr1D = class(TTestCase)
private
  V1, V2: IIArr1D;
  i, j, d1, l1, h1: integer;
  n, m: TInt;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
end;

Check_TIArr2D = class(TTestCase)
private
  i1, i2: integer;
  V1, V2, V3, V4: IIArr1D;
  M1, M2: IIArr2D;
  i, j, d1, d2, l1, h1, l2, h2: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
  procedure VerfyGetVector;
  procedure VerfySetVector;
end;

Check_TIArr3D = class(TTestCase)
private
  i1, i2, i3: integer;
//  M1, M2, M3, M4: IIArr2D;
  MM1, MM2: IIArr3D;
  d1, d2, d3, l1, h1, l2, h2, l3, h3: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
//  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
end;

Check_TFArr1D = class(TTestCase)
private
  V1, V2: IFArr1D;
  i, j, d1, l1, h1: integer;
  eps: TFloat;
  n, m: TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
  procedure VerifyCreate;
  procedure VerifyDot;
end;

Check_TFArr2D = class(TTestCase)
private
  i1, i2: integer;
  V1, V2, V3, V4: IFArr1D;
  M1, M2: IFArr2D;
  i, j, d1, d2, l1, h1, l2, h2: integer;
  eps: TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
  procedure VerfyGetVector;
  procedure VerfySetVector;
  procedure VerfyNorm;
end;

Check_TFArr3D = class(TTestCase)
private
  i1, i2, i3: integer;
//  M1, M2, M3, M4: IFArr2D;
  MM1, MM2: IFArr3D;
  d1, l1, h1, d2, l2, h2, l3, h3, d3: integer;
  eps :TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFill;
  procedure VerifyCopy;
//  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
end;

Check_TCArr1D = class(TTestCase)
private
  V1, V2: ICArr1D;
  i, j, d1, l1, h1: integer;
  eps: TFloat;
  n, m: Complex;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyConjugate;
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
end;

Check_TCArr2D = class(TTestCase)
private
  i1, i2: integer;
  V1, V2, V3, V4: ICArr1D;
  M1, M2: ICArr2D;
  i, j, d1, d2, l1, h1, l2, h2: integer;
  eps: TFloat;
  msg: string;
  n: Complex;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyConjugate;
  procedure VerifyFill;
  procedure VerifyCopy;
  procedure VerifySwap;
  procedure VerifyGetHiLo;
  procedure VerifyGetDimLo;
  procedure VerifySetGetValue;
//  procedure VerfyGetVector;
end;

function Suite : ITestSuite;

implementation

uses
  uDynArrUtils,
  uDynArrUtilsX, // for tests
  uDynObjAlg,
  uDynArrIO,
  math;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDynArrays Tests');

//  result.addTest(testSuiteOf(Check_TBArr1D));
  result.addTest(Check_TBArr1D.Suite);
  result.addTest(Check_TBArr2D.Suite);

  result.addTest(Check_TIArr1D.Suite);
  result.addTest(Check_TIArr2D.Suite);
  result.addTest(Check_TIArr3D.Suite);

  result.addTest(Check_TFArr1D.Suite);
  result.addTest(Check_TFArr2D.Suite);
  result.addTest(Check_TFArr3D.Suite);

  result.addTest(Check_TCArr1D.Suite);
  result.addTest(Check_TCArr2D.Suite);
end;

{ Check_TBArr1D }

procedure Check_TBArr1D.setUp;
begin
  d1 := 4;
  Randomize;
end;

procedure Check_TBArr1D.tearDown;
begin
end;


procedure Check_TBArr1D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  V1 := TBArr1D.Create(l1,h1);
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo');
  i := RndNumEx(-5,5,l1);
  V1.Lo1 := i;
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo #2');
end;

procedure Check_TBArr1D.VerifySetGetValue;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TBArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndBool;
  V1[i] := n;
  b := (V1[i] = n);
  Check( b, 'Fail SetValue');
end;

procedure Check_TBArr1D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  V1 := TBArr1D.Create(h1);
  Check( (h1 = V1.Hi1), 'Fail Hi=Dim');
  Check( (1=V1.Lo1), 'Fail Lo=1');
end;

procedure Check_TBArr1D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TBArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndBool;
  V1[i] := n;
  j := RndNumEx(l1,h1,i);
  m := RndBool;
  V1[j] := m;
  V1.Swap(i,j);
  Check( (V1[i] = m) and (V1[j]=n), 'Fail Swap');
end;

procedure Check_TBArr1D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TBArr1D.Create(l1,h1);
  V2 := TBArr1D.Create(l1,h1);
  V1.Fill(true);
  V2.Fill(true);
  Check( SameArr(V1,V2,msg), 'Fail Fill'+msg);
end;

procedure Check_TBArr1D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TBArr1D.Create(l1,h1);
  RandArr1D(V1);
  V2 := TBArr1D.Create(V1,true);
  Check(SameArr(V1,V2,msg), 'Fail VerifyCopy:'+msg);
end;

{ Check_TBArr2D }

procedure Check_TBArr2D.setUp;
begin
  d1 := 4;
  d2 := 5;
  Randomize;
end;

procedure Check_TBArr2D.tearDown;
begin
end;

procedure Check_TBArr2D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1);

  V1 := TBArr1D.Create(l2,h2);
  V2 := TBArr1D.Create(l2,h2);
  V3 := TBArr1D.Create(l2,h2);
  V4 := TBArr1D.Create(l2,h2);
  i := RndNum(l1,h1);
  j := RndNumEx(l1,h1,i);
  V1.Assign(GetArr1D(M1,i,_Row));
  V2.Assign(GetArr1D(M1,j,_Row));
  M1.Swap(i,j,_Row);
  V3.Assign(GetArr1D(M1,i,_Row));
  V4.Assign(GetArr1D(M1,j,_Row));
  Check( SameArr(V1, V4, msg), 'Fail Swap Rows, 1: '+msg);
  Check( SameArr(V2, V3, msg), 'Fail Swap Rows, 2: '+msg);

  V1 := TBArr1D.Create(l1,h1);
  V2 := TBArr1D.Create(l1,h1);
  V3 := TBArr1D.Create(l1,h1);
  V4 := TBArr1D.Create(l1,h1);
  i := RndNum(l2,h2);
  j := RndNumEx(l2,h2,i);
  V1.Assign(GetArr1D(M1,i,_Col));
  V2.Assign(GetArr1D(M1,j,_Col));
  M1.Swap(i,j,_Col);
  V3.Assign(GetArr1D(M1,i,_Col));
  V4.Assign(GetArr1D(M1,j,_Col));
  Check( SameArr(V1, V4, msg), 'Fail Swap Cols, 1: '+msg);
  Check( SameArr(V2, V3, msg), 'Fail Swap Cols, 2: '+msg);
end;

procedure Check_TBArr2D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1');
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2');
  i1 := RndNumEx(-5,5,l1);   M1.Lo1 := i1;
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1 #2');
  i2 := RndNumEx(-5,5,l2);   M1.Lo2 := i2;
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2 #2');
end;

procedure Check_TBArr2D.VerifySetGetValue;
var
  i1, i2: integer;
  b, n: boolean;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  i1 := RndNum(l1,h1);
  i2 := RndNum(l2,h2);
  n := RndBool;
  M1[i1,i2] := n;
  b := M1[i1,i2] = n;
  Check(b, 'Fail SetValue');
end;

procedure Check_TBArr2D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  h2 := RndNum(2,7);
  M1 := TBArr2D.Create(h1,h2);
  Check( (h1 = M1.Hi1), 'Fail Hi1=Dim1');
  Check( (h2 = M1.Hi2), 'Fail Hi2=Dim2');
  Check( (1=M1.Lo1), 'Fail Lo1=1');
  Check( (1=M1.Lo2), 'Fail Lo2=1');
end;

procedure Check_TBArr2D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  M2 := TBArr2D.Create(l1,h1,l2,h2);
  M1.Fill(true);
  Fill(M2,true);
  Check(SameArr(M1,M2,msg), 'Fail Fill:'+msg);;
end;

procedure Check_TBArr2D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1);
  M2 := TBArr2D.Create(M1,true);
  Check(SameArr(M1,M2,msg), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TBArr2D.VerfyGetVector;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1);
  i1 := RndNum(l1,h1);
  V1 := M1.GetSlice(i1,_);
  V2 := GetArr1D(M1, i1, _Row);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Row:'+msg);
  i2 := RndNum(l2,h2);
  V1 := M1.GetSlice(_,i2);
  V2 := GetArr1D(M1, i2, _Col);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Col:'+msg);
end;

procedure Check_TBArr2D.VerfySetVector;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TBArr2D.Create(l1,h1,l2,h2);
  V1 := TBArr1D.Create(l2,h2);
  RandArr1D(V1);
  i1 := RndNum(l1,h1);
  M1.SetSlice(i1,_,V1);
  V2 := GetArr1D(M1, i1, _Row);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Row:'+msg);
  V1 := TBArr1D.Create(l1,h1);
  RandArr1D(V1);
  i2 := RndNum(l2,h2);
  M1.SetSlice(_,i2,V1);
  V2 := GetArr1D(M1, i2, _Col);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Col:'+msg);
end;

{ Check_TIArr1D }

procedure Check_TIArr1D.setUp;
begin
  d1 := 4;
  Randomize;
end;

procedure Check_TIArr1D.tearDown;
begin
end;

procedure Check_TIArr1D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);   h1 := l1+d1-1;
  V1 := TIArr1D.Create(l1,h1);
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo');
  i := RndNumEx(-5,5,l1);
  V1.Lo1 := i;
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo #2');
end;

procedure Check_TIArr1D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  V1 := TIArr1D.Create(h1);
  Check( (h1 = V1.Hi1), 'Fail Hi=Dim');
  Check( (1=V1.Lo1), 'Fail Lo=1');
end;

procedure Check_TIArr1D.VerifySetGetValue;
var
  i, n: integer;
  b: boolean;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TIArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndNum(1,13);
  V1[i] := n;
  b := V1[i] = n;
  Check( b, 'Fail SetValue');
end;

procedure Check_TIArr1D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TIArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndNum(-9,9);
  V1[i] := n;
  j := RndNumEx(l1,h1,i);
  m := RndNum(-9,9);
  V1[j] := m;
  V1.Swap(i,j);
  Check( (V1[i] = m) and (V1[j]=n), 'Fail Swap');
end;

procedure Check_TIArr1D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TIArr1D.Create(l1,h1);
  V2 := TIArr1D.Create(l1,h1);
  V1.Fill(222);
  Fill(V2,222);
  Check( SameArr(V1,V2,msg), 'Fail Fill'+msg);
end;

procedure Check_TIArr1D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TIArr1D.Create(l1,h1);
  RandArr1D(V1,-9,9);
  V2 := TIArr1D.Create(V1,true);
  Check(SameArr(V1,V2,msg), 'Fail VerifyCopy:'+msg);
end;

{ Check_TIArr2D }

procedure Check_TIArr2D.setUp;
begin
  d1 := 4;
  d2 := 5;
  Randomize;
end;

procedure Check_TIArr2D.tearDown;
begin
end;

procedure Check_TIArr2D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1');
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2');
  i1 := RndNumEx(-5,5,l1);   M1.Lo1 := i1;
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1 #2');
  i2 := RndNumEx(-5,5,l2);   M1.Lo2 := i2;
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2 #2');
end;

procedure Check_TIArr2D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  h2 := RndNum(2,7);
  M1 := TIArr2D.Create(h1,h2);
  Check( (h1 = M1.Hi1), 'Fail Hi1=Dim1');
  Check( (h2 = M1.Hi2), 'Fail Hi2=Dim2');
  Check( (1=M1.Lo1), 'Fail Lo1=1');
  Check( (1=M1.Lo2), 'Fail Lo2=1');
end;

procedure Check_TIArr2D.VerifySetGetValue;
var
  i1, i2, n: integer;
  b: boolean;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  i1 := RndNum(l1,h1);
  i2 := RndNum(l2,h2);
  n := RndNum(1,13);
  M1[i1,i2] := n;
  b := M1[i1,i2] = n;
  Check(b, 'Fail SetValue');
end;

procedure Check_TIArr2D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9,9);

  V1 := TIArr1D.Create(l2,h2);
  V2 := TIArr1D.Create(l2,h2);
  V3 := TIArr1D.Create(l2,h2);
  V4 := TIArr1D.Create(l2,h2);
  i := RndNum(l1,h1);
  j := RndNumEx(l1,h1,i);
  V1.Assign(GetArr1D(M1,i,_Row));
  V2.Assign(GetArr1D(M1,j,_Row));
  M1.Swap(i,j,_Row);
  V3.Assign(GetArr1D(M1,i,_Row));
  V4.Assign(GetArr1D(M1,j,_Row));
  Check( SameArr(V1, V4, msg), 'Fail Swap Rows, 1: '+msg);
  Check( SameArr(V2, V3, msg), 'Fail Swap Rows, 2: '+msg);

  V1 := TIArr1D.Create(l1,h1);
  V2 := TIArr1D.Create(l1,h1);
  V3 := TIArr1D.Create(l1,h1);
  V4 := TIArr1D.Create(l1,h1);
  i := RndNum(l2,h2);
  j := RndNumEx(l2,h2,i);
  V1.Assign(GetArr1D(M1,i,_Col));
  V2.Assign(GetArr1D(M1,j,_Col));
  M1.Swap(i,j,_Col);
  V3.Assign(GetArr1D(M1,i,_Col));
  V4.Assign(GetArr1D(M1,j,_Col));
  Check( SameArr(V1, V4, msg), 'Fail Swap Cols, 1: '+msg);
  Check( SameArr(V2, V3, msg), 'Fail Swap Cols, 2: '+msg);
end;

procedure Check_TIArr2D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  M2 := TIArr2D.Create(l1,h1,l2,h2);
  M1.Fill(222);
  Fill(M2,222);
  Check(SameArr(M1,M2,msg), 'Fail Fill:'+msg);;
end;

procedure Check_TIArr2D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9,9);
  M2 := TIArr2D.Create(M1,true);
  Check(SameArr(M1,M2,msg), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TIArr2D.VerfyGetVector;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9,9);
  i1 := RndNum(l1,h1);
  V1 := M1.GetSlice(i1,_);
  V2 := GetArr1D(M1, i1, _Row);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Row:'+msg);
  i2 := RndNum(l2,h2);
  V1 := M1.GetSlice(_,i2);
  V2 := GetArr1D(M1, i2, _Col);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Col:'+msg);
end;

procedure Check_TIArr2D.VerfySetVector;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TIArr2D.Create(l1,h1,l2,h2);
  V1 := TIArr1D.Create(l2,h2);
  RandArr1D(V1,-9,9);
  i1 := RndNum(l1,h1);
  M1.SetSlice(i1,_,V1);
  V2 := GetArr1D(M1, i1, _Row);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Row:'+msg);
  V1 := TIArr1D.Create(l1,h1);
  RandArr1D(V1,-9,9);
  i2 := RndNum(l2,h2);
  M1.SetSlice(_,i2,V1);
  V2 := GetArr1D(M1, i2, _Col);
  Check(SameArr(V1,V2,msg), 'Fail Verify Get Vector-Col:'+msg);
end;

{ Check_TIArr3D }

procedure Check_TIArr3D.setUp;
begin
  inherited;
  d1 := 3;
  d2 := 4;
  d3 := 5;
  randomize;
end;

procedure Check_TIArr3D.tearDown;
begin
  inherited;
end;

procedure Check_TIArr3D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TIArr3D.Create(l1,h1,l2,h2,l3,h3);
  RandArr3D(MM1,-9,9);
  MM2 := TIArr3D.Create(MM1,true);
  Check(SameArr(MM1,MM2,msg), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TIArr3D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TIArr3D.Create(l1,h1,l2,h2,l3,h3);
  MM2 := TIArr3D.Create(l1,h1,l2,h2,l3,h3);
  MM1.Fill(222);
  Fill(MM2,222);
  Check(SameArr(MM1,MM2,msg), 'Fail Fill:'+msg);;
end;

procedure Check_TIArr3D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  h2 := RndNum(2,7);
  h3 := RndNum(2,7);
  MM1 := TIArr3D.Create(h1,h2,h3);
  Check( (h1 = MM1.Hi1), 'Fail Hi1=Dim1');
  Check( (h2 = MM1.Hi2), 'Fail Hi2=Dim2');
  Check( (h3 = MM1.Hi3), 'Fail Hi3=Dim3');
  Check( (1=MM1.Lo1), 'Fail Lo1=1');
  Check( (1=MM1.Lo2), 'Fail Lo2=1');
  Check( (1=MM1.Lo3), 'Fail Lo3=1');
end;

procedure Check_TIArr3D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TIArr3D.Create(l1,h1,l2,h2,l3,h3);
  Check( (MM1.Hi1=MM1.Lo1+d1-1), 'Fail Hi1, Lo1');
  Check( (MM1.Hi2=MM1.Lo2+d2-1), 'Fail Hi2, Lo2');
  Check( (MM1.Hi3=MM1.Lo3+d3-1), 'Fail Hi3, Lo3');
  i1 := RndNumEx(-5,5,l1);   MM1.Lo1 := i1;
  Check( (MM1.Hi1=MM1.Lo1+d1-1), 'Fail Hi1, Lo1 #2');
  i2 := RndNumEx(-5,5,l2);   MM1.Lo2 := i2;
  Check( (MM1.Hi2=MM1.Lo2+d2-1), 'Fail Hi2, Lo2 #2');
  i3 := RndNumEx(-5,5,l3);   MM1.Lo3 := i3;
  Check( (MM1.Hi3=MM1.Lo3+d3-1), 'Fail Hi3, Lo3 #2');
end;

procedure Check_TIArr3D.VerifySetGetValue;
var
  i1, i2, i3, n: integer;
  b: boolean;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TIArr3D.Create(l1,h1,l2,h2,l3,h3);
  i1 := RndNum(l1,h1);
  i2 := RndNum(l2,h2);
  i3 := RndNum(l3,h3);
  n := RndNum(1,13);
  MM1[i1,i2,i3] := n;
  b := (MM1[i1,i2,i3]= n);
  Check( b, 'Fail GetValue');
end;

{ Check_TFArr1D }

procedure Check_TFArr1D.setUp;
begin
  d1 := 4;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_TFArr1D.tearDown;
begin
end;

procedure Check_TFArr1D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5); h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo');
  i := RndNumEx(-5,5,l1);
  V1.Lo1 := i;
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo #2');
end;

procedure Check_TFArr1D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  V1 := TFArr1D.Create(h1);
  Check( (h1 = V1.Hi1), 'Fail Hi=Dim');
  Check( (1=V1.Lo1), 'Fail Lo=1');
end;

procedure Check_TFArr1D.VerifySetGetValue;
var
  i: integer;
  b: boolean;
  n: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndFloat(-5.5,11.0);
  V1[i] :=n;
  b := abs(V1[i] - n) < eps;
  Check( b, 'Fail SetValue');
end;

procedure Check_TFArr1D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndFloat(-9.0,9.0);
  V1[i] := n;
  j := RndNumEx(l1,h1,i);
  m := RndFloat(-9.0,9.0);
  V1[j] := m;
  V1.Swap(i,j);
  Check( (V1[i]=m) and (V1[j]=n), 'Fail Swap');
end;

procedure Check_TFArr1D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  V2 := TFArr1D.Create(l1,h1);
  V1.Fill(sqrt(2.0));
  Fill(V2,sqrt(2.0));
  Check(SameArr(V1,V2,msg,eps), 'Fail Fill:'+msg);
  V1.Times(0.5);
  CxArr(0.5,V2);
  Check(SameArr(V1,V2,msg,eps), 'Fail Times:'+msg);
end;

procedure Check_TFArr1D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  RandArr1D(V1,-9.0,9.0);
  V2 := TFArr1D.Create(V1,true);
  Check(SameArr(V1,V2,msg,eps), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TFArr1D.VerifyCreate;
var
  A: TFArr;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1,A);
  RandArr1D(V1,-2.0,2.0);
  V2 := TFArr1D.Create(l1,_,A);
  SetLength(A,0);
  Check(SameArr(V1, V2, msg, 8*MachEps),'Fail VerifyCreate:'+msg);
end;

procedure Check_TFArr1D.VerifyDot;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  RandArr1D(V1,-2.0,3.0);
  V2 := TFArr1D.Create(V1,false);
  RandArr1D(V2,-2.0,3.0);
  Check(SameValue(V1.Dot(V2),VDotV(V1,V2)), 'Fail VerifyDot');
end;
{ Check_TFArr2D }

procedure Check_TFArr2D.setUp;
begin
  d1 := 4;
  d2 := 5;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_TFArr2D.tearDown;
begin
end;

procedure Check_TFArr2D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1');
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2');
  i1 := RndNumEx(-5,5,l1);   M1.Lo1 := i1;
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1 #2');
  i2 := RndNumEx(-5,5,l2);   M1.Lo2 := i2;
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2 #2');
end;

procedure Check_TFArr2D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  h2 := RndNum(2,7);
  M1 := TFArr2D.Create(h1,h2);
  Check( (h1 = M1.Hi1), 'Fail Hi1=Dim1');
  Check( (h2 = M1.Hi2), 'Fail Hi2=Dim2');
  Check( (1=M1.Lo1), 'Fail Lo1=1');
  Check( (1=M1.Lo2), 'Fail Lo2=1');
end;

procedure Check_TFArr2D.VerifySetGetValue;
var
  i1, i2: integer;
  b: boolean;
  n: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  i1 := RndNum(l1,h1);
  i2 := RndNum(l2,h2);
  n := RndFloat(-5.5,11.0);
  M1[i1,i2] := n;
  b := abs(M1[i1,i2] - n) < eps;
  Check(b, 'Fail SetValue');
end;

procedure Check_TFArr2D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9.0,9.0);

  V1 := TFArr1D.Create(l2,h2);
  V2 := TFArr1D.Create(l2,h2);
  V3 := TFArr1D.Create(l2,h2);
  V4 := TFArr1D.Create(l2,h2);
  i := RndNum(l1,h1);
  j := RndNumEx(l1,h1,i);
  V1.Assign(GetArr1D(M1,i,_Row));
  V2.Assign(GetArr1D(M1,j,_Row));
  M1.Swap(i,j,_Row);
  V3.Assign(GetArr1D(M1,i,_Row));
  V4.Assign(GetArr1D(M1,j,_Row));
  Check( SameArr(V1, V4, msg, eps), 'Fail Swap Rows, 1: '+msg);
  Check( SameArr(V2, V3, msg, eps), 'Fail Swap Rows, 2: '+msg);

  V1 := TFArr1D.Create(l1,h1);
  V2 := TFArr1D.Create(l1,h1);
  V3 := TFArr1D.Create(l1,h1);
  V4 := TFArr1D.Create(l1,h1);
  i := RndNum(l2,h2);
  j := RndNumEx(l2,h2,i);
  V1.Assign(GetArr1D(M1,i,_Col));
  V2.Assign(GetArr1D(M1,j,_Col));
  M1.Swap(i,j,_Col);
  V3.Assign(GetArr1D(M1,i,_Col));
  V4.Assign(GetArr1D(M1,j,_Col));
  Check( SameArr(V1, V4, msg, eps), 'Fail Swap Cols, 1: '+msg);
  Check( SameArr(V2, V3, msg, eps), 'Fail Swap Cols, 2: '+msg);
end;

procedure Check_TFArr2D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l1,h1,l2,h2);
  M1.Fill(sqrt(2.0));
  Fill(M2,sqrt(2.0));
  Check(SameArr(M1,M2,msg,eps), 'Fail Fill:'+msg);
  M1.Times(0.5);
  CxArr(0.5,M2);
  Check(SameArr(M1,M2,msg,eps), 'Fail Times:'+msg);
end;

procedure Check_TFArr2D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9.0,9.0);
  M2 := TFArr2D.Create(M1,true);
  Check(SameArr(M1,M2,msg,eps), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TFArr2D.VerfyGetVector;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9.0,9.0);
  i1 := RndNum(l1,h1);
//  V1 := M1.GetSlice(i1,_);
  V1 := M1.Slice[i1,_];
  V2 := GetArr1D(M1, i1, _Row);
  Check(SameArr(V1,V2,msg,eps), 'Fail Verify Get Vector-Row:'+msg);
  i2 := RndNum(l2,h2);
//  V1 := M1.GetSlice(_,i2);
  V1 := M1.Slice[_,i2];
  V2 := GetArr1D(M1, i2, _Col);
  Check(SameArr(V1,V2,msg,eps), 'Fail Verify Get Vector-Col:'+msg);
end;

procedure Check_TFArr2D.VerfySetVector;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  V1 := TFArr1D.Create(l2,h2);
  RandArr1D(V1,-9.0,9.0);
  i1 := RndNum(l1,h1);
//  M1.SetSlice(i1,_,V1);
  M1.Slice[i1,_] := V1;
  V2 := GetArr1D(M1, i1, _Row);
  Check(SameArr(V1,V2,msg,eps), 'Fail Verify Get Vector-Row:'+msg);
  V1 := TFArr1D.Create(l1,h1);
  RandArr1D(V1,-9.0,9.0);
  i2 := RndNum(l2,h2);
//  M1.SetSlice(_,i2,V1);
  M1.Slice[_,i2] := V1;
  V2 := GetArr1D(M1, i2, _Col);
  Check(SameArr(V1,V2,msg,eps), 'Fail Verify Get Vector-Col:'+msg);
end;

procedure Check_TFArr2D.VerfyNorm;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-9.0,9.0);
  i1 := RndNum(l1,h1);
  Check(abs(M1.Norm(i1,_) - NormRow(M1, i1)) < eps, 'Fail Norm Row');
  i2 := RndNum(l2,h2);
  Check(abs(M1.Norm(_,i2) - NormCol(M1, i2)) < eps, 'Fail Norm Col');
end;

{ Check_TFArr3D }

procedure Check_TFArr3D.setUp;
begin
  inherited;
  d1 := 3;
  d2 := 4;
  d3 := 5;
  eps := 1.0e-12;
  randomize;
end;

procedure Check_TFArr3D.tearDown;
begin
  inherited;
end;

procedure Check_TFArr3D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  RandArr3D(MM1,-9.0,9.0);
  MM2 := TFArr3D.Create(MM1,true);
  Check(SameArr(MM1,MM2,msg,eps), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TFArr3D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  MM2 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  MM1.Fill(sqrt(2.0));
  Fill(MM2,sqrt(2.0));
  Check(SameArr(MM1,MM2,msg,eps), 'Fail Fill:'+msg);
  MM1.Times(0.5);
  CxArr(0.5,MM2);
  Check(SameArr(MM1,MM2,msg,eps), 'Fail Times:'+msg);
end;

procedure Check_TFArr3D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  h2 := RndNum(2,7);
  h3 := RndNum(2,7);
  MM1 := TFArr3D.Create(h1,h2,h3);
  Check( (h1 = MM1.Hi1), 'Fail Hi1=Dim1');
  Check( (h2 = MM1.Hi2), 'Fail Hi2=Dim2');
  Check( (h3 = MM1.Hi3), 'Fail Hi3=Dim3');
  Check( (1=MM1.Lo1), 'Fail Lo1=1');
  Check( (1=MM1.Lo2), 'Fail Lo2=1');
  Check( (1=MM1.Lo3), 'Fail Lo3=1');
end;

procedure Check_TFArr3D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  Check( (MM1.Hi1=MM1.Lo1+d1-1), 'Fail Hi1, Lo1');
  Check( (MM1.Hi2=MM1.Lo2+d2-1), 'Fail Hi2, Lo2');
  Check( (MM1.Hi3=MM1.Lo3+d3-1), 'Fail Hi3, Lo3');
  i1 := RndNumEx(-5,5,l1);   MM1.Lo1 := i1;
  Check( (MM1.Hi1=MM1.Lo1+d1-1), 'Fail Hi1, Lo1 #2');
  i2 := RndNumEx(-5,5,l2);   MM1.Lo2 := i2;
  Check( (MM1.Hi2=MM1.Lo2+d2-1), 'Fail Hi2, Lo2 #2');
  i3 := RndNumEx(-5,5,l3);   MM1.Lo3 := i3;
  Check( (MM1.Hi3=MM1.Lo3+d3-1), 'Fail Hi3, Lo3 #2');
end;

procedure Check_TFArr3D.VerifySetGetValue;
var
  i1, i2, i3: integer;
  n: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  MM1 := TFArr3D.Create(l1,h1,l2,h2,l3,h3);
  i1 := RndNum(l1,h1);
  i2 := RndNum(l2,h2);
  i3 := RndNum(l3,h3);
  n := RndFloat(-5.5,11.0);
  MM1[i1,i2,i3] := n;
  Check( abs(MM1[i1,i2,i3] - n) < eps, 'Fail GetValue');
end;

{ Check_TCArr1D }

procedure Check_TCArr1D.setUp;
begin
  d1 := 4;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_TCArr1D.tearDown;
begin
end;

procedure Check_TCArr1D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5); h1 := l1+d1-1;
  V1 := TCArr1D.Create(l1,h1);
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo');
  i := RndNumEx(-5,5,l1);
  V1.Lo1 := i;
  Check( (V1.Hi1=V1.Lo1+d1-1), 'Fail Hi, Lo #2');
end;

procedure Check_TCArr1D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  V1 := TCArr1D.Create(h1);
  Check( (h1 = V1.Hi1), 'Fail Hi=Dim');
  Check( (1=V1.Lo1), 'Fail Lo=1');
end;

procedure Check_TCArr1D.VerifySetGetValue;
var
  i: integer;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TCArr1D.Create(l1,h1);
  V2 := TCArr1D.Create(l1,h1);
  i := RndNum(l1,h1);
  n := RndComplex(-5.5,11.0,-5.5,11.0);
  V1[i] := n;
  Check(Cabs(Csub(V1[i],n)) < eps, 'Fail SetValue');
  m := V1[i];
  Check(Cabs(Csub(m,n)) < eps, 'Fail GetValue');
  V2[i] := V1[i];
  Check(Cabs(Csub(V1[i],V2[i])) < eps, 'Fail assign');
end;

procedure Check_TCArr1D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TCArr1D.Create(l1,h1);
//  V1.Fill(Cmplx0);
  i := RndNum(l1,h1);
  n := RndComplex(-9.0,9.0,-9.0,9.0);
  V1[i] := n;
  j := RndNumEx(l1,h1,i);
  m := RndComplex(-9.0,9.0,-9.0,9.0);
  V1[j] := m;
  V1.Swap(i,j);
  Check(SameComplex(V1[i],m,eps) and SameComplex(V1[j],n,eps), 'Fail Swap');
end;

procedure Check_TCArr1D.VerifyFill;
//var s: string;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TCArr1D.Create(l1,h1);
  V2 := TCArr1D.Create(l1,h1);
  V1.Fill(cmplx(1.0,1.0));
  Fill(V2,cmplx(1.0,1.0));
  Check(SameArr(V1,V2,msg,eps), 'Fail Fill:'+msg);
  V1.Times(cmplx(1.0,1.0));
  CxArr(cmplx(1.0,1.0),V2);
  Check(SameArr(V1,V2,msg,eps), 'Fail Times:'+msg);
end;

procedure Check_TCArr1D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TCArr1D.Create(l1,h1);
  RandArr1D(V1.Re,-9.0,9.0);
  RandArr1D(V1.Im,-9.0,9.0);
  V2 := TCArr1D.Create(V1,true);
  Check(SameArr(V1,V2,msg,eps), 'Fail VerifyCopy:'+msg);
end;

procedure Check_TCArr1D.VerifyConjugate;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TCArr1D.Create(l1,h1);
  RandArr1D(V1.Re,-9.0,9.0);
  RandArr1D(V1.Im,-9.0,9.0);
  V2 := Conjugate(V1);
  V1.Conjugate;
  Check(SameArr(V1,V2,msg,eps), 'Fail VerifyConjugate:'+msg);
end;

{ Check_TCArr2D }

procedure Check_TCArr2D.setUp;
begin
  d1 := 4;
  d2 := 5;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_TCArr2D.tearDown;
begin
end;

procedure Check_TCArr2D.VerifyGetHiLo;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TCArr2D.Create(l1,h1,l2,h2);
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1');
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2');
  i1 := RndNumEx(-5,5,l1);   M1.Lo1 := i1;
  Check( (M1.Hi1=M1.Lo1+d1-1), 'Fail Hi1, Lo1 #2');
  i2 := RndNumEx(-5,5,l2);   M1.Lo2 := i2;
  Check( (M1.Hi2=M1.Lo2+d2-1), 'Fail Hi2, Lo2 #2');
end;

procedure Check_TCArr2D.VerifyGetDimLo;
begin
  h1 := RndNum(2,7);
  h2 := RndNum(2,7);
  M1 := TCArr2D.Create(h1,h2);
  Check( (h1 = M1.Hi1), 'Fail Hi1=Dim1');
  Check( (h2 = M1.Hi2), 'Fail Hi2=Dim2');
  Check( (1=M1.Lo1), 'Fail Lo1=1');
  Check( (1=M1.Lo2), 'Fail Lo2=1');
end;

procedure Check_TCArr2D.VerifySetGetValue;
var
  i1, i2: integer;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TCArr2D.Create(l1,h1,l2,h2);
  i1 := RndNum(l1,h1);
  i2 := RndNum(l2,h2);
  n := RndComplex(-5.5,11.0, -5.5,11.0);
  M1[i1,i2] := n;
  Check(SameComplex(M1[i1,i2],n, eps), 'Fail SetValue');
end;

procedure Check_TCArr2D.VerifySwap;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TCArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1.Re,-9.0,9.0);
  RandArr2D(M1.Im,-9.0,9.0);
  V1 := TCArr1D.Create(l2,h2);
  V2 := TCArr1D.Create(l2,h2);
  V3 := TCArr1D.Create(l2,h2);
  V4 := TCArr1D.Create(l2,h2);
  i := RndNum(l1,h1);
  j := RndNumEx(l1,h1,i);
  V1.Re.Assign(GetArr1D(M1.Re,i,_Row));
  V1.Im.Assign(GetArr1D(M1.Im,i,_Row));
  V2.Re.Assign(GetArr1D(M1.Re,j,_Row));
  V2.Im.Assign(GetArr1D(M1.Im,j,_Row));
  M1.Swap(i,j,_Row);
  V3.Re.Assign(GetArr1D(M1.Re,i,_Row));
  V3.Im.Assign(GetArr1D(M1.Im,i,_Row));
  V4.Re.Assign(GetArr1D(M1.Re,j,_Row));
  V4.Im.Assign(GetArr1D(M1.Im,j,_Row));
  Check( SameArr(V1, V4, msg, eps), 'Fail Swap Rows, 1: '+msg);
  Check( SameArr(V2, V3, msg, eps), 'Fail Swap Rows, 2: '+msg);

  V1 := TCArr1D.Create(l1,h1);
  V2 := TCArr1D.Create(l1,h1);
  V3 := TCArr1D.Create(l1,h1);
  V4 := TCArr1D.Create(l1,h1);
  i := RndNum(l2,h2);
  j := RndNumEx(l2,h2,i);
  V1.Re.Assign(GetArr1D(M1.Re,i,_Col));
  V1.Im.Assign(GetArr1D(M1.Im,i,_Col));
  V2.Re.Assign(GetArr1D(M1.Re,j,_Col));
  V2.Im.Assign(GetArr1D(M1.Im,j,_Col));
  M1.Swap(i,j,_Col);
  V3.Re.Assign(GetArr1D(M1.Re,i,_Col));
  V3.Im.Assign(GetArr1D(M1.Im,i,_Col));
  V4.Re.Assign(GetArr1D(M1.Re,j,_Col));
  V4.Im.Assign(GetArr1D(M1.Im,j,_Col));
  Check( SameArr(V1, V4, msg, eps), 'Fail Swap Cols, 1: '+msg);
  Check( SameArr(V2, V3, msg, eps), 'Fail Swap Cols, 2: '+msg);
end;

procedure Check_TCArr2D.VerifyFill;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TCArr2D.Create(l1,h1,l2,h2);
  M2 := TCArr2D.Create(l1,h1,l2,h2);
  M1.Fill(cmplx(1.0,1.0));
  Fill(M2,cmplx(1.0,1.0));
  Check(SameArr(M1,M2,msg,eps), 'Fail Fill:'+msg);
  M1.Times(cmplx(1.0,1.0));
  CxArr(cmplx(1.0,1.0),M2);
  Check(SameArr(M1,M2,msg,eps), 'Fail Times:'+msg);
end;

procedure Check_TCArr2D.VerifyCopy;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TCArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1.Re,-9.0,9.0);
  RandArr2D(M1.Im,-9.0,9.0);
  M2 := TCArr2D.Create(M1,true);
  Check(SameArr(M1,M2,msg,eps), 'Fail VerifyCopy:'+msg);
end;
(*
procedure Check_TCArr2D.VerfyGetVector;
begin

end;
*)
procedure Check_TCArr2D.VerifyConjugate;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TCArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1.Re,-9.0,9.0);
  RandArr2D(M1.Im,-9.0,9.0);
  M2 := Conjugate(M1);
  M1.Conjugate;
  Check(SameArr(M1,M2,msg,eps), 'Fail VerifyCopy:'+msg);
end;

initialization

  RegisterTest('Dyn FArr Tests',Suite);

end.
