{
 Tests for Typed Lists
 @author Nikolai Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 Nikolai V. Shokhirev, 2008-2008
 @version 2008.08.08 - created
 @version 2008.08.23 - added Delete
}
unit Test_uTypedLists;

interface

uses
  TestFramework
  , uMatTypes
  , uTypedLists
  , uDynArrays
  ;

type

Check_TSimpleIntList = class(TTestCase)
private
  x: TSimpleIntList;
  y: Array of TInt;
  n: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifySaveLoad;
   procedure VerifyAppend;
end;

Check_TSimpleFloatList = class(TTestCase)
private
  x: TSimpleFloatList;
  y: Array of TFloat;
  n: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifySaveLoad;
   procedure VerifyAppend;
   procedure VerifySort;
   procedure VerifyIndexOfSortedList;
end;

Check_TSimpleStrList = class(TTestCase)
private
  x: TSimpleStrList;
  y: Array of string;
  n: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifySaveLoad;
   procedure VerifyAppend;
end;

Check_TBinList = class(TTestCase)
private
  x: TBinList;
  y: Array of integer;
  n: integer;
  Vmin, Vmax: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyCount;
   procedure VerifyEmptyCount;
   procedure VerifyMinBinNumber;
   procedure VerifyMaxBinNumber;
   procedure VerifyBin;
end;

Check_THistorgam = class(TTestCase)
private
  x: THistorgam;
  y: Array of TFloat;
  n: integer;
  Vmin, Vmax, step, halfstep: TFloat;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyMinValue;
   procedure VerifyMaxValue;
   procedure VerifyBinCount;
end;

Check_IIntList = class(TTestCase)
private
  x: IIntList;
  y: Array of TInt;
  n: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifySaveLoad;
   procedure VerifyAppend;
end;

Check_IFloatList = class(TTestCase)
private
  x: IFloatList;
  y: Array of TFloat;
  n: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifySaveLoad;
   procedure VerifyAppend;
   procedure VerifySort;
   procedure VerifyIndexOfSortedList;
end;

Check_IStringList = class(TTestCase)
private
  x: IStringList;
  y: Array of string;
  n: integer;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifySaveLoad;
   procedure VerifyAppend;
end;

function Suite : ITestSuite;

implementation

uses
  Math
  , SysUtils
  , uStrFun
  ;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uSimpleIntLiss Tests');
  result.addTest(Check_TSimpleIntList.Suite);
  result.addTest(Check_TSimpleFloatList.Suite);
  result.addTest(Check_TSimpleStrList.Suite);
  result.addTest(Check_TBinList.Suite);
  result.addTest(Check_THistorgam.Suite);
  result.addTest(Check_IIntList.Suite);
  result.addTest(Check_IFloatList.Suite);
  result.addTest(Check_IStringList.Suite);
end;

{ Check_TSimpleIntList }

procedure Check_TSimpleIntList.setUp;
var
  i: integer;
  r: TInt;
begin
  inherited;
  n := 197;
  SetLength(y, n);
  x := TSimpleIntList.Create;
  Randomize;
  for i := 0 to n-1 do
  begin
    r := RandomRange(-n, n);
    x.Append(r);
    y[i] := r;
  end;
end;

procedure Check_TSimpleIntList.tearDown;
begin
  inherited;
  SetLength(y, 0);
  x.Free;
end;

procedure Check_TSimpleIntList.VerifyAppend;
var
  i: integer;
begin
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

procedure Check_TSimpleIntList.VerifySaveLoad;
var
  i: integer;
begin
  x.SaveToFile('tmp.txt');
  x.Clear;
  CheckEquals(0, x.Count, 'Fail Clear');
  x.LoadFromFile('tmp.txt');
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

{ Check_TSimpleFloatList }

procedure Check_TSimpleFloatList.setUp;
var
  i: integer;
  r: TFloat;
begin
  inherited;
  n := 197;
  SetLength(y, n);
  x := TSimpleFloatList.Create;
  Randomize;
  for i := 0 to n-1 do
  begin
    r := RandG(0.0, 9.0);
    x.Append(r);
    y[i] := r;
  end;
end;

procedure Check_TSimpleFloatList.tearDown;
begin
  inherited;
  SetLength(y, 0);
  x.Free;
end;

procedure Check_TSimpleFloatList.VerifyAppend;
var
  i: integer;
begin
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 1.0e-9, 'i='+IntToStr(i));
end;

procedure Check_TSimpleFloatList.VerifyIndexOfSortedList;
var
  i, j: integer;
  v: TFloat;
begin
  x.Sort;

  Check(IndexOfSortedList(x[0], x) = 0, 'value = MinValue');

  Check(IndexOfSortedList(x[n-1], x) = n-1, 'value = MaxValue');

  for i := 1 to n*2 do
  begin
    v := RandG(0.0, 9.9);
    j := IndexOfSortedList(v, x);
    if v < x[0] then
      Check(j = -1, 'value < MinValue')
    else if v > x[n-1] then
      Check(j = n-1, 'value > MaxValue')
    else
    if x[j] < x[j+1] then
       Check((x[j] <= v) and (v < x[j+1]), 'i='+IntToStr(i))
    else
       Check((x[j] <= v) and (v <= x[j+1]), 'i='+IntToStr(i))
  end;

  // Returnrs idx so that
  // List[idx] <= value < List[idx+1]
  // if value < MinValue then idx = -1;
  // if value > MaxValue then idx = Count
//  function IndexOfSortedList(value: TFloat; List: TSimpleFloatList): integer;
end;

procedure Check_TSimpleFloatList.VerifySaveLoad;
var
  i: integer;
begin
  x.SaveToFile('tmp.txt');
  x.Clear;
  CheckEquals(0, x.Count, 'Fail Clear');
  x.LoadFromFile('tmp.txt');
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 1.0e-9, 'i='+IntToStr(i));
end;

procedure Check_TSimpleFloatList.VerifySort;
var
  i: integer;
//  b: boolean;
begin
  x.Sort;
  for i := 1 to n-1 do
    Check(x[i-1] <= x[i], 'i='+IntToStr(i));
end;

{ Check_TSimpleStrList }

procedure Check_TSimpleStrList.setUp;
var
  i, m: integer;
  r: string;
begin
  inherited;
  n := 197;
  SetLength(y, n);
  x := TSimpleStrList.Create;
  Randomize;
  for i := 0 to n-1 do
  begin
    m := RandomRange(1, 19);
    r := RandStr(m);
    x.Append(r);
    y[i] := r;
  end;
end;

procedure Check_TSimpleStrList.tearDown;
begin
  inherited;
  SetLength(y, 0);
  x.Free;
end;

procedure Check_TSimpleStrList.VerifyAppend;
var
  i: integer;
begin
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

procedure Check_TSimpleStrList.VerifySaveLoad;
var
  i: integer;
begin
  x.SaveToFile('tmp.txt');
  x.Clear;
  CheckEquals(0, x.Count, 'Fail Clear');
  x.LoadFromFile('tmp.txt');
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

{ Check_TBinList }

procedure Check_TBinList.setUp;
var
  i: integer;
begin
  inherited;
  Randomize;
  x := TBinList.Create;
  n := 6;
  SetLength(y, 2*n+1);
  Vmin := n;
  Vmax:= -n;
  for i := 0 to 2*n do
  begin
    y[i] := RndNum(-n, n);
    Vmin := min(Vmin, y[i]);
    Vmax := max(Vmax, y[i]);
    x.IncBinCount(Y[i]);
  end;
end;

procedure Check_TBinList.tearDown;
begin
  inherited;
  x.Free;
  y := nil;
end;

procedure Check_TBinList.VerifyBin;
var
  i, j, k: integer;
begin
  for j := Vmin to Vmax do
  begin
    k := 0;
    for i := 0 to 2*n do
      if y[i] = j then
        inc(k);
    if k <> 0 then
      CheckEquals(k, x[x.IndexOfBinNumber(j)].BinCount, 'j='+IntToStr(j));
  end;
end;

procedure Check_TBinList.VerifyCount;
var
  i, j, c, k: integer;
begin
  c := 0;
  for j := Vmin to Vmax do
  begin
    k := 0;
    for i := 0 to 2*n do
      if y[i] = j then
      begin
        k := 1;
        break;
      end;
    c := c + k;
  end;
  CheckEquals(c, x.Count, ' ');
end;

procedure Check_TBinList.VerifyEmptyCount;
var
  i, j, c, k: integer;
begin
  c := 0;
  for j := Vmin to Vmax do
  begin
    k := 0;
    for i := 0 to 2*n do
      if y[i] = j then
      begin
        k := 1;
        break;
      end;
    if k = 0 then inc(c)
  end;
  CheckEquals(c, x.EmptyCount, ' ');
end;

procedure Check_TBinList.VerifyMaxBinNumber;
begin
  CheckEquals(Vmax, x.MaxBinNumber, ' ');
end;

procedure Check_TBinList.VerifyMinBinNumber;
begin
  CheckEquals(Vmin, x.MinBinNumber, ' ');
end;

{ Check_THistorgam }

procedure Check_THistorgam.setUp;
var
  i: integer;
begin
  inherited;
  Randomize;
  step := 0.1;
  halfstep := step/2.0;
  x := THistorgam.Create(0.0, step);
  n := 50;
  SetLength(y, 2*n+1);
  Vmin := 0.0;
  Vmax := 0.0;
  for i := 0 to 2*n do
  begin
    y[i] := RndFloat(-0.95, 0.95);
    Vmin := min(Vmin, y[i]);
    Vmax := max(Vmax, y[i]);
    x.AddValue(Y[i]);
  end;
end;

procedure Check_THistorgam.tearDown;
begin
  inherited;
  x.Free;
  y := nil;
end;

procedure Check_THistorgam.VerifyBinCount;
var
  i, j, k: integer;
  v: TFloat;
begin
  for j := -9 to 9 do
  begin
    v := j;
    v := 0.1*v;
    k := 0;
    for i := 0 to 2*n do
      if (y[i] >= (v-halfstep)) and (y[i] < (v+halfstep)) then
        inc(k);
    CheckEquals(k, x.BinCount[j],  'j='+IntToStr(j));
  end;
end;

procedure Check_THistorgam.VerifyMaxValue;
begin
  CheckEquals(Vmax, x.MaxValue, 0.05);
end;

procedure Check_THistorgam.VerifyMinValue;
begin
  CheckEquals(Vmin, x.MinValue, 0.05);
end;

{ Check_IIntList }

procedure Check_IIntList.setUp;
var
  i: integer;
  r: TInt;
begin
  inherited;
  n := 197;
  SetLength(y, n);
  x := TSimpleIntList.Create;
  Randomize;
  for i := 0 to n-1 do
  begin
    r := RandomRange(-n, n);
    x.Append(r);
    y[i] := r;
  end;
end;

procedure Check_IIntList.tearDown;
begin
  inherited;
  SetLength(y, 0);
end;

procedure Check_IIntList.VerifyAppend;
var
  i: integer;
begin
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

procedure Check_IIntList.VerifySaveLoad;
var
  i: integer;
begin
  x.SaveToFile('tmp.txt');
  x.Clear;
  CheckEquals(0, x.Count, 'Fail Clear');
  x.LoadFromFile('tmp.txt');
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

{ Check_IFloatList }

procedure Check_IFloatList.setUp;
var
  i: integer;
  r: TFloat;
begin
  inherited;
  n := 197;
  SetLength(y, n);
  x := TSimpleFloatList.Create;
  Randomize;
  for i := 0 to n-1 do
  begin
    r := RandG(0.0, 9.0);
    x.Append(r);
    y[i] := r;
  end;
end;

procedure Check_IFloatList.tearDown;
begin
  inherited;
  SetLength(y, 0);
end;

procedure Check_IFloatList.VerifyAppend;
var
  i: integer;
begin
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 1.0e-9, 'i='+IntToStr(i));
end;

procedure Check_IFloatList.VerifyIndexOfSortedList;
var
  i, j: integer;
  v: TFloat;
begin
  x.Sort;

  Check(IndexOfSortedList(x[0], x) = 0, 'value = MinValue');

  Check(IndexOfSortedList(x[n-1], x) = n-1, 'value = MaxValue');

  for i := 1 to n*2 do
  begin
    v := RandG(0.0, 9.9);
    j := IndexOfSortedList(v, x);
    if v < x[0] then
      Check(j = -1, 'value < MinValue')
    else if v > x[n-1] then
      Check(j = n-1, 'value > MaxValue')
    else
    if x[j] < x[j+1] then
       Check((x[j] <= v) and (v < x[j+1]), 'i='+IntToStr(i))
    else
       Check((x[j] <= v) and (v <= x[j+1]), 'i='+IntToStr(i))
  end;

  // Returnrs idx so that
  // List[idx] <= value < List[idx+1]
  // if value < MinValue then idx = -1;
  // if value > MaxValue then idx = Count
//  function IndexOfSortedList(value: TFloat; List: TSimpleFloatList): integer;
end;

procedure Check_IFloatList.VerifySaveLoad;
var
  i: integer;
begin
  x.SaveToFile('tmp.txt');
  x.Clear;
  CheckEquals(0, x.Count, 'Fail Clear');
  x.LoadFromFile('tmp.txt');
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 1.0e-9, 'i='+IntToStr(i));
end;

procedure Check_IFloatList.VerifySort;
var
  i: integer;
//  b: boolean;
begin
  x.Sort;
  for i := 1 to n-1 do
    Check(x[i-1] <= x[i], 'i='+IntToStr(i));
end;

{ Check_IStringList }

procedure Check_IStringList.setUp;
var
  i, m: integer;
  r: string;
begin
  inherited;
  n := 197;
  SetLength(y, n);
  x := TSimpleStrList.Create;
  Randomize;
  for i := 0 to n-1 do
  begin
    m := RandomRange(1, 19);
    r := RandStr(m);
    x.Append(r);
    y[i] := r;
  end;
end;

procedure Check_IStringList.tearDown;
begin
  inherited;
  SetLength(y, 0);
end;

procedure Check_IStringList.VerifyAppend;
var
  i: integer;
begin
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

procedure Check_IStringList.VerifySaveLoad;
var
  i: integer;
begin
  x.SaveToFile('tmp.txt');
  x.Clear;
  CheckEquals(0, x.Count, 'Fail Clear');
  x.LoadFromFile('tmp.txt');
  CheckEquals(n, x.Count, 'Fail Count');
  for i := 0 to n-1 do
    CheckEquals(y[i], x[i], 'i='+IntToStr(i));
end;

initialization

  RegisterTest('Typed Lists Tests', Suite);

end.
