{@abstract(Linear algebra tests )
@author(Nikolai Shokhirev <nikolai@shokhirev.com>)
@created(2003.02.02)
@lastmod(2004.11.11)
@lastmod(2010.01.01)
Nikolai V. Shokhirev, 2002-2010 }
unit Test_uDynLinAlg;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays;

type

Check_Solve = class(TTestCase)
private
  V0, V1, V2, V3, V4, V5: IFArr1D;
  M1, M2, M3: IFArr2D;
  x, eps: TFloat;
  dd, err, d, n: integer;
  d1, l1, h1, d2, l2, h2: integer;
  s, msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyDecompSolve2;
  procedure VerifyDecompSolve1;
  procedure VerifyFastInverse;
  procedure VerifyChSolve;
  procedure VerifyQRSolve;
  procedure VerifyGaussJordan;
  procedure VerifyGJSolve;
  procedure VerifyGJInverse;
  procedure VerifyFastSolve;
  procedure VerifyFastSolve1;
  procedure VerifyQRhouseholder;
  procedure VerifyBiDiagSolve;
  procedure VerifyTriDiagMxV;
  procedure VerifyTriDiagSolve;
  procedure VerifyTriDiagSolve1;
  procedure VerifyMaskSolve;
end;

Check_Diag = class(TTestCase)
private
  V1, V2: IFArr1D;
  M1, M2, M3: IFArr2D;
  eps: TFloat;
  x : TFloat;
  err, dm: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyRestore;
  procedure VerifySortVal;
  procedure VerifyDiag;
  procedure VerifyJacobi;
  procedure VerifyCJacobi;
  procedure VerifyJacobiC;
  procedure VerifyHTDiag;
  procedure VerifyDiagC;
  procedure VerifyDiagC2;
end;

Check_SVD = class(TTestCase)
private
  W, T: IFArr1D;
  A, U, V, A1: IFArr2D;
  err, N1, N2: integer;
  del: TFloat;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifySVD_1;
  procedure VerifySVD_2;
  procedure VerifySVD1;
  procedure VerifySVD2;
  procedure VerifyGramSchmidt;
  procedure VerifyPseudoInverse;
  procedure VerifyHouseholder;
  procedure VerifyHouseholderFull;
  procedure VerifyXSVD;
  procedure VerifyXSVDFull;
  procedure VerifyXSVD3x2;
end;


function Suite : ITestSuite;

implementation

uses
  math,
  uDynArrUtils,
  uDynArrUtilsX,
  uDynLinAlg,
  uDynArrIO,
  uDynObjAlg;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uLinAlg Tests');

  result.addTest(Check_Solve.Suite);
  result.addTest(Check_Diag.Suite);
  result.addTest(Check_SVD.Suite);
end;

{ Check_Solve }

procedure Check_Solve.setUp;
begin
  d := 4;
  d1 := 4;
  d2 := 5;
  dd := 6;
  eps := 1.0e-12;
  Randomize;
  x := d;
  x := x*x*x;
  n := 8;
end;

procedure Check_Solve.tearDown;
begin
end;

{ check: FastInverse, MtxMt }
procedure Check_Solve.VerifyFastInverse;
var
  i :  TInt;
  x, Det : TFloat;
begin
  M1 := TFArr2D.Create(d,d);
  M2 := TFArr2D.Create(d,d);
  RandArr2D(M1,-1.0,1.0,0);
  x := d + 1.0;
  for i := M1.Lo1 to M1.Hi1 do
    M1[i,i] := M1[i,i] + x; // make diagonal dominant
  M2.Assign(M1);
  FastInverse(M1, Det);
  M3 := MtxMt(M2, M1);
  AssignScalar(M2,1.0);
  Check(SameArr(M3, M2, msg, x*MachEps), 'Fail VerifyFastInverse:'+ msg);
end;

{ check: Cholesky, MTtxMt, MtxMtT, MtxVt, LSolve, LTSolve,ChSolve }
procedure Check_Solve.VerifyChSolve;
var
  x : TFloat;
  S: IFArr1D;
begin
  { Decomposition check }
  M1 := TFArr2D.Create(d,d);
  x := d; x := x*x*x*8.0;
  RandArr2D(M1,-1.0,1.0,0);
  M2 := MTtxMt(M1, M1);  // positive-definite matrix
  V1 := TFArr1D.Create(d);
  MShiftDiag(M2, 1.0, M2); // increase cond
  M1.Assign(M2);
  Cholesky(M2);
  M3 := MtxMtT(M2, M2);
  { Cholesky decomposition check }
  Check(SameArr(M1, M3, msg, x*MachEps),'Fail Cholesky:'+msg);
  RandArr1D(V1,-1.0,1.0);
  S := TFArr1D.Create(d);
  { ChSolve check }
  V2 := MtxVt(M1,V1);  // r.h.s.
  ChSolve (M2, V2, S);
  Check((SameArr(V1, S, msg, x*MachEps)),'Fail ChSolve:'+msg);
end;

procedure Check_Solve.VerifyQRSolve;
var
  x : TFloat;
  B, S: IFArr1D;
begin
  V1 := TFArr1D.Create(d);
  V2 := TFArr1D.Create(d);
  M1 := TFArr2D.Create(d,d);
  x := d;
  x := x*x*x;
  RandArr2D(M1, 0.0,2.0);
  B := TFArr1D.Create(d);
  RandArr1D(B,-1.0,1.0);
  S := MtxVt(M1,B);
{ QR Decomposition of  M1 }
  QRDecomp(M1, V1, V2);
  QRSolve(M1, V1, V2, S);
  Check(SameArr(B, S, msg, 2.0*x*MachEps), 'Fail QRSolve: '+msg);
end;

procedure Check_Solve.VerifyGaussJordan;
var
  i2: TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+d-1;
  l2 := RndNum(-5,5);  h2 := l2+d-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l1,h1,l2,h2+1);
  V1 := TFArr1D.Create(l2,h2);
  RandArr2D(M1,-1.0,1.0,0);

  MShiftDiag(M1, 1.0, M1, false);
  RandArr1D(V1,-1.0,1.0);
  V2 := MtxVt(M1,V1);
  for i2 := l2 to h2 do
    CopyCC(M2,i2,M1,i2);
  M2.SetSlice(_,h2+1,V2); // CopyArr1D(M2, V2, d+1, _Col);
  GaussJordan(M2, true, err);
  Check((err=0),'Fail GaussJordan:  err = '+IntToStr(err));
  V2 := M2.GetSlice(_,h2+1); // V2.Assign(GetArr1D(M2,d+1,_Col));
  if err = 0 then
  begin
    V2.Lo1 := V1.Lo1; // Make the same Limits
    Check(SameArr(V2, V1, msg, x*MachEps),'Fail GaussJordan:'+msg);
  end;
end;

procedure Check_Solve.VerifyGJInverse;
var
  i2: TInt;
  x : TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d-1;
  M1 := TFArr2D.Create(l1,h1,l1,h1);
  M2 := TFArr2D.Create(l1,h1,l1,h1);
  M3 := TFArr2D.Create(l1,h1,l1,h1+d);

  x := d;
  x := x*x*x;
  RandArr2D(M1,-1.0,1.0,0);
  MShiftDiag(M1, 2.0, M1, false);
  AssignScalar(M2,1.0);

  for i2 := l1 to h1 do
  begin
    CopyCC(M3,i2,  M1,i2);
    CopyCC(M3,i2+d,M2,i2);
  end;

  GaussJordan(M3, false, err);
//  MatrixToStr(M3,s);
  Check((err=0),'Fail Gauss-Jordan Inverse:  err = '+s+IntToStr(err));

  if err = 0 then
  begin
    for i2 := l1 to h1 do
      CopyCC(M2,i2,M3,i2+d);
    M3 := MtxMt(M1,M2);
    Check((err=0),'Fail Gauss-Jordan MtxMt:  err = '+IntToStr(err));
    AssignScalar(M1,1.0);
    Check((err=0),'Fail Gauss-Jordan MaxDifMt:  err = '+IntToStr(err));
    Check(SameArr(M1, M3, msg, x*MachEps),'Fail Gauss-Jordan Inverse:'+msg);
  end;
end;

procedure Check_Solve.VerifyGJSolve;
var
  x : TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d-1;
  l2 := RndNum(-5,5);  h2 := l2+d-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l1,h1,l2,h2+1);
  V1 := TFArr1D.Create(l2,h2);
  x := d;
  x := x*x*x;
  RandArr2D(M1,-1.0,1.0,0);
  MShiftDiag(M1, 1.0, M1, false);
  RandArr1D(V1,-1.0,1.0);
  V2 := MtxVt(M1,V1);
  GJSolve(M1, true, V2, V2, err);
  Check((err=0),'Fail GJSolve:  err = '+IntToStr(err));

  if err = 0 then
  begin
    Check(SameArr(V2, V1, msg, x*MachEps) and (err=0),'Fail GJSolve:'+msg);
  end;
end;

procedure Check_Solve.VerifyFastSolve1;
var
  x : TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d-1;
  l2 := RndNum(-5,5);  h2 := l2+d-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l1,h1,l2,h2+1);
  V1 := TFArr1D.Create(l2,h2);

  x := d;  x := x*x*x;
  RandArr2D(M1,-1.0,1.0,0);
  MShiftDiag(M1, 3.0, M1, false);
  RandArr1D(V1,-1.0,1.0);
  V2 := MtxVt(M1,V1);

  FastSolve(M1, V2, V2);
  Check(SameArr(V2, V1, msg, x*MachEps),'Fail FastSolve1:'+msg);
end;

procedure Check_Solve.VerifyFastSolve;
var
  x : TFloat;
begin
  x := d;
  x := x*x*x;
  M1 := TFArr2D.Create(d,d);
  M1.Lo1 := RndNum(-4,3);
  M1.Lo2 := RndNum(-3,4);
  V1 := TFArr1D.Create(d);
  V1.Lo1 := M1.Lo2;
  RandArr2D(M1,-1.0,1.0,0);
  MShiftDiag(M1, 3.0, M1, false);
  RandArr1D(V1,-1.0,1.0);

  V2 := MtxVt(M1,V1);
  FastSolve(M1, V2, V2);
  Check(SameArr(V2, V1, msg, x*MachEps),'Fail FastSolve:'+msg);
end;

procedure Check_Solve.VerifyDecompSolve2;
var
  x : TFloat;
  cond: TFloat;
  ipvt: IIArr1D;
begin
  d := 6;
  x := d; x := x*x*x;
  M1 := TFArr2D.Create(d,d);
  ipvt := TIArr1D.Create(d);
  RandArr2D(M1,-1.0,1.0,0);
  MtShiftDiag(M1, 3.0);
  V1 := TFArr1D.Create(d);
  RandArr1D(V1,-1.0,1.0);
  V2 := MtxVt(M1,V1);
  FMMDecomp(M1, cond, ipvt);
  FMMSolve(M1, V2, ipvt);
    Check(SameArr(V2, V1, msg, x*MachEps),'Fail VerifyDecompSolve:'+msg);
end;

procedure Check_Solve.VerifyDecompSolve1;
var
  x : TFloat;
  cond: TFloat;
begin
  d := 6;
  x := d; x := x*x*x;
  M1 := TFArr2D.Create(d,d);
  RandArr2D(M1,-1.0,1.0,0);
  MtShiftDiag(M1, 3.0);
  M1.Lo1 := RndNum(-4,3);
  M1.Lo2 := RndNum(-3,4);
  V1 := TFArr1D.Create(d);
  V1.Lo1 := M1.Lo2;
  RandArr1D(V1,-1.0,1.0);
  V2 := MtxVt(M1,V1);
  DecompSolve(M1, V2, cond);
  Check(cond<1.0e32,'Fail VerifyDecompSolve: cond='+FloatToStr(cond));
  Check(SameArr(V2, V1, msg, x*MachEps),'Fail VerifyDecompSolve:'+msg);
end;

procedure Check_Solve.VerifyQRhouseholder;
var
  x : TFloat;
begin
  x := d; x := x*x*x;
  M1 := TFArr2D.Create(dd,d);  // a
  M2 := TFArr2D.Create(dd,d);  // u
  M3 := TFArr2D.Create(d,d);   // v
  V1 := TFArr1D.Create(d);     // q
  V2 := TFArr1D.Create(d);     // e
  V0 := TFArr1D.Create(d);     // x
  V5 := TFArr1D.Create(d);     // y
  RandArr2D(M1, 0.0,2.0);
  // a = u*w*vT, w = BiDiag(q,e)
  householder(M1,V1,V2,M2,M3,MachEps, MinFloat);
  // a*x = b,  u*w*vT*x = b,   w*vT*x = uT*b
  // w*y = uT*b,  vT*x = y
  RandArr1D(V0, -2.0,2.0);       // x
  V3 := MtxVt(M1,V0);            // b = a*x
  V3 := MtxVt(Transpose(M2),V3); // b := uT*b
  BiDiagSLE(V1, V2, V3, V5);     // y = w**(-1)*uT*b
  V4 := MtxVt(M3,V5);            // x := v*y
  Check(SameArr(V0, V4, msg, x*MachEps),'Fail QRhouseholder: '+msg);
end;

procedure Check_Solve.VerifyBiDiagSolve;
var
  x : TFloat;
begin
  V1 := TFArr1D.Create(n); // d
  V2 := TFArr1D.Create(n); // e
  V0 := TFArr1D.Create(n); // x
  V4 := TFArr1D.Create(n);
  M1 := TFArr2D.Create(n,n);
  RandArr1D(V1, 1.0,2.0);
  RandArr1D(V2, 0.0,1.0);
  BiDiagMt(V1,V2,M1);
  RandArr1D(V0, -1.0,1.0);
  V3 := MtxVt(M1,V0);
  BiDiagSLE(V1, V2, V3, V4);
  x := n*n;
  Check(SameArr(V0, V4, msg, x*MachEps),'Fail BiDiagSolve: '+msg);
end;

procedure Check_Solve.VerifyTriDiagMxV;
var
  x : TFloat;
begin
  V0 := TFArr1D.Create(n); // x
  V1 := TFArr1D.Create(n); // a
  V2 := TFArr1D.Create(n); // b
  V5 := TFArr1D.Create(n); // c
  V4 := TFArr1D.Create(n);
  M1 := TFArr2D.Create(n,n);
  RandArr1D(V1, 1.0,3.0);
  RandArr1D(V2, -1.0,0.0);
  RandArr1D(V5, -1.0,0.0);
  TriDiagMt(V1,V2,V5, M1);
  RandArr1D(V0, -1.0,1.0);
  V3 := MtxVt(M1,V0);
  V4 := TriDiagMxV(V1,V2,V5,V0);
  x := n*n;
  Check(SameArr(V3, V4, msg, x*MachEps),'Fail TriDiagMxV: '+msg);
end;

procedure Check_Solve.VerifyTriDiagSolve;
var
  x : TFloat;
begin
  V0 := TFArr1D.Create(n); // x
  V1 := TFArr1D.Create(n); // a
  V2 := TFArr1D.Create(n); // b
  V5 := TFArr1D.Create(n); // c
  V4 := TFArr1D.Create(n);
  M1 := TFArr2D.Create(n,n);
  RandArr1D(V1, 1.0,3.0);
  RandArr1D(V2, -1.0,0.0);
  RandArr1D(V5, -1.0,0.0);
  TriDiagMt(V1,V2,V5, M1);
  RandArr1D(V0, -1.0,1.0);
  V3 := MtxVt(M1,V0);
  TriDiagSLE(V1,V2,V5, V3, V4);
  x := n*n;
  Check(SameArr(V0, V4, msg, x*MachEps),'Fail TriDiagSolve: '+msg);
end;

procedure Check_Solve.VerifyTriDiagSolve1;
var
  x: TFloat;
  n: TInt;
begin
  n := 0;
  V0 := TFArr1D.Create(n, d+n-1); // x
  V1 := TFArr1D.Create(n, d+n-1); // a
  V2 := TFArr1D.Create(n, d+n-1); // b
  V5 := TFArr1D.Create(n, d+n-1); // c
  V4 := TFArr1D.Create(n, d+n-1);
  M1 := TFArr2D.Create(n, d+n-1,n, d+n-1);
  RandArr1D(V1, 1.0,3.0);
  RandArr1D(V2, -1.0,0.0);
  RandArr1D(V5, -1.0,0.0);
  TriDiagMt(V1,V2,V5, M1);
  RandArr1D(V0, -1.0,1.0);
  V3 := MtxVt(M1,V0);
  TriDiagSLE(V1,V2,V5, V3, V4);
  x := d*d;
  Check(SameArr(V0, V4, msg, x*MachEps),'Fail TriDiagSolve1: '+msg);
end;

procedure Check_Solve.VerifyMaskSolve;
var
  Use: IBArr1D;
begin
  Use := TBArr1D.Create(dd);
  Use.Fill(true);
  V0 := TFArr1D.Create(dd); // x
  RandArr1D(V0, -1.0,1.0);
  V2 := TFArr1D.Create(dd); // b
  V4 := TFArr1D.Create(V0);  // x
  V5 := TFArr1D.Create(V0,true);  // x - copy for the next test
  M1 := TFArr2D.Create(dd,dd);
  RandArr2D(M1, -1.0,1.0);
  MShiftDiag(M1,2.0,M1);
  V2 := MtxVt(M1,V0);
  V3 := TFArr1D.Create(V2,true); // copy for the next test
  // use all test
  MaskSolve(M1, V4, V2, Use);
  Check(SameArr(V0, V4, msg, x*MachEps),'Fail MaskSolve: '+msg);
  // use not all test
  Use[1] := false;
  Use[3] := false;
  Use[dd] := false;
  MaskSolve(M1, V5, V3, Use);
  Check(SameArr(V0, V5, msg, x*MachEps),'Fail MaskSolve: '+msg);
end;

{ Check_Diag }

procedure Check_Diag.setUp;
begin
  inherited;
  dm := 6;
  x := dm;
  x := x*x*x*4.0;
  eps := 4.0e-8;
  Randomize;
end;

procedure Check_Diag.tearDown;
begin
  inherited;

end;

procedure Check_Diag.VerifySortVal;
var
  l1, h1 : TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+dm-1;
  M1 := TFArr2D.Create(l1,h1,l1,h1);
  M2 := TFArr2D.Create(M1);
  RandArr2D(M1,-2.0,2.0);
  M2.Assign(M1);
  V1 := TFArr1D.Create(l1,h1);
  RandArr1D(V1,-2.0,2.0);
  V2 := TFArr1D.Create(V1);
  V2.Assign(V1);
  // SortVal - vectors = False
  SortVal(V1, M1, false, true);
  SortVal0(V2, M2, false, true);
  Check(SameArr(V1,V2,msg,x*MachEps),'Fail SortVal - vectors = F; ascending = T :'+msg);
  SortVal(V1, M1, false, false);
  SortVal0(V2, M2, false, false);
  Check(SameArr(V1,V2,msg,x*MachEps),'Fail SortVal - vectors = F; ascending = F:'+msg);
  // SortVal - vectors = True
  RandArr1D(V1,-2.0,2.0);
  V2.Assign(V1);
  SortVal(V1, M1, false, true);
  SortVal0(V2, M2, false, true);
  Check(SameArr(V1,V2,msg,x*MachEps),'Fail SortVal_V - vectors = T; ascending = T:'+msg);
  Check(SameArr(M1,M2,msg,x*MachEps),'Fail SortVal_M - vectors = T; ascending = T:'+msg);
  SortVal(V1, M1, false, false);
  SortVal0(V2, M2, false, false);
  Check(SameArr(V1,V2,msg,x*MachEps),'Fail SortVal_V - vectors = T; ascending = F:'+msg);
  Check(SameArr(M1,M2,msg,x*MachEps),'Fail SortVal_M - vectors = T; ascending = F:'+msg);
end;

procedure Check_Diag.VerifyDiag;
var
  l1, h1 : TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+dm-1;
  M1 := TFArr2D.Create(l1,h1,l1,h1);
  M2 := TFArr2D.Create(M1);
  RandArr2D(M1,-2.0,2.0,1);
  M2.Assign(M1);
  V1 := TFArr1D.Create(l1,h1); // Base is assigned in Diag
  diag(M1, V1);
  Check((err=0),'Fail Diag:  err = '+IntToStr(err));
  V1.Lo1 := M1.Lo2;
  M3 := MtxDxMtT(M1, V1);
  Check((err=0),'Fail Diag MtxDxMtT:  err = '+IntToStr(err));
  if err = 0 then
  begin
    Check(SameArr(M3, M2, msg, x*MachEps) and (err=0),'Fail Diag:'+msg+'  err = '+IntToStr(err));
  end;
end;

procedure Check_Diag.VerifyJacobi;
//var s: string;
begin
  M1 := TFArr2D.Create(dm,dm);
  M1.Lo1 := RndNum(-4,4);
  M1.Lo2 := M1.Lo1;
  M2 := TFArr2D.Create(M1);
  M3 := TFArr2D.Create(M1);
  RandArr2D(M1,-2.0,2.0,1);
  M3.Assign(M1);
  V1 := TFArr1D.Create(dm); // Base is assigned in Diag
  V1.Lo1 := M1.Lo1;
  jacobi(M1,M2,V1, err);
//  MatrixToStr(M1,s);
  Check((err=0),'Fail Jacobi:  err = '+IntToStr(err));
  M1 := MtxDxMtT(M2, V1);
  Check((err=0),'Fail Jacobi MtxDxMtT:  err = '+IntToStr(err));
  if err = 0 then
  begin
    Check(SameArr(M3, M1, msg, eps) and (err=0),'Fail Jacobi:'+msg+'  err = '+IntToStr(err));
  end;
end;

procedure Check_Diag.VerifyJacobiC;
var
  A, B, U, V: IFArr2D;
begin
  A := TFArr2D.Create(dm,dm);
  M1 := TFArr2D.Create(A);
  B := TFArr2D.Create(A);
  M2 := TFArr2D.Create(A);
  U := TFArr2D.Create(A);
  V := TFArr2D.Create(A);
  RandArr2D(A,-2.0,2.0,1);
  M1.Assign(A);
  RandArr2D(B,-2.0,2.0,-1);
  M2.Assign(B);
  V1 := TFArr1D.Create(dm);
  JacobiC(A, B, U, V, V1, err);
//  VectorToStr(V1,s);
  Check((err=0),'Fail JacobiC:  err = '+IntToStr(err));
  CMxDxCMH(U, V, V1, A, B);
    Check(SameArr(A, M1, msg, eps) and (err=0),'Fail JacobiC delRe:'+msg);
    Check(SameArr(B, M2, msg, eps) and (err=0),'Fail JacobiC delIm:'+msg);
end;

procedure Check_Diag.VerifyCJacobi;
var
  A, B, U, V: IFArr2D;
begin
  A := TFArr2D.Create(dm,dm);
  M1 := TFArr2D.Create(A);
  B := TFArr2D.Create(A);
  M2 := TFArr2D.Create(A);
  U := TFArr2D.Create(A);
  V := TFArr2D.Create(A);
  RandArr2D(A,-2.0,2.0,1);
  MShiftDiag(A,2.0,A);
  M1.Assign(A);
  RandArr2D(B,-2.0,2.0,-1);
  M2.Assign(B);
  V1 := TFArr1D.Create(dm);
  CJacobi(A, B, U, V, V1,err);
  Check((err=0),'Fail CJacobi:  err = '+IntToStr(err));
  CMxDxCMH(U, V, V1, A, B);
  Check(SameArr(A, M1, msg, eps) and (err=0),'Fail CJacobi Re:'+msg);
  Check(SameArr(B, M2, msg, eps) and (err=0),'Fail CJacobi Im:'+msg);
end;

procedure Check_Diag.VerifyHTDiag;
var
  ar, ai, tau, zr, zi: IFArr2D;
  d, e, e2: IFArr1D;
  j: TInt;
begin
  tau := TFArr2D.Create(2,dm);
  ar := TFArr2D.Create(dm,dm);
  M1 := TFArr2D.Create(ar);
  ai := TFArr2D.Create(ar);
  M2 := TFArr2D.Create(ar);
  zr := TFArr2D.Create(ar);
  for j := 1 to  dm do
    zr[j,j] := 1.0;
  zi := TFArr2D.Create(ar);
  RandArr2D(ar,-2.0,2.0,1);
  M1.Assign(ar);
  RandArr2D(ai,-2.0,2.0,-1);
  M2.Assign(ai);
  d := TFArr1D.Create(dm);
  e := TFArr1D.Create(d);
  e2 := TFArr1D.Create(d);
  htridi(ar, ai, tau, d, e, e2);
  tqli(d, e, zr, err);
  htribk(ar, ai, tau, dm, zr,zi);
  Check((err=0),'Fail HTDiag:  err = '+IntToStr(err));
  CMxDxCMH(zr, zi, d, ar, ai);
  Check(SameArr(ar, M1, msg, x*MachEps) and (err=0),'Fail HTDiag Re:'+msg);
  Check(SameArr(ai, M2, msg, x*MachEps) and (err=0),'Fail HTDiag Im:'+msg);
end;

procedure Check_Diag.VerifyDiagC;
var
  ar, ai, zr, zi: IFArr2D;
  d: IFArr1D;
begin
  ar := TFArr2D.Create(dm,dm);
  M1 := TFArr2D.Create(ar);
  ai := TFArr2D.Create(ar);
  M2 := TFArr2D.Create(ar);
  zr := TFArr2D.Create(ar);
  zi := TFArr2D.Create(ar);
  RandArr2D(ar,-2.0,2.0,1);
  M1.Assign(ar);
  RandArr2D(ai,-2.0,2.0,-1);
  M2.Assign(ai);
  d := TFArr1D.Create(dm);
  diagc(ar, ai, zr, zi, d, dm, err);
  Check((err=0),'Fail HTDiag:  err = '+IntToStr(err));
  CMxDxCMH(zr, zi, d, ar, ai);
  if err = 0 then
  begin
    Check(SameArr(ar, M1, msg, x*MachEps),'Fail HTDiag Re:'+msg);
    Check(SameArr(ai, M2, msg, x*MachEps),'Fail HTDiag Im:'+msg);
  end;
end;

procedure Check_Diag.VerifyDiagC2;
var
  ar, ai, zr, zi: IFArr2D;
  d: IFArr1D;
begin
  ar := TFArr2D.Create(dm,dm);
  ar.Lo1 := RndNum(-5,5);
//  ar.Lo2 := RndNum(-5,5);
  ar.Lo2 := ar.Lo1;
  M1 := TFArr2D.Create(ar);
  ai := TFArr2D.Create(ar);
  M2 := TFArr2D.Create(ar);
  zr := TFArr2D.Create(ar);
  zi := TFArr2D.Create(ar);
  RandArr2D(ar,-2.0,2.0,1);
  M1.Assign(ar);
  RandArr2D(ai,-2.0,2.0,-1);
  M2.Assign(ai);
  d := TFArr1D.Create(dm);
  d.Lo1 := ar.Lo1;
  diagc(ar, ai, zr, zi, d, dm, err);
  Check((err=0),'Fail HTDiag:  err = '+IntToStr(err));
  CMxDxCMH(zr, zi, d, ar, ai);
  if err = 0 then
  begin
    Check(SameArr(ar, M1, msg, x*MachEps),'Fail HTDiag Re:'+msg);
    Check(SameArr(ai, M2, msg, x*MachEps),'Fail HTDiag Im:'+msg);
  end;
end;

procedure Check_Diag.VerifyRestore;

  procedure DoNothing1(const V1,V2: IFArr1D);
  var
    R: IRestore;
  begin
    R := TRestore1D.Create([V1,V2]);
    if (V1.Lo1 <> 1) or (V2.Lo1 <> 1) then
      raise Exception.Create('Vector Base <> 1' );
  end;

  procedure DoNothing2(const M1,M2: IFArr2D);
  var
    R: IRestore;
  begin
    R := TRestore2D.Create([M1,M2]);
    if (M1.Lo1 <> 1) or (M2.Lo1 <> 1) or
       (M1.Lo2 <> 1) or (M2.Lo2 <> 1)
    then
      raise Exception.Create('Matrix Base <> 1' );
  end;

begin
  V1 := TFArr1D.Create(3);
  V1.Lo1 := -1;
  V2 := TFArr1D.Create(5);
  V2.Lo1 := 2;
  DoNothing1(V1,V2);
  Check((V1.Lo1 = -1) and (V2.Lo1 = 2), 'V.Lo1s are not Recoverd');
  Check((V1.Dim1 = 3) and (V2.Dim1 = 5), 'V.Dims are not Recoverd');
  M1 := TFArr2D.Create(3,5);
  M1.Lo1 := -1;
  M1.Lo2 := 2;
  M2 := TFArr2D.Create(4,4);
  M2.Lo1 := 2;
  M2.Lo2 := -1;
  DoNothing2(M1,M2);
  Check((M1.Lo1 = -1) and (M1.Lo2 = 2) and
        (M2.Lo1 = 2) and (M2.Lo2 = -1), 'M.Lo1s are not Recoverd');
end;

{ Check_SVD }

procedure Check_SVD.setUp;
begin
  inherited;
//  eps := 1.0e-8;
  Randomize;
end;

procedure Check_SVD.tearDown;
begin
  inherited;

end;

procedure Check_SVD.VerifySVD_1;
var
  x : TFloat;
  MatU, MatV: boolean;
begin
  N1 := 6;  N2 := 4;
  x := N1;  x := x*x*x;
  A  := TFArr2D.Create(N1,N2);
  A1 := TFArr2D.Create(N1,N2);
  U  := TFArr2D.Create(N1,N2);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N2);
  T  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);
  A1.Assign(A);

  MatU := true;
  MatV := true;
  SVD_(A, U, V, W, err, MatU, MatV);
  Check((err=0),'Fail SVD_1 - SVD:  err = '+IntToStr(err));
  if err <> 0 then exit;

  A := Mt1xDxMt2T(U,W,V);
  Check((err=0),'Fail SVD_1 - Mt1xDxMt2T:  err = '+IntToStr(err));
  if err <> 0 then exit;

  Check(SameArr(A, A1, msg, x*MachEps) and (err=0),'Fail SVD_1:'+msg);
end;

procedure Check_SVD.VerifySVD_2;
var
  i: TInt;
  x : TFloat;
  FullMat, MatU, MatV: boolean;
begin
  N1 := 6;  N2 := 4;
  x := N1;  x := x*x*x;
  A  := TFArr2D.Create(N1,N2);
  A1 := TFArr2D.Create(N1,N2);
  U  := TFArr2D.Create(N1,N1);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N1);
  T  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);
  A1.Assign(A);

  FullMat := true;
  MatU := true;
  MatV := true;
  SVD_(A, U, V, W, err, MatU, MatV, FullMat);
  Check((err=0),'Fail SVD_2 - SVD_:  err = '+IntToStr(err));
  if err <> 0 then exit;
  A := MtTxMt(U,A1);
  A := MtxMt(A,V);
  for i := 1 to N2 do
    T[i] := A[i,i];
  Check((err=0),'Fail SVD_2 - Mt1xDxMt2T:  err = '+IntToStr(err));
  if err <> 0 then exit;
  del := 0.0;
  for i := 1 to N2 do
    del := max(abs(W[i]-T[i]),del);
  for i := N2+1 to N1 do
    del := max(abs(W[i]),del);
  Check((del < x*MachEps) and (err=0),'Fail SVD_2: del = '+FloatToStr(del)+'  err = '+IntToStr(err));
end;

procedure Check_SVD.VerifySVD1;
var
  x : TFloat;
  MatU, MatV: boolean;
begin
  N1 := 6;  N2 := 4;
  x := N1;  x := x*x*x;
  A  := TFArr2D.Create(N1,N2);
  A.Lo1 := RndNum(-4,4);
  A.Lo2 := RndNum(-4,4);
  U  := TFArr2D.Create(N1,N2);
  U.Lo1 := A.Lo1;
  V  := TFArr2D.Create(N2,N2);
  V.Lo1 := A.Lo2;
  W  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);

  MatU := true;
  MatV := true;
  SVD(A, U, V, W, err, MatU, MatV);
  Check((err=0),'Fail SVD1 - SVD:  err = '+IntToStr(err));
  if err <> 0 then exit;

  A1 := Mt1xDxMt2T(U,W,V);
  Check((err=0),'Fail SVD1 - Mt1xDxMt2T:  err = '+IntToStr(err));
  if err <> 0 then exit;

  Check(SameArr(A, A1, msg, x*MachEps) and (err=0),'Fail SVD1:'+msg);
end;

procedure Check_SVD.VerifySVD2;
var
  x : TFloat;
  MatU, MatV: boolean;
begin
  N1 := 4;  N2 := 6;
  x := N1;  x := x*x*x;
  A  := TFArr2D.Create(N1,N2);
  A.Lo1 := RndNum(-4,4);
  A.Lo2 := RndNum(-4,4);
  U  := TFArr2D.Create(N1,N1);
  U.Lo1 := A.Lo1;
  V  := TFArr2D.Create(N2,N1);
  V.Lo1 := A.Lo2;
  W  := TFArr1D.Create(N1);
  RandArr2D(A,-2.0,2.0,0);

  MatU := true;
  MatV := true;
  SVD(A, U, V, W, err, MatU, MatV);
  Check((err=0),'Fail SVD2 - SVD:  err = '+IntToStr(err));
  if err <> 0 then exit;

  A1 := Mt1xDxMt2T(U,W,V);
  Check((err=0),'Fail SVD2 - Mt1xDxMt2T:  err = '+IntToStr(err));
  if err <> 0 then exit;

  Check(SameArr(A, A1, msg, x*MachEps) and (err=0),'Fail SVD2:'+msg);
end;

procedure Check_SVD.VerifyGramSchmidt;
var
  k, i1, i2 : TInt;
  x : TFloat;
begin
  N1 := 5;  N2 := N1;
  A  := TFArr2D.Create(N1,N2);
  A1 := TFArr2D.Create(A);
  U  := TFArr2D.Create(A);
  AssignScalar(U,1.0);
  RandArr2D(A,-2.0,2.0,0);
  MShiftDiag(A, 1.0, A);
  Check((err=0),'Fail GramSchmidt: err = '+IntToStr(err));
  GramSchmidt(A,N1);
  for i2 := 1 to A.Dim2 do
    for i1 := 1 to A.Dim2 do
    begin
      x := 0.0;
      for k := 1 to A.Dim1 do
        x := x + A[k,i1]*A[k,i2];
      A1[i1,i2] := x;
    end;
  x := N1;  x := x*x*x;
  Check(SameArr(U, A1, msg, x*MachEps),'Fail GramSchmidt:'+msg);
end;

// A = U*W*VT  A^(-1) = V*(W^(-1))*UT    }
procedure Check_SVD.VerifyPseudoInverse;
var
  Ainv, AA, A, U, V: IFArr2D;
  W: IFArr1D;
  i, N: integer;
  tol, eps: TFloat;
begin
  N := 6;
  tol := 1.0e-12; // tolerance
  A := TFArr2D.Create(N,N); // one-based matrix N x N
  U := TFArr2D.Create(N,N); // same size matrix [1..N,1..N] - Singular vectors
  V := TFArr2D.Create(A); // can be done this way as well - Singular vectors
//  Ainv := TArr2D.Create(A); // not necessary to create
  W := TFArr1D.Create(N); // one-based vector [1..N] of singular values
  RandMt(-2.0,2.0,A,1); // random non-symmetric matrix

  // --- begin of Pseudo inverse ---
  SVD(A, U, V, W, err, true, true);  // A is not destroyed
  // Pseudo inverse of W
  if err = 0 then
    for i := W.Lo1 to W.Hi1 do
      if W[i] > tol then W[i] := 1.0/W[i] else W[i] := 0.0;

  // Pseudo inverse or inverse if all W[i] > 0
  // ( for safety better to use  some tolerance level )
  Ainv := Mt1xDxMt2T(V, W, U); // Ainv - (Pseudo) Inverse
  // --- end of Pseudo inverse ---

  // Test: A = A*Ainv*A
  AA := MtxMt(MtxMt(A,Ainv),A); // or B := MtxMt(A,Ainv,err); AA := MtxMt(B,A,err);
  eps := 1.0e-11;
  Check(SameArr(A, AA, msg, eps),'Fail PseudoInverse:'+msg);
  // actual del was ~ 1e-15

  // Should not destroy all matrices because they are declared as interfaces
end;

procedure Check_SVD.VerifyHouseholder;
var
  x: TFloat;
  A2: IFArr2D;
begin
  N1 := 6;  N2 := 4; // M = N1   N = N2
  A  := TFArr2D.Create(N1,N2); // M > N
  U  := TFArr2D.Create(N1,N2);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N2);
  T  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);
  householder(A,W,T,U,V,MachEps, MinFloat);
  A2  := TFArr2D.Create(N2,N2);
  BiDiagMt(W,T,A2);
  A1 := MtxMtT(MtxMt(U,A2),V);
  x := N2*MachEps*A.Norm(_,_);
  Check(SameArr(A, A1, msg,x) and (err=0),'Fail Householder:'+msg);
end;

procedure Check_SVD.VerifyHouseholderFull;
var
  x: TFloat;
  A2: IFArr2D;
  k: TInt;
begin
  N1 := 6;  N2 := 4; // M = N1   N = N2
  A  := TFArr2D.Create(N1,N2); // M > N
  U  := TFArr2D.Create(N1,N1);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N2);
  T  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);
  householder(A,W,T,U,V,MachEps, MinFloat,true);
  A2  := TFArr2D.Create(N1,N2);
  for k := 1 to N2 do
  begin
    A2[k,k] := W[k];
    if k < N2 then
      A2[k,k+1] := T[k+1];
  end;    
  A1 := MtxMt(U,A2);
  A1 := MtxMtT(A1,V);
  x := N2*MachEps*A.Norm(_,_);
  Check(SameArr(A, A1, msg,x) and (err=0),'Fail Householder:'+msg);
end;

procedure Check_SVD.VerifyXSVD;
var
  x: TFloat;
begin
  N1 := 6;  N2 := 4; // M = N1   N = N2
  A  := TFArr2D.Create(N1,N2); // M > N
  U  := TFArr2D.Create(N1,N2);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);
  xsvd(MachEps, MinFloat,A,W,U,V);
  A1 := Mt1xDxMt2T(U,W,V);
  x := N2*MachEps*A.Norm(_,_);
  Check(SameArr(A, A1, msg, x) and (err=0),'Fail XSVD:'+msg);
end;

procedure Check_SVD.VerifyXSVD3x2;
var
  x: TFloat;
begin
  N1 := 3;  N2 := 2; // M = N1   N = N2
  x := N1;  x := x*x*x;
  A  := TFArr2D.Create(N1,N2); // M > N
  U  := TFArr2D.Create(N1,N2);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N2);
  A[1,1] := 1.316496580927726;
  A[2,1] := 0.31649658092772603;
  A[3,1] := 0.816496580927726;
  A[1,2] := 0.31649658092772603;
  A[2,2] := 1.316496580927726;
  A[3,2] := 0.816496580927726;
  xsvd(MachEps, MinFloat,A,W,U,V);
  Check((err=0),'Fail XSVD:  err = '+IntToStr(err));
  if err <> 0 then exit;
  A1 := Mt1xDxMt2T(U,W,V);
  Check((err=0),'Fail XSVD - Mt1xDxMt2T:  err = '+IntToStr(err));
  if err <> 0 then exit;

  Check(SameArr(A, A1, msg, x*MachEps) and (err=0),'Fail XSVD:'+msg);
end;

procedure Check_SVD.VerifyXSVDFull;
var
  x: TFloat;
  A2: IFArr2D;
  k: TInt;
begin
  N1 := 6;  N2 := 4; // M = N1   N = N2
  A  := TFArr2D.Create(N1,N2); // M > N
  U  := TFArr2D.Create(N1,N1);
  V  := TFArr2D.Create(N2,N2);
  W  := TFArr1D.Create(N2);
  RandArr2D(A,-2.0,2.0,0);
  xsvd(MachEps, MinFloat,A,W,U,V,true,true,true);
  A2  := TFArr2D.Create(N1,N2);
  for k := 1 to N2 do
    A2[k,k] := W[k];
  A1 := MtxMt(U,A2);
  A1 := MtxMtT(A1,V);
  x := N2*MachEps*A.Norm(_,_);
  Check(SameArr(A, A1, msg, x) and (err=0),'Fail XSVD:'+msg);
end;

initialization

  RegisterTest('uLinAlg Tests',Suite);

end.
