{@abstract(Object algebra tests )
@author(Nikolai Shokhirev <nikolai@shokhirev.com>)
@created(2003.02.02)
@lastmod(2004.11.11)
Nikolai V. Shokhirev, 2002-2007 }
unit Test_uDynObjAlg;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays;

type

Check_DynObjAlg = class(TTestCase)
private
  V1, V2, V3, V4: IFArr1D;
  M1, M2, M3, M4: IFArr2D;
  x,  eps: TFloat;
  l1, l2,h1,h2,l3,h3,
  d1, d2, d3, b1, b2: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyTranspose;
  procedure VerifyMtTxMtT;
  procedure VerifyMtxDxMtT;
  procedure VerifyMt1xDxMt2T;
  procedure VerifyMt1TxDxMt2;
  procedure VerifySumMt;
  procedure VerifySumVt;
//  procedure VerifyMtxDxMtT;
end;

Check_DynObjLinAlg = class(TTestCase)
private
  M1, M2: IFArr2D;
  MC, MC1, MC2: ICArr2D;
  x: TFloat;
  l1, l2,h1,h2, dim: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyDiagMt;
  procedure VerifyDiagMt1x1;
  procedure VerifySVDMt1;
  procedure VerifySVDMt2;
  procedure VerifySVDMtFull1;
  procedure VerifySVDMtFull2;
  procedure VerifyDiagHMt;
  procedure VerifyPseudoInv1;
  procedure VerifyPseudoInv2;
end;

function Suite : ITestSuite;

implementation

uses
  math,
  uDynArrUtils,
  uDynArrUtilsX,
  uDynObjAlg,
  uDynArrIO;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDyn Obj Alg Tests');

  result.addTest(Check_DynObjAlg.Suite);
  result.addTest(Check_DynObjLinAlg.Suite);
end;

{ Check_DynObjAlg }

procedure Check_DynObjAlg.setUp;
begin
  inherited;
  d1 := 5;
  d2 := 4;
  d3 := 6;
  b1 := -1;
  b2 := 2;
  x := d1*d2;
end;

procedure Check_DynObjAlg.tearDown;
begin
  inherited;

end;

procedure Check_DynObjAlg.VerifyTranspose;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l2,h2,l1,h1);
  RandArr2D(M1,-1.0,1.0,0);
  M3 := Transpose(M1);
  MtT(M1,M2);
  Check(SameArr(M3, M2, msg, eps), 'Fail VerifyTranspose: '+ msg);
end;

{ Check: MxM, MT_MT }
procedure Check_DynObjAlg.VerifyMtTxMtT;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(l3,h3,l1,h1);
  M3 := TFArr2D.Create(l2,h2,l3,h3);
  RandArr2D(M1,-1.0,1.0,0);
  RandArr2D(M2,-1.0,1.0,0);
  MT_MT(M1, M2, M3);
  M4 := MtTxMtT(M1,M2);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail VerifyMtTxMtT: '+msg);
end;

{ check: MxD, MxMT, MxDxMT }
procedure Check_DynObjAlg.VerifyMtxDxMtT;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,0);
  V1 := TFArr1D.Create(l2,h2);
  RandArr1D(V1,-1.0,1.0);
  M3 := TFArr2D.Create(l1,h1,l1,h1);
  MxDxMT(M1, V1, M3);
  M4 := MtxDxMtT(M1,V1);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail MxDxMT: '+ msg);
end;

{ check: MxD, MxMT, M1xDxM2T }
procedure Check_DynObjAlg.VerifyMt1xDxMt2T;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,0);
  V1 := TFArr1D.Create(l2,h2);
  RandArr1D(V1,-1.0,1.0);
  M2 := TFArr2D.Create(l3,h3,l2,h2);
  RandArr2D(M2,-1.0,1.0,0);
  M3 := TFArr2D.Create(l1,h1,l3,h3);
  M1xDxM2T(M1,M2,V1,M3);
  M4 := Mt1xDxMt2T(M1,V1,M2);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail M1xDxM2T: '+ msg);
end;

{ check: DxM, MTxM, M1TxDxM2 }
procedure Check_DynObjAlg.VerifyMt1TxDxMt2;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  l3 := RndNum(-5,5);  h3 := l3+d3-1;
  M1 := TFArr2D.Create(l2,h2,l1,h1);
  RandArr2D(M1,-1.0,1.0,0);
  V1 := TFArr1D.Create(l2,h2);
  RandArr1D(V1,-1.0,1.0);
  M2 := TFArr2D.Create(l2,h2,l3,h3);
  RandArr2D(M2,-1.0,1.0,0);
  M3 := TFArr2D.Create(l1,h1,l3,h3);
  M1TxDxM2(M1,M2,V1,M3);
  M4 := Mt1TxDxMt2(M1,V1,M2);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail M1TxDxM2: '+ msg);
end;

{ check: MAddM, SumArr2D }
procedure Check_DynObjAlg.VerifySumMt;
var
  c1, c2, c3: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  l2 := RndNum(-5,5);  h2 := l2+d2-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  M2 := TFArr2D.Create(M1);
  M3 := TFArr2D.Create(M1);
  M4 := TFArr2D.Create(M1);
  RandArr2D(M1,-1.0,1.0,0);  c1 := RndFloat(-2.0,2.0);
  RandArr2D(M2,-1.0,1.0,0);  c2 := RndFloat(-2.0,2.0);
  RandArr2D(M3,-1.0,1.0,0);  c3 := RndFloat(-2.0,2.0);
  SumArr2D([c1,c2,c3],[M1,M2,M3],M4);
  M3 := SumMt([c1,c2,c3],[M1,M2,M3]);
  Check(SameArr(M3, M4, msg, x*MachEps), 'Fail SumArr1D: '+ msg);
end;

{ check: VAddV, SumArr1D }
procedure Check_DynObjAlg.VerifySumVt;
var
  c1, c2, c3: TFloat;
begin
  l1 := RndNum(-5,5);  h1 := l1+d1-1;
  V1 := TFArr1D.Create(l1,h1);
  V2 := TFArr1D.Create(V1);
  V3 := TFArr1D.Create(V1);
  V4 := TFArr1D.Create(V1);
  RandArr1D(V1,-1.0,1.0);  c1 := RndFloat(-2.0,2.0);
  RandArr1D(V2,-1.0,1.0);  c2 := RndFloat(-2.0,2.0);
  RandArr1D(V3,-1.0,1.0);  c3 := RndFloat(-2.0,2.0);
  SumArr1D([c1,c2,c3],[V1,V2,V3],V4);
  V3 := SumVt([c1,c2,c3],[V1,V2,V3]);
  Check(SameArr(V3, V4, msg, x*MachEps), 'Fail SumArr1D: '+ msg);
end;


{ Check_DynObjLinAlg }

procedure Check_DynObjLinAlg.setUp;
begin
  inherited;
  dim := 6;
  x := dim;
  x := x*x*x;
end;

procedure Check_DynObjLinAlg.tearDown;
begin
  inherited;

end;

procedure Check_DynObjLinAlg.VerifyDiagHMt;
var
  HES: IHEigenSys;
  code: TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+dim-1;
  MC1 := TCArr2D.Create(l1,h1,l1,h1);
  RandArr2D(MC1.Re,-1.0,1.0,1);
  RandArr2D(MC1.Im,-1.0,1.0,-1);
  MC := TCArr2D.Create(MC1,true);
  HES := DiagHMt(MC1,code);
  Check(code=0, 'Fail VerifyDiagHMt: code <> 0');
  MC2 := HEigenSysToMt(HES);
  Check(SameArr(MC,MC2, msg, x*MachEps), 'Fail VerifyDiagHMt: '+ msg);
end;

procedure Check_DynObjLinAlg.VerifyDiagMt;
var
  ES: IEigenSys;
  code: TInt;
begin
  l1 := RndNum(-5,5);  h1 := l1+dim-1;
  M1 := TFArr2D.Create(l1,h1,l1,h1);
  RandArr2D(M1,-1.0,1.0,1);
  ES := DiagMt(M1,code);
  Check(code=0, 'Fail VerifyDiagMt: code <> 0');
  M2 := EigenSysToMt(ES);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifyDiagMt: '+ msg);
end;

procedure Check_DynObjLinAlg.VerifyDiagMt1x1;
var
  ES: IEigenSys;
  code: TInt;
begin
  l1 := RndNum(-5,5); ;
  M1 := TFArr2D.Create(l1,l1,l1,l1);
  RandArr2D(M1,-1.0,1.0,1);
  ES := DiagMt(M1,code);
  Check(code=0, 'Fail VerifyDiagMt1x1: code <> 0');
  M2 := EigenSysToMt(ES);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifyDiagMt1x1: '+ msg);
end;

procedure Check_DynObjLinAlg.VerifyPseudoInv1;
begin
  l1 := RndNum(-5,5);  h1 := l1+6-1;
  l2 := RndNum(-5,5);  h2 := l2+4-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,1);
  M2 := MtxMt(PseudoinverseMt(M1),M1);
  M2 := MtxMt(M1,M2);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifyPseudoInv: M*(M~*M)'+ msg);
  M2 := MtxMt(M1,PseudoinverseMt(M1));
  M2 := MtxMt(M2,M1);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifyPseudoInv: (M*M~)*M'+ msg);
end;

{ Dim1 < Dim2 }
procedure Check_DynObjLinAlg.VerifyPseudoInv2;
begin
  l1 := RndNum(-5,5);  h1 := l1+4-1;
  l2 := RndNum(-5,5);  h2 := l2+6-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,1);
  M2 := MtxMt(M1,PseudoinverseMt(M1));
  M2 := MtxMt(M2,M1);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifyPseudoInv: (M*M~)*M'+ msg);
end;

procedure Check_DynObjLinAlg.VerifySVDMt1;
var
  SVDSys: ISVDSys;
begin
  l1 := RndNum(-5,5);  h1 := l1+6-1;
  l2 := RndNum(-5,5);  h2 := l2+4-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,1);
  SVDSys := SVDMt(M1);
  M2 := SVDSysToMt(SVDSys);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifySVDMt m > n: '+ msg);
end;

procedure Check_DynObjLinAlg.VerifySVDMt2;
var
  SVDSys: ISVDSys;
begin
  l1 := RndNum(-5,5);  h1 := l1+4-1;
  l2 := RndNum(-5,5);  h2 := l2+6-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,1);
  SVDSys := SVDMt(M1);
  M2 := SVDSysToMt(SVDSys);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifySVDMt m < n: '+ msg);
end;

procedure Check_DynObjLinAlg.VerifySVDMtFull1;
var
  SVDSys: ISVDSys;
begin
  l1 := RndNum(-5,5);  h1 := l1+6-1;
  l2 := RndNum(-5,5);  h2 := l2+4-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,1);
  SVDSys := SVDMt(M1,true);
  M2 := SVDSysToMt(SVDSys, );
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifySVDMt Full m > n: '+ msg);
end;

procedure Check_DynObjLinAlg.VerifySVDMtFull2;
var
  SVDSys: ISVDSys;
begin
  l1 := RndNum(-5,5);  h1 := l1+4-1;
  l2 := RndNum(-5,5);  h2 := l2+6-1;
  M1 := TFArr2D.Create(l1,h1,l2,h2);
  RandArr2D(M1,-1.0,1.0,1);
  SVDSys := SVDMt(M1, true);
  M2 := SVDSysToMt(SVDSys);
  Check(SameArr(M1,M2, msg, x*MachEps), 'Fail VerifySVDMt Full m < n: '+ msg);
end;

initialization

  RegisterTest('uLinAlg Tests',Suite);

end.

