{@abstract(Numerical integration tests)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>
                          http://www.shokhirev.com/nikolai.html)
@created(2004.12.12)
@lastmod(2004.12.12)
Nikolai V. Shokhirev, 2003-2004}
unit Test_uIntegration;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes;

type

Check_Quanc8 = class(TTestCase)
private
  x1, x2, abserr, relerr, eps: TFloat;
  r0, result, errest, flag: TFloat;
  nofun: TInt;
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyIntSin;
   procedure VerifyPoly08;
   procedure VerifySqrt;
end;


function Suite : ITestSuite;

implementation

uses
  uIntegration,
  math;

var
  nexp: TInt;
  alfasqD: TFloat;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uFMM Tests');

  result.addTest(Check_Quanc8.Suite);
end;

{ Check_Quanc8 }

procedure Check_Quanc8.setUp;
begin

end;

procedure Check_Quanc8.tearDown;
begin

end;

function F5(const x: TFloat): TFloat;
begin
  if x = 0.0 then
    result := 1.0
  else
    result := sin(x)/x;
end;

procedure Check_Quanc8.VerifyIntSin;
begin
  x1 := 0.0;
  x2 := 2.0*Pi;
  r0 := 1.4181515761326282;  // from Mathematica
  abserr := 2.0e-12;
  relerr := 1.0e-12;
  eps := max(abserr, relerr*r0);
  Quanc8(F5, x1, x2, abserr, relerr, result, errest, flag, nofun);
  Check(SameValue(r0,result,eps),'VerifyIntSin faild; flag = '+FloatToStr(flag));
end;


function FPn(const x: TFloat): TFloat;
begin
    result := (nexp+1.0)*IntPower(x,nexp);;
end;

procedure Check_Quanc8.VerifyPoly08;
var
  n: TInt;
begin
  x1 := 0.0;
  x2 := 1.0;
  r0 := 1.0;
  abserr := 1.0e-12;
  relerr := 1.0e-12;
  eps := max(abserr, relerr*r0);
  for n := 0 to 8 do
  begin
    nexp := n;
    Quanc8(FPn, x1, x2, abserr, relerr, result, errest, flag, nofun);
    Check(SameValue(r0, result, eps), 'VerifyPoly08 faild n = '+IntToStr(n));
  end;
end;

function FSqrt(const x: TFloat): TFloat;
begin
  result := sqrt(x);
end;

procedure Check_Quanc8.VerifySqrt;
begin
  x1 := 0.0;
  x2 := 4.0;
  r0 := 16.0/3.0;
  abserr := 1.0e-12;
  relerr := 1.0e-12;
  eps := max(abserr, relerr*r0);
  Quanc8(FSqrt, x1, x2, abserr, relerr, result, errest, flag, nofun);
  Check(SameValue(r0,result,eps),'VerifySqrt faild; flag = '+FloatToStr(flag));
end;


initialization

  RegisterTest('uFMM Tests',Suite);

end.
