{@abstract(Ordinary differential equations tests)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>
                          http://www.shokhirev.com/nikolai.html)
@created(2004.12.12)
@lastmod(2004.12.12)
Nikolai V. Shokhirev, 2003-2004}
unit Test_uODE;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes;

type

Check_RKF45 = class(TTestCase)
private
public
   procedure setUp;  override;
   procedure tearDown; override;
published
   procedure VerifyOrbit;
end;

function Suite : ITestSuite;

implementation

uses
  uODE, uDynArrays,
  uDynArrIO,
  math;

var
  nexp: TInt;
  alfasqD: TFloat;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uFMM Tests');

  result.addTest(Check_RKF45.Suite);
end;


{ Check_RKF45 }

procedure Check_RKF45.setUp;
begin
  inherited;

end;

procedure Check_RKF45.tearDown;
begin
  inherited;

end;

function f3(const x: TFloat):TFloat;
begin
  result := 1.0/(x+1.0)+1/(1.0-x);
end;


// Sample6
procedure Orbit(const t: TFloat; const y, yp: IFArr1D);                            
var
  r: TFloat;
begin
//----------------------------------------------------------------------------
  r:=y[1]*y[1]+y[2]*y[2];
  r:=r*SQRT(r)/alfasqD; // alfasqD is a module variable, used here and Sample6
  yp[1]:=y[3];
  yp[2]:=y[4];
  yp[3]:=-y[1]/r;
  yp[4]:=-y[2]/r;
  exit ;
end;//

// Sample6  from PDAS: http://www.pdas.com/free.htm
procedure Check_RKF45.VerifyOrbit;
var
  i, NEQN, iflag, nfe, kop, init, jflag, kflag : TInt;
  y, yp, f1, f2, f3, f4, f5: IFArr1D;
  y1, y2: IFArr1D;
  t,  tout,  tprint,  tfinal,  alfa,  ecc,  relerr,  abserr: TFloat;
  err, h, savre, savae: TFloat;
  ss: string;
begin
  NEQN := 4;
  y  := TFArr1D.Create(NEQN);
  yp := TFArr1D.Create(NEQN);
  f1 := TFArr1D.Create(NEQN);
  f2 := TFArr1D.Create(NEQN);
  f3 := TFArr1D.Create(NEQN);
  f4 := TFArr1D.Create(NEQN);
  f5 := TFArr1D.Create(NEQN);

  y1 := TFArr1D.Create(25);
  ss := '(0.750000000,0.619766772,0.294413418,-0.105183668,-0.490308911,'+
        '-0.813951671,-1.054037690,-1.200736640,-1.249995470,-1.200723410,'+
        '-1.054012660,-0.813917398,-0.490270019,-0.105147965,0.294436693,'+
        '0.619762182,0.749918401,0.619556844,0.294083089,-0.105574392,'+
        '-0.490691930,-0.814271748,-1.054254890,-1.200823430,-1.249934080)';
  StrToVector(y1, ss);
  y2 := TFArr1D.Create(25);
  ss := '(0.000000000,0.477790743,0.812175512,0.958031714,0.939863801,'+
        '0.799574375,0.575683534,0.300133795,-0.000028848,-0.300188512,'+
        '-0.575729311,-0.799605250,-0.939875066,-0.958015144,-0.812122822,'+
        '-0.477689207,0.000150567,0.477936178,0.812232316,0.957946718,'+
        '0.939628422,0.799207032,0.575215459,0.299602211,-0.000583312)';
  StrToVector(y2, ss);
  iflag := 1;
  tprint := 0.5;
  tfinal := 12.0;
  alfa := 3.141592653589/4.0;
  relerr := 1.0E-5;
  abserr := 1.0E-5;

  ecc:=0.25;
  alfasqD:=alfa*alfa;
  y[1]:=1.0-ecc;
  y[2]:=0.0;
  y[3]:=0.0;
  y[4]:=alfa*SQRT((1.0+ecc)/(1.0-ecc));
  t:=0.0;
  tout:=t;
  ss := '';
  err := 0.0;
  i := 0;
  repeat
    inc(i);
    Rkfs45(Orbit,t,tout, abserr, relerr, iflag, y, yp, f1, f2, f3, f4, f5,
           h, savre, savae, nfe, kop, init, jflag, kflag);
    err := max(err,abs(y[1]-y1[i])+abs(y[2]-y2[i]));
    CASE (iflag) of
    1: begin
         ss := ss + ' IMPROPER CALL';
         break;
       end;
    3: ss := ss + ' Tolerances reset due to iflag = 3, relerr = '+
                   FloatToStr(relerr)+' abserr = '+ FloatToStr(abserr);
    4: ss := ss + 'Warning.  Many steps...';
    5: begin
         abserr := 1.0E-9;
         ss := ss + ' Tolerances reset due to iflag = 5, relerr = '+
                     FloatToStr(relerr)+' abserr = '+ FloatToStr(abserr);
       end;
    6: begin
         relerr := 10.0*relerr;
         ss := ss + ' Tolerances reset due to iflag = 6, relerr = '+
                     FloatToStr(relerr)+' abserr = '+ FloatToStr(abserr);
       end;
    7: begin
         ss := ss + ' WARNING. Much output';
         iflag := 2;
       end;
    else
        tout := t+ tprint;
    end;
  until (t >= tfinal);
  ss := ss + ' err = '+FloatToStr(err);
  ss := ss + ' Successful completion of Sample6';
  Check((iflag=2) and (err < 2.0E-5), 'faild VerifyOrbit ' + ss);
end;



initialization

  RegisterTest('uFMM Tests',Suite);

end.
