{*
 Test for Panel Data - Rectangular table data object
 Nikolai V. Shokhirev, 2006-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2006.10.09 - created
 @version 2007.08.04: Last modified
 }
unit Test_uPanelData;

interface

uses
  TestFramework
  , SysUtils
  , uMatTypes
  , uDynArrays
  , uPanelData
  ;

type

Check_TPanelData = class(TTestCase)
private
  Titles: ISArr1D;
  Values: ISArr2D;
  NCol, NRow: integer;
  pd: IPanelData;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyCreate;
  procedure VerifyLim;
  procedure VerifyTitles;
  procedure VerifyValues;
  procedure VerifyLines;
end;


Check_TPanelDataConvert = class(TTestCase)
private
  NCol, NRow: integer;
  pd: IPanelData;
  eps: TFloat;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyFloat;
  procedure VerifyInteger;
  procedure VerifyDateTime;
end;

Test_TPanelData = class(TTestCase)
private
  NCol, NRow: integer;
  pd, pd1: IPanelData;
//  eps: TFloat;
//  msg: string;
  procedure SetData;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure TestSaveToFileCSV;
  procedure TestSaveToFileTab;
  procedure TestSetLengthRowM;
  procedure TestSetLengthRowP;
  procedure TestSetLengthColM;
  procedure TestSetLengthColP;
end;

Test_TRansform = class(TTestCase)
private
  NCol, NRow: integer;
  pd, pd1: IPanelData;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure TestDateTimeToInt;
  procedure TestIntToDateTime;
  procedure TestReturnToPrice;
  procedure TestPriceToReturn;
end;

function Suite : ITestSuite;

implementation

uses
  uDynArrUtils
  , uDynArrUtilsX // for tests
  , uDynObjAlg
  , uDynArrIO
  , math
  , Classes
  , uStrFun
  ;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uPanelData Tests');

  result.addTest(Check_TPanelData.Suite);
  result.addTest(Check_TPanelDataConvert.Suite);
  result.addTest(Test_TPanelData.Suite);
  result.addTest(Test_TRansform.Suite);  
end;

{ Check_TPanelData }

procedure Check_TPanelData.setUp;
var
  r, c: TInt;
begin
  inherited;
  NRow := 4;
  NCol := 3;
//  Randomize;
  Titles := TSArr1D.Create(NCol);
  Values := TSArr2D.Create(NRow,NCol);
  pd := TPanelData.Create(NRow,NCol);
  for c := 1 to NCol do
  begin
    Titles[c] := 'Title'+IntToStr(c);
    pd.Titles[c] := Titles[c];
    for r := 1 to NRow do
    begin
      Values[r,c] := 'Value'+IntToStr(r)+','+IntToStr(c);
      pd[r,c] := Values[r,c];
    end;
  end;
end;

procedure Check_TPanelData.tearDown;
begin
  inherited;
end;


procedure Check_TPanelData.VerifyCreate;
begin
  CheckNotNull(pd, 'Fail to create PanelData');
end;

procedure Check_TPanelData.VerifyLim;
begin
  CheckEquals(NCol, pd.NCol, ' Fail NCol');
  CheckEquals(NRow, pd.NRow, ' Fail NRow');
end;

procedure Check_TPanelData.VerifyLines;
var
  i: TInt;
  sl: TStringlist;
begin
  sl := TStringlist.Create;
  for i := 1 to 5 do
  begin
    sl.Append('Comment'+IntToStr(i));
    pd.Lines.Append('Comment'+IntToStr(i));
  end;

  CheckEquals(sl.Count, pd.NLines, ' Fail NLines');

  for i := 0 to pd.NLines-1 do
  begin
    CheckEquals(sl[i], pd.Lines[i], ' @ i ='+IntToStr(i));
  end;
  sl.Free;
end;

procedure Check_TPanelData.VerifyTitles;
var
  c: TInt;
begin
  for c := 1 to NCol do
    CheckEquals(Titles[c], pd.Titles[c], ' @ c ='+IntToStr(c));
end;

procedure Check_TPanelData.VerifyValues;
var
  r, c: TInt;
begin
  for c := 1 to NCol do
    for r := 1 to NRow do
      CheckEquals(Values[r,c], pd[r,c], ' @ r ='+IntToStr(r)+ ' c ='+IntToStr(c));
end;

{ Check_TPanelDataConvert }

procedure Check_TPanelDataConvert.setUp;
begin
  inherited;
  NRow := 4;
  NCol := 3;
  pd := TPanelData.Create(NRow,NCol);
  eps := 1.0e-12;
end;

procedure Check_TPanelDataConvert.tearDown;
begin
  inherited;

end;

procedure Check_TPanelDataConvert.VerifyDateTime;
var
  Values: IFArr2D;
  r, c: TInt;
begin
//  fail('Test Not Implemented Yet');
  Values := TFArr2D.Create(NRow,NCol);
  for c := 1 to NCol do
    for r := 1 to NRow do
    begin
      Values[r,c] := 36543.21*random;
      pd.AsDateTime[r,c] := TDateTime(Values[r,c]);
    end;

  for c := 1 to NCol do
    for r := 1 to NRow do
      CheckEquals(DateTimeToStr(TDateTime(Values[r,c])), pd[r,c], ' @ r ='+IntToStr(r)+ ' c ='+IntToStr(c));
end;

procedure Check_TPanelDataConvert.VerifyFloat;
var
  Values: IFArr2D;
  r, c: TInt;
begin
  Values := TFArr2D.Create(NRow,NCol);
  for c := 1 to NCol do
    for r := 1 to NRow do
    begin
      Values[r,c] := 10.0*random - 5.0;
      pd.AsFloat[r,c] := Values[r,c];
    end;

  for c := 1 to NCol do
    for r := 1 to NRow do
      CheckEquals(Values[r,c], pd.AsFloat[r,c], eps, ' @ r ='+IntToStr(r)+ ' c ='+IntToStr(c));
end;

procedure Check_TPanelDataConvert.VerifyInteger;
var
  Values: IIArr2D;
  r, c: TInt;
begin
  Values := TIArr2D.Create(NRow,NCol);

  for c := 1 to NCol do
    for r := 1 to NRow do
    begin
      Values[r,c] := RandomRange(-22,22);
      pd.AsInt[r,c] := Values[r,c];
    end;

  for c := 1 to NCol do
    for r := 1 to NRow do
      CheckEquals(Values[r,c], pd.AsFloat[r,c],  ' @ r ='+IntToStr(r)+ ' c ='+IntToStr(c));
end;

{ Test_TPanelData }

procedure Test_TPanelData.setUp;
begin
  inherited;
  NRow := 4;
  NCol := 5;
end;

procedure Test_TPanelData.tearDown;
begin
  inherited;

end;

procedure Test_TPanelData.SetData;
var
  r, c: TInt;
begin
  pd := TPanelData.Create(NRow,NCol);
  for r := 1 to NRow do
  begin
    pd[r,1] := RandDateTimeStr;
    pd[r,2] := FloatToStr(RndFloat(-1.0e22,1.0e22));
    pd[r,3] := IntToStr(RndNum(-123456789,123456789));
    pd[r,4] := BoolToStr(RndBool);
    pd[r,5] := RandStr(9,1);
    pd.Types[1] := btDateTime;
    pd.Types[2] := btFloat;
    pd.Types[3] := btInteger;
    pd.Types[4] := btBoolean;
    pd.Types[5] := btString;
  end;
  pd.Lines.Append(RandStr(9,1));
  pd.Lines.Append(RandStr(9,1));
  for c := 1 to NCol do
    pd.Titles[c] := RandStr(9,1);
end;

procedure Test_TPanelData.TestSaveToFileCSV;
var
  r, c, err: TInt;
  s: AnsiString;
begin
  SetData;
  pd.SaveToFile('test.csv');
  pd1 := TPanelData.Create;
  pd1.LoadFromFile('test.csv', err, s);
  for c := 1 to NCol do
    for r := 1 to NRow do
      Check( pd[r,c] = pd1[r,c], 'Fail TestSaveToFile - data');
  for c := 1 to NCol do
    Check( pd.Titles[c] = pd1.Titles[c], 'Fail TestSaveToFile - Titles');
  for c := 1 to NCol do
    Check( pd.Types[c] = pd1.Types[c], 'Fail TestSaveToFile - Types');
  for c := 0 to pd.Lines.Count-1 do
    Check( pd.Lines[c] = pd1.Lines[c], 'Fail TestSaveToFile - Lines');
end;

procedure Test_TPanelData.TestSaveToFileTab;
var
  r, c, err: TInt;
  s: AnsiString;
begin
  SetData;
  pd.SaveToFile('test.tab',TabDelimiter,';');
  pd1 := TPanelData.Create;
  pd1.LoadFromFile('test.tab', err, s, TabDelimiter,';');
  for c := 1 to NCol do
    for r := 1 to NRow do
      Check( pd[r,c] = pd1[r,c], 'Fail TestSaveToFile - data');
  for c := 1 to NCol do
    Check( pd.Titles[c] = pd1.Titles[c], 'Fail TestSaveToFile - Titles');
  for c := 1 to NCol do
    Check( pd.Types[c] = pd1.Types[c], 'Fail TestSaveToFile - Types');
  for c := 0 to pd.Lines.Count-1 do
    Check( pd.Lines[c] = pd1.Lines[c], 'Fail TestSaveToFile - Lines');
end;

procedure Test_TPanelData.TestSetLengthRowM;
var
  r, c, NewNRow: TInt;
begin
  SetData;
  pd1 := TPanelData.Create(NRow, NCol);
  for c := 1 to NCol do
    for r := 1 to NRow do
      pd1[r,c] := pd[r,c];
  pd1.SetLengths(NRow-1,Ncol);
  for c := 1 to NCol do
    for r := 1 to NRow-1 do
      Check( pd[r,c] = pd1[r,c], 'Test Set Length Row-1 ');
end;

procedure Test_TPanelData.TestSetLengthRowP;
var
  r, c, NewNRow: TInt;
begin
  SetData;
  pd1 := TPanelData.Create(NRow, NCol);
  for c := 1 to NCol do
    for r := 1 to NRow do
      pd1[r,c] := pd[r,c];
  pd1.SetLengths(NRow+1,Ncol);
  for c := 1 to NCol do
    for r := 1 to NRow do
      Check( pd[r,c] = pd1[r,c], 'Test Set Length Row+1 ');
end;

procedure Test_TPanelData.TestSetLengthColM;
var
  r, c, NewNRow: TInt;
begin
  SetData;
  pd1 := TPanelData.Create(NRow, NCol);
  for c := 1 to NCol do
    for r := 1 to NRow do
      pd1[r,c] := pd[r,c];
  pd1.SetLengths(NRow,Ncol-1);
  for c := 1 to NCol-1 do
    for r := 1 to NRow-1 do
      Check( pd[r,c] = pd1[r,c], 'Test Set Length Row-1 ');
end;

procedure Test_TPanelData.TestSetLengthColP;
var
  r, c, NewNRow: TInt;
begin
  SetData;
  pd1 := TPanelData.Create(NRow, NCol);
  for c := 1 to NCol do
    for r := 1 to NRow do
      pd1[r,c] := pd[r,c];
  pd1.SetLengths(NRow,Ncol+1);
  for c := 1 to NCol do
    for r := 1 to NRow do
      Check( pd[r,c] = pd1[r,c], 'Test Set Length Row+1 ');
end;

{ Test_TRansform }

procedure Test_TRansform.setUp;
begin
  inherited;
  NRow := 4;
  NCol := 5;
  pd := TPanelData.Create(NRow,NCol);
  pd[1,1] := '1/2/1996';
  pd[2,1] := '1/3/1996';
  pd[3,1] := '1/4/1996';
  pd[4,1] := '11/23/1996';

  pd[1,2] := '19960102';
  pd[2,2] := '19960103';
  pd[3,2] := '19960104';
  pd[4,2] := '19961122';

  pd[1,3] := '960102';
  pd[2,3] := '960103';
  pd[3,3] := '960104';
  pd[4,3] := '961122';

  pd[1,4] := '1.0';
  pd[2,4] := '1.2';
  pd[3,4] := '1.44';
  pd[4,4] := '1.728';

  pd[1,5] := '1.0';
  pd[2,5] := '1.2';
  pd[3,5] := '1.2';
  pd[4,5] := '1.2';

  pd.Types[1] := btDateTime;
  pd.Types[2] := btInteger;
  pd.Types[3] := btInteger;
  pd.Types[4] := btFloat;
  pd.Types[5] := btFloat;
end;

procedure Test_TRansform.tearDown;
begin
  inherited;

end;

procedure Test_TRansform.TestDateTimeToInt;
var
  r: integer;
begin
  pd.DateTimeToInt(1,dtfDate8);
  for r := 1 to NRow do
    Check( pd[r,1] = pd[r,2], 'Test DateTimeToInt Row+1 ');
  pd.DateTimeToInt(1,dtfDate6);
  for r := 1 to NRow do
    Check( pd[r,1] = pd[r,2], 'Test DateTimeToInt Row+1 ');
end;

procedure Test_TRansform.TestIntToDateTime;
begin

end;

procedure Test_TRansform.TestPriceToReturn;
begin

end;

procedure Test_TRansform.TestReturnToPrice;
begin

end;

initialization

  RegisterTest('Panel Data Tests',Suite);
  Randomize;
end.
