{@abstract(Tests for Data and DFT)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(January 1, 2003)
@lastmod(June 06, 2004)
Nikolai V. Shokhirev, 2003-2004 }
unit Test_uDFTUtils;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays,
  uDFT;

type

Check_ForwardInverse = class(TTestCase)
private
  A: Idft1D;
  F1, F2: IFArr1D;
  C1, C2, C3, C4: ICArr1D;
  N, err: integer;
  x, eps, del: TFloat;
  msg, msg0: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyDFT;
  procedure VerifyDFTR;
  procedure VerifyDFTSym;
  procedure VerifyCFTG;
  procedure VerifyDFT_CFTG;
  procedure VerifyDFT_CFTG_TCArr1D;
end;

function Suite : ITestSuite;

implementation

uses
  math,
  uDynArrUtils,
  uDynArrUtilsX,
//  uDynObjAlg,
  uDFTUtils,
  uComplexType;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDFTUtils Tests');

  result.addTest(Check_ForwardInverse.Suite);
end;

{ Check_ForwardInverse }

procedure Check_ForwardInverse.setUp;
begin
  inherited;
  inherited;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_ForwardInverse.tearDown;
begin
  inherited;

end;

procedure Check_ForwardInverse.VerifyDFT;
begin
  msg0 := 'Fail VerifyDFT - Forward-Inverse: ';
  N := RndNum(3,16);
  C1 := TCArray1D.Create(1,N);
  RandArr1D(C1,-1.1,1.1);
  del := NormCV(C1);
  C2 := DFT(C1);
  C3 := InvDFT(C2);
  x := N; x := x*x*x;
  Check((del > x*MachEps) and SameArr(C3, C1, msg, x*MachEps),msg0+msg);
end;

procedure Check_ForwardInverse.VerifyDFTR;
begin
  msg0 := 'Fail VerifyDFTR - Forward-Inverse: ';
  N := RndNum(3,16);
  F1 := TFArray1D.Create(1,N);
  RandArr1D(F1,-1.1,1.1);
  C1 := DFTR(F1);
  F2 := InvDFTR(C1,N);
  x := N; x := x*x*x;
  Check(SameArr(F2, F1, msg, x*MachEps),msg0+msg);
end;

procedure Check_ForwardInverse.VerifyDFTSym;
begin
  msg0 := 'Fail VerifyDFTSym - Forward-Inverse: ';
  N := RndNum(2,8); N := N+N;
  C1 := TCArray1D.Create(1,N);
  RandArr1D(C1,-1.1,1.1);
  C2 := DFTSym(C1);
  C3 := InvDFTSym(C2);
  x := N; x := x*x*x;
  Check(SameArr(C3, C1, msg, x*MachEps),msg0+msg);
end;

procedure Check_ForwardInverse.VerifyCFTG;
begin
  msg0 := 'Fail VerifyCFTG - Forward-Inverse ';
  N := RndNum(2,8); N := N+N;
  C1 := TCArray1D.Create(1,N);
  C2 := TCArray1D.Create(1,N);
  C3 := TCArray1D.Create(1,N);
  C4 := TCArray1D.Create(1,N);
  RandArr1D(C1,-1.1,1.1);
  C3.Assign(C1);
  CFTG(C1,C2,C4,+N);
  C1.Fill(Cmplx0);
  CFTG(C2,C1,C4,-N);
  x := N; x := x*x*x;
  Check(SameArr(C3, C1, msg, x*MachEps),msg0+' #1'+msg);
  // use  two arrays
  CFTG(C1,C2,C1,+N);
  C1.Fill(Cmplx0);
  CFTG(C2,C1,C2,-N);
  Check(SameArr(C3, C1, msg, x*MachEps),msg0+' #2'+msg);
end;

procedure Check_ForwardInverse.VerifyDFT_CFTG;
begin
  msg0 := 'Fail VerifyDFT_CFTG - Forward ';
  N := RndNum(2,8); N := N+N;
  C1 := TCArray1D.Create(1,N);
  C3 := TCArray1D.Create(1,N);
  C4 := TCArray1D.Create(1,N);
  RandArr1D(C1,-1.1,1.1);
  C2 := DFT(C1);
  CFTG(C1,C3,C4,+N);
  x := N; x := x*x*x;
  Check(SameArr(C3, C2, msg, x*MachEps),msg0+msg);

  msg0 := 'Fail VerifyDFT_CFTG - Inverse ';
  RandArr1D(C1,-1.1,1.1);
  C2 := InvDFT(C1);
  CFTG(C1,C3,C4,-N);
  Check(SameArr(C3, C2, msg, x*MachEps),msg0+msg);
end;

procedure Check_ForwardInverse.VerifyDFT_CFTG_TCArr1D;
begin
  msg0 := 'Fail VerifyDFT_CFTG - Forward ';
  N := RndNum(2,8); N := N+N;
  C1 := TCArr1D.Create(1,N);
  C3 := TCArr1D.Create(1,N);
  C4 := TCArr1D.Create(1,N);
  RandArr1D(C1,-1.1,1.1);
  C2 := DFT(C1);
  CFTG(C1,C3,C4,+N);
  x := N; x := x*x*x;
  Check(SameArr(C3, C2, msg, x*MachEps),msg0+msg);

  msg0 := 'Fail VerifyDFT_CFTG - Inverse ';
  RandArr1D(C1,-1.1,1.1);
  C2 := InvDFT(C1);
  CFTG(C1,C3,C4,-N);
  Check(SameArr(C3, C2, msg, x*MachEps),msg0+msg);
end;

initialization

  RegisterTest('uLinAlg Tests',Suite);

end.

