{@abstract(Tests for Data and DFT)
@author(Nikolai Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2003.01.01)
@lastmod(2004.12.12)
Nikolai V. Shokhirev, 2003-2004 }
unit Test_uFourier;

interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uDynArrays,
  uFourier,
  uDFT;

type

Check_ZeroToCentric = class(TTestCase)
private
//  A: TFArr;
  Cinp, Cout, Cout0: ICArr1D;
  i, j, d1, l1, h1: integer;
  eps: TFloat;
  n, m: TFloat;
  msg: string;
private
  procedure SetData0(n: TInt);
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure ZeroToCentric8;
  procedure ZeroToCentric7;
  procedure CentricToZero8;
  procedure CentricToZero7;
end;

Check_FFTG = class(TTestCase)
private
  Cinp, Cout, CScr, CInv: ICArr1D;
  i, j, d1, l1, h1: integer;
  eps: TFloat;
  n, m: Complex;
  msg: string;
private
  procedure SetDataNK(N, K: TInt; MultI: boolean);
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyRealK;
  procedure VerifyRealKInv;
  procedure VerifyImK;
  procedure VerifyImKInv;
  procedure VerifyCentered;
end;

Check_Tdft1D = class(TTestCase)
private
  A: Idft1D;
  F1, F2: IFArr1D;
  C1, C2: ICArr1D;
  N, err: integer;
  x, eps, del: TFloat;
  msg, msg0: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure Verifydft1DCreate;
  procedure VerifyForwardInverse;
  procedure VerifyForwardInvSignal;
  procedure VerifyHarmonicRe;
  procedure VerifyHarmonicIm;
end;


function Suite : ITestSuite;

implementation

uses
  uDynArrUtils, uDynLinAlg, math,
  uDynArrUtilsX,
  uDynArrIO,
  uDynObjAlg;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDFT Tests');

  result.addTest(Check_ZeroToCentric.Suite);
  result.addTest(Check_FFTG.Suite);
  result.addTest(Check_Tdft1D.Suite);
end;

{ Check_ZeroToCentric }

procedure Check_ZeroToCentric.SetData0(n: TInt);
var
  i, h, l: TInt;
begin
  n := 7;
  l := MinFreqIndex(n);
  h := MaxFreqIndex(n);
  Cout0 := TCArr1D.Create(l,h);
  Cout := TCArr1D.Create(n);
  Cinp := TCArr1D.Create(0,n-1);
  for i := l to h do Cout0[i] := cmplx(i,-i);
  for i := 0 to h do Cinp[i] := cmplx(i,-i);
  for i := h+1 to n-1 do Cinp[i] := cmplx(i-n,n-i);
end;

procedure Check_ZeroToCentric.setUp;
begin
  d1 := 16;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_ZeroToCentric.tearDown;
begin
end;

procedure Check_ZeroToCentric.ZeroToCentric7;
begin
  SetData0(7);
  ZeroToCentric(Cinp, Cout);
  Check(SameArr(Cout, Cout0,msg,eps), 'Fail ZeroToCentric7:'+msg);
end;

procedure Check_ZeroToCentric.ZeroToCentric8;
begin
  SetData0(8);
  ZeroToCentric(Cinp, Cout);
  Check(SameArr(Cout, Cout0,msg,eps), 'Fail ZeroToCentric8:'+msg);
end;

procedure Check_ZeroToCentric.CentricToZero7;
begin
  SetData0(7);
  CentricToZero(Cout0, Cout);
  Check(SameArr(Cout, Cinp,msg,eps), 'Fail CentricToZero7:'+msg);
end;

procedure Check_ZeroToCentric.CentricToZero8;
begin
  SetData0(8);
  CentricToZero(Cout0, Cout);
  Check(SameArr(Cout, Cinp,msg,eps), 'Fail CentricToZero8:'+msg);
end;

{ Check_FFTG }

procedure Check_FFTG.setUp;
begin
  d1 := 16;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_FFTG.tearDown;
begin
end;

procedure Check_FFTG.SetDataNK(N, K: TInt; MultI: boolean);
var
  j, m: TInt;
  t, rN, a , f, ff: TFloat;
  s: string;
  z: Complex;
begin
  rN := 1.0/N;

  Cinp := TCArr1D.Create(N);
  Cout := TCArr1D.Create(N);
  CScr := TCArr1D.Create(N);
  CInv := TCArr1D.Create(N);

  f := K;
  ff := (1+abs(K));
  for j := 0 to N-1 do
begin
    a := 2.0*Pi*f*j/N;
    if MultI then
      Cinp[j+1] := cmplx(-sin(a)*ff, cos(a)*ff)
    else
      Cinp[j+1] := cmplx(cos(a)*ff, sin(a)*ff);
end;
end;

procedure Check_FFTG.VerifyRealK;
var
  N, K: TInt;
begin
  N := 8;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, false);
    FFTG(Cinp,Cout,CScr,ttDirect);
    Cout[K] := Csub( Cout[K],(1+abs(K)) );
    Check(SameValue(Cout.MaxAbs,0.0,eps), 'Fail VerifyRealK8, K='+IntToStr(K));
end;
  N := 7;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, false);
    FFTG(Cinp,Cout,CScr,ttDirect);
    Cout[K] := Csub( Cout[K],(1+abs(K)) );
    Check(SameValue(Cout.MaxAbs,0.0,eps), 'Fail VerifyRealK7, K='+IntToStr(K));
end;
end;

procedure Check_FFTG.VerifyRealKInv;
var
  N, K: TInt;
begin
  N := 8;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, false);
    FFTG(Cinp,Cout,CScr,ttDirect);
    FFTG(Cout,CInv,CScr,ttInverse);
    Check(SameArr(Cinp, Cinv,msg,eps), 'Fail VerifyRealKInv8:'+msg);
end;
  N := 7;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, false);
    FFTG(Cinp,Cout,CScr,ttDirect);
    FFTG(Cout,CInv,CScr,ttInverse);
    Check(SameArr(Cinp, Cinv,msg,eps), 'Fail VerifyRealKInv7:'+msg);
end;
end;

procedure Check_FFTG.VerifyImK;
var
  N, K: TInt;
begin
  N := 8;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, true);
    FFTG(Cinp,Cout,CScr,ttDirect);
    Cout[K] := Csub( Cout[K],cmplx(0.0,(1+abs(K))) );
    Check(SameValue(Cout.MaxAbs,0.0,eps), 'Fail VerifyImK8, K='+IntToStr(K));
end;
  N := 7;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, true);
    FFTG(Cinp,Cout,CScr,ttDirect);
    Cout[K] := Csub( Cout[K],cmplx(0.0,(1+abs(K))) );
    Check(SameValue(Cout.MaxAbs,0.0,eps), 'Fail VerifyImK7, K='+IntToStr(K));
  end;
end;

procedure Check_FFTG.VerifyImKInv;
var
  N, K: TInt;
begin
  N := 8;
  for K := 0 to N-1 do
begin
    SetDataNK(N, K, true);
    FFTG(Cinp,Cout,CScr,ttDirect);
    FFTG(Cout,CInv,CScr,ttInverse);
    Check(SameArr(Cinp, Cinv,msg,eps), 'Fail VerifyImKInv8:'+msg);
  end;
  N := 7;
  for K := 0 to N-1 do
  begin
    SetDataNK(N, K, true);
    FFTG(Cinp,Cout,CScr,ttDirect);
    FFTG(Cout,CInv,CScr,ttInverse);
    Check(SameArr(Cinp, Cinv,msg,eps), 'Fail VerifyImKInv7:'+msg);
  end;
end;

procedure Check_FFTG.VerifyCentered;
var
  N, K: TInt;
begin
  N := 8;
  for K := MinFreqIndex(N) to MaxFreqIndex(N) do
  begin
    SetDataNK(N, K, true);
    FFTG(Cinp,Cout,CScr,ttDirect);
    ZeroToCentric(Cout, CScr);
    CScr[K] := Csub( CScr[K],cmplx(0.0,(1+abs(K))) );
    Check(SameValue(CScr.MaxAbs,0.0,eps), 'Fail VerifyCentered8, K='+IntToStr(K));
  end;
  N := 7;
  for K := MinFreqIndex(N) to MaxFreqIndex(N) do
  begin
    SetDataNK(N, K, true);
    FFTG(Cinp,Cout,CScr,ttDirect);
    ZeroToCentric(Cout, CScr);
    CScr[K] := Csub( CScr[K],cmplx(0.0,(1+abs(K))) );
    Check(SameValue(CScr.MaxAbs,0.0,eps), 'Fail VerifyCentered7, K='+IntToStr(K));
  end;
end;

{ Check_Tdft1D }

procedure Check_Tdft1D.setUp;
begin
  inherited;
  eps := 1.0e-12;
  Randomize;
end;

procedure Check_Tdft1D.tearDown;
begin
  inherited;

end;

procedure Check_Tdft1D.Verifydft1DCreate;
var
  i: TInt;
  fi, fN : TFloat;
  MatU, MatV: boolean;
  c: complex;
begin
  msg0 := 'Fail Verifydft1DCreate - ';
  N := RndNum(7,16);
  fN := N;
  A := Tdft1D.Create(N);
  Check(IsEqual(1, A.Data1D.Lo1, msg),msg0+'Data.Lo1:'+ msg);
  Check(IsEqual(N, A.Data1D.Hi1, msg),msg0+'Data.Hi1:'+ msg);
  Check(IsEqual(1, A.dft1D1.Lo1, msg),msg0+'dft1.Lo1:'+ msg);
  Check(IsEqual(N, A.dft1D1.Hi1, msg),msg0+'dft1.Hi1:'+ msg);
  Check(IsEqual(-((N-1) div 2), A.dft1DCenter.Lo1, msg),msg0+'dft1DCenter.Lo1:'+ msg);
  Check(IsEqual((N div 2), A.dft1DCenter.Hi1, msg),msg0+'dft1DCenter.Hi1:'+ msg);

  for i := 1 to N do
  begin
    fi := i-1;
    A.Data1D[i] := cmplx(fi,fN-fi-1.0);
    A.dft1D1[i] := A.Data1D[i];
  end;
  Check(SameArr(A.Data1D, A.dft1D1, msg, eps),msg0+'Data=Generic:'+ msg);

  for i := 1 to N do
    Check(IsEqual(A.dft1D1[i],A.dft1D0[i-1],msg),msg0+'dft1[i] <> dft0[i-1] at i='+IntToStr(i)+':'+msg);

  for i := 1 to N do
    Check(IsEqual(A.Data1D[i],A.Signal1D[i-1],msg),msg0+'dft1[i] <> dft0[i-1] at i='+IntToStr(i)+':'+msg);

  for i := A.dft1DCenter.Lo1 to A.dft1DCenter.Hi1 do
  begin
    fi := i;
    if i < 0 then
      c := cmplx(fN+fi,-fi-1.0)
    else
      c := cmplx(fi,(fN-fi-1.0));
    Check(IsEqual(c,A.dft1DCenter[i],msg),msg0+'Freq at i='+IntToStr(i)+':'+msg);
  end;
end;

procedure Check_Tdft1D.VerifyForwardInverse;
begin
  msg0 := 'Fail VerifyForwardInverse - Data';
  N := RndNum(7,16);
  A := Tdft1D.Create(N);
  C1 := TCArray1D.Create(1,N);
  RandArr1D(A.Data1D,-1.1,1.1);
  C1.Assign(A.Data1D);
  A.ForwardDFT;
  A.Data1D.Fill(Cmplx0);
  A.InverseDFT;
  x := N; x := x*x*x;
  Check(SameArr(A.Data1D, C1, msg, x*MachEps),msg0+msg);
end;

procedure Check_Tdft1D.VerifyForwardInvSignal;
begin
  msg0 := 'Fail VerifyForwardInverse - Signal';
  N := RndNum(7,16);
  A := Tdft1D.Create(N);
  C1 := TCArray1D.Create(0,N-1);
  RandArr1D(A.Data1D,-1.1,1.1);
  C1.Assign(A.Signal1D);
  A.ForwardDFT;
  A.Signal1D.Fill(Cmplx0);
  A.InverseDFT;
  x := N; x := x*x*x;
  Check(SameArr(A.Signal1D, C1, msg, x*MachEps),msg0+msg);
end;

procedure Check_Tdft1D.VerifyHarmonicRe;
var
  M, j, k: TInt;
begin
  msg0 := 'Fail VerifyHarmonicRe M=';
  N := RndNum(7,16);
  A := Tdft1D.Create(N);
  C1 := TCArray1D.Create(1,N,atCentered);
  for M := A.dft1DCenter.Lo1 to A.dft1DCenter.Hi1 do
  begin
    for j := 0 to N-1 do
      A.Signal1D[j] := CexpIm(2.0*Pi*M*j/N);
    A.ForwardDFT;
    for k := A.dft1DCenter.Lo1 to A.dft1DCenter.Hi1 do
      if k=M then
        Check(IsEqual(Cmplx1, A.dft1DCenter[k], msg),msg0+IntToStr(M)+' : '+ msg)
      else
        Check(IsEqual(Cmplx0, A.dft1DCenter[k], msg),msg0+IntToStr(M)+' : '+ msg);
  end;
end;

procedure Check_Tdft1D.VerifyHarmonicIm;
var
  M, j, k: TInt;
begin
  msg0 := 'Fail VerifyHarmonicIm M=';
  N := RndNum(7,16);
  A := Tdft1D.Create(N);
  C1 := TCArray1D.Create(1,N,atCentered);
  for M := A.dft1DCenter.Lo1 to A.dft1DCenter.Hi1 do
  begin
    for j := 0 to N-1 do
      A.Signal1D[j] := cMul(CexpIm(2.0*Pi*M*j/N),CmplxIm1);
    A.ForwardDFT;
    for k := A.dft1DCenter.Lo1 to A.dft1DCenter.Hi1 do
      if k=M then
        Check(IsEqual(CmplxIm1, A.dft1DCenter[k], msg),msg0+IntToStr(M)+' : '+ msg)
      else
        Check(IsEqual(Cmplx0, A.dft1DCenter[k], msg),msg0+IntToStr(M)+' : '+ msg);
  end;
end;

initialization

  RegisterTest('uLinAlg Tests',Suite);

end.
