{@abstract(uPolynom unit test)
@author(Nikolai V. Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
@created(2004.12.12)
@lastmod(2005.01.22)
Nikolai V. Shokhirev, 2004-2005 }
{ 2005.05.29 Added Tests for TPolynom2D and TPolySystem2D }
unit Test_uPolynom;
interface

uses
  TestFramework,
  SysUtils,
  uMatTypes,
  uPolynom, uDynObjAlg;

type

Check_Polynom = class(TTestCase)
private
  del, eps: TFloat;
  P1, P2: IPolynom;
  d: integer;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyAssign;
  procedure VerifyCreatePoly;
  procedure VerifyCreateCoeff;
  procedure VerifyTimes;
  procedure VerifyAdd;
  procedure VerifySub;
  procedure VerifyMul;
  procedure VerifyIntDiff;
  procedure VerifyEval;
//  procedure VerifySaveLoad;
end;

Check_Polynom2D = class(TTestCase)
private
  del, eps: TFloat;
  P1, P2: IPolynom2D;
  d1, d2: TInt;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyAssign;
  procedure VerifyCreatePoly;
  procedure VerifyCreateCoeff;
  procedure VerifyTimes;
  procedure VerifyAdd;
  procedure VerifySub;
  procedure VerifyMul;
  procedure VerifyIntDiff;
  procedure VerifyEval;
  procedure VerifyEval1;
  procedure VerifyEval2;
  procedure VerifyPolynom1;
  procedure VerifyPolynom2;
//  procedure VerifySaveLoad;
end;

Check_PolySys = class(TTestCase)
private
  eps: TFloat;
  P1, P2: IPolynom;
  S1, S2: IPolySystem;
  d: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyAdd;
  procedure VerifySaveLoad;
  procedure VerifyDel;
  procedure VerifyChange;
  procedure VerifyGrammSchmidt;
end;

Check_PolySys2D = class(TTestCase)
private
  eps: TFloat;
  P1, P2: IPolynom2D;
  S1, S2: IPolySystem2D;
  d: integer;
  msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure VerifyAdd;
  procedure VerifySaveLoad;
  procedure VerifyDel;
  procedure VerifyGrammSchmidt2D;
end;

function Suite : ITestSuite;

implementation

uses
  math, TypInfo, uDynArrays, uDynArrUtils, uDynArrUtilsX, uDynData;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('Polynom Tests');
  result.addTest(Check_Polynom.Suite);
  result.addTest(Check_Polynom2D.Suite);
  result.addTest(Check_PolySys.Suite);
  result.addTest(Check_PolySys2D.Suite);
//  result.addTest(testSuiteOf(Check_TBTensor1D));
end;


{ Check_Polynom }

procedure Check_Polynom.setUp;
begin
  inherited;
  eps := 1.0E-12;
  d := 5;
  Randomize;
end;

procedure Check_Polynom.tearDown;
begin
  inherited;

end;

procedure Check_Polynom.VerifyCreatePoly;
begin
  P1 := TPolynom.Create(d);
  RandPolynom(P1);
  P2 := TPolynom.Create(P1);
  P2.Assign(P1);
  Check(SamePolynom(P1,P2,eps, true),'Fail VerifyCreatePoly' );
end;

procedure Check_Polynom.VerifyAssign;
begin
  P1 := TPolynom.Create(d);
  RandPolynom(P1);
  P2 := TPolynom.Create;
  P2.Assign(P1);
  Check(SamePolynom(P1,P2,eps, true),'Fail VerifyAssign' );
end;

procedure Check_Polynom.VerifyCreateCoeff;
var
  s: string;
begin
  P1 := TPolynom.Create(d);
  RandPolynom(P1);
  s := P1.Coefficients;
  P2 := TPolynom.Create(s);
  Check(SamePolynom(P1,P2,eps, true),'Fail VerifyCreateCoeff' );
end;

procedure Check_Polynom.VerifyTimes;
var
  k: TInt;
  c: TFloat;
begin
  P1 := TPolynom.Create(d);
  RandPolynom(P1);
  P2 := TPolynom.Create(P1);
  c := 1.23456789;
  for k := 0 to P1.Degree do P1[k] := c*P1[k];
  P2.Times(c);
  Check(SamePolynom(P1,P2,eps, true),'Fail VerifyTimes' );
end;

procedure Check_Polynom.VerifyAdd;
var
  k: TInt;
  c: TFloat;
  P3, P4: IPolynom;
begin
  P2 := TPolynom.Create(d);
  RandPolynom(P2);
  P3 := TPolynom.Create(d);
  RandPolynom(P3);
  P2 := TPolynom.Create(d);
  c := 1.23456789;
  P1 := TPolynom.Create(d);
  for k := 0 to P1.Degree do P1[k] := P2[k]+c*P3[k];
  P4 := P2.Add(P3,c);
  Check(SamePolynom(P1,P4,eps, true),'Fail VerifyAdd' );
end;

procedure Check_Polynom.VerifySub;
var
  k: TInt;
  c: TFloat;
  P3, P4: IPolynom;
begin
  P2 := TPolynom.Create(d);
  RandPolynom(P2);
  P3 := TPolynom.Create(d);
  RandPolynom(P3);
  P2 := TPolynom.Create(d);
  c := 1.23456789;
  P1 := TPolynom.Create(d);
  for k := 0 to P1.Degree do P1[k] := P2[k]-c*P3[k];
  P4 := P2.Sub(P3,c);
  Check(SamePolynom(P1,P4,eps, true),'Fail VerifySub' );
end;

procedure Check_Polynom.VerifyMul;
var
  P3, P4: IPolynom;
begin
  P2 := TPolynom.Create(d);
  RandPolynom(P2);
  P3 := TPolynom.Create(4);
  RandPolynom(P3);
  P1 := PxP(P2, P3);
  P4 := P2.Mul(P3);
  Check(SamePolynom(P1,P4,eps),'Fail VerifyMul' );
end;

procedure Check_Polynom.VerifyIntDiff;
var
  P3: IPolynom;
begin
  P2 := TPolynom.Create(d);
  RandPolynom(P2);
  P1 := P2.Integrate;
  P3 := P1.Diff;
  Check(SamePolynom(P2,P3,eps),'Fail VerifyIntDiff' );
  P1 := P2.Diff;
  P3 := P1.Integrate;
  P2[0] := 0.0; // integration const is 0
  Check(SamePolynom(P2,P3,eps),'Fail VerifyDiffInt' );
end;

procedure Check_Polynom.VerifyEval;
var
  i, k: TInt;
  x, y: TFloat;
begin
  del := 0.0;
  P1 := TPolynom.Create(d);
  RandPolynom(P1);
  for i := 1 to 11 do
  begin
    x := 10.0*(random-0.5);
    y := 0.0;
    for k := 0 to P1.Degree do y := y + P1[k]*IntPower(x,k);
    del := max(del, abs(y-P1.Eval(x)));
  end;
  Check(abs(del) < 3.0*eps,'Fail VerifyEval, del='+FloatToStr(del));
end;

{ Check_Polynom2D }

procedure Check_Polynom2D.setUp;
begin
  inherited;
  eps := 1.0E-11;
  d1 := 5;
  d2 := 4;
end;

procedure Check_Polynom2D.tearDown;
begin
  inherited;

end;

procedure Check_Polynom2D.VerifyAdd;
var
  k1, k2: TInt;
  c: TFloat;
  P3, P4: IPolynom2D;
begin
//  del := 0.0;
  P2 := TPolynom2D.Create(d1,d2);
  for k1 := 0 to P2.Degree1 do
    for k2 := 0 to P2.Degree2 do
      P2[k1,k2] := 10.0*(random-0.5);
  P3 := TPolynom2D.Create(d1,d2);
  for k1 := 0 to P3.Degree1 do
    for k2 := 0 to P3.Degree2 do
      P3[k1,k2] := 10.0*(random-0.5);
  P2 := TPolynom2D.Create(d1,d2);
  c := sqrt(2.0);
  P1 := TPolynom2D.Create(d1,d2);
  for k1 := 0 to P1.Degree1 do
    for k2 := 0 to P1.Degree2 do
      P1[k1,k2] := P2[k1,k2]+c*P3[k1,k2];
  P4 := P2.Add(P3,c);
  Check(SamePolynom2D(P1,P4,eps, true),'Fail VerifyAdd' );
end;

procedure Check_Polynom2D.VerifyAssign;
var
  k1, k2: TInt;
begin
  P1 := TPolynom2D.Create(d1,d2);
  for k1 := 0 to P1.Degree1 do
    for k2 := 0 to P1.Degree2 do
      P1[k1,k2] := 10.0*(random-0.5);
  P2 := TPolynom2D.Create;
  P2.Assign(P1);
  Check(SamePolynom2D(P1,P2,eps, true),'Fail VerifyAssign' );
end;

procedure Check_Polynom2D.VerifyCreateCoeff;
var
  s: string;

  procedure CreateRandom(d1, d2: integer);
  var
    k1, k2: TInt;
  begin
    P1 := TPolynom2D.Create(d1,d2);
    for k1 := 0 to P1.Degree1 do
      for k2 := 0 to P1.Degree2 do
        P1[k1,k2] := 10.0*(random-0.5);
  end;

begin

  CreateRandom(0, 0);
  s := P1.Coefficients;
  P2 := TPolynom2D.Create(s);
  Check(SamePolynom2D(P1,P2,eps, true),'Fail VerifyCreateCoeff 0 0' );

  CreateRandom(d1, 0);
  s := P1.Coefficients;
  P2 := TPolynom2D.Create(s);
  Check(SamePolynom2D(P1,P2,eps, true),'Fail VerifyCreateCoeff d1, 0' );

  CreateRandom(0, d2);
  s := P1.Coefficients;
  P2 := TPolynom2D.Create(s);
  Check(SamePolynom2D(P1,P2,eps, true),'Fail VerifyCreateCoeff 0,d2' );

  CreateRandom(d1, d2);
  s := P1.Coefficients;
  P2 := TPolynom2D.Create(s);
  Check(SamePolynom2D(P1,P2,eps, true),'Fail VerifyCreateCoeff d1, d2' );
end;

procedure Check_Polynom2D.VerifyCreatePoly;
begin
  P1 := TPolynom2D.Create(d1,d2);
  RandPolynom2D(P1);
  P2 := TPolynom2D.Create(P1);
  P2.Assign(P1);
  Check(SamePolynom2D(P1,P2,eps, true),'Fail VerifyCreatePoly' );
end;

procedure Check_Polynom2D.VerifyEval;
var
  k1, k2: TInt;
  x1, x2, y: TFloat;
begin
  P1 := TPolynom2D.Create(d1,d2);
  RandPolynom2D(P1);

  x1 := 10.0*(random-0.5);
  x2 := 10.0*(random-0.5);
  y := 0.0;
  for k1 := 0 to P1.Degree1 do
    for k2 := 0 to P1.Degree2 do
      y := y + P1[k1,k2]*IntPower(x1,k1)*IntPower(x2,k2);
  del := y-P1.Eval(x1,x2);

  Check(abs(del) < 16.0*eps,'Fail VerifyEval, del='+FloatToStr(del));
end;

procedure Check_Polynom2D.VerifyEval1;
var
  x1, x2: TFloat;
  Q: IPolynom;
begin
  P1 := TPolynom2D.Create(d1,d2);
  RandPolynom2D(P1);
  x1 := 10.0*(random-0.5);
  x2 := 10.0*(random-0.5);
  Q := P1.Polynom1(x2);
  del := Q.Eval(x1)-P1.Eval(x1,x2);
  Check(abs(del) < 16.0*eps,'Fail VerifyEval1, del='+FloatToStr(del));
end;

procedure Check_Polynom2D.VerifyEval2;
var
  x1, x2: TFloat;
  Q: IPolynom;
begin
  P1 := TPolynom2D.Create(d1,d2);
  RandPolynom2D(P1);
  x1 := 10.0*(random-0.5);
  x2 := 10.0*(random-0.5);
  Q := P1.Polynom2(x1);
  del := Q.Eval(x2)-P1.Eval(x1,x2);
  Check(abs(del) < 16.0*eps,'Fail VerifyEval2, del='+FloatToStr(del));
end;

procedure Check_Polynom2D.VerifyPolynom1;
var
  k1, k2: TInt;
  x2, y: TFloat;
  Q1, Q2: IPolynom;
begin
  P1 := TPolynom2D.Create(d1,d2);
  RandPolynom2D(P1);
  x2 := 10.0*(random-0.5);
  Q1 := P1.Polynom1(x2);
  Q2 := Tpolynom.Create(P1.Degree1);
  for k1 := 0 to P1.Degree1 do
  begin
    y := 0.0;
    for k2 := 0 to P1.Degree2 do
      y := y + P1[k1,k2]*IntPower(x2,k2);
    Q2[k1] := y;
  end;
  Check(SamePolynom(Q1,Q2,2.0*eps, true),'Fail VerifyPolynom1' );
end;

procedure Check_Polynom2D.VerifyPolynom2;
var
  k1, k2: TInt;
  x1, y: TFloat;
  Q1, Q2: IPolynom;
begin
  P1 := TPolynom2D.Create(d1,d2);
  RandPolynom2D(P1);
  x1 := 10.0*(random-0.5);
  Q1 := P1.Polynom2(x1);
  Q2 := Tpolynom.Create(P1.Degree2);
  for k2 := 0 to P1.Degree2 do
  begin
    y := 0.0;
    for k1 := 0 to P1.Degree1 do
      y := y + P1[k1,k2]*IntPower(x1,k1);
    Q2[k2] := y;
  end;
  Check(SamePolynom(Q1,Q2,2.0*eps, true),'Fail VerifyPolynom1' );
end;

procedure Check_Polynom2D.VerifyIntDiff;
var
  k1, k2: TInt;
  P3: IPolynom2D;
  idx: IndexType;

begin
  P2 := TPolynom2D.Create(d1,d2);
  for idx := Index1 to Both do
  begin
    RandPolynom2D(P2);
    P1 := P2.Integrate(idx);
    P3 := P1.Diff(idx);
    Check(SamePolynom2D(P2,P3,eps),'Fail VerifyIntDiff I-D for '+
                                     getEnumName(typeInfo(IndexType),Ord(idx)));
    P1 := P2.Diff(idx);
    P3 := P1.Integrate(idx);
    case idx of
    Index1: for k2 := 0 to P2.Degree2 do
              P2[0, k2] := 0.0; // integration const is 0
    Index2: for k1 := 0 to P2.Degree1 do
              P2[k1, 0] := 0.0; // integration const is 0
    Both: begin
            for k1 := 0 to P2.Degree1 do P2[k1, 0] := 0.0;
            for k2 := 0 to P2.Degree2 do P2[0, k2] := 0.0;
          end;
    end;
    Check(SamePolynom2D(P2,P3,eps),'Fail VerifyDiffInt D-I for '+
                                     getEnumName(typeInfo(IndexType),Ord(idx)));
  end;
end;

procedure Check_Polynom2D.VerifyMul;
begin
  Check(true,'Fail VerifyCreateCoeff' );
end;

procedure Check_Polynom2D.VerifySub;
begin
  Check(true,'Fail VerifyCreateCoeff' );
end;

procedure Check_Polynom2D.VerifyTimes;
begin
  Check(true,'Fail VerifyCreateCoeff' );
end;

{ Check_PolySys }

procedure Check_PolySys.setUp;
begin
  inherited;
  eps := 1.0E-12;
  d := 5;
end;

procedure Check_PolySys.tearDown;
begin
  inherited;

end;

procedure Check_PolySys.VerifyAdd;
var
  i: integer;
begin
  S1 := TPolySystem.Create;
  S2 := TPolySystem.Create(d);
  for i := 0 to d do
  begin
    P1 := Tpolynom.Create(i);
    RandPolynom(P1);
    S1.Add(P1);
    S2.Polynom[i] := P1;
  end;
  for i := 0 to S1.MaxDegree do
  begin
    P1 := S1[i];
    P2 := S2[i];
    Check(SamePolynom(P1,P2, eps, true),'Fail VerifyAdd, Deg='+IntToStr(i));
  end;
end;

procedure Check_PolySys.VerifyChange;
var
  i: integer;
begin
  S1 := TPolySystem.Create;
  S2 := TPolySystem.Create;
  for i := 0 to d do
  begin
    P1 := Tpolynom.Create(i);
    RandPolynom(P1);
    S1.Add(P1);
    if i = 3 then
      S2.Add(TPolynom.Create(P1)) // Different objects
    else
      S2.Add(P1); // same objects
  end;
  S2[2].Coeff[0] := 1.23;
  S2[3].Coeff[2] := Pi;
  for i := 0 to S1.MaxDegree do
  begin
    P1 := S1[i];
    P2 := S2[i];
    if i <> 3 then
      Check( SamePolynom(P1,P2, eps, true),'Fail VerifyDel, Deg='+IntToStr(i))
    else
      Check( not SamePolynom(P1,P2, eps, true),'Fail VerifyDel, Deg='+IntToStr(i));
  end;
end;

procedure Check_PolySys.VerifyDel;
var
  i: integer;
begin
  S1 := TPolySystem.Create;
  S2 := TPolySystem.Create(d);
  for i := 0 to d do
  begin
    P1 := Tpolynom.Create(i);
    RandPolynom(P1);
    if i <> 3 then
      S1.Add(P1);
    S2.Polynom[i] := P1;
  end;
  S2.Delete(3);
  for i := 0 to S1.MaxDegree do
  begin
    P1 := S1[i];
    P2 := S2[i];
    Check(SamePolynom(P1,P2, eps, true),'Fail VerifyDel, Deg='+IntToStr(i));
  end;
end;

procedure Check_PolySys.VerifyGrammSchmidt;
var
  X: IFArr1D;
  S, E: IFArr2D;
  t: TFloat;
  i, j, idx: TInt;
//  c: string;
begin
  S1 := GenPolySys(d);
  X := TFArr1D.Create(0,d);

  for i := 0 to d do X[i] := i/d;

  idx := 0;
  for i := 0 to d do
  begin
    if not NormalizePoly(S1[i], X) then break;
    inc(idx);
//    c := S1[i].Coefficients;
    for j := i+1 to d do
    begin
      t := DotProdPoly(S1[i], S1[j], X);
      S1[j].Combine(S1[i],-t);
    end;
  end;

  Check(idx > d,'Fail VerifyGrammSchmidt: Cannot normalize for idx='+IntToStr(idx));

  E := IdentityMt(d+1,0);
  S := TFArr2D.Create(E);
  for i := 0 to d do
  begin
    for j := 0 to d do
    begin
      S[i,j] := DotProdPoly(S1[i], S1[j], X);
    end;
  end;
  Check( SameArr(E, S, msg, eps),'Fail VerifyGrammSchmidt:'+msg );
end;

procedure Check_PolySys.VerifySaveLoad;
var
  i: integer;
begin
  S1 := TPolySystem.Create;
  for i := 0 to d do
  begin
    P1 := Tpolynom.Create(i);
    RandPolynom(P1);
    S1.Add(P1);
  end;
  S1.Comment := 'test1';
  S1.SaveToFile('test1.txt');
  S2 := TPolySystem.Create('test1.txt');
  for i := 0 to S1.MaxDegree do
  begin
    P1 := S1[i];
    P2 := S2[i];
    Check(SamePolynom(P1,P2, eps, true),'Fail VerifySaveLoad, Deg='+IntToStr(i));
  end;
end;

{ Check_PolySys2D }

procedure Check_PolySys2D.setUp;
begin
  inherited;
  eps := 1.0E-11;
  d := 3;
end;

procedure Check_PolySys2D.tearDown;
begin
  inherited;

end;

procedure Check_PolySys2D.VerifyAdd;
var
  i, d1, d2: integer;
begin
  S1 := TPolySystem2D.Create;
  S2 := TPolySystem2D.Create;
  for d2 := 0 to d do
    for d1 := 0 to d do
    begin
      P1 := TPolynom2D.Create(d1,d2);
      RandPolynom2D(P1);
      S1.Add(P1);
      S2.Add(P1);
    end;
  for i := 0 to S1.Count-1 do
  begin
    P1 := S1[i];
    P2 := S2[i];
    Check(SamePolynom2D(P1,P2, eps, true),'Fail VerifyAdd, idx='+IntToStr(i));
  end;
end;

procedure Check_PolySys2D.VerifyDel;
var
  i, d1, d2: integer;
begin
  S1 := TPolySystem2D.Create;
  S2 := TPolySystem2D.Create;
  i := 0;
  for d2 := 0 to d do
    for d1 := 0 to d do
    begin
      P1 := TPolynom2D.Create(d1,d2);
      RandPolynom2D(P1);
      if (i<>3) and (i<>6) then
        S1.Add(P1);
      S2.Add(P1);
      inc(i);
    end;
  S2.Delete(6);
  S2.Delete(3);
  for i := 0 to S1.Count-1 do
  begin
    P1 := S1[i];
    P2 := S2[i];
    Check(SamePolynom2D(P1,P2, eps, true),'Fail VerifyDel, idx='+IntToStr(i));
  end;
end;

procedure Check_PolySys2D.VerifyGrammSchmidt2D;
var
  m, k: integer;
  R: IFData1D;
  M1, M2: IFArr2D;
begin
  S1 := GenPolySys2D(2,true);
  R := TFData1D.Create(9);
  RandArr1D(R.x1, 0.0, 2.0);
  RandArr1D(R.x2, 0.0, 2.0);
  GrammSchmidt2D(S1, R);

  M1 := TFArr2D.Create(0,S1.Count-1,0,S1.Count-1);
  M2 := TFArr2D.Create(M1);
  for m := 0 to S1.Count-1 do
  begin
    for k := 0 to S1.Count-1 do
    begin
      M2[k,m] := DotProdPoly2D(S1[m], S1[k], R);
      M1[m,k] := 0.0;
    end;
    M1[m,m] := 1.0;
  end;

  Check(SameArr(M1,M2,msg,eps),'Fail VerifyGrammSchmidt2D: '+msg);
end;

procedure Check_PolySys2D.VerifySaveLoad;
var
  i, d1, d2: integer;
begin
  S1 := TPolySystem2D.Create;
  for d2 := 0 to d do
    for d1 := 0 to d do
    begin
      P1 := TPolynom2D.Create(d1,d2);
      RandPolynom2D(P1);
      S1.Add(P1);
    end;
  S1.Comment := 'test2';
  S1.SaveToFile('test2.txt');
  S2 := TPolySystem2D.Create('test2.txt');
  for i := 0 to S1.Count-1 do
  begin
    P1 := S1[i];
    P2 := S2[i];
    Check(SamePolynom2D(P1,P2, eps, true),'Fail VerifyAdd, idx='+IntToStr(i));
  end;
end;

initialization

  RegisterTest('uPolynom Tests',Suite);

end.
