{*
 PasMatLib - Utils Test
 Nikolai V. Shokhirev, 2003-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2003.01.01 - created
 @version 2007.11.11 - added date-time tests
 }
unit Test_uUtils;

interface

uses
  TestFramework
  , uMatTypes
  , Classes
  , uFileReader
  , uTimeObjects
  ;

type

Check_Time = class(TTestCase)
private
  Year, Month, Day: Word;
  sYear, sMonth, sDay: string;
  d, t: TDateTime;
  Hour, Min, Sec, MSec: Word;
  s, ss, msg: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure Validate_DateToYYYYMMDD;
  procedure Validate_YYYYMMDDToDate;
  procedure CheckCompareDateTime;
  procedure CheckTicksDateTime;
  procedure CheckEncodeDateTimeF;
  procedure CheckDecodeDateTimeToStr;
end;

Check_TimeObjects = class(TTestCase)
private
//  Year, Month, Day: Word;
//  sYear, sMonth, sDay: string;
//  d, t: TDateTime;
//  Hour, Min, Sec, MSec: Word;
//  s, ss, msg: string;
//  t1, t2: TTimeSpan;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure CheckCompare;
end;

Check_RandStr = class(TTestCase)
private
  s: string;
  len: integer;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure Validate_RandStr1;
  procedure Validate_RandStr2;
  procedure Validate_RandStr0;
end;

Test_TypeFunctions = class(TTestCase)
private
  s: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure Validate_IsFloat;
  procedure Validate_IsInteger;
  procedure Validate_IsFloat0;
end;

Test_uFileReader = class(TTestCase)
private
  sl1, sl2: TStringList;
  FR: TIncrementalFileReader;
  NLines, TotalLines: integer;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure TestGetLines;
  procedure TestGetLastLine;
end;

Test_uFileUtils = class(TTestCase)
private
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure TestFileSize;
end;

Test_MyStrFun = class(TTestCase)
private
//  s: string;
public
  procedure setUp;  override;
  procedure tearDown; override;
published
  procedure Validate_TimeStepToStr;
  procedure Validate_Split;
  procedure Validate_SplitToStrArr;
  procedure Validate_SplitToSL;
  procedure Validate_SplitToISL;
  procedure Validate_IsDate;
  procedure Validate_IsTime;
  procedure Validate_SplitToStringList;
end;

function Suite : ITestSuite;

implementation

uses
  SysUtils
  , math
  , uStrFun
  , uDateTimeUtils
  , DateUtils
  , uFileUtils
  , uDynArrays
  , uTypedLists
  ;

function Suite : ITestSuite;
begin
  result := TTestSuite.Create('uDynArrays Tests');
  result.addTest(Check_Time.Suite);
  result.addTest(Check_RandStr.Suite);
  result.addTest(Test_TypeFunctions.Suite);
  result.addTest(Test_uFileReader.Suite);
  result.addTest(Test_uFileUtils.Suite);
  result.addTest(Test_MyStrFun.Suite);
  result.addTest(Check_TimeObjects.Suite);
//  result.addTest(Test_MyStrFun.Suite);

end;


{ Check_Time }

procedure Check_Time.setUp;
begin
  inherited;
  msg := 'Fail Validate_';
  sYear := '2007';
  sMonth := '05';
  sDay :='07';
  Year := 2007;
  Month := 5;
  Day := 7;
  Hour := 8;
  Min := 27;
  Sec := 33;
  MSec := 333;
end;

procedure Check_Time.tearDown;
begin
  inherited;

end;

procedure Check_Time.CheckCompareDateTime;
begin
  d := StrToDateTime('11/13/2007 8:27:07 PM');
  t := d + 1.158e-8;
  CheckEquals(-1, CompareDateTimeEx(d, t), 'd < t');
  t := d - 1.158e-8;
  CheckEquals(1, CompareDateTimeEx(d, t), 'd > t');
  t := d + 1.157e-8;
  CheckEquals(0, CompareDateTimeEx(d, t), 'd = t');
end;

procedure Check_Time.CheckEncodeDateTimeF;
begin
  d := EncodeDateTime(Year, Month, Day, Hour, Min, Sec, MSec);
  t := EncodeDateTimeF(Year, Month, Day, Hour, Min, 33.3333);
  Check(d < t, 'd < t');
end;

procedure Check_Time.CheckTicksDateTime;
begin
  d := StrToDateTime('12/30/1899 12:00:01 AM'); // = 1.15740740741e-5
  t := TicksToDateTime(DateTimeToTicks(d));
  CheckEquals(d, t, 2.5e-14,'d = t  1');
  d := StrToDateTime('11/13/2007 8:27:07 PM'); // 39399.852164
  t := TicksToDateTime(DateTimeToTicks(d));
  CheckEquals(d, t, 0,'d = t  2');
end;

procedure Check_Time.Validate_DateToYYYYMMDD;
begin
//  fail('Test Not Implemented Yet');
  d := EncodeDate(StrToInt(sYear), StrToInt(sMonth), StrToInt(sDay));
  s := DateToYYYYMMDD(d);
  ss := sYear + sMonth + sDay;
  CheckEquals(ss, s, msg+'DateToYYYYMMDD');
end;

procedure Check_Time.Validate_YYYYMMDDToDate;
begin
  ss := sYear + sMonth + sDay;
  d := YYYYMMDDToDate(ss);
  s := DateToYYYYMMDD(d);
  CheckEquals(ss, s, msg+'YYYYMMDDToDate');
end;

procedure Check_Time.CheckDecodeDateTimeToStr;
begin
  s := '20071203233845123';
  d := DecodedStrToDateTime(s);
  ss := DecodeDateTimeToStr(d);
  CheckEquals(ss, s, '');
end;

{ Check_RandStr }

procedure Check_RandStr.setUp;
begin
  inherited;
  len := 128;
end;

procedure Check_RandStr.tearDown;
begin
  inherited;

end;

procedure Check_RandStr.Validate_RandStr0;
var
  i : integer;
begin
  s := RandStr(len,1);
  for i := 1 to len do
    Check( (Ord(s[i]) <=127) and (Ord(s[i]) >=32));
end;

procedure Check_RandStr.Validate_RandStr1;
var
  i : integer;
begin
  s := RandStr(len,1);
  for i := 1 to len do
    Check( s[i] in [' ','A'..'Z','a'..'z','0'..'9']);
end;

procedure Check_RandStr.Validate_RandStr2;
var
  i : integer;
begin
  s := RandStr(len,2);
  for i := 1 to len do
    Check( s[i] in [' ','A'..'Z','a'..'z']);
end;

{ Test_TypeFunctions }

procedure Test_TypeFunctions.setUp;
begin
  inherited;

end;

procedure Test_TypeFunctions.tearDown;
begin
  inherited;

end;

procedure Test_TypeFunctions.Validate_IsFloat;
begin
  s := '123.456';
  Check( IsFloat(s), 'Float f');
  s := '123.456e-2';
  Check( IsFloat(s), 'Float e');
  s := '12345';
  Check( not IsFloat(s), 'Integer');
  s := '1/3';
  Check( not IsFloat(s), 'Not Float');
end;

procedure Test_TypeFunctions.Validate_IsFloat0;
begin
  check(IsFloat0('-.123'),'Fail IsFloat0 -.123');
  check(IsFloat0('+.234'),'Fail IsFloat0 +.234');
end;

procedure Test_TypeFunctions.Validate_IsInteger;
begin
  s := '123.0';
  Check( not IsInt(s), 'Float f');
  s := '-1234';
  Check( IsInt(s), 'Integer -');
  s := '12345';
  Check( IsInt(s), 'Integer +');
  s := '1/3';
  Check( not IsInt(s), 'Not integer');
end;

{ Test_uFileReader }

procedure Test_uFileReader.setUp;
var
 i, j: integer;
 s: string;
begin
  inherited;
  sl1 := TStringList.Create;
  TotalLines := RandomRange(111,999);
  NLines := RandomRange(11, 111);
//  TotalLines := 22;
//  NLines := 8;

  for i := 1 to TotalLines do
  begin
    s := '';
    for j := 1 to i do
      s := s + IntToStr(j mod 10);
    sl1.Append(s);
  end;
  sl1.SaveToFile('test.txt');
  sl2 := TStringList.Create;
end;

procedure Test_uFileReader.tearDown;
begin
  inherited;
  sl1.Free;
  sl2.Free;
end;

procedure Test_uFileReader.TestGetLastLine;
begin
  FR := TIncrementalFileReader.Create('Test.txt');
  CheckEquals(sl1[sl1.Count-1], FR.GetLastLine, 'GetLastLine');
  FR.Free;
end;

procedure Test_uFileReader.TestGetLines;
var
  done: boolean;
  i1, i2, j : integer;
  s: string;
begin
  FR := TIncrementalFileReader.Create('Test.txt');
  i1 := 0;
  repeat
    FR.GetLines(sl2, NLines, done);
    s := '';
    i2 := min(i1 + NLines,sl1.Count);
    for j := i1 to i2 - 1 do
      s := s + sl1[j] + CRLF;
    CheckEquals(sl2.Text, s, 'GetLines');
    inc(i1,NLines);
  until done;
  FR.Free;
end;

{ Test_uFileUtils }

procedure Test_uFileUtils.setUp;
begin
  inherited;
  Randomize;
end;

procedure Test_uFileUtils.tearDown;
begin
  inherited;

end;

procedure Test_uFileUtils.TestFileSize;
var
  f: file of byte;
  n1, n2: Int64;
  FileName, s: string;
  i, j, n, m, k: integer;
  sl: TStringList;
begin
  FileName := 'test.txt';
  n :=50000;
  m :=200;
  sl := TStringList.Create;
  try
    for i := 1 to n do
    begin
      k := random(m)+1;
      SetLength(s,k);
      for j := 1 to k do
      begin
        s[j] := Char(random(128)+1);
      end;
      sl.Append(s);
    end;
    sl.SaveToFile(FileName);
  finally
    sl.Free;
  end;

  AssignFile(f, FileName);
  try
    System.Reset(f);
    n1 := System.FileSize(f);
  finally
    CloseFile(f);
  end;

  n2 := uFileUtils.FileSize(FileName);

  CheckEquals(n1, n2, 'int <-> int64');
  Check(DeleteFile(fileName), 'DeleteFile');

end;

{ Test_MyStrFun }

procedure Test_MyStrFun.setUp;
begin
  inherited;
  Randomize;
end;

procedure Test_MyStrFun.tearDown;
begin
  inherited;

end;

procedure Test_MyStrFun.Validate_IsDate;
begin
  check(IsDate('1/23/2009'),'Fail Is Date');
  check(not IsDate('1:23:45'),'Fail Not Is Date 1');
  check(not IsDate('1/23/2009 1:23:45'),'Fail Not Is Date 2');
end;

procedure Test_MyStrFun.Validate_IsTime;
begin
  check(not IsTime('1/23/2009'),'Not Fail Is Time');
  check(IsTime('1:23:45'),'Fail Is Time');
end;

procedure Test_MyStrFun.Validate_Split;
var
  s1, s2: string;
  a1, a2: ISArr1D;

  procedure CheckIt(s2: string);
  var
    i: integer;
  begin
    s1 := StringReplace(s2,',',_t,[rfReplaceAll]);
    a2 := TSArr1D.Create(s2);
    a1 := SplitToISArr(s1, _t);
    Check(SameLim(a1,a2), 's2='+s2+'Split tab: Not SameLim ');
    for i := a2.Lo1 to a2.Hi1 do
      CheckEquals(a1[i], a2[i], 's2='+s2+'Split tab: i = '+IntToStr(i));
  end;

begin
  s2 := '1,12,123,1234,12345';
  CheckIt(s2);

  s2 := ',1,12,123,1234,1234,';
  CheckIt(s2);

  s2 := ',,1,12,123,,';
  CheckIt(s2);

  s2 := '1,,,1234,12345';
  CheckIt(s2);

  s2 := ',,';
  CheckIt(s2);

  s2 := ',';
  CheckIt(s2);

  s2 := '';
  CheckIt(s2);
end;

procedure Test_MyStrFun.Validate_SplitToISL;
var
  s1, s2: string;
  sl1: IStringList;
  sl2: TStringList;

  procedure CheckIt(s2: string);
  var
    i: integer;
  begin
    sl2.Clear;
    s1 := StringReplace(s2,',',_t,[rfReplaceAll]);
    if s2 = '' then sl2.Append(s2)
               else sl2.CommaText := s2;
    sl1 := SplitToISL(s1, _t);
    CheckEquals(sl2.Count, sl1.Count, 's2='+s2+'Split tab: Count ');
    for i := 0 to sl2.Count-1 do
      CheckEquals(sl1[i], sl2[i], 's2='+s2+'Split tab: i = '+IntToStr(i));
  end;

begin
  sl2 := TStringList.Create;

  s2 := '1,12,123,1234,12345';
  CheckIt(s2);

  s2 := ',1,12,123,1234,1234,';
  CheckIt(s2);

  s2 := ',,1,12,123,,';
  CheckIt(s2);

  s2 := '1,,,1234,12345';
  CheckIt(s2);

  s2 := ',,';
  CheckIt(s2);

  s2 := ',';
  CheckIt(s2);

  s2 := '';
  CheckIt(s2);

  sl2.Free;
end;

procedure Test_MyStrFun.Validate_SplitToStrArr;
var
  s: string;
  a1, a2: TSArr;
  Count: TInt;

  procedure CheckIt(a1, a2: TSArr);
  var
    i, L: integer;
  begin
    L := Length(a1);
    for i := 0 to High(a1) do
      CheckEquals(a1[i], a2[i], 's='+s+'Split idx: i = '+IntToStr(i));
  end;
begin
  s := '1,12,123,1234,12345';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '1';
  a2[1] := '12';
  a2[2] := '123';
  a2[3] := '1234';
  a2[4] := '12345';
  CheckIt(a1, a2);

  s := '1,,,1234,12345';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '1';
  a2[1] := '';
  a2[2] := '';
  a2[3] := '1234';
  a2[4] := '12345';
  CheckIt(a1, a2);

  s := ',1,12,123,1234,12345,';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '';
  a2[1] := '1';
  a2[2] := '12';
  a2[3] := '123';
  a2[4] := '1234';
  a2[5] := '12345';
  a2[6] := '';
  CheckIt(a1, a2);

  s := ',,1,12,123,,';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '';
  a2[1] := '';
  a2[2] := '1';
  a2[3] := '12';
  a2[4] := '123';
  a2[5] := '';
  a2[6] := '';
  CheckIt(a1, a2);

  s := ',,';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '';
  a2[1] := '';
  a2[2] := '';
  CheckIt(a1, a2);

  s := ',';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '';
  a2[1] := '';
  CheckIt(a1, a2);

  s := '';
  SetSplitToStrArr(s, ',',a1, Count);
  SetLength(a2, Count);
  a2[0] := '';
  CheckIt(a1, a2);
end;

procedure Test_MyStrFun.Validate_SplitToSL;
var
  s1, s2: string;
  sl1, sl2: TStringList;

  procedure CheckIt(s2: string);
  var
    i: integer;
  begin
    sl2.Clear;
    s1 := StringReplace(s2,',',_t,[rfReplaceAll]);
    if s2 = '' then sl2.Append(s2)
               else sl2.CommaText := s2;
    SplitToSL(s1, _t, sl1);
    CheckEquals(sl2.Count, sl1.Count, 's2='+s2+'Split tab: Count ');
    for i := 0 to sl2.Count-1 do
      CheckEquals(sl1[i], sl2[i], 's2='+s2+'Split tab: i = '+IntToStr(i));
  end;
begin
  sl1 := TStringList.Create;
  sl2 := TStringList.Create;

  s2 := '1,12,123,1234,12345';
  CheckIt(s2);

  s2 := ',1,12,123,1234,1234,';
  CheckIt(s2);

  s2 := ',,1,12,123,,';
  CheckIt(s2);

  s2 := '1,,,1234,12345';
  CheckIt(s2);

  s2 := ',,';
  CheckIt(s2);

  s2 := ',';
  CheckIt(s2);

  s2 := '';
  CheckIt(s2);

  sl1.Free;
  sl2.Free;
end;

procedure Test_MyStrFun.Validate_SplitToStringList;
var
  s1, s2, d: string;
  sl1, sl2: TStringList;

  procedure CheckIt(s2: string);
  var
    i: integer;
  begin
    sl2.Clear;
    s1 := StringReplace(s2,',',d,[rfReplaceAll]);
    if s2 = '' then sl2.Append(s2)
               else sl2.CommaText := s2;
    SplitToStringList(s1, d, sl1);
    CheckEquals(sl2.Count, sl1.Count, 's2='+s2+'Split delim: Count ');
    for i := 0 to sl2.Count-1 do
      CheckEquals(sl1[i], sl2[i], 's2='+s2+'Split delim: i = '+IntToStr(i));
  end;
begin

  d := '_|_';

  sl1 := TStringList.Create;
  sl2 := TStringList.Create;

  s2 := '1,12,123,1234,12345';
  CheckIt(s2);

  s2 := ',1,12,123,1234,1234,';
  CheckIt(s2);

  s2 := ',,1,12,123,,';
  CheckIt(s2);

  s2 := '1,,,1234,12345';
  CheckIt(s2);

  s2 := ',,';
  CheckIt(s2);

  s2 := ',';
  CheckIt(s2);

  s2 := '';
  CheckIt(s2);

  sl1.Free;
  sl2.Free;
end;

procedure Test_MyStrFun.Validate_TimeStepToStr;
var
  f, s1, s2: string;
  i, d, h, m, s, ms: integer;
  t: double;
begin
  s1 := ' 2 d 3 h 4 m 5 s 6 ms';
  s2 :=  TimeStepToStr(2.127835718);
  CheckEquals(s1, s2, 'non-zero');

  s1 := ' 2 d 4 m 6 ms';
  s2 :=  TimeStepToStr(2.002777847);
  CheckEquals(s1, s2, 'two zeroes');

  f := ' %d d %d h %d m %d s %d ms';
  for i := 1 to 22 do
  begin
    d := random(30)+1;
    h := random(23)+1;
    m := random(59)+1;
    s := random(59)+1;
    ms := random(999);
    s1 := Format(f,[d, h, m, s, ms]);

    t := d+(h+(m+(s+ms/1000.0)/60.0)/60.0)/24.0;
    s2 :=  TimeStepToStr(t);
    CheckEquals(s1, s2, 'TimeStepToStr');
  end;
end;

{ Check_TimeObjects }

procedure Check_TimeObjects.CheckCompare;
begin

end;

procedure Check_TimeObjects.setUp;
begin
  inherited;
//  t1, t2: TTimeSpan;
end;

procedure Check_TimeObjects.tearDown;
begin
  inherited;

end;

initialization

  RegisterTest('Utils Tests',Suite);

end.
