{*
 PasMatLib - DateTime Utils
 Nikolai V. Shokhirev, 2001-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2007.11.11 - created
 @version 2007.12.03 - Added DecodeDateTimeToStr, DecodedStrToDateTime
 }
unit uDateTimeUtils;

interface

uses
  uMatTypes
  , SysUtils
  , Types
  ;
  
const

  /// Time Accuracy = 0.9999999936 MilliSecond, used in CompareDateTime
  TimeAccuracyMSec = 1.1574074e-8;
  /// MilliSecond = 1.1574074074074074074074074074074e-8
  MilliSecond: extended = 1.1574074074074074074074074074074e-8;
  ///  DotNetTick = 1.1574074074074074074074074074074e-12
  DotNetTick: extended = 1.1574074074074074074074074074074e-12;
  ///  TimeAccuracyTick = 1.1574e-12;
  TimeAccuracyTick = 1.1574e-12;
  /// DateTime('01/01/0001')
  Date00010101 = -693593.0;
  /// TicksPerDay: int64 = 864000000000;
  TicksPerDay: int64 = 864000000000;
  /// Length(YYYYMMDDHHMMSSmmm)
  DateTimeStrLength = 17;

  function CompareDateTimeEx(const T1, T2: TDateTime;
                   TimeAccuracy: TFloat = TimeAccuracyMSec): TValueRelationship;

  /// .Net Ticks To DateTime
  function TicksToDateTime(Ticks: Int64): TDateTime;

  /// DateTime To .Net Ticks
  function DateTimeToTicks(DateTime: TDateTime): Int64;

  /// Encodes with fractions of milliseconds
  function EncodeDateTimeF(const AYear, AMonth, ADay, AHour, AMinute: Word;
                           const FSecond: TFloat): TDateTime;

  /// AValue -> YYYYMMDDHHMMSSmmm
  function DecodeDateTimeToStr(const AValue: TDateTime): string;

  function DecodedStrToDateTime(const AValue: string): TDateTime;

implementation

uses
  DateUtils
  ;

function CompareDateTimeEx(const T1, T2: TDateTime; TimeAccuracy: TFloat): TValueRelationship;
begin
  if abs(T2 - T1) < TimeAccuracy then
    result := 0
  else if T1 < T2 then
    result := -1
  else
    result := 1;
end;

function TicksToDateTime(Ticks: Int64): TDateTime;
begin
  result := DotNetTick*Ticks + Date00010101;
end;

function DateTimeToTicks(DateTime: TDateTime): Int64;
begin
  result := round(extended(DateTime - Date00010101)/DotNetTick);
end;

function EncodeDateTimeF(const AYear, AMonth, ADay, AHour, AMinute: Word;
                        const FSecond: TFloat): TDateTime;
var
  ASecond, AMilliSecond: Word;
  FMilliSecond: TFloat;
begin
  ASecond := Trunc(FSecond);
  FMilliSecond := 1000.0*Frac(FSecond);
  AMilliSecond := Round(FMilliSecond);
  result := EncodeDateTime(AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond);
  FMilliSecond := FMilliSecond - AMilliSecond;
  result := result + FMilliSecond*Millisecond; // correction to the fraction of MSec
end;

function DecodeDateTimeToStr(const AValue: TDateTime): string;
var
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond: Word;
begin
  DecodeDateTime(AValue, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond);
  result := SysUtils.Format('%.4d%.2d%.2d%.2d%.2d%.2d%.3d',
                [AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond]) ;
end;

function DecodedStrToDateTime(const AValue: string): TDateTime;
var
  L, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond: Word;
  S: string;
begin
  L := DateTimeStrLength - Length(AValue);
  Assert(L >= 0, 'Too long date-time string');
  S := AValue + StringOfChar('0', L);
  AYear := StrToInt(Copy(S, 1, 4));
  AMonth := StrToInt(Copy(S, 5, 2));
  ADay := StrToInt(Copy(S, 7, 2));
  AHour := StrToInt(Copy(S, 9, 2));
  AMinute := StrToInt(Copy(S, 11, 2));
  ASecond := StrToInt(Copy(S, 13, 2));
  AMilliSecond := StrToInt(Copy(S, 15, 3));
  Assert(TryEncodeDateTime(AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond, result),
         'CannotEncode:'+AValue);
end;

end.
