{*
 PasMatLib - FileUtils
 Nikolai V. Shokhirev, 2008-2008
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2007.11.11 - created
 }
unit uFileReader;

interface

uses
  Classes
  ;

const
  BuffSize = 32768;

type

  TIncrementalFileReader = class
  private
    fFileStream: TFileStream;
    fFileName: string;
    fBuffSize: integer;
    fPosition: int64;
    fBuff: array[0..BuffSize-1] of char;
    fFileSize: Int64;
    fFileDateTime: TDateTime;
  protected
    fBufferIsSmall: boolean;
    fLineNo: Int64;
    fReminder: string;
    function GetBuffer(Position: Int64): string; virtual;
  public
//    constructor Create(FileName: string);
    constructor Create(FileName: string; StartPosition: Int64 = 0);
    /// Reads NLines of current lines from the File and advances File position
    procedure GetLines(Lines: TStrings; NLines: integer; out EndOfFile: boolean); virtual;
    procedure GetLeadingLines(var Line0, Line1: string);
    function GetLastLine: string;
    procedure SetPosition(StartPosition: Int64);
    procedure ResetFileReader;
    function GetFileDateTime: TDateTime;
    function FileModified: boolean;
    property FileSize: Int64 read fFileSize;
    property BufferIsSmall: boolean read fBufferIsSmall;
  end;

implementation

uses
  SysUtils
  , uMatTypes
  , uStrFun
  , StrUtils
  , uFileUtils
  ;

{ TIncrementalFileReader }

procedure TIncrementalFileReader.ResetFileReader;
begin
  fPosition := 0;
  fReminder := '';
  fLineNo := 0;
end;

constructor TIncrementalFileReader.Create(FileName: string; StartPosition: Int64 = 0);
//var
//  f: file of byte;
begin
//  inherited Create;
  fBufferIsSmall := false;
  fFileName := FileName;
  fBuffSize := BuffSize;
  fPosition := StartPosition;
  fReminder := '';
//  if FileAge(FileName, fFileDateTime) then
  if FileExists(FileName) then
  begin
      fFileDateTime := FileDateTime(FileName);
    try
      fFileStream := TFileStream.Create(fFileName, fmOpenRead);
      fFileSize := fFileStream.Size;  
    finally
      FreeAndNil(fFileStream);
    end;
  end else
        fFileSize := -1;
end;

function TIncrementalFileReader.GetLastLine: string;
var
  p, size, i, position, count, BytesToRead: integer;
  LastLine, s: string;
  FirstTime: boolean;
begin
  BytesToRead := fBuffSize;
  LastLine := '';
  result := LastLine;
  FirstTime := true;
  fFileStream := TFileStream.Create(fFileName, fmOpenRead);
  fFileSize := fFileStream.Size;
  position := fFileSize;
  try
    while (position > 0) and (Pos(LF, LastLine) = 0) do
    begin
      position := position - fBuffSize;
      if (position < 0) then
      begin
        BytesToRead := fBuffSize - position;
        position := 0;
      end;

      fFileStream.Position := position;
      count := fFileStream.Read(fBuff, BytesToRead);

      SetLength(s, count);
      for i := 1 to count do
        s[i] := fBuff[i-1];

      if FirstTime then  // remove empty lines in the end, hopefully  less than fBuffSize
        s := TrimRight(s);

      LastLine := s + LastLine;
      FirstTime := false;
    end;

    size := Length(LastLine);
    p := LastDelimiter(LF, LastLine);
    if (p > 0) then
      LastLine := CopyFromTo(LastLine, p+1, size);
      result := LastLine;
  finally
    FreeAndNil(fFileStream);
  end;
end;

procedure TIncrementalFileReader.GetLeadingLines(var Line0, Line1: string);
var
  TF: TextFile;
begin
  AssignFile(TF, fFileName);
  try
    System.Reset(TF);
    ReadLn(TF, Line0);
    ReadLn(TF, Line1);
  finally
    CloseFile(TF);
  end;
end;

function TIncrementalFileReader.GetBuffer(Position: Int64): string;
var
  i, count: integer;
begin
  result := '';
  fFileStream.Position :=fPosition;
  count := fFileStream.Read(fBuff, fBuffSize);
  inc(fPosition, count);

  SetLength(result, count);
  for i := 0 to count - 1 do
    result[i+1] := fBuff[i];
end;

procedure TIncrementalFileReader.GetLines(Lines: TStrings; NLines: integer;
                                          out EndOfFile: boolean); 
var
  p, offset, size, FileSize: integer;
  s: string;
begin
  fFileStream := TFileStream.Create(fFileName, fmOpenRead);
  FileSize := fFileStream.Size;
  EndOfFile := false;
  try
    Lines.Clear;
    fLineNo := 0;
    repeat
      if (fPosition < FileSize) and (Pos(CRLF, fReminder) = 0) then
        fReminder := fReminder + GetBuffer(fPosition);
      size := Length(fReminder);
      p := 0;
      if (size > 0) then
      begin
        offset := 1;
        while (offset < size) and (fLineNo < NLines) do
        begin
          p := PosEx(CRLF, fReminder, offset);
          if (p = 0) then
            break;
          s := CopyFromTo(fReminder,offset, p-1);
          if Length(s) > (BuffSize-2) then
            fBufferIsSmall := true;
          Lines.Append(s);
          inc(fLineNo);
          offset := p + 2; // + Length(CRLF)
        end;
        fReminder := CopyFromTo(fReminder, offset, size);
      end;
    until (fLineNo = NLines) or ((fPosition = FileSize) and (p = 0));

    if ((fPosition = FileSize) and (p = 0)) then
    begin
      EndOfFile := true;
      if (fReminder <> '') then
      begin
        Lines.Append(fReminder);
        fReminder := '';
      end;
    end;
  finally
    FreeAndNil(fFileStream);
  end;
end;

function TIncrementalFileReader.FileModified: boolean;
begin

end;

function TIncrementalFileReader.GetFileDateTime: TDateTime;
begin

end;

procedure TIncrementalFileReader.SetPosition(StartPosition: Int64);
begin

end;

end.
