{*
 PasMatLib - FileUtils
 Nikolai V. Shokhirev, 2003-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2006.10.09 - created
 @version 2008.02.02 - added AttrToStr, GetFiles, ExtractFileData
 @version 2010.05.05 - Added ListFileDir, IsTextFile
 }
unit uFileUtils;

interface

uses
  SysUtils
  , Classes;

  {
    FullName=C:\exe\SciCalculator\Calculator.exe     FullName=C:\winzip.log
    FileName=Calculator.exe                          FileName=winzip.log
    NameOnly=Calculator                              NameOnly=winzip
    FileExt=.exe                                     FileExt=.log
    ExtOnly=exe                                      ExtOnly=log
    FileDir=C:\exe\SciCalculator                     FileDir=C:\
    FilePath=C:\exe\SciCalculator\                   FilePath=C:\
    DirOnly=SciCalculator                            DirOnly=
  }
  procedure FileNameInfo(const FullName: string; var FileName, NameOnly, FileExt,
                                     ExtOnly, FileDir, DirOnly, FilePath: string);

  function FileNameOnly(const FullName: string): string;

  {
    Write/ Append Text to a file
    http://www.swissdelphicenter.ch/en/showcode.php?id=674
    Autor: Mileriam Homepage: http://mileriam.de.vu
  }
  function AppendOrWriteTextToFile(FileName : TFilename; WriteText : string): boolean;

  function SetDateToFile(const FileName: string; Value: TDateTime): Boolean;

  ///////////////////////////////////////////////////////////
  //                                                       //
  //  Converts File attribute to the string                //
  //  of the following form: 'r_h_s_v_d_a'                 //
  //  (only existing attributes are displayed)             //
  //                                                       //
  //  File Attribute abbreviations:                        //
  //  r ReadOnly     faReadOnly  =  1  $00000001           //
  //  h Hidden       faHidden    =  2  $00000002           //
  //  s System File  faSysFile   =  4  $00000004           //
  //  v Volume ID    faVolumeID  =  8  $00000008           //
  //  d Directory    faDirectory = 16  $00000010           //
  //  a Archive      faArchive   = 32  $00000020           //
  //                 faAnyFile         $0000003F           //
  ///////////////////////////////////////////////////////////
  function AttrToStr(iAttr : integer) : string;

  procedure GetFiles(const ADirectory: string; Files: Tstrings; SubFolders: Boolean);
                   
  procedure ExtractFileData(var FileName, sTime, sAttr, sSize, sName, sExt, sPath: string);

  function IsDirEmpty (const ADirPath :string) :boolean;

  // return the exact file size for a file. Return zero if the file is not found.
  // http://www.jpgriffiths.com/tutorial/filesize.html
  function FileSize(FileName : String) : Int64;

  procedure FileSizeToString(FileByteSize: Int64;
                     out FileSizeB, FileSizeKB, FileSizeMB, FileSizeGB: string);

  function FileDateTime(FileName: string): TDateTime;

  // Find all files and directories in a directory (non-recursive)
  // use: FindAllFiles('C:\Windows\*.*',faAnyFile,ListBox1.Items)
  procedure FindAllFIles (const Path: String; Attr: Integer; List: TStrings) ;

  // Find all files (excluding directories) in a directory (non-recursive)
  // Author: Fabio Torres
  // modified by Nikolai Shokhirev  05.05.2010
  // use:   ListFileDir('C:\WINDOWS\', ListBox1.Items);
  procedure ListFileDir(Path: string; FileList: TStrings);

  // Check if a file contains only printable symbols + TAB, LF, CR, SUB
  // Based on IsTextFile by Marcelo Castro, http://www.mcgsoftwares.com
  // and  IsAscii by Fabio Cillo, http://web.tiscalinet.it/fabiocillo
  function IsTextFile(const sFile: TFileName; AsciiOnly: boolean = false;
                            SampleSize: integer = 2048): boolean;
implementation

uses
  StrUtils
  , uMatTypes
  , uStrFun
  , math
  ;

procedure FileNameInfo(const FullName: string; var FileName, NameOnly, FileExt,
                                   ExtOnly, FileDir, DirOnly, FilePath: string);
var
  p, pp: integer;

begin

  FileExt := ExtractFileExt(FullName);
  FileName := ExtractFileName(FullName);
  FileDir := ExtractFileDir(FullName);
  FilePath := ExtractFilePath(FullName);
  NameOnly := FileNameOnly(FullName);
  pp := Length(FileDir);
  p := pp+1;
  repeat
    dec(p);
  until (FileDir[p] = '\');
  DirOnly := Copy(FileDir,p+1,pp-p+1);
  if DirOnly = '\' then DirOnly := '';
  ExtOnly := Copy(FileExt,2,31);
end;

  {
    FullName=C:\exe\SciCalculator\Calculator.exe     FullName=C:\winzip.log
    FileName=Calculator.exe                          FileName=winzip.log
    NameOnly=Calculator                              NameOnly=winzip
    FileExt=.exe                                     FileExt=.log
    ExtOnly=exe                                      ExtOnly=log
    FileDir=C:\exe\SciCalculator                     FileDir=C:\
    FilePath=C:\exe\SciCalculator\                   FilePath=C:\
    DirOnly=SciCalculator                            DirOnly=
  }
procedure FileInfo(const FullName: string; var DateTime, FileSize: string);
var
  sr: TSearchRec;
begin
  DateTime := DateTimeToStr(FileDateToDateTime(FileAge(FullName)));

  if FindFirst(FullName, faAnyFile, sr) = 0 then
    FileSize := IntToStr(sr.Size);
  FindClose(sr);
end;

function FileNameOnly(const FullName: string): string;
var
  p: integer;
  FileName, FileExt: string;
begin
  FileExt := ExtractFileExt(FullName);
  FileName := ExtractFileName(FullName);
  p := Pos(FileExt,FileName);
  result := Copy(FileName,1,p-1);
end;

{
  Write/ Append Text to a file
  http://www.swissdelphicenter.ch/en/showcode.php?id=674
  Autor: Mileriam Homepage: http://mileriam.de.vu
}
function AppendOrWriteTextToFile(FileName : TFilename; WriteText : string): boolean;
var
  f : Textfile;
begin
  Result := False;
  AssignFile(f, FileName);
  try
    if not FileExists(FileName) then
      Rewrite(f)
    else
    begin
      Append(f);
    end;
    Writeln(f, WriteText);
    Result := True;
  finally
    CloseFile(f);
  end;
end;

function SetDateToFile(const FileName: string; Value: TDateTime): Boolean;
var
  hFile: THandle;
begin
  Result := False;
  try
    {open a file handle}
    hFile := FileOpen(FileName, fmOpenWrite or fmShareDenyNone);

    {if opened succesfully}
    if (hFile > 0) then
      {convert a datetime into DOS format and set a date}
      Result := (FileSetDate(hFile, DateTimeToFileDate(Value)) = 0)
  finally
    {close an opened file handle}
    FileClose(hFile);
  end;
end;

function AttrToStr(iAttr : integer) : string;
var
  s : string;
begin
  s:= '';
  if (iAttr and $00000001)   <> 0 then s := s + 'r ' else s := s + '_ ';
  if (iAttr and faHidden)    <> 0 then s := s + 'h ' else s := s + '_ ';
  if (iAttr and faSysFile)   <> 0 then s := s + 's ' else s := s + '_ ';
  if (iAttr and faVolumeID)  <> 0 then s := s + 'v ' else s := s + '_ ';
  if (iAttr and faDirectory) <> 0 then s := s + 'd ' else s := s + '_ ';
  if (iAttr and faArchive)   <> 0 then s := s + 'a ' else s := s + '_ ';
  Result := s;
end;

// Helper function to remove any slashes or add them if needed
function SlashSep(const Path, S: string): string;
begin
  if AnsiLastChar(Path)^ <> '\' then
    Result := Path + '\' + S
  else
    Result := Path + S;
end;

function SearchRecToStr(SearchRec: TSearchRec): string;
//TSearchRec = record
//             Time: Integer;
//             Size: Integer;
//             Attr: Integer;
//             Name: TFileName;
//             ExcludeAttr: Integer;
//             FindHandle: THandle;
//             FindHandle:  TWin32FindData;
//             end;
begin
  with SearchRec do
  begin
    Result :=  ';' + IntToStr(Attr) + ';' + IntToStr(Size)
             + ';' + IntToStr(Time);
  end;
end;

// this code from:
//   http://community.borland.com/article/0,1410,23175,00.html
//   Article ID: 23175   Publish Date: September 18, 2000
procedure GetFiles(const ADirectory: string; Files: Tstrings;
                   SubFolders: Boolean);
var
  SearchRec: TSearchRec;
  nStatus: Integer;
begin
  nStatus := FindFirst(PChar(SlashSep(ADirectory, '*.*')), 0, SearchRec);
  while nStatus = 0 do
  begin
//    Files.Add(SlashSep(ADirectory, SearchRec.Name)+SearchRecToStr(SearchRec));
    Files.Add(SlashSep(ADirectory, SearchRec.Name)+','+IntToStr(SearchRec.Size));
    nStatus := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
  // Next look for subfolders and search them if required to do so
  if SubFolders then
  begin
    nStatus := FindFirst(PChar(SlashSep(ADirectory, '*.*')), faDirectory,
      SearchRec);
    while nStatus = 0 do
    begin
      // If it is a directory, then use recursion
      if ((SearchRec.Attr and faDirectory) <> 0) then
      begin
        if ( (SearchRec.Name <> '.') and (SearchRec.Name <> '..') )  then
          GetFiles(SlashSep(ADirectory, SearchRec.Name), Files, SubFolders);
      end;
      nStatus := FindNext(SearchRec)
    end;
    FindClose(SearchRec);
  end;
end;

procedure ExtractFileData(var FileName, sTime, sAttr, sSize, sName, sExt, sPath: string);
var
  s: string;
  Index: integer;
begin
  Index := LastDelimiter(';',FileName);
  s := Copy(FileName, Index+1, 512);
  Delete(FileName, Index, 512);
  sTime := s;
  Index := LastDelimiter(';',FileName);
  s := Copy(FileName, Index+1, 512);
  Delete(FileName, Index, 512);
  sSize := s;
  Index := LastDelimiter(';',FileName);
  s := Copy(FileName, Index+1, 512);
  Delete(FileName, Index, 512);
  sAttr := s;
  sPath := ExtractFilePath( FileName);
  sName := ExtractFileName( FileName);
  sExt := ExtractFileExt( FileName);
//  sName := StringReplace(sName, sExt, '', [rfReplaceAll,rfIgnoreCase]);
end;

// http://www.delphi3000.com/articles/article_2084.asp
function IsDirEmpty (const ADirPath :string) :boolean;
var
  F :TSearchRec;
begin
  try
    result := (FindFirst(ADirPath+'\*.*', faAnyFile,F) = 0) and
              (FindNext(F) = 0) and (FindNext(F) <> 0);
  finally
     FindClose(F);
  end;
end;

// return the exact file size for a file. Return -1 if the file is not found.
// http://www.jpgriffiths.com/tutorial/filesize.html
function FileSize(FileName : String) : Int64;
var
  SearchRec : TSearchRec;
begin
  if FindFirst(FileName, faAnyFile, SearchRec ) = 0 then                  // if found
     Result := Int64(SearchRec.FindData.nFileSizeHigh) shl Int64(32) +    // calculate the size
               Int64(SearchREc.FindData.nFileSizeLow)
  else
     Result := -1;
  FindClose(SearchRec);                                                   // close the find
end;

procedure FileSizeToString(FileByteSize: Int64;
                     out FileSizeB, FileSizeKB, FileSizeMB, FileSizeGB: string);
var
  fs: double;
begin
  FileSizeB := IntToStr(FileByteSize) + ' B';
  fs := FileByteSize;
  fs := fs/1024.0;
  FileSizeKB := FloatToStrF(fs, ffGeneral, 8, 4) + ' KB';
  fs := fs/1024.0;
  FileSizeMB := FloatToStrF(fs, ffGeneral, 8, 4) + ' MB';
  fs := fs/1024.0;
  FileSizeGB := FloatToStrF(fs, ffGeneral, 8, 4) + ' GB';
end;

 // Kanade's Delphi Stuff
// http://kanadepro.com/delphistuff/sampcode.mv?listTipsScreen+2
function MyGetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
  begin
    Result := SearchRec.Size;
    FindClose(SearchRec);
  end
  else Result := -1;
end;

// http://www.scalabium.com/faq/dct0046.htm
function FileDateTime(FileName: string): TDateTime;
var intFileAge: LongInt;
begin
  intFileAge := FileAge(FileName);
  if intFileAge = -1 then
    Result := 0
  else
    Result := FileDateToDateTime(intFileAge)
end;

// use: FindAllFiles('C:\Windows\*.*',faAnyFile,ListBox1.Items)
procedure FindAllFIles (const Path: String; Attr: Integer; List: TStrings) ;

var
   Res: TSearchRec;
   EOFound: Boolean;
begin
   EOFound:= False;
   if FindFirst(Path, Attr, Res) < 0 then
     exit
   else
     while not EOFound do
     begin
       List.Add(Res.Name) ;
       EOFound:= FindNext(Res) <> 0;
     end;
   FindClose(Res) ;
end;


// Author: Fabio Torres
// modified by Nikolai Shokhirev  05.05.2010
// use:   ListFileDir('C:\WINDOWS\', ListBox1.Items);
procedure ListFileDir(Path: string; FileList: TStrings);
var
  SR: TSearchRec;
begin
  if FindFirst(Path + '*.*', faAnyFile, SR) = 0 then
  begin
    repeat
//      if (SR.Attr <> faDirectory) then
      if (SR.Attr and faDirectory) = 0 then   // make working for Path = '.\'
      begin
        FileList.Add(SR.Name);
      end;
    until FindNext(SR) <> 0;
    FindClose(SR);
  end;
end;

// Check if a file contains only printable symbols + TAB, LF, CR, SUB
// Based on IsTextFile by Marcelo Castro, http://www.mcgsoftwares.com
// and  IsAscii by Fabio Cillo, http://web.tiscalinet.it/fabiocillo
function IsTextFile(const sFile: TFileName; AsciiOnly: boolean = false;
                          SampleSize: integer = 2048): boolean;
var
  oIn: TFileStream;
  iRead: Integer;
  iMaxRead: Integer;
  iData: Byte;
  dummy:string;
begin
  result := true;
  dummy :='';
  oIn := TFileStream.Create(sFile, fmOpenRead or fmShareDenyNone);
  try
    iMaxRead := min(SampleSize, oIn.Size); // only the first SampleSize bytes
    for iRead := 1 to iMaxRead do
    begin
      oIn.Read(iData, 1);
      if AsciiOnly then
        result := (iData < 128);
      if result then
        result := (iData > 31) or (iData in [9, 10, 13, 26]);
      if not result then
        break;
//      if (iData < 32) and (not (c[i] in [9, 10, 13, 26])) then a := False;
//      if (iData) > 127 then result:=false;
    end;
 finally
   FreeAndNil(oIn);
 end;
end;

end.
