{*********************************************************}
{*                     FUNSTR.PAS                        *}
{*          Copyright (c)  Sergey Perevoznik             *}
{* Portions Copyright (c) TurboPower Software 1987, 1992 *}
{* Portions Copyright 1991 TechnoJock Software, Inc.     *}
{*                 All rights reserved.                  *}
{*********************************************************}

{-Basic string manipulation routines}

unit uFunStr;

interface

uses
  SysUtils;

type
  TLong = record
            LowWord,
            HighWord: Word;
         end;

  TWord = record
            LowByte,
            HighByte: Byte;
         end;

  TDouble = record
              b0, b1, b2, b3, b4, b5, b6, b7: Byte;
            end;

const
  CRLF=#13#10;

{-------- Numeric conversion -----------}

{-Return hex string for byte}
function HexB(B: Byte): string;

{-Return hex string for word}
function HexW(W: Word): string;

{-Return hex string for longint}
function HexL(L: LongInt): string;

{-Return hex string for pointer}
//function HexPtr(P: Pointer): string;

{-Return binary string for byte}
function BinaryB(B: Byte): string;

{-Return binary string for word}
function BinaryW(W: Word): string;

{-Return binary string for longint}
function BinaryL(L: LongInt): string;

{-Return octal string for byte}
function OctalB(B: Byte): string;

{-Return octal string for word}
function OctalW(W: Word): string;

{-Return octal string for longint}
function OctalL(L: LongInt): string;

{-Return byte for string}
function  Str2Byte(S: String)     : Byte;

{-Return byte for hex string}
function  HexStr2Byte(Str: String): Byte;

{-Return word for string}
function  Str2Word(S: String)     : Word;

{-Return word for hex string}
function  HexStr2Word(Str: String): Word;

{-Return integer for string}
function  Str2Int(Str: string)    : Integer;

{-Return integer for hex string}
function  HexStr2Int(Str: string) : Integer;

{-Return longint for string}
function  Str2TLong(Str: string)   : LongInt;

{-Return longint for hex string}
function  HexStr2TLong(Str: String): LongInt;

{-Return real for string}
function Str2Real(S: string)    : Real;

{-Return real for hex string}
function HexStr2Real(Str: string)      : Real;

{-Return string for integer}
function  Int2Str(i: LongInt)     : String;

{-Return string for word}
function  Word2Str(w: Word)       : String;

{-Return string for byte}
function  Byte2Str(b: Byte)       : String;

{-Return string for real}
function  Real2Str(r: Real;Len,Decim: Byte): String;

{-Convert a longint/word/integer/byte/shortint to a string}
function TLong2Str(L: LongInt): string;


{-------- General purpose string manipulation --------}

{-Return uppercase of char, with international character support}
function Upcase(Ch: Char): Char;

{-Convert lower case letters in string to uppercase, with intl chars}
//function StUpcase(S: string): string;

{-Return lowercase of char, with international character support}
function LoCase(Ch: Char): Char;

{-Convert upper case letters in string to lowercase, with intl chars}
//function StLocase(S: string): string;

{-Return a string of length len filled with ch}
function CharStr(Ch: Char; Len: Byte): string;

//function  FillCh(Sym: Char;L: Byte): String;
{-Return a string of length len filled with ch}

//function  Space(L: Byte)   : String;
{-Return a string of length len filled with space}

{-Return a string right-padded to length len with ch}
function PadCh(S: string; Ch: Char; Len: Byte): string;

{-Return a string right-padded to length len with blanks}
function Pad(S: string; Len: Byte): string;

{-Return a string left-padded to length len with ch}
function LeftPadCh(S: string; Ch: Char; Len: Byte): string;

{-Return a string left-padded to length len with blanks}
function LeftPad(S: string; Len: Byte): string;

{-Return a string with leading white space removed}
function TrimLead(S: string): string;

{-Return a string with trailing white space removed}
function TrimTrail(S: string): string;

{-Return a string with leading and trailing white space removed}
function Trim(S: string): string;

{-Return a string centered in a string of Ch with specified width}
function CenterCh(S: string; Ch: Char; Width: Byte): string;

{-Return a string centered in a blank string of specified width}
function Center(S: string; Width: Byte): string;

//function  TruncL(N: Byte;Str: String): String;
{-Return a string of length N }

//function  TruncR(N: Byte;Str: String): String;
{-Return a string of length N }

{--------------- Word manipulation -------------------------------}

function PosWord(Wordno: byte;Str: string): byte;

function WordCnt(Str: string): byte;

function ExtractWords(StartWord,NoWords: byte;Str: string): string;

function ValidNumeric(Value: string): boolean;

{==========================================================================}

implementation

const
  Digits: array[0..$F] of Char = '0123456789ABCDEF';

{-Return hex string for byte}
function HexB(B: Byte): string;
begin
  SetLength(result, 2);
  result[1] := Digits[B shr 4];
  result[2] := Digits[B and $F];
end;

{-Return hex string for word}
function HexW(W: Word): string;
begin
  SetLength(result, 4);
  result[1] := Digits[hi(W) shr 4];
  result[2] := Digits[hi(W) and $F];
  result[3] := Digits[lo(W) shr 4];
  result[4] := Digits[lo(W) and $F];
end;

{-Return hex string for LongInt}
function HexL(L: LongInt): string;
begin
  with TLong(L) do
    result := HexW(HighWord)+HexW(LowWord);
end;

{-Return hex string for pointer}
{
function HexPtr(P: Pointer): string;
begin
//  result := HexW(Seg(P^))+': '+HexW(Ofs(P^));
  result := '';
end;
}

{-Return binary string for byte}
function BinaryB(B: Byte): string;
var
  I, N: Word;
begin
  N := 1;
  SetLength(result, 8);
  for I := 7 downto 0 do
  begin
    result[N] := Digits[Ord(B and (1 shl I) <> 0)]; {0 or 1}
    Inc(N);
  end;
end;

{-Return binary string for word}
function BinaryW(W: Word): string;
var
  I, N: Word;
begin
  N := 1;
  SetLength(result, 16);
  for I := 15 downto 0 do
  begin
    result[N] := Digits[Ord(W and (1 shl I) <> 0)]; {0 or 1}
    Inc(N);
  end;
end;

{-Return binary string for LongInt}
function BinaryL(L: LongInt): string;
var
  I: LongInt;
  N: Byte;
begin
  N := 1;
  SetLength(result, 16);

  for I := 31 downto 0 do
  begin
    result[N] := Digits[Ord(L and LongInt(1 shl I) <> 0)]; {0 or 1}
    Inc(N);
  end;
end;

{-Return octal string for byte}
function OctalB(B: Byte): string;
var
  I: Word;
begin
  SetLength(result, 3);
  for I := 0 to 2 do begin
    result[3-I] := Digits[B and 7];
    B := B shr 3;
  end;
end;

function OctalW(W: Word): string;
{-Return octal string for word}
var
  I: Word;
begin
  SetLength(result, 6);
  for I := 0 to 5 do begin
    result[6-I] := Digits[W and 7];
    W := W shr 3;
  end;
end;

function OctalL(L: LongInt): string;
{-Return octal string for word}
var
  I: Word;
begin
  SetLength(result, 12);
  for I := 0 to 11 do begin
    result[12-I] := Digits[L and 7];
    L := L shr 3;
  end;
end;

function Str2Int(Str: string): integer;
var
  temp, code: integer;
begin
   if length(Str) = 0 then
      Str2Int := 0
   else
   begin
      val(Str,temp,code);
      if code = 0 then
         Str2Int := temp
      else
       raise EConvertError.Create('  ');
   end;
end; {Str2Int}

function  Str2Word(S: string): Word;
var
  code: integer;
  SLen: Byte absolute S;
  I   : Word;
begin
  while S[SLen] = ' ' do
    Dec(SLen);
  Val(S, I, code);
  IF ( code = 0  ) THEN
    Str2Word := I
     else
      raise EConvertError.Create('  ');

end;

function Str2TLong(Str: string): LongInt;
var
  code: integer;
  Temp: longint;
begin
   if length(Str) = 0 then
      Str2TLong := 0
   else
   begin
      val(Str,temp,code);
      if code = 0 then
         Str2TLong := temp
      else
        raise EConvertError.Create('Incorrect number presentation');

   end;
end; {Str2TLong}

function Str2Real(S: string): Real;
var
  Code: integer;
  SLen: Byte absolute S;
  R: real;
begin
  while S[SLen] = ' ' do
    Dec(SLen);

  Val(S, R, Code);
  if Code <> 0 then
      raise EConvertError.Create('Incorrect number presentation')
  else
    Str2Real := R;
end;

{-Convert a long/word/integer/byte/shortint to a string}
function TLong2Str(L: LongInt): string;
var
  S: string;
begin
  Str(L, S);
  TLong2Str := S;
end;

function Real2Str(r: Real;Len,Decim: Byte): String;
var
  s: String;
  L: Byte ABSOLUTE s;
  i: Byte;
begin
  STR(r: Len: Decim,s);
  s:=Trim(s);
 { IF Decim>0 Then While (s[l]='0') AND (s[l-1]<>'.') DO DEC(l);}
  IF l>Len Then
   begin
    STR(r: Len,s); s:=Trim(s);
    i:=POS('E',s)+2;
    IF COPY(s,i,2)='00' Then DELETE(s,i,2);
   end;
 Real2Str:=s;
end;


function CharStr(Ch: Char; Len: Byte): string;
{-Return a string of length len filled with ch}
var
  S: string;
begin
  if Len = 0 then
    SetLength(result, 0)
  else
  begin
    SetLength(result, Len);
    FillChar(S[1], Len, Ch);
    CharStr := S;
  end;
end;

{-Return a string right-padded to length len with ch}
function PadCh(S: string; Ch: Char; Len: Byte): string;
var
  o: string;
  SLen: Byte absolute S;
begin
  if Length(S) >= Len then
    PadCh := S
  else
  begin
    SetLength(o, Len);
    Move(S[1], o[1], SLen);
    if SLen < 255 then
      FillChar(o[Succ(SLen)], Len-SLen, Ch);
    result := o;
  end;
end;

{-Return a string right-padded to length len with blanks}
function Pad(S: string; Len: Byte): string;
begin
  result := PadCh(S, ' ', Len);
end;

{-Return a string left-padded to length len with ch}
function LeftPadCh(S: string; Ch: Char; Len: Byte): string;
var
  o: string;
  SLen: Byte absolute S;
begin
  if Length(S) >= Len then
    LeftPadCh := S
  else if SLen < 255 then
  begin
    SetLength(o, Len);
    Move(S[1], o[Succ(Word(Len))-SLen], SLen);
    FillChar(o[1], Len-SLen, Ch);
    result := o;
  end;
end;

{-Return a string left-padded to length len with blanks}
function LeftPad(S: string; Len: Byte): string;
begin
  result := LeftPadCh(S, ' ', Len);
end;

{-Return a string with leading white space removed}
function TrimLead(S: string): string;
var
  I: Word;
begin
  I := 1;
  while (I <= Length(S)) and (S[I] <= ' ') do
    Inc(I);
  Dec(I);
  if I > 0 then
    Delete(S, 1, I);
  TrimLead := S;
end;

{-Return a string with trailing white space removed}
function TrimTrail(S: string): string;
var
  SLen: Byte absolute S;
begin
  while (SLen > 0) and (S[SLen] <= ' ') do
    Dec(SLen);
  TrimTrail := S;
end;

{-Return a string with leading and trailing white space removed}
function Trim(S: string): string;
var
  I: Word;
  SLen: Byte absolute S;
begin
  while (SLen > 0) and (S[SLen] <= ' ') do
    Dec(SLen);

  I := 1;
  while (I <= SLen) and (S[I] <= ' ') do
    Inc(I);
  Dec(I);
  if I > 0 then
    Delete(S, 1, I);

  Trim := S;
end;

{-Return a string centered in a string of Ch with specified width}
function CenterCh(S: string; Ch: Char; Width: Byte): string;
var
  o: string;
  SLen: Byte absolute S;
begin
  if SLen >= Width then
    CenterCh := S
  else if SLen < 255 then begin
    SetLength(o, Width);
    FillChar(o[1], Width, Ch);
    Move(S[1], o[Succ((Width-SLen) shr 1)], SLen);
    result := o;
  end;
end;

{-Return a string centered in a blank string of specified width}
function Center(S: string; Width: Byte): string;
begin
  Center := CenterCh(S, ' ', Width);
end;

function UpCase(Ch: Char): Char;  Assembler;
               ASM
                MOV AL,&Ch
                CMP AL,$61
                JL @@1
                CMP AL,$7A
                JG @@1
                JMP @@2
               @@1: 
                CMP AL,$A0
                JL @@3
                CMP AL,$AF
                JG @@3
                JMP @@2
               @@3: 
                CMP AL,$E0
                JL @@4
                CMP AL,$EF
                JG @@4
                SUB AL,$50
                JMP @@4
               @@2: 
                SUB AL,$20
               @@4: 
end;

function LoCase(Ch: Char): Char;  Assembler;
               ASM
                MOV AL,&Ch
                CMP AL,$41
                JL @@1
                CMP AL,$5A
                JG @@1
                JMP @@2
               @@1: 
                CMP AL,$80
                JL @@3
                CMP AL,$8F
                JG @@3
                JMP @@2
               @@3: 
                CMP AL,$90
                JL @@4
                CMP AL,$9F
                JG @@4
                ADD AL,$50
                JMP @@4
               @@2: 
                ADD AL,$20
               @@4: 
end;
{
function StUpCase(S: String): String; assembler;
asm
        PUSH DS
        CLD
//        LDS  SI, S
//        LES  DI, @Result
        LODSB
        STOSB
        XOR AH,AH
        XCHG AX,CX
        JCXZ @2
  @1: 
        LODSB
        CMP  AL, 'a'
        JB   @3
        CMP  AL, 'z'
        JA   @3
        SUB  AL, 20h
  @3:   CMP  AL, ''
        JB   @4
        CMP  AL, ''
        JA   @4
        SUB  AL, 20h
  @4:   CMP  AL, ''
        JB   @5
        CMP  AL, ''
        JA   @5
        SUB  AL,50h
        JMP  @5
  @5:   STOSB
        LOOP @1
  @2: 
        POP DS
end;


function StLoCase(S: String): String; assembler;
asm
        PUSH DS
        CLD
//        LDS  SI, S
//        LES  DI, @Result
        LODSB
        STOSB
        XOR AH,AH
        XCHG AX,CX
        JCXZ @@4

   @@1: 
       LODSB
       CMP AL, 'A'
       JB  @@3
       CMP AL, 'Z'
       JBE @@2
       CMP AL, ''
       JB  @@3
       CMP AL, ''
       JBE @@2
       CMP AL, ''
       JA  @@3
       ADD AL, 80
       JMP @@3
   @@2: 
       ADD AL, 32
   @@3: 
       STOSB
       LOOP  @@1
   @@4: 
   POP DS
end;
}
{
function  FillCh(Sym: Char;L: Byte): String;  Assembler;
ASM
 PUSH DS
// LES   DI,@Result
 XOR  CX,CX
 MOV   CL,L
 CMP   CL,0
 MOV AL,CL
 STOSB
 MOV CL,AL
 JE  @@1
 MOV   AL,SYM
 CLD
 REP   STOSB
@@1: 
 POP DS
end;
}

function Str2Byte(S: String): Byte;
var
  VByte: Byte;
  i    : Byte;
begin
  VByte:=0;
  for I:=1 to Length(S)  do
    VByte:=VByte or ((Byte(S[Length(S)-i+1])-48) SHL (i-1));
  result := VByte;
end;

{}

function HexChar2Byte(const c: char): Byte;
begin
  case c of
  '0'..'9': begin
              result := ord(c)-ord('0');
            end;
  'a'..'f': begin
              result := ord(c)-ord('a')+10;
            end;
  'A'..'F': begin
              result := ord(c)-ord('A')+10;
            end;
  else
    raise EConvertError.Create('char out of [0..9, a..f, A..F]')
  end;
end;

{ Two digits }
function HexStr2Byte(Str: string): Byte;
var
  c1, c0: char;
  i: integer;
begin
  if Str[1] = '$' then
    i := 2
  else
    i := 1;
  c1 := Str[i];
  c0 := Str[i+1];
  result := (HexChar2Byte(c1) shl 4)+HexChar2Byte(c0);
{   if Str = '' then
      HexStr2Byte := 0
   else
   begin
      if Str[1] <> '$' then
         Str := '$'+Str;
      result := Str2Byte(Str);
   end;
}
end; {HexStr2Byte}

{}
function HexStr2Word(Str: String): Word;
begin
   if Str = '' then
      HexStr2Word := 0
   else
   begin
      if Str[1] <> '$' then
         Str := '$'+Str;
      HexStr2Word := Str2Word(Str);
   end;
end; {HexStr2Word}

{}
function HexStr2Int(Str: String): Integer;
begin
   if Str = '' then
      HexStr2Int := 0
   else
   begin
      if Str[1] <> '$' then
         Str := '$'+Str;
      HexStr2Int := Str2Int(Str);
   end;
end; {HexStr2Int}

{}
function HexStr2Real(Str: String): Real;
begin
   if Str = '' then
      HexStr2Real := 0
   else
   begin
      if Str[1] <> '$' then
         Str := '$'+Str;
      HexStr2Real := Str2Real(Str);
   end;
end; {HexStr2Real}

{}
function HexStr2TLong(Str: String): LongInt;
begin
   if Str = '' then
      HexStr2TLong := 0
   else
   begin
      if Str[1] <> '$' then
         Str := '$'+Str;
      HexStr2TLong := Str2TLong(Str);
   end;
end; {HexStr2TLong}

function Int2Str(i: LongInt): String;
var
  s: string;
begin
  STR(i,s);
  Int2Str:=s;
end;


function Word2Str(w: Word): String;
VAR
  s: String;
begin
  STR(w,s);
  Word2Str:=s;
end;

function Byte2Str(B: Byte): String;
VAR
  s: String;
begin
  STR(B,s);
  Byte2Str:=s;
end;

(*

function TruncR(N: Byte;Str: String): String; assembler;
asm
  PUSH DS
  CLD
//  LDS SI,STR
//  LES DI, @RESULT
  LODSB
  XOR AH,AH
  MOV BL,AL    {࠭塞    BL}
  XCHG AX,CX
  JCXZ @3
  MOV BH,N
  CMP BL,BH
  JBE @2
  SUB BL,BH
  MOV CL,BL
  MOV AL,N
  STOSB
@1: 
  LODSB
  LOOP @1
  JMP @6
@2: 
  MOV AL,BL
  STOSB
  MOV CL,BL
  JMP @5
@6: 
  MOV CL,N
@5: 
  LODSB
  STOSB
  LOOP @5
  JMP @4
@3: 
  MOV AL,0
  STOSB
@4: 
  POP DS
end;{TruncR}
*)
{
function TruncL(N: Byte;Str: String): String; assembler;
asm
  PUSH DS
  CLD
//  LDS SI,STR
//  LES DI,@RESULT
  LODSB
  MOV BL,AL
  XOR AH,AH
  XCHG AX,CX
  JCXZ @3
  MOV BH,N
  CMP BL,BH
  JAE @2
  MOV AL,BL
  STOSB
  XOR CX,CX
  MOV CL,BL
  JMP @1
@2: 
  MOV CL,N
  MOV AL,N
  STOSB
@1: 
  LODSB
  STOSB
  LOOP @1
  JMP @4
@3: 
  MOV AL,0
  STOSB
@4: 
  POP DS
end;{TruncL}

(*
function  Space(L: Byte): String;  Assembler;
ASM
 PUSH DS
// LES   DI,@Result
 XOR  CX,CX
 MOV   CL,L
 CMP   CL,0
 MOV AL,CL
 STOSB
 MOV CL,AL
 JE  @@1
 MOV   AL,32
 CLD
 REP   STOSB
@@1: 
 POP DS
end;
*)

{local proc used by PosWord and Extract word}
function LocWord(StartAT,Wordno: byte;Str: string): byte;
var
  W: integer;
  Spacebefore: boolean;
begin
   if (Str = '') or (wordno < 1) or (StartAT > length(Str)) then
   begin
       LocWord := 0;
       exit;
   end;
   SpaceBefore := true;
   W := 0;
//   L := length(Str);
   StartAT := pred(StartAT);
   while (W < Wordno) and (StartAT <= length(Str)) do
   begin
      StartAT := succ(StartAT);
      if SpaceBefore and (Str[StartAT] <> ' ') then
      begin
         W := succ(W);
         SpaceBefore := false;
      end
      else
         if (SpaceBefore = false) and (Str[StartAT] = ' ') then
            SpaceBefore := true;
   end;
   if W = Wordno then
      LocWord := StartAT
   else
      LocWord := 0;
end;

function PosWord(Wordno: byte;Str: string): byte;
begin
   PosWord := LocWord(1,wordno,Str);
end;{Word}

function WordCnt(Str: string): byte;
var
  W,I: integer;
  SpaceBefore: boolean;
begin
   if Str = '' then
   begin
      WordCnt := 0;
      exit;
   end;
   SpaceBefore := true;
   W := 0;
   For  I :=  1 to length(Str) do
   begin
      if SpaceBefore and (Str[I] <> ' ') then
      begin
         W := succ(W);
         SpaceBefore := false;
      end
      else
         if (SpaceBefore = false) and (Str[I] = ' ') then
            SpaceBefore := true;
   end;
   WordCnt := W;
end;

function ExtractWords(StartWord,NoWords: byte;Str: string): string;
var Start, finish: integer;
begin
   if Str = '' then
   begin
      ExtractWords := '';
      exit;
   end;
   Start := LocWord(1,StartWord,Str);
   if Start <> 0 then
      finish := LocWord(Start,succ(NoWords),Str)
   else
   begin
      ExtractWords := '';
      exit;
   end;
   if finish = 0 then
      finish := succ(length(Str));
   repeat
      finish := pred(finish);
   until Str[finish] <> ' ';
   ExtractWords := copy(Str,Start,succ(finish-Start));
end;{ExtractWords}


{*******************************************************************************
  ValidNumeric - 24.09.97 15: 31
 BY: 

********************************************************************************}
function ValidNumeric(Value: string): boolean;
var
  I: integer;
begin
     ValidNumeric := true;
     for I := 1 to Length(Value) do
     begin
        if (Value[I] in ['0'..'9']) = false then{1.00b}
          begin
           ValidNumeric := false;
           Exit;
          end;
     end;
end; {ValidNumeric}

end.
