{*
 PasMatLib - HTML tag utilities
 Nikolai V. Shokhirev, 2001-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2006.10.09 - created
 @version 2007.08.28 - modified
 }
unit uHTMLUtils;

interface

uses
  Classes, StdCtrls, Grids;

type
  Thtml = class(TObject)
  private
    sli: TStringList;
  public
    constructor Create(const Name: string);
    destructor Destroy; override;
    function comment(const text: string): string;
    function header: string;
    function start_html(const title: string ='' ): string;
    function h1(const text: string): string;
    function h2(const text: string): string;
    function h3(const text: string): string;
    function b(const text: string): string;
    function i(const text: string): string;
    function p(const text: string; const align: string = ''): string;
    function end_html: string;
  end;

const
  CRLF = #13#10;
  eol = CRLF;
  br = '<br />';
  nbsp = '&nbsp;';
  breol = br + eol;
  NChar = 4;
  ind = '  ';

var
  specialchar:  array[1..NChar,1..2] of string
                     = (('&','&amp;'),('"','&quot;'),('<','&lt;'),('>','&gt;'));

function header(const title: string = 'document'): string;

function footer: string;

function p(const text: string; const align: string = ''): string;
function p_(const text: string; const align: string = ''): string;

function pcenter(const text: string): string;

function img(const FileName: string; width, height: integer;
                   centered: boolean=true; border: integer = 0): string;

function h(n: integer; const text: string): string;
function h1(const text: string): string;
function h2(const text: string): string;

function b(const text: string): string;
function b_(const text: string): string;

function i(const text: string): string;
function i_(const text: string): string;

/// formatted tag
function ftag(const tagstr, text: string; const attributs: string = ''): string;
function tag(const tagstr, text: string; const attributs: string = ''): string;

function a(const url, text: string): string;

function TextToHTML(const text: string): string;

function MemoToHTML(const Memo: TMemo): string;

function StrGridToHTML(const StrGrid: TStringGrid;
                         Caption: string =''; centered: boolean = true): string;

function Gethtmlstr(const MemoHTML: TMemo): string;

function row(const data: array of string): string;

function table(const rows: array of string; border: integer = 1;
                                            cellpadding: integer = 1): string;
implementation

uses
  SysUtils;

function header(const title: string = 'document'): string;
begin
  result := '<html>'+CRLF+'<head>'+CRLF+'<title>'+TextToHTML(title)+'</title>'
            +CRLF+'</head>'+CRLF+'<body>'+CRLF;
end;

function footer: string;
begin
  result := CRLF+'</body>'+CRLF+'</html>';
end;

function p(const text: string; const align: string): string;
begin
  if align ='' then
    result := '<p>'+TextToHTML(text)+'</p>'+eol
  else
    result := '<p align="'+align+'">'+TextToHTML(text)+'</p>'+eol;
end;

function p_(const text: string; const align: string): string;
begin
  if align ='' then
    result := '<p>'+text+'</p>'+eol
  else
    result := '<p align="'+align+'">'+text+'</p>'+eol;
end;

function pcenter(const text: string): string;
begin
  result := '<p align="center">'+TextToHTML(text)+'</p>';
end;

function img(const FileName: string; width, height: integer;
                   centered: boolean=true; border: integer = 0): string;
var
  s: string;
begin
  if centered then s := ' align="center"' else s := '';
  result := '<p'+s+'><img border="'+IntToStr(border)+'" src="'+FileName+
          '" width="'+IntToStr(width)+'" height="'+IntToStr(height)+'"></p>';
end;

function h(n: integer; const text: string): string;
var
  sn: string;
begin
  sn := IntToStr(n);
  result := '<h'+sn+'>'+TextToHTML(text)+'</h'+sn+'>';
end;

function h1(const text: string): string;
begin
  result := '<h1>'+TextToHTML(text)+'</h1>';
end;

function h2(const text: string): string;
begin
  result := '<h2>'+TextToHTML(text)+'</h2>';
end;

function b(const text: string): string;
begin
  result := '<b>'+TextToHTML(text)+'</b>';
end;

function b_(const text: string): string;
begin
  result := '<b>'+text+'</b>';
end;

function i(const text: string): string;
begin
  result := '<i>'+TextToHTML(text)+'</i>';
end;

function i_(const text: string): string;
begin
  result := '<i>'+text+'</i>';
end;

function ftag(const tagstr, text: string; const attributs: string = ''): string;
var
  s: string;
begin
  if attributs <> '' then
    s := ' '+attributs
  else
    s := '';
  result := '<'+tagstr+s+'>'+eol+ind+ text +eol+'</'+tagstr+'>'+eol;
//  result := '<'+tagstr+s+'>'+eol+ind+TextToHTML(text)+eol+'</'+tagstr+'>'+eol;
end;

function tag(const tagstr, text: string; const attributs: string = ''): string;
var
  s: string;
begin
  if attributs <> '' then
    s := ' '+attributs
  else
    s := '';
  result := '<'+tagstr+s+'>'+TextToHTML(text)+'</'+tagstr+'>';
end;

function a(const url, text: string): string;
begin
  result := '<a href="'+url+'>'+TextToHTML(text)+'</a>';
end;

function TextToHTML(const text: string): string;
var
  i: integer;
  s: string;
begin
  s := text;
  for i := 1 to NChar do
   s := StringReplace(s, specialchar[i,1], specialchar[i,2], [rfReplaceAll, rfIgnoreCase]);
  result := s;
end;

function MemoToHTML(const Memo: TMemo): string;
var
  i: integer;
  s: string;
begin
  s := '';
  for i := 0 to Memo.Lines.Count-1 do
    s := s + TextToHTML(Memo.Lines[i])+br+CRLF;
  result := s;
end;

function StrGridToHTML(const StrGrid: TStringGrid; Caption: string;
                                                   centered: boolean): string;
var
  r, c, fr, fc: integer;
  s, ss: string;
begin

  fr := StrGrid.FixedRows;
  fc := StrGrid.FixedCols;
  if centered then
    s := '<table border="1" align="center">'+CRLF
  else
    s := '<table border="1">'+CRLF;
  if Caption <> '' then
    s := s + '<caption>'+Caption+'</caption>'+CRLF;
  for r := 0 to fr-1 do
  begin
    s := s + '<tr>'+CRLF;
    for c := 0 to StrGrid.ColCount-1 do
    begin
      ss := StrGrid.Cells[c,r];
      if ss = '' then
        s := s + '<th>'+nbsp+'</th>'+CRLF
      else
        s := s + '<th>'+TextToHTML(ss)+'</th>'+CRLF;
   end;
    s := s + '</tr>'+CRLF;
  end;

  for r := fr to StrGrid.RowCount-1 do
  begin
    s := s + '<tr>'+CRLF;
    for c := 0 to fc-1 do
      s := s + '<th>'+TextToHTML(StrGrid.Cells[c,r])+'</th>'+CRLF;

    for c := fc to StrGrid.ColCount-1 do
      s := s + '<td>'+TextToHTML(StrGrid.Cells[c,r])+'</td>'+CRLF;
    s := s + '</tr>'+CRLF;
  end;
  result := s+'</table>';
end;

function Gethtmlstr(const MemoHTML: TMemo): string;
var
  i: integer;
  s, ss: string;
begin
  ss := '';
  for i := 0 to MemoHTML.Lines.Count-3 do
  begin
    s := MemoHTML.Lines[i];
    ss := ss + s + CRLF;
  end;
  result := ss;
end;

function row(const data: array of string): string;
var
  s: string;
  i: integer;
begin
  s := '<tr>'+eol;
  for i := Low(data) to High(data) do
    s := s + '  <td>'+data[i]+'</td>'+eol;
  result := s+'</tr>'+eol;
end;


function table(const rows: array of string; border: integer = 1;
                                            cellpadding: integer = 1): string;
var
  s: string;
  i: integer;
begin
  s := Format('<table border="%d" cellpadding="%d">', [border, cellpadding])+eol;

  for i := Low(rows) to High(rows) do
    s := s + rows[i]+eol;
  result := s+'</table>'+eol;
end;

{ Thtml }

function Thtml.b(const text: string): string;
begin
  result := '<b>'+text+'</b>'+eol;
end;

function Thtml.comment(const text: string): string;
begin
  result := '<!--'+text+'-->'+eol;
end;

constructor Thtml.Create(const Name: string);
begin

end;

destructor Thtml.Destroy;
begin

  inherited;
end;

function Thtml.end_html: string;
begin
  result := '</body></html>'+eol;
end;

function Thtml.h1(const text: string): string;
begin
  result := '<h1>'+text+'</h1>'+eol;
end;

function Thtml.h2(const text: string): string;
begin
  result := '<h2>'+text+'</h2>'+eol;
end;

function Thtml.h3(const text: string): string;
begin
  result := '<h3>'+text+'</h3>'+eol;
end;

function Thtml.header: string;
begin
  result := '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">'+eol;
end;

function Thtml.i(const text: string): string;
begin
  result := '<i>'+text+'</i>'+eol;
end;

function Thtml.p(const text, align: string): string;
begin
  if align ='' then
    result := '<p>'+text+'</p>'+eol
  else
    result := '<p align="'+align+'">'+text+'</p>'+eol;
end;

function Thtml.start_html(const title: string): string;
begin
  result := '<html><head><title>'+title+'</title><head><body>'+eol;
end;

end.
