{*
 String utils
 Nikolai V. Shokhirev, 2006-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2006.10.09 - created
 @version 2007.08.05: Added chek type, YYYYMMDD function and DelphiCodeToDoc comments
 @version 2009.05.23 renamed: Split -> SplitToISArr, added SplitToISL
 @version 2009.07.11 Added Isdate, IsTime, IsFloat0.
 @version 2010.11.11 Added SetSplitToStrArr and SplitToStrArr.
 }
unit uStrFun;

interface

uses
  classes
  , uMatTypes
  , uDynArrays
  , SysUtils
  , uTypedLists
  ;
const
  NullValues: array[1..5] of string = ('', 'N.A.','N/A', 'NULL', 'NAN');
  BoolValues: array[1..4] of string = ('FALSE','TRUE','NO','YES');
  _t: char = #09;
  TabDelimiter = #09;


{*
  Random string of length len
  code = 0 : s[i] in [' ','A'..'Z','a'..'z','0'..'9']
}
function RandStr(len: integer; code: integer = 0): string;
/// result = s[i1]+s[i1+1]+...+s[i2]
function CopyFromTo(const s: string; i1, i2: integer): string;

function RemoveDuplicateSpaces(s: string): string;

function StrFmt(Value: integer; w: integer): string; overload;

function StrFmt(Value: TFloat; w, d: integer): string; overload;

// Sets array length and splits
/// s - input string
/// schar - delimiter
/// sa - output array
/// Count = Length(sa) - will be passed to SplitToStrArr
procedure SetSplitToStrArr(const s: AnsiString; schar: AnsiChar; var sa: TSArr; var Count: TInt);

// Checks Count and splits
/// s - input string
/// schar - delimiter
/// sa - output array
/// Count = Length(sa) - expected number of fields
procedure SplitToStrArr(const s: AnsiString; schar: AnsiChar; const sa: TSArr; Count: TInt);

function SplitToISArr(const s: string; schar: char = ','):ISArr1D;

function CombineSArr(const sa: TSArr1D; schar: char): string;

procedure SplitToSL(const s: string; schar: char; const SL :TStringList);

/// s - input string
/// d - delimiter
procedure SplitToStringList(const s, d: string; const SL :TStringList);

function SplitToISL(const s: string; schar: char):IStringList;
///
function CombineSL(const SL :TStringList; schar: char): string;
/// valid integer
function IsInt(const s: string): boolean;
/// valid float
function IsFloat(const s: string): boolean;
/// float starting with .
function IsFloat0(const s: string): boolean;
/// one of ('FALSE','TRUE','NO','YES')
function IsBool(const s: string): boolean;
/// valid DateTime
function IsDateTime(const s: string): boolean;
/// valid Date
function IsDate(str: string): Boolean;
/// valid Time
function IsTime(str: string): Boolean;
/// one of ('', 'N.A.','N/A', 'NULL', 'NAN')
function IsNULL(const s: string): boolean;
/// e.g. 20070817 -> date
function DateToYYYYMMDD(date: TDateTime): string;
/// e.g. date -> 20070817
function YYYYMMDDToDate(date: string): TDateTime;
/// from 12/30/1899 12:00 am  To 1/18/2000 5:02:24 AM
function RandDateTime: TDateTime;
/// from '12/30/1899 12:00 AM' To '1/18/2000 5:02:24 AM'
function RandDateTimeStr: string;
/// Usage: AddLeadingZeroes(2005, 10) ;
/// Will result in a '0000002005' string value.
//  http://delphi.about.com/od/adptips2005/qt/leadingzero.htm
function AddLeadingZeroes(const aNumber, Length : integer): string;

function TimeStepToStr(TimeStep: TDateTime): string;

function QuotedCSV(StringListCSV: string): string;

implementation

uses
  math
  , StrUtils
  ;

function RandStr(len: integer; code: integer = 0): string;
var
  idx: integer;
  c: char;
begin
  result := '';
  case code of
  1 : repeat  // Space + Letters + numbers
        c := Chr(Random(95) + 32);
        if c in [' ','A'..'Z','a'..'z','0'..'9'] then
          result := result + c;
      until Length(result) >= len;
  2 : repeat  // Space + Letters
        c := Chr(Random(95) + 32);
        if c in [' ','A'..'Z','a'..'z'] then
          result := result + c;
      until Length(result) >= len;
  else
      for idx := 1 to len do  // All printable characters: 32 <= ord <= 126
        result := result + Chr(Random(95) + 32);
  end;
end;

function CopyFromTo(const s: string; i1, i2: integer): string;
begin
  result := Copy(s, i1, i2-i1+1);
end;

function RemoveDuplicateSpaces(s: string): string;
begin
  result := s;
  while Pos('  ', result) > 0 do
    result := StringReplace(result, '  ', ' ', [rfReplaceAll]);
end;

function StrFmt(Value: integer; w: integer): string;
var
  s: string;
begin
  if Value = _ then
    result := StringOfChar(' ', w)
  else
  begin
    str(Value:w, s);
    result := s;
  end;
end;

function StrFmt(Value: TFloat; w, d: integer): string;
var
  s: string;
begin
  str(Value:w:d, s);
  result := s;
end;

// Sets length and splits
procedure SetSplitToStrArr(const s: AnsiString; schar: AnsiChar; var sa: TSArr; var Count: TInt);
var
  j: integer;
begin
  Count := 1;
  for j := 1 to Length(s) do
    if s[j] = schar then inc(Count);

  SetLength(sa, Count);

  SplitToStrArr(s, schar, sa, Count);
end;

// Checks Count and splits
procedure SplitToStrArr(const s: AnsiString; schar: AnsiChar; const sa: TSArr; Count: TInt);
var
  j, n, ia, L: integer;
begin
  L := Length(s);
  ia := 0;
  n := 1;
  for j := 1 to L do
    if s[j] = schar then
    begin
    if ia < Count then
      sa[ia] := Copy(s, n, j-n)
    else
      raise Exception.CreateFmt('String %s does not match array dim %d', [s, Count]);
      n := j+1;
      inc(ia);
    end;;
    if ia = (Count-1) then
      sa[ia] := Copy(s, n, L-n+1)
    else
      raise Exception.CreateFmt('String %s does not match array dim %d', [s, Count]);
end;

function SplitToISArr(const s: string; schar: char):ISArr1D;
var
  j, n, m, ia: integer;
  sa: TSArr1D;
//  ss: string;
begin
  n := 0;
  m := Length(s);
  for j := 1 to m do
    if s[j] = schar then inc(n);

  sa := TSArr1D.Create(n+1);
  ia := 1;
//  ss := '';
  n := 1;
  for j := 1 to m do
    if s[j] = schar then
    begin
      sa[ia] := CopyFromTo(s, n, j-1);
      n := j+1;
      inc(ia);
    end;;
  if s <> '' then
    sa[ia] := CopyFromTo(s, n, m)
  else
    sa[ia] := '';
  result := sa;
end;

function CombineSArr(const sa: TSArr1D; schar: char): string;
var
  j: integer;
  ss, s: string;
begin
  s := schar;
  ss := sa[sa.Lo1];
  for j := sa.Lo1+1 to sa.Hi1 do
  begin
    ss := ss+s+sa[j];
  end;
  result := ss;
end;

procedure SplitToSL(const s: string; schar: char; const SL :TStringList);
var
  j, n, m: integer;
begin
  m := Length(s);
  SL.Clear;
  n := 1;
  for j := 1 to m do
    if s[j] = schar then
    begin
      SL.Append(CopyFromTo(s, n, j-1));
      n := j+1;
    end;
  if s <> '' then
    SL.Append(CopyFromTo(s, n, m))
  else
    SL.Append('');
end;

procedure SplitToStringList(const s, d: string; const SL :TStringList);
var
  p, i, l, k: integer;
  done: boolean;
begin
  done := false;
  i := 1;
  l := Length(s);
  k := Length(d);
  SL.Clear;
  repeat
    p := PosEx(d, s, i);
    if p = 0 then
    begin
      p := l + 1;
      done := true;
    end;
    SL.Append(copy(s, i, p-i));
    i := p + k;

    if not done and (i > l ) then
    begin
      SL.Append('');
      done := true;
    end;
  until done;
end;

function CombineSL(const SL :TStringList; schar: char): string;
var
  j: integer;
  ss, s: string;
begin
  s := schar;
  ss := SL[0];
  for j := 1 to SL.Count-1 do
  begin
    ss := ss+s+SL[j];
  end;
  result := ss;
end;

function SplitToISL(const s: string; schar: char):IStringList;
var
  j, n, m: integer;
  sa: IStringList;
  ss: string;
begin
  m := Length(s);
  sa := TSimpleStrList.Create;
  n := 1;
  for j := 1 to m do
    if s[j] = schar then
    begin
      ss := CopyFromTo(s, n, j-1);
      sa.Append(ss);
      n := j+1;
    end;
  if s <> '' then
    ss := CopyFromTo(s, n, m)
  else
    ss := '';
  sa.Append(ss);
  result := sa;
end;

function IsInt(const s: string): boolean;
begin
  Result := True;
  try
    StrToInt(s);
  except
    Result := False;
  end;
end;

function IsFloat(const s: string): boolean;
var
  Code: Integer;
  x: TFloat;
begin
  Val(s, x, Code);
  result := (Code = 0) and (Pos(DecimalSeparator,s) > 0)
end;

function StrToFloat0(const s: string): TFloat;
begin
//  result := StrToFloat(StringReplace(s,DecimalSeparator,'0'+DecimalSeparator,[]));
  result := StrToFloat(StringReplace(s,'.','0.',[]));
end;

function IsFloat0(const s: string): boolean;
begin
  Result := True;
  try
    StrToFloat0(s);
  except
    Result := False;
  end;
end;

function IsBool(const s: string): boolean;
var
  i: Integer;
begin
  result := false;
  for i := Low(BoolValues) to High(BoolValues) do
    if SameText(BoolValues[i],Trim(s)) then
    begin
      result := true;
      break;
    end;
end;

function IsDateTime(const s: string): boolean;
var
  t: TDateTime;
  d: double;
begin
{
  result := true;
  try
    t := StrToDateTime(s);
  except
    result := false;
  end;
}
  d := -1.0;
  t := StrToDateTimeDef(s, TDateTime(d));
  result := (t > 0.0);
end;

function IsDate(str: string): Boolean;
begin
  Result := True;
  try
    StrToDate(str);
  except
    Result := False;
  end;
end;

function IsTime(str: string): Boolean;
begin
  Result := True;
  try
    StrToTime(str);
  except
    Result := False;
  end;
end;

function IsNULL(const s: string): boolean;
var
  i: Integer;
begin
  result := false;
  for i := Low(NullValues) to High(NullValues) do
    if SameText(NullValues[i],Trim(s)) then
    begin
      result := true;
      break;
    end;
end;

function DateToYYYYMMDD(date: TDateTime): string;
var
  Year, Month, Day: Word;
begin
  DecodeDate(date, Year, Month, Day);
  result := Format('%4.4d%2.2d%2.2d', [Year, Month, Day]);
end;

function YYYYMMDDToDate(date: string): TDateTime;
var
  Year, Month, Day: Word;
begin
  Year := StrToInt(CopyFromTo(date,1,4));
  Month  := StrToInt(CopyFromTo(date,5,6));
  Day  := StrToInt(CopyFromTo(date,7,8));
  result := EncodeDate(Year, Month, Day);
end;

function RandDateTime: TDateTime;
var
  d: double;
begin
  d := 36543.21*random;
  result := TDateTime(d);
end;

function RandDateTimeStr: string;
begin
  result := DateTimeToStr(RandDateTime);
end;

function AddLeadingZeroes(const aNumber, Length : integer) : string;
begin
   result := SysUtils.Format('%.*d', [Length, aNumber]) ;
end;

function TimeStepToStr(TimeStep: TDateTime): string;
var
  t: double;
  d, h, m, s, ms: integer;
begin
  t := double(TimeStep);
  result := '';
  d  := Floor(t);
  t  := Frac(t)*24.0;
  h  := Floor(t);
  t  := Frac(t)*60.0;
  m  := Floor(t);
  t  := Frac(t)*60.0;
  s  := Floor(t);
  t  := Frac(t)*1000.0;
  ms := Round(t);
//  ms := Floor(t);
//  t  := Frac(t);
  if d > 0 then result := result + Format(' %d d', [d]);
  if h > 0 then result := result + Format(' %d h', [h]);
  if m > 0 then result := result + Format(' %d m', [m]);
  if s > 0 then result := result + Format(' %d s', [s]);
  if ms > 0 then result := result + Format(' %d ms', [ms]);
end;

// Fix  StringList CSV
function QuotedCSV(StringListCSV: string): string;
begin
  result := '"'+StringReplace(StringListCSV,',','","',[rfReplaceAll])+'"';
end;

// http://delphi.about.com/od/adptips2005/qt/parsedelimited.htm
procedure ParseDelimited(const sl : TStrings; const value : string; const delimiter : string) ;
var
   dx : integer;
   ns : string;
   txt : string;
   delta : integer;
begin
   delta := Length(delimiter) ;
   txt := value + delimiter;
   sl.BeginUpdate;
   sl.Clear;
   try
     while Length(txt) > 0 do
     begin
       dx := Pos(delimiter, txt) ;
       ns := Copy(txt,0,dx-1) ;
       sl.Add(ns) ;
       txt := Copy(txt,dx+delta,MaxInt) ;
     end;
   finally
     sl.EndUpdate;
   end;
end;

end.
