{*
 I/O routines for dynamic arrays
 Nikolai V. Shokhirev, 2002-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2002.02.02 - created
 @version 2005.04.12 moved (Read/Save) XYDValues routines to the interface section
            for the use in uDynDataIO
 @version 2005.05.27 aReplaced: FloatToStr -> FloatToStrF,
            WriteLn(TF, x) -> WriteLn(TF, FloatToStrF(x,ffGeneral,15,3))
 @version 2005.09.25 in FArr2DSave added parameter AsTable, default = false
 @version 2007.07.07 - removed compiler warnings, extended comments, other cleanup
 @version 2007.07.15 - Added DelphiCodeToDoc comments
 @version 2008.08.24 - Switched to default parameters
}
unit  uDynArrIO;

interface

uses
  SysUtils, uMatTypes, uDynArrays, uDynArrUtils;

const

  sIOVersion = '1.2'; /// file label
  sSArr1D = 'SArr1D'; /// file label
  sIArr1D = 'IArr1D'; /// file label
  sIArr2D = 'IArr2D'; /// file label
  sIArr3D = 'IArr3D'; /// file label
  sFArr1D = 'FArr1D'; /// file label
  sFArr2D = 'FArr2D'; /// file label
  sFArr3D = 'FArr3D'; /// file label
  sCArr1D = 'CArr1D'; /// file label
  sCArr2D = 'CArr2D'; /// file label

type
  /// CLASS(Exception);
  EDynReadErr = CLASS(Exception);

/// like in const arrays 
procedure IVectorToStr(const V: IIArr1D; var s: string);

/// like in const arrays 
procedure StrToVector(const V: IFArr1D; ss: string);
/// like in const arrays 
procedure VectorToStr(const V: IFArr1D; var s: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; Digits: integer = 2);

/// like in const arrays 
procedure StrToMatrix(const M: IFArr2D; ss: string);
/// like in const arrays 
procedure MatrixToStr(const M: IFArr2D; var s: string; ds: string = #13#10;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; Digits: integer = 2);

///attr is a table attribute(s0, e.g. ' border="1" cellpadding="2" align="center"'
procedure MatrixToHTML(const M: IFArr2D; var s: string; const attr: string ='';
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; Digits: integer = 2);

/// Create from File 
function SArr1DCreate(FileName: string): ISArr1D;
/// Save to File 
procedure SArr1DSave(const A: ISArr1D; FileName: string);
/// Load data from File 
procedure SArr1DRead(const A: ISArr1D; FileName: string);

/// Create from File 
function IArr1DCreate(FileName: string): IIArr1D;
/// Save to File 
procedure IArr1DSave(const A: IIArr1D; FileName: string);
/// Load data from File 
procedure IArr1DRead(const A: IIArr1D; FileName: string);

/// Create from File 
function IArr2DCreate(FileName: string): IIArr2D;
/// Save to File 
procedure IArr2DSave(const A: IIArr2D; FileName: string);
/// Load data from File 
procedure IArr2DRead(const A: IIArr2D; FileName: string);

/// Create from File 
function IArr3DCreate(FileName: string): IIArr3D;
/// Save to File 
procedure IArr3DSave(const A: IIArr3D; FileName: string);
/// Load data from File 
procedure IArr3DRead(const A: IIArr3D; FileName: string);

/// Create from File 
function FArr1DCreate(FileName: string): IFArr1D;
/// Save to File 
procedure FArr1DSave(const A: IFArr1D; FileName: string);
/// Load data from File 
procedure FArr1DRead(const A: IFArr1D; FileName: string);

/// Create from File 
function FArr2DCreate(FileName: string): IFArr2D;

{*
 Save to File
Version ( 1.2 )
Type    (FArr2D)
Comment
Lo1 Hi1 -4 -1
if AsTable = false
<Values>
A[Lo1,  Lo2]
A[Lo1+1,Lo2]
A[Lo1+2,Lo2]
. . .
A[H11,Hi2]
</Values>

if AsTable = true
<Table>
A[Lo1,  Lo2] A[Lo1,  Lo2+1] . . A[Lo1,  Hi2]
A[Lo1+1,Lo2] A[Lo1+1,Lo2+1] . . A[Lo1+1,Hi2]
. . . . . . . . . . . . . . . . . . . . . .
A[Hi1,  Lo2] A[Hi1,  Lo2+1] . . A[Hi1,  Hi2]
</Table>                                          
}
procedure FArr2DSave(const A: IFArr2D; FileName: string;
                     AsTable: boolean = false);
/// Load data from File 
procedure FArr2DRead(const A: IFArr2D; FileName: string);
//procedure FArr2DRead(const A: IFArr2D; FileName: string; AsTable: boolean = false);

/// Create from File 
function FArr3DCreate(FileName: string): IFArr3D;
/// Save to File 
procedure FArr3DSave(const A: IFArr3D; FileName: string);
/// Load data from File 
procedure FArr3DRead(const A: IFArr3D; FileName: string);

/// Create from File 
function CArr1DCreate(FileName: string): ICArr1D;
/// Save to File 
procedure CArr1DSave(const A: ICArr1D; FileName: string);
/// Load data from File 
procedure CArr1DRead(const A: ICArr1D; FileName: string);

/// Create from File 
function CArr2DCreate(FileName: string): ICArr2D;
/// Save to File 
procedure CArr2DSave(const A: ICArr2D; FileName: string);
/// Load data from File 
procedure CArr2DRead(const A: ICArr2D; FileName: string);

/// helper
procedure ReadS1DValues(const A: ISArr1D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveS1DValues(const A: ISArr1D; var TF: TextFile; const TagName: string);
/// helper
procedure ReadI1DValues(const A: IIArr1D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveI1DValues(const A: IIArr1D; var TF: TextFile; const TagName: string);
/// helper
procedure ReadI2DValues(const A: IIArr2D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveI2DValues(const A: IIArr2D; var TF: TextFile; const TagName: string);
/// helper
procedure ReadI3DValues(const A: IIArr3D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveI3DValues(const A: IIArr3D; var TF: TextFile; const TagName: string);
/// helper
procedure ReadF1DValues(const A: IFArr1D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveF1DValues(const A: IFArr1D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
/// helper
procedure ReadF2DValues(const A: IFArr2D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveF2DValues(const A: IFArr2D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
/// helper
procedure ReadF3DValues(const A: IFArr3D; var TF: TextFile; var TagName: string);
/// helper
procedure SaveF3DValues(const A: IFArr3D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);

/// helper
procedure SaveF2DTable(const A: IFArr2D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
/// helper
procedure ReadF2DTable(const A: IFArr2D; var TF: TextFile; var TagName: string);

ResourceString
  /// 'Error Creating From File:%s';
  RS_ErrorCreatingFromFile = 'Error Creating From File:%s';
  /// 'Saving To File:%s, Error:%s';
  RS_ErrorSavingToFile = 'Saving To File:%s, Error:%s';
  /// 'Incorrect Type:%s';
  RS_IncorrectType = 'Incorrect Type:%s';
  // 'Lim Mismatch';
  RS_LimMismatch = 'Lim Mismatch';
  /// 'Tag Mismatch: %s <> %s';
  RS_TagMismatch = 'Tag Mismatch: %s <> %s';

implementation

uses
  Math, classes, Dialogs;

procedure StrToVector(const V: IFArr1D; ss: string);
var
  j1: integer;
  s: string;
  sl: TStringList;
begin
  s := trim(ss);
  s := copy(s,2,Length(s)-2);
  sl := TStringList.Create;
  sl.CommaText := s;
  try
    for j1 := V.Lo1 to V.Hi1 do
     V[j1] := StrToFloat(sl[j1-1]);
  finally
    sl.Free;
  end;
end;

procedure IVectorToStr(const V: IIArr1D; var s: string);
var
  i1: integer;
begin
  s := '(';
  for i1 := V.Lo1 to V.Hi1 do
  begin
    if i1 > V.Lo1 then
      s := s + ',';
    s := s + IntToStr(V[i1]);
  end;
  s := s + ')';
end;

procedure VectorToStr(const V: IFArr1D; var s: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1: integer;
begin
  s := '(';
  for i1 := V.Lo1 to V.Hi1 do
  begin
    if i1 > V.Lo1 then
      s := s + ',';
    s := s + FloatToStrF(V[i1], fmt, precision, digits);
  end;
  s := s + ')';
end;

procedure StrToMatrix(const M: IFArr2D; ss: string);
var
  i1, i2, j1, j2: integer;
  s: string;
  sl: TStringList;
begin
  s := trim(ss);
  s := copy(s,2,Length(s)-2);
  sl := TStringList.Create;
  j1 := M.Lo1;
  for i1 := 1 to Length(s) do
  begin
    if s[i1] = '(' then
    begin
      sl.Clear;
      for i2 := i1+1 to Length(s) do
      begin
        if s[i2] = ')' then
        begin
          sl.CommaText := copy(s,i1+1,i2-i1-1);
          break;
        end;
      end;

      for j2 := M.Lo2 to M.Hi2 do
        M[j1, j2] := StrToFloat(sl[j2-1]);
     inc(j1);
    end;
  end;
  sl.Free;
end;

procedure MatrixToStr(const M: IFArr2D; var s: string; ds: string = #13#10;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1, i2: integer;
begin
  s := '(';
  for i1 := M.Lo1 to M.Hi1 do
  begin
    if i1 > M.Lo1 then
      s := s + ',';
    s := s + '(';
    for i2 := M.Lo2 to M.Hi2 do
    begin
      if i2 > M.Lo2 then
        s := s + ',';
      s := s + FloatToStrF(M[i1,i2], fmt, precision, digits);
    end;
    s := s + ')'+ds;
  end;
    s := s + ')';
end;

procedure MatrixToHTML(const M: IFArr2D; var s: string; const attr: string ='';
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1, i2: integer;
  b: string;
begin
  b := '';
  if attr <> '' then
    if attr[1] <> ' ' then b := ' ';
  s := '<table'+b+attr+'>'#13#10;
  for i1 := M.Lo1 to M.Hi1 do
  begin
    s := s + '<tr>'#13#10;
    for i2 := M.Lo2 to M.Hi2 do
      s := s + '<td>'+FloatToStrF(M[i1,i2], fmt, precision, digits)+'</td>'#13#10;
    s := s + '</tr>'#13#10;
  end;
    s := s + '</table>';
end;

{ Internal Procedures }

function GetOpenTagName(const s: string): string;
var
  p1, p2: integer;
begin
  p1 := Pos('<',s);
  p2 := Pos('>',s);
  result := Copy(s,P1+1,p2-p1-1);
end;

function GetCloseTagName(const s: string): string;
var
  p1, p2: integer;
begin
  p1 := Pos('</',s);
  p2 := Pos('>',s);
  result := Copy(s,P1+2,p2-p1-2);
end;

function SetOpenTag(const TagName: string): string;
begin
  result := '<'+TagName+'>';
end;

function SetCloseTag(const TagName: string): string;
begin
  result := '</'+TagName+'>';
end;

procedure ReadS1DValues(const A: ISArr1D; var TF: TextFile; var TagName: string);
var
  i1: TInt;
  s: string;
  x: string;
begin
  try
    ReadLn(TF, s); // <Values>
    TagName := GetOpenTagName(s);
    for i1 := A.Lo1 to A.Hi1 do
    begin
      ReadLn(TF, x);
      A[i1] := x;
    end;
    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: EInOutError do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveS1DValues(const A: ISArr1D; var TF: TextFile; const TagName: string);
var
  i1: TInt;
  x: string;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i1 := A.Lo1 to A.Hi1 do
    begin
      x := A[i1];
      WriteLn(TF, x);
    end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure ReadI1DValues(const A: IIArr1D; var TF: TextFile; var TagName: string);
var
  i1: TInt;
  s: string;
  x: TInt;
begin
  try
    ReadLn(TF, s); // <Values>
    TagName := GetOpenTagName(s);
    for i1 := A.Lo1 to A.Hi1 do
    begin
      ReadLn(TF, x);
      A[i1] := x;
    end;
    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveI1DValues(const A: IIArr1D; var TF: TextFile; const TagName: string);
var
  i1: TInt;
  x: TInt;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i1 := A.Lo1 to A.Hi1 do
    begin
      x := A[i1];
      WriteLn(TF, x);
    end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure ReadI2DValues(const A: IIArr2D; var TF: TextFile; var TagName: string);
var
  i1, i2: TInt;
  s: string;
  x: TInt;
begin
  try
    ReadLn(TF, s); // <Values>
    TagName := GetOpenTagName(s);
    for i2 := A.Lo2 to A.Hi2 do
      for i1 := A.Lo1 to A.Hi1 do
      begin
        ReadLn(TF, x);
        A[i1,i2] := x;
      end;
    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveI2DValues(const A: IIArr2D; var TF: TextFile; const TagName: string);
var
  i1, i2: TInt;
  x: TInt;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i2 := A.Lo2 to A.Hi2 do
      for i1 := A.Lo1 to A.Hi1 do
      begin
        x := A[i1,i2];
        WriteLn(TF, x);
      end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure ReadI3DValues(const A: IIArr3D; var TF: TextFile; var TagName: string);
var
  i1, i2, i3: TInt;
  s: string;
  x: TInt;
begin
  try
    ReadLn(TF, s); // <Values>
    TagName := GetOpenTagName(s);
    for i3 := A.Lo3 to A.Hi3 do
      for i2 := A.Lo2 to A.Hi2 do
        for i1 := A.Lo1 to A.Hi1 do
        begin
          ReadLn(TF, x);
          A[i1,i2,i3] := x;
        end;
    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveI3DValues(const A: IIArr3D; var TF: TextFile; const TagName: string);
var
  i1, i2, i3: TInt;
  x: TInt;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i3 := A.Lo3 to A.Hi3 do
      for i2 := A.Lo2 to A.Hi2 do
        for i1 := A.Lo1 to A.Hi1 do
        begin
          x := A[i1,i2,i3];
          WriteLn(TF, x);
        end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure ReadF1DValues(const A: IFArr1D; var TF: TextFile; var TagName: string);
var
  i1: TInt;
  s: string;
  x: TFloat;
begin
  try
    ReadLn(TF, s); // <Values>
    TagName := GetOpenTagName(s);
    for i1 := A.Lo1 to A.Hi1 do
    begin
      ReadLn(TF, x);
      A[i1] := x;
    end;
    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveF1DValues(const A: IFArr1D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1: TInt;
  s: string;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i1 := A.Lo1 to A.Hi1 do
    begin
//      x := A[i1];
      s := FloatToStrF(A[i1], fmt, precision, digits);
      WriteLn(TF, s);
    end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure ReadF2DValues(const A: IFArr2D; var TF: TextFile; var TagName: string);
var
  i1, i2: TInt;
  s: string;
  x: TFloat;
begin
  try
    ReadLn(TF, s); // <Values> or <Table>
    TagName := GetOpenTagName(s);

    if Pos('Values', TagName) > 0 then
      for i2 := A.Lo2 to A.Hi2 do
        for i1 := A.Lo1 to A.Hi1 do
        begin
          ReadLn(TF, x);
          A[i1,i2] := x;
        end
    else
      for i1 := A.Lo1 to A.Hi1 do
      begin
        for i2 := A.Lo2 to A.Hi2 do
        begin
          Read(TF, x);
          A[i1,i2] := x;
        end;
        ReadLn(TF);
      end;

    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure ReadF2DTable(const A: IFArr2D; var TF: TextFile; var TagName: string);
var
  i1, i2: TInt;
  s: string;
  x: TFloat;
begin
  try
    ReadLn(TF, s); // <Table>
    TagName := GetOpenTagName(s);
    for i1 := A.Lo1 to A.Hi1 do
    begin
      for i2 := A.Lo2 to A.Hi2 do
      begin
        Read(TF, x);
        A[i1,i2] := x;
      end;
      ReadLn(TF);
    end;
    ReadLn(TF, s); // </Table>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveF2DValues(const A: IFArr2D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1, i2: TInt;
  s: string;
//  x: TFloat;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i2 := A.Lo2 to A.Hi2 do
      for i1 := A.Lo1 to A.Hi1 do
      begin
//        x := A[i1,i2];
        s := FloatToStrF(A[i1,i2], fmt, precision, digits);
        WriteLn(TF, s);
      end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure SaveF2DTable(const A: IFArr2D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1, i2: TInt;
  s: string;
//  x: TFloat;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i1 := A.Lo1 to A.Hi1 do
    begin
      for i2 := A.Lo2 to A.Hi2 do
      begin
        s := ' '+FloatToStrF(A[i1,i2], fmt, precision, digits);
        Write(TF, s);
      end;
      WriteLn(TF);
    end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

procedure ReadF3DValues(const A: IFArr3D; var TF: TextFile; var TagName: string);
var
  i1, i2, i3: TInt;
  s: string;
  x: TFloat;
begin
  try
    ReadLn(TF, s); // <Values>
    TagName := GetOpenTagName(s);
    for i3 := A.Lo3 to A.Hi3 do
      for i2 := A.Lo2 to A.Hi2 do
        for i1 := A.Lo1 to A.Hi1 do
        begin
          ReadLn(TF, x);
          A[i1,i2,i3] := x;
        end;
    ReadLn(TF, s); // </Values>
    s := GetCloseTagName(s);
    if s<>TagName then
      raise EDynReadErr.CreateFmt(RS_TagMismatch, [TagName,s]);
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorCreatingFromFile,[E.Message]));
  end;
end;

procedure SaveF3DValues(const A: IFArr3D; var TF: TextFile; const TagName: string;
   fmt: TFloatFormat = ffGeneral; precision: integer = 13; digits: integer = 2);
var
  i1, i2, i3: TInt;
  s: string;
//  x: TFloat;
begin
  try
    WriteLn(TF, SetOpenTag(TagName));
    for i3 := A.Lo3 to A.Hi3 do
      for i2 := A.Lo2 to A.Hi2 do
        for i1 := A.Lo1 to A.Hi1 do
        begin
//          x := A[i1,i2,i3];
          s := FloatToStrF(A[i1,i2,i3], fmt, precision, digits);
          WriteLn(TF, s);
        end;
    WriteLn(TF, SetCloseTag(TagName));
  except
    on E: Exception do
      ShowMessage(Format(RS_ErrorSavingToFile,[E.Message]));
  end;
end;

{ Dynamic array I/O routines }

function SArr1DCreate(FileName: string): ISArr1D;
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: ISArr1D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sSArr1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TSArr1D.Create(L1, H1);
    t.Comment := aComment;
    ReadS1DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure SArr1DRead(const A: ISArr1D; FileName: string);
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: ILim1d;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sSArr1d then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TLim1d.Create(L1, H1);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadS1DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure SArr1DSave(const A: ISArr1D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sIArr1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,  ' ',A.Hi1);
    SaveS1DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

function IArr1DCreate(FileName: string): IIArr1D;
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: IIArr1D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sIArr1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TIArr1D.Create(L1, H1);
    t.Comment := aComment;
    ReadI1DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure IArr1DRead(const A: IIArr1D; FileName: string);
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: ILim1d;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sIArr1d then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TLim1d.Create(L1, H1);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadI1DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure IArr1DSave(const A: IIArr1D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sIArr1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,  ' ',A.Hi1);
    SaveI1DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

function IArr2DCreate(FileName: string): IIArr2D;
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: IIArr2D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sIArr2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TIArr2D.Create(L1, H1, L2, H2);
    t.Comment := aComment;
    ReadI2DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure IArr2DRead(const A: IIArr2D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: ILim2D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sIArr2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TLim2D.Create(L1, H1, L2, H2);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadI2DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure IArr2DSave(const A: IIArr2D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sIArr2D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2);
    SaveI2DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

function IArr3DCreate(FileName: string): IIArr3D;
var
  TF: TextFile;
  L1, H1, L2, H2, L3, H3: TInt;
  aVersion, aComment, s: string;
  t: IIArr3D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sIArr3D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2, L3, H3);
    t := TIArr3D.Create(L1, H1, L2, H2, L3, H3);
    t.Comment := aComment;
    ReadI3DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure IArr3DRead(const A: IIArr3D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2, L3, H3: TInt;
  aVersion, aComment, s: string;
  t: ILim3D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sIArr3D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2, L3, H3);
    t := TLim3D.Create(L1, H1, L2, H2, L3, H3);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadI3DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure IArr3DSave(const A: IIArr3D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sIArr3D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2,' ',A.Lo3,' ',A.Hi3);
    SaveI3DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

function FArr1DCreate(FileName: string): IFArr1D;
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: IFArr1D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArr1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TFArr1D.Create(L1, H1);
    t.Comment := aComment;
    ReadF1DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure FArr1DRead(const A: IFArr1D; FileName: string);
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: ILim1d;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArr1d then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TLim1d.Create(L1, H1);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadF1DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure FArr1DSave(const A: IFArr1D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFArr1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,  ' ',A.Hi1);
    SaveF1DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

function FArr2DCreate(FileName: string): IFArr2D;
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: IFArr2D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArr2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TFArr2D.Create(L1, H1, L2, H2);
    t.Comment := aComment;
    ReadF2DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

//procedure FArr2DRead(const A: IFArr2D; FileName: string; AsTable: boolean);
procedure FArr2DRead(const A: IFArr2D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: ILim2D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArr2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TLim2D.Create(L1, H1, L2, H2);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
      ReadF2DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure FArr2DSave(const A: IFArr2D; FileName: string; AsTable: boolean);
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFArr2D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2);
    if AsTable then
      SaveF2DTable(A, TF, 'Table')
    else
      SaveF2DValues(A, TF, 'Values');
  finally
    CloseFile(TF);
  end;
end;

function FArr3DCreate(FileName: string): IFArr3D;
var
  TF: TextFile;
  L1, H1, L2, H2, L3, H3: TInt;
  aVersion, aComment, s: string;
  t: IFArr3D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArr3D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2, L3, H3);
    t := TFArr3D.Create(L1, H1, L2, H2, L3, H3);
    t.Comment := aComment;
    REadF3DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure FArr3DRead(const A: IFArr3D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2, L3, H3: TInt;
  aVersion, aComment, s: string;
  t: ILim3D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArr3D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2, L3, H3);
    t := TLim3D.Create(L1, H1, L2, H2, L3, H3);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadF3DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure FArr3DSave(const A: IFArr3D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFArr3D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2,' ',A.Lo3,' ',A.Hi3);
    SaveF3DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

{ Create from File }
function CArr1DCreate(FileName: string): ICArr1D;
//var
//  TF: TextFile;
begin
  assert(false,' Not implemented');
end;

{ Save to File }
procedure CArr1DSave(const A: ICArr1D; FileName: string);
const
  sr = 'ReValues';
  si = 'ImValues';
var
  TF: TextFile;
//  aVersion, aComment, s: string;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sCArr1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1);
    SaveF1DValues(A.Re, TF, sr);
    SaveF1DValues(A.Im, TF, si);
  finally
    CloseFile(TF);
  end;
end;

{ Load data from File }
procedure CArr1DRead(const A: ICArr1D; FileName: string);
//var
//  TF: TextFile;
begin
  assert(false,' Not implemented');
end;

{ Create from File }
function CArr2DCreate(FileName: string): ICArr2D;
//var
//  TF: TextFile;
begin
  assert(false,' Not implemented');
end;

{ Save to File }
procedure CArr2DSave(const A: ICArr2D; FileName: string);
//var
//  TF: TextFile;
begin
  assert(false,' Not implemented');
end;

{ Load data from File }
procedure CArr2DRead(const A: ICArr2D; FileName: string);
//var
//  TF: TextFile;
begin
  assert(false,' Not implemented');
end;

end.
