{*
  Linear algebra utilities and basic routines
  Nikolai V. Shokhirev, 2001-2007
 @author(Nikolai Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2001.02.02 - created
 @version 2005.01.22 - added SortIArr1D
 @version 2005.06.05 - added Row1xRow2, Col1xCol2
 @version  2007.07.07 - added AssignVector, ResizeF1D and ResizeF2D, extended comments
 @version 2007.07.15 - Added DelphiCodeToDoc comments
 @version 2010.01.10 - Added MaxAbsDiff
 }
unit uDynArrUtils;

interface

uses
  uMatTypes, uDynArrays;

{*
  if minIdx = MaxInt then Lo := Lo1 else Lo := max(minIdx, Lo1);
  if maxIdx = MaxInt then Hi := Hi1 else Hi := min(maxIdx, Hi1);
  Dim := Hi - Lo + 1;
}
procedure SetMinMaxIdx(minIdx, maxIdx, Lo1, Hi1: TInt; var Lo, Hi, Dim: TInt);

{*
 OldA -> NewA
 OldA =     |x x x x x x|
 NewA = |* * * * * * *|
  If CopyData :
 NewA=  |0 0|x x x x x|
}
procedure ResizeF1D(var A: IFArr1D; NewLo1, NewHi1: TInt; CopyData: boolean = true); overload;
///  OldA -> NewA
procedure ResizeF1D(var A: IFArr1D; NewLim: ILim1D; CopyData: boolean = true); overload;
///  OldA -> NewA
procedure ResizeF1D(var A: IFArr1D; NewDim1: TInt; CopyData: boolean = true); overload;

{*
 OldA -> NewA
                                              If CopyData :
                                 |* * * * * *|              |0 0 0 0 0 0|
            |x x x x x x|        |* * * * * *|              |0 0|x x x x|. .|
 OldA =     |x x x x x x| NewA = |* * * * * *|       NewA = |0 0|x x x x|. .|
            |x x x x x x|        |* * * * * *|              |0 0|x x x x|. .|
                                 |* * * * * *|              |0 0 0 0 0 0|      
}
procedure ResizeF2D(var A: IFArr2D; NewLo1, NewHi1, NewLo2, NewHi2: TInt;
                                          CopyData: boolean = true); overload;
///  OldA -> NewA
procedure ResizeF2D(var A: IFArr2D; NewDim1, NewDim2: TInt;
                                          CopyData: boolean = true); overload;
///  OldA -> NewA
procedure ResizeF2D(var A: IFArr2D; NewLim: ILim2D; CopyData: boolean = true); overload;

/// Replaces the last positions of Prefix with Lo1, Lo1+1,.., Hi1 
procedure SetStringArr(const A: ISArr1D; Prefix: string; Truncate: boolean = false);

/// A := Diag(c) 
procedure AssignScalar(const A: IFArr2D; c: TFloat);

{*
  A := B
  If not MatchLim :

A = |* * * * * *| B =     |x x x x x x|  A := B |* *|x x x x|. .|
}
procedure AssignVector(const A, B: IFArr1D; MatchLim: boolean = true);

{*
  A := B
  If not MatchLim :
    |* * * * * *|                               |* * * * * *|
    |* * * * * *|         |x x x x x x|         |* *|x x x x|x x|
A = |* * * * * *| B =     |x x x x x x|  A := B |* *|x x x x|x x|
    |* * * * * *|         |x x x x x x|         |* *|x x x x|x x|
    |* * * * * *|                               |* * * * * *|         
}
procedure AssignMatrix(const A, B: IFArr2D; MatchLim: boolean);

/// AL.Col[c1] := AR.Col[c2] 
procedure CopyCC(const AL: IFArr2D; c1: TInt; AR: IFArr2D; c2: TInt);
/// AL.Row[r1] := AR.Row[r2] 
procedure CopyRR(const AL: IFArr2D; r1: TInt; AR: IFArr2D; r2: TInt);

/// random boolean vector 
procedure RandArr1D(const V: IBArr1D); overload;
/// fills with r cMin <= r <= cMax 
procedure RandArr1D(const V: IIArr1D; cMin, cMax: TInt); overload;
/// random vector: cMin <= V[i] <= cMax 
procedure RandArr1D(const V: IFArr1D; cMin, cMax: TFloat); overload;
/// random vector: cMin <= V.Re[i] <= cMax, cMin <= V.Im[i] <= cMax  
procedure RandArr1D(const V: ICArr1D; cMin, cMax: TFloat); overload

{*
random boolean matrix 
  if sym =  1 then M[i,j] = M[j,i]   
  if sym <> 1 - no symmetry 
}
procedure RandArr2D(const M: IBArr2D; sym: TInt = 0); overload;
{*
fills with r cMin <= r <= cMax 
  if sym = -1 then M[i,j] = - M[j,i] 
  if sym =  1 then M[i,j] = M[j,i]   
  if sym =  0 - no symmetry 
}
procedure RandArr2D(const M: IIArr2D; cMin, cMax: TInt; sym: TInt = 0); overload;
{*
fills with r cMin <= r < cMax  
 if sym = -1 then M[i,j] = - M[j,i] 
 if sym =  1 then M[i,j] = M[j,i]   
 if sym =  0 - no symmetry  
}
procedure RandArr2D(const M: IFArr2D; cMin, cMax: TFloat; sym: TInt = 0); overload;

/// fills with r cMin <= r < cMax  
procedure RandArr3D(const M: IIArr3D; cMin, cMax: TInt); overload;
/// fills with r cMin <= r < cMax  
procedure RandArr3D(const M: IFArr3D; cMin, cMax: TFloat); overload;

/// random boolean vector - Same as RandArr1D, backward compatibility
procedure RandBVt(const V: IBArr1D);
/// random boolean matrix  - Same as RandArr2D, backward compatibility
procedure RandBMt(const M: IBArr2D);

/// fills with r iMin <= r <= iMax  - Same as RandArr1D, backward compatibility
procedure RandIVt(iMin, iMax: TInt; const V: IIArr1D);
{*
fills with r iMin <= r <= iMax 
  if sym = -1 then M[i,j] = - M[j,i] 
  if sym =  1 then M[i,j] = M[j,i]   
  if sym =  0 - no symmetry  - Same as RandArr2D, backward compatibility
}
procedure RandIMt(iMin, iMax: TInt; const M: IIArr2D; sym: TInt);

/// random vector: cMin <= V[i] <= cMax  - Same as RandArr1D, backward compatibility
procedure RandVt(cMin, cMax: TFloat; const V: IFArr1D);
{*
fills with r cMin <= r < cMax  
 if sym = -1 then M[i,j] = - M[j,i] 
 if sym =  1 then M[i,j] = M[j,i]   
 if sym =  0 - no symmetry  - Same as RandArr2D, backward compatibility 
}
procedure RandMt(cMin, cMax: TFloat; const M: IFArr2D; sym: TInt);

/// fills with r cMin <= r < cMax  - Same as RandArr3D, backward compatibility
procedure Rand3Mt(cMin, cMax: TFloat; const M: IFarr3D);

/// if |B| &gt; SqrtMinReal then A := B/|B| , allows A = B, Norm := |B| 
procedure NormalizeV(const A, B: IFArr1D; var Norm: TFloat);
/// NormV := |B| 
function NormV(const B: IFArr1D; Normalize: boolean = false): TFloat;
/// NormV := |B| 
function NormCV(const B: ICArr1D; Normalize: boolean = false): TFloat;
/// Norm2V := (B|B) 
function Norm2V(const B: IFArr1D): TFloat;

/// Norm1MCol := Max(Norm1(M.Col[j])) 
function Norm1MCol(const M: IFArr2D): TFloat;
/// Norm1MCol := Max(Norm1(M.Row[i])) 
function Norm1MRow(const M: IFArr2D): TFloat;

/// A = c1*A1 + c2*A2 + . . . ; A can coincide with any Vi 
procedure SumArr1D(cc: array of TFloat; const AA: array of IFArr1D; const A: IFArr1D);
/// A = c1*A1 + c2*A2 + . . . ; A can coincide with any Vi 
procedure SumArr2D(cc: array of TFloat; const AA: array of IFArr2D; const A: IFArr2D);

/// C = rA*A + rB*B C = A or C = B is OK
procedure CombineArr(const A, B: IFArr1D; rA, rB: TFloat; const C: IFArr1D); overload;
/// C = rA*A + rB*B C = A or C = B is OK
procedure CombineArr(const A, B: IFArr2D; rA, rB: TFloat; const C: IFArr2D); overload;

/// C = cA*A + cB*B C = A or C = B is OK
procedure CombineCArr(const A, B: ICArr1D; cA, cB: Complex; const C: ICArr1D); overload;
/// C = cA*A + cB*B C = A or C = B is OK
procedure CombineCArr(const A, B: ICArr2D; cA, cB: Complex; const C: ICArr2D); overload;

/// MaxAbsV = max( |V[i]| 
function MaxAbsDiff(const V1, V2: IFArr1D): TFloat;

/// MaxAbsV = max( |V[i]|
function MaxAbsV(const V: IFArr1D): TFloat;
/// MaxAbsIndexV = IndexOf(MaxAbsV), val = V[MaxAbsIndexV] 
function MaxAbsIndexV(const V: IFArr1D; var val: TFloat):TInt; overload;
/// MaxAbsIndexV = IndexOf(MaxAbsV), val = V[MaxAbsIndexV] 
function MaxAbsIndexV(const V: IFArr1D; var val: TFloat; imin, imax: TInt):TInt; overload;

/// MaxAbsCol = IndexOf(Max(Abs(A.Col[c]))), val = Abs(Max(A.Col[c])) 
function MaxAbsIndexCol(const A: IFArr2D; var val: TFloat; c, imin, imax: TInt):TInt; overload;
/// MaxAbsCol = IndexOf(Max(Abs(A.Col[c]))), val = Abs(Max(A.Col[c])) 
function MaxAbsIndexCol(const A: IFArr2D; var val: TFloat; c: TInt):TInt; overload;
/// MaxAbsCol = IndexOf(Max(Abs(A.Row[r]))), val = Abs(Max(A.Row[r])) 
function MaxAbsIndexRow(const A: IFArr2D; var val: TFloat; r, imin, imax: TInt):TInt; overload;
/// MaxAbsCol = IndexOf(Max(Abs(A.Row[r]))), val = Abs(Max(A.Row[r])) 
function MaxAbsIndexRow(const A: IFArr2D; var val: TFloat; r: TInt):TInt; overload;


{*
  if First then
    MinVal = Min(A[i]);  MaxVal = Max(A[i])
  else  // min for many files
    MinVal = Min(OldMinval, A[i]);  MaxVal = Max(OldMavVal, A[i])
}
procedure MinMaxVec(const A: IFArr1D; var MinVal, MaxVal: TFloat; First: boolean = true);
{*
  if First then
    MinVal = Min(A[i]);  MaxVal = Max(A[i])  IdxMini <= i <= IdxMax
  else
    MinVal = Min(OldMinval, A[i]);  MaxVal = Max(OldMavVal, A[i])
}
procedure MinMaxVecIdx(const A: IFArr1D; var MinVal, MaxVal: TFloat;
                                   IdxMin, IdxMax: TInt; First: boolean = true);

{*
  if First then
    MinVal = Min(A[i,j]);  MaxVal = Max(A[i,j])
  else
    MinVal = Min(OldMinval, A[i,j]);  MaxVal = Max(OldMavVal, A[i,j])
}
procedure MinMaxMatr(const A: IFArr2D; var MinVal, MaxVal: TFloat; First: boolean = true);

/// MaxAbsCol = A.Col[c].MaxAbs 
function MaxAbsCol(const A: IFArr2D; c: TInt):TFloat;
/// MaxAbsCol = A.Row[r].MaxAbs 
function MaxAbsRow(const A: IFArr2D; r: TInt):TFloat;

/// V1 := V1 + V2*h 
procedure VAddV(const V1, V2: IFArr1D; h: TFloat; imin, imax: TInt); overload;
/// V1 := V1 + V2*h 
procedure VAddV(const V1, V2: IFArr1D; h: TFloat); overload;

/// V1[i] := V2[i] + h 
procedure ShiftV(const V1, V2: IFArr1D; var err: TInt; h: TFloat; imin, imax: TInt); overload;
/// V1[i] := V2[i] + h 
procedure ShiftV(const V1, V2: IFArr1D; var err: TInt; h: TFloat); overload;

/// VDotV := (V1|V2) 
function VDotV(const V1, V2: IFArr1D; imin, imax: TInt): TFloat; overload;
/// VDotV := (V1|V2) 
function VDotV(const V1, V2: IFArr1D): TFloat; overload;
/// InnerProd := c + Sum(i=low,high,step; A[i]*B[i]) 
function InnerProd(low, step, high: integer; const A, B: IFArr1D; c: TFloat = 0.0): TFloat;
/// M1 := M1 + M2*h 
procedure MAddM(const M1, M2: IFArr2D; h: TFloat);

/// Mt := M + Diag(d);  Mt = M - OK 
procedure MShiftDiag(const M: IFArr2D; const d: TFloat; const Mt: IFArr2D;
                     RequireSquare: boolean = true);
/// Mt := M + D; Mt = M - OK 
procedure MAddDiag(const M: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);

/// Norm2Col = U.Col[i].GetNorm2 
function Norm2Col(const U: IFArr2D; i: TInt):TFloat;
/// dot product: ColxCol = (U.Col[i]|U.Col[i]) 
function ColxCol(const U: IFArr2D; i, j: TInt):TFloat;
/// U.Col[i] := U.Col[i] + C*U.Col[j] 
procedure ColAddCol(const U: IFArr2D; i, j: TInt; C: TFloat);
/// U.Col[i] := U.Col[i] + f*U.Col[j] 
procedure CxCol(const U: IFArr2D; i: TInt; C: TFloat);
/// U.Col[i] := U.Col[i]/U.Col[i].norm 
procedure NormalizeCol(const U: IFArr2D; i: TInt; var err: TInt);
/// |U.Col[i]| if Normalize then U.Col[i] := U.Row[i]/U.Row[i].norm 
function NormCol(const U: IFArr2D; i: TInt; Normalize: boolean = false): TFloat;

/// M.Col[j].AssignValues(A); (regardless of V.Base)
procedure ValuesToCol(const M: IFArr2D; j: TInt; const V: IFArr1D);
/// V.AssignValues(M.Col[j]); (regardless of V.Base)
procedure ColValuesToV(const M: IFArr2D; j: TInt; const V: IFArr1D);

/// dot product: RowxCol = (U.Row[i]|U.Col[j]) 
function RowxCol(const U: IFArr2D; i, j: TInt):TFloat;
/// dot product: RowxCol = (U1.Row[i1]|U2.Col[j2]) 
function Row1xCol2(const U1, U2: IFArr2D; i1, j2: TInt):TFloat;
/// dot product: ColxCol = (U1.Col[i1]|U2.Col[j2]) 
function Col1xCol2(const U1, U2: IFArr2D; i1, j2: TInt):TFloat;
/// dot product: Row1xRow2 = (U1.Row[i1]|U2.Row[j2]) 
function Row1xRow2(const U1, U2: IFArr2D; i1, j2: TInt):TFloat;

/// dot product: VxCol = (V |U.Col[j]) 
function VxCol(const V: IFArr1D; const U: IFArr2D; j: TInt):TFloat;
/// dot product: VxCol = (V |U.Row[j]) 
function VxRow(const V: IFArr1D; const U: IFArr2D; j: TInt):TFloat;

/// Norm2Row = U.Row[i].GetNorm2 
function Norm2Row(const U: IFArr2D; i: TInt):TFloat;
/// dot product: RowxRow = (U.Row[i]|U.Row[i]) 
function RowxRow(const U: IFArr2D; i, j: TInt):TFloat;

/// U.Row[i] := U.Row[i] + C*U.Row[j] 
procedure RowAddRow(const U: IFArr2D; i, j: TInt; C: TFloat); overload;
/// U.Row[i] := U.Row[i] + C*U.Row[j] 
procedure RowAddRow(const U: IFArr2D; i, j: TInt; C: TFloat; imin, imax: TInt); overload;

/// U.Row[i] := U.Row[i] + f*U.Row[j] 
procedure CxRow(const U: IFArr2D; i: TInt; C: TFloat); overload;
/// U.Row[i] := U.Row[i] + f*U.Row[j] 
procedure CxRow(const U: IFArr2D; i: TInt; C: TFloat; imin, imax: TInt); overload;

/// U.Row[i] := U.Row[i]/U.Row[i].norm 
procedure NormalizeRow(const U: IFArr2D; i: TInt; var err: TInt);
/// |U.Row[i]| if Normalize then U.Row[i] := U.Row[i]/U.Row[i].norm 
function NormRow(const U: IFArr2D; i: TInt; Normalize: boolean = false): TFloat;

// M.Row[j].AssignValues(A); (regardless of A.Base)
procedure ValuesToRow(const M: IFArr2D; j: TInt; const A: IFArr1D);
// V.AssignValues(M.Row[j]); (regardless of V.Base)
procedure RowValuesToV(const M: IFArr2D; j: TInt; const V: IFArr1D);

/// VL := VR*C
procedure VxC(const VL, VR: IFArr1D; C: TFloat);
/// Dot (scalar) product of two vectors VtxVt = VL*VR without limit checks
function VxV(const VL, VR: IFArr1D): TFloat;
/// vector-matrix product: VxM = V*M = MT*V 
procedure VxM(const VL: IFArr1D; const MR: IFArr2D; const Vt: IFArr1D);
/// matrix-vector product: MxV = M*V = V*MT 
procedure MxV(const ML: IFArr2D; const VR: IFArr1D; const Vt: IFArr1D);
/// matrix-matrix product: Mt = ML*MR 
procedure MxM(const ML, MR: IFArr2D; const Mt: IFArr2D);
/// matrix-matrix product: MtTxMt = Transpose(ML)*MR  
procedure MTxM(const ML, MR: IFArr2D; const Mt: IFArr2D);
/// matrix-matrix product: MtxMtT = ML*Transpose(MR) 
procedure MxMT(const ML, MR: IFArr2D; const Mt: IFArr2D);
/// matrix-matrix product: Mt = Transpose(ML)*Transpose(MR) 
procedure MT_MT(const ML, MR: IFArr2D; const Mt: IFArr2D);

/// Mt = Transpose(M1)*M2*M1 
procedure MTxMxM(const M1, M2: IFArr2D; const Mt: IFArr2D);

/// matrix-matrix product: Mt = DiagonalMatrix(DL)*MR 
procedure DxM(const DL: IFArr1D; const MR: IFArr2D; const Mt: IFArr2D);
/// matrix-matrix product: Mt = ML*DiagonalMatrix(DR) 
procedure MxD(const ML: IFArr2D; const DR: IFArr1D; const Mt: IFArr2D);
/// Mt = Transpose(ML)*DiagonalMatrix(DR) 
procedure MTrxD(const ML: IFArr2D; const DR: IFArr1D; const Mt: IFArr2D);

/// matrix-matrix product: Mt = MxDxMT = M*DiagonalMatrix(D)*Transpose(M) 
procedure MxDxMT(const M: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
/// matrix-matrix product: Mt = MxDxMT = Transpose(M)*DiagonalMatrix(D)*M 
procedure MTxDxM(const M: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
/// matrix-matrix product: Mt = M1xDxM2T = M*DiagonalMatrix(D)*Transpose(M) 
procedure M1xDxM2T(const M1, M2: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
/// matrix-matrix product: Mt = M1TxDxM2 = Transpose(M)*DiagonalMatrix(D)*M 
procedure M1TxDxM2(const M1, M2: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);

/// idx is in IArr 
function IsInArr(idx: TInt; const IArr: IIArr1D): boolean;

/// Ascending Buble Sort 
procedure BubleSortI1D(const A: IIArr1D);

/// Ascending Buble Sort 
procedure BubleSortF1D(const A: IFArr1D);

{*
 Ascending Insert Sort: Based on the Algorithm by D.F.Stermole
  http://www.macdonald.egate.net/CompSci/Pascal/hsorting.html
}
procedure InsertSortI1D(const A: IIArr1D);

{*
 Ascending NONRECURSIVE quicksort routine
  Alan R. Miller. Borland Pascal Programs for Scientists and Engineers,
  Copyright  1993, (SYBEX) ISBN 0-7821-1150-5
  Adapted from 'Software-Tools', B.Kernighan, Addison Wesley, 1976 
}
procedure NRQuickSortI1D(const x: IIArr1D);

{*
 Ascending RECURSIVE quicksort routine
  Alan R. Miller. Borland Pascal Programs for Scientists and Engineers,
  Copyright  1993, (SYBEX) ISBN 0-7821-1150-5
  Adapted from 'The design of Well-Structured and Correct Programs',
	S. Alagic, Springer-Verlag, 1978 
}
procedure QuickSortI1D(const x: IIArr1D);

{*
 1D array sorting Based on: Sebastian Boung, Sorting Algorithms in Pascal
  @link(http://privat.schlund.de/b/bossung/prog/delphi/sorting.html) 
}
procedure QSortI1D(const A: IIArr1D; Ascending: boolean);

{*
 1D array sorting Based on: Sebastian Boung, Sorting Algorithms in Pascal
  @link(http://privat.schlund.de/b/bossung/prog/delphi/sorting.html)
}
procedure QSortF1D(const A: IFArr1D; Ascending: boolean);

{*
 Ascending 1D-array sorting with sorting of the associated 2D-array
  by columns (idx = 2) or  by rows (idx = 1) 
  Based on: Sebastian Boung, Sorting Algorithms in Pascal 
  http://privat.schlund.de/b/bossung/prog/delphi/sorting.html 
}
procedure QSortF2D(const A: IFArr1D; const B: IFArr2D; Ascending: boolean; st: TSliceType);

/// MT = Transpose(M) 
procedure MtT(const M, MT: IBArr2D); overload;
/// MT = Transpose(M) 
procedure MtT(const M, MT: IIArr2D); overload;
/// MT = Transpose(M) 
procedure MtT(const M, MT: IFArr2D); overload;

/// Complex Algebra - backward compatibility support 

/// MT = Transpose(M) 
procedure CMT(const MR, MI, MTR, MTI: IFArr2D);
/// MC = Conjugate(M); M = MC is OK 
procedure CMC(const MR, MI, MCR, MCI: IFArr2D);
/// MH = HTranspose(M) 
procedure CMH(const MR, MI, MHR, MHI: IFArr2D);

/// Dot (scalar) product of two vectors 
procedure CVxCV(const V1R, V1I, V2R, V2I: IFArr1D; var ReResult, ImResult: TFloat);

/// HConjugate(V1) Dot V2 
procedure CVHxCV(const V1R, V1I, V2R, V2I: IFArr1D; var ReResult, ImResult: TFloat);
{*
 (V3R + i*V3I) = (M1R + i*M1I)*(V2R + i*V2I)
   V3R = M1R*V2R - M1I*V2I;  M3I = M1R*V2I + M1I*V2R 
}
procedure CMxCV(const M1R, M1I: IFArr2D; const V2R, V2I, V3R, V3I: IFArr1D);
{*
 (V3R + i*V3I) = (V1R + i*V1I)*(M2R + i*M2I)
   V3R = V1R*M2R - V1I*M2I;  V3I = V1R*M2I + V1I*M2R 
}
procedure CVxCM(const V1R, V1I: IFArr1D; const M2R, M2I: IFArr2D; const V3R, V3I: IFArr1D);

{*
 (M3R + i*M3I) = (M1R + i*M1I)*(M2R + i*M2I)
   M3R = M1R*M2R - M1I*M2I;  M3I = M1R*M2I + M1I*M2R 
}
procedure CMxCM(const M1R, M1I, M2R, M2I, M3R, M3I: IFArr2D);

{*
 (M3R + i*M3I) = (M1RT + i*M1IT)*(M2R + i*M2I)
   M3R = M1RT*M2R - M1IT*M2I;  M3I = M1RT*M2I + M1IT*M2R 
}
procedure CMHxCM(const M1R, M1I, M2R, M2I, M3R, M3I: IFArr2D);

{*
 (M3R + i*M3I) = (M1R + i*M1I)*(M2RT - i*M2IT)
   M3R = M1R*M2RT + M1I*M2IT;  M3I = -M1R*M2IT + M1I*M2RT 
}
procedure CMxCMH(const M1R, M1I, M2R, M2I, M3R, M3I: IFArr2D);

{*
 (M3R + i*M3I) = (M1RT - i*M1IT)*D*(M2R + i*M2I)
   M3R = M1RT*D*M1R + M1IT*D*M1I;  M3I = M1RT*D*M2I - M1IT*D*M1R 
}
procedure CMHxDxCM(const M1R, M1I: IFArr2D; const D: IFArr1D; const M3R, M3I: IFArr2D);

{*
 (M3R + i*M3I) = (M1R + i*M1I)*D*(M2RT - i*M2IT)
   M3R = M1R*D*M1RT + M1I*D*M1IT;  M3I = -M1R*D*M1IT + M1I*D*M1RT 
}
procedure CMxDxCMH(const M1R, M1I: IFArr2D; const D: IFArr1D; const M3R, M3I: IFArr2D);

// ==========================================================================

implementation

uses
  Math, sysutils, classes, uDisplay;

ResourceString
//  RS_MinMax = 'imin = %d > imax = %d';
//  RS_DimError = 'Index No= %d : out of dimension %d';
  ///'Matrix is not square';
  RS_NotSquare = 'Matrix is not square';
//  RS_LimMismatch = 'Lim Mismatch';
  /// 'Cannot Normalize: Zero Vector';
  RS_ZeroVector = 'Cannot Normalize: Zero Vector';
  /// 'Min/Max Error';
  RS_MinMaxError = 'Min/Max Error';

{*
  if minIdx = MaxInt then Lo := Lo1 else Lo := max(minIdx, Lo1);
  if maxIdx = MaxInt then Hi := Hi1 else Hi := min(maxIdx, Hi1);
  Dim := Hi - Lo + 1;
}
procedure SetMinMaxIdx(minIdx, maxIdx, Lo1, Hi1: TInt; var Lo, Hi, Dim: TInt);
begin
  if minIdx = MaxInt then
    Lo := Lo1
  else
    Lo := max(minIdx, Lo1);
  if maxIdx = MaxInt then
    Hi := Hi1
  else
    Hi := min(maxIdx, Hi1);
  assert( Lo <= Hi, 'Incorrect MinMaxIdx');
  Dim := Hi - Lo + 1;
end;

procedure ResizeF1D(var A: IFArr1D; NewLo1, NewHi1: TInt; CopyData: boolean); overload;
var
  temp: IFArr1D;
  i1, i1Lo, i1Hi: TInt;
begin
  temp := TFArr1D.Create(NewLo1, NewHi1);
  temp.Comment := A.Comment;
  if CopyData then
  begin
    i1Lo := max(A.Lo1,temp.Lo1);
    i1Hi := min(A.Hi1,temp.Hi1);
    for i1 := i1Lo to i1Hi do
      temp[i1] := a[i1];
  end;
  A := temp;
end;

procedure ResizeF1D(var A: IFArr1D; NewDim1: TInt; CopyData: boolean); overload;
begin
  ResizeF1D(A, 1, NewDim1, CopyData);
end;

procedure ResizeF1D(var A: IFArr1D; NewLim: ILim1D; CopyData: boolean); overload;
begin
  ResizeF1D(A, NewLim.Lo1, NewLim.Hi1, CopyData);
end;

procedure ResizeF2D(var A: IFArr2D; NewLo1, NewHi1, NewLo2, NewHi2: TInt;
                                          CopyData: boolean = true); overload;
var
  temp: IFArr2D;
  i1, i1Lo, i1Hi: TInt;
  i2, i2Lo, i2Hi: TInt;
begin
  temp := TFArr2D.Create(NewLo1, NewHi1, NewLo2, NewHi2);
  temp.Comment := A.Comment;
  if CopyData then
  begin
    i1Lo := max(A.Lo1,temp.Lo1);
    i1Hi := min(A.Hi1,temp.Hi1);
    i2Lo := max(A.Lo2,temp.Lo2);
    i2Hi := min(A.Hi2,temp.Hi2);
    for i1 := i1Lo to i1Hi do
      for i2 := i2Lo to i2Hi do
        temp[i1,i2] := a[i1,i2];
  end;
  A := temp;
end;

procedure ResizeF2D(var A: IFArr2D; NewDim1, NewDim2: TInt;
                                          CopyData: boolean = true); overload;
begin
  ResizeF2D(A, 1, NewDim1, 1, NewDim1, CopyData);
end;

procedure ResizeF2D(var A: IFArr2D; NewLim: ILim2D; CopyData: boolean = true); overload;
begin
  ResizeF2D(A, NewLim.Lo1, NewLim.Hi1, NewLim.Lo2, NewLim.Hi2 , CopyData);
end;


{ AL.Col[c1] := AR.Col[c2] }
procedure CopyCC(const AL: IFArr2D; c1: TInt; AR: IFArr2D; c2: TInt);
var
  i1: TInt;
begin
  if (AL.Lo1<>AR.Lo1) or (AL.Hi1<>AR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    AL[i1,c1] := AR[i1,c2];
end;

{ AL.Row[r1] := AR.Row[r2] }
procedure CopyRR(const AL: IFArr2D; r1: TInt; AR: IFArr2D; r2: TInt);
var
  i2: TInt;
begin
  if (AL.Lo2<>AR.Lo2) or (AL.Hi2<>AR.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i2 := AR.Lo2 to AR.Hi2 do
    AL[r1,i2] := AR[r2,i2];
end;

{ Replaces the last positions of Prefix with Lo1, Lo1+1,.., Hi1 }
procedure SetStringArr(const A: ISArr1D; Prefix: string; Truncate: boolean);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := IntgToStrPrefix(i1, Prefix, Truncate);
end;

{ A := Diag(c) }
procedure AssignScalar(const A: IFArr2D; c: TFloat);
var
  i1, i2: TInt;
begin
  if not IsSquare(A) then
    Raise ERangeError.Create(RS_NotSquare);
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      if i1 = i2 then
        A[i1,i2] := c
      else
        A[i1,i2] := 0.0;
end;

procedure AssignVector(const A, B: IFArr1D; MatchLim: boolean);
var
  i1, L1, H1: TInt;
begin
  if MatchLim then
  begin
    Assert((B.Hi1=A.Hi1) and (B.Lo1=A.Lo1) , RS_LimMismatch);
    for i1 := A.Lo1 to A.Hi1 do
      A[i1] := B[i1];
  end else
  begin
    L1 := max(A.Lo1,B.Lo1);
    H1 := min(A.Hi1,B.Hi1);
    for i1 := L1 to H1 do
      A[i1] := B[i1];
  end;
end;

{ A := B }
procedure AssignMatrix(const A, B: IFArr2D; MatchLim: boolean);
var
  i1, i2, L1, L2, H1, H2: TInt;
begin
  if MatchLim then
  begin
    Assert((B.Hi1=A.Hi1) and (B.Lo1=A.Lo1) and
           (B.Hi2=A.Hi2) and (B.Lo2=A.Lo2), RS_LimMismatch);
    for i1 := A.Lo1 to A.Hi1 do
      for i2 := A.Lo2 to A.Hi2 do
        A[i1,i2] := B[i1,i2];
  end else
  begin
    L1 := max(A.Lo1,B.Lo1);
    H1 := min(A.Hi1,B.Hi1);
    L2 := max(A.Lo2,B.Lo2);
    H2 := min(A.Hi2,B.Hi2);
    for i1 := L1 to H1 do
      for i2 := L2 to H2 do
        A[i1,i2] := B[i1,i2];
  end;
end;

{ random boolean vector }
procedure RandArr1D(const V: IBArr1D);
var
  i: TInt;
begin
  for i := V.Lo1 to V.Hi1 do
    V[i] := (random(2) = 0);
end;

{ fills with r iMin <= r <= iMax }
procedure RandArr1D(const V: IIArr1D; cMin, cMax: TInt);
var
  i: TInt;
begin
  for i := V.Lo1 to V.Hi1 do
    V[i] := cMin+random(cMax-cMin+1);
end;

{ random vector: cMin <= V[i] <= cMax }
procedure RandArr1D(const V: IFArr1D; cMin, cMax: TFloat);
var
  i: TInt;
begin
//  randomize;
  for i := V.Lo1 to V.Hi1 do
    V[i] := cMin + random*(cMax-cMin);
end;

{ random vector: cMin <= V.Re[i] <= cMax, cMin <= V.Im[i] <= cMax  }
procedure RandArr1D(const V: ICArr1D; cMin, cMax: TFloat);
begin
  RandArr1D(V.Re, cMin, cMax);
  RandArr1D(V.Im, cMin, cMax);
end;

{random boolean matrix 
  if sym =  1 then M[i,j] = M[j,i]   
  if sym <> 1 - no symmetry }
procedure RandArr2D(const M: IBArr2D; sym: TInt = 0);
var
  i1, i2: TInt;
begin
  if IsSquare(M) and (sym > 0) then
  begin
    for i1 := M.Lo1 to M.Hi1-1 do
    begin
      for i2 := i1+1 to M.Hi2 do
      begin
        M[i1,i2] := (random(2) = 0);
        M[i2,i1] := M[i1,i2];
      end;
      M[i1,i1] := (random(2) = 0);
    end;
    exit;
  end;

  for i1 := M.Lo1 to M.Hi1 do
    for i2 := M.Lo2 to M.Hi2 do
      M[i1,i2] := (random(2) = 0);
end;

{fills with r cMin <= r <= cMax 
  if sym = -1 then M[i,j] = - M[j,i] 
  if sym =  1 then M[i,j] = M[j,i]   
  if sym =  0 - no symmetry }
procedure RandArr2D(const M: IIArr2D; cMin, cMax: TInt; sym: TInt = 0);
var
  i1, i2: TInt;
begin
  if IsSquare(M) and (sym < 0) then
  begin
    for i1 := M.Lo1 to M.Hi1-1 do
    begin
      for i2 := i1+1 to M.Hi2 do
      begin
        M[i1,i2] := cMin+random(cMax-cMin+1);;
        M[i2,i1] := -M[i1,i2];
      end;
      M[i1,i1] := 0;
    end;
    exit;
  end;

  if IsSquare(M) and (sym > 0) then
  begin
    for i1 := M.Lo1 to M.Hi1-1 do
    begin
      for i2 := i1+1 to M.Hi2 do
      begin
        M[i1,i2] := cMin+random(cMax-cMin+1);
        M[i2,i1] := M[i1,i2];
      end;
      M[i1,i1] := cMin+random(cMax-cMin+1);
    end;
    exit;
  end;

  for i1 := M.Lo1 to M.Hi1 do
    for i2 := M.Lo2 to M.Hi2 do
      M[i1,i2] := cMin+random(cMax-cMin+1);
end;

{fills with r cMin <= r < cMax  
 if sym = -1 then M[i,j] = - M[j,i] 
 if sym =  1 then M[i,j] = M[j,i]   
 if sym =  0 - no symmetry  }
procedure RandArr2D( const M: IFArr2D; cMin, cMax: TFloat;sym: TInt = 0);
var
  i1, i2: TInt;
begin
  if IsSquare(M) and (sym < 0) then
  begin
    for i1 := M.Lo1 to M.Hi1-1 do
    begin
      for i2 := i1+1 to M.Hi2 do
      begin
          M[i1,i2] := cMin + random*(cMax-cMin);
          M[i2,i1] := -M[i1,i2];
      end;
      M[i1,i1] := 0;
    end;
    exit;
  end;

  if IsSquare(M) and (sym > 0) then
  begin
    for i1 := M.Lo1 to M.Hi1-1 do
    begin
      for i2 := i1+1 to M.Hi2 do
      begin
          M[i1,i2] := cMin + random*(cMax-cMin);
          M[i2,i1] := M[i1,i2];
      end;
      M[i1,i1] := cMin + random*(cMax-cMin);
    end;
    exit;
  end;

  for i1 := M.Lo1 to M.Hi1 do
    for i2 := M.Lo2 to M.Hi2 do
      M[i1,i2] := cMin + random*(cMax-cMin);
end;

{ fills with r cMin <= r < cMax  }
procedure RandArr3D(const M: IIArr3D; cMin, cMax: TInt);
var
  i1, i2, i3: TInt;
begin
  for i1 := M.Lo1 to M.Hi1 do
    for i2 := M.Lo2 to M.Hi2 do
      for i3 := M.Lo3 to M.Hi3 do
        M[i1,i2,i3] := cMin+random(cMax-cMin+1);
end;

{ fills with r cMin <= r < cMax  }
procedure RandArr3D(const M: IFArr3D; cMin, cMax: TFloat);
var
  i1, i2, i3: TInt;
begin
  for i1 := M.Lo1 to M.Hi1 do
    for i2 := M.Lo2 to M.Hi2 do
      for i3 := M.Lo3 to M.Hi3 do
        M[i1,i2,i3] := cMin + random*(cMax-cMin);
end;

{ random boolean vector }
procedure RandBVt(const V: IBArr1D);
begin
  RandArr1D(V);
end;

{ random boolean matrix }
procedure RandBMt(const M: IBArr2D);
begin
  RandArr2D(M);
end;

{ fills with r iMin <= r <= iMax }
procedure RandIVt(iMin, iMax: TInt; const V: IIArr1D);
begin
  RandArr1D(V,iMin, iMax);
end;

{fills with r iMin <= r <= iMax 
  if sym = -1 then M[i,j] = - M[j,i] 
  if sym =  1 then M[i,j] = M[j,i]   
  if sym =  0 - no symmetry }
procedure RandIMt(iMin, iMax: TInt; const M: IIArr2D; sym: TInt);
begin
  RandArr2D(M,iMin, iMax, sym);
end;

{ random vector: cMin <= V[i] <= cMax }
procedure RandVt(cMin, cMax: TFloat; const V: IFArr1D);
begin
  RandArr1D(V,cMin, cMax);
end;

{fills with r cMin <= r < cMax
 if sym = -1 then M[i,j] = - M[j,i] 
 if sym =  1 then M[i,j] = M[j,i]   
 if sym =  0 - no symmetry  }
procedure RandMt(cMin, cMax: TFloat; const M: IFArr2D; sym: TInt);
begin
  RandArr2D(M,cMin, cMax, sym);
end;

{ fills with r cMin <= r < cMax  }
procedure Rand3Mt(cMin, cMax: TFloat; const M: IFArr3D);
begin
  RandArr3D(M,cMin, cMax);
end;

{ if |B| &gt; SqrtMinReal then A := B/|B| , allows A = B, Norm := |B| }
procedure NormalizeV(const A, B: IFArr1D; var Norm: TFloat);
var
  x: TFloat;
begin
  x := sqrt(VxV(B, B));
  if x > SqrtMinFloat then
    VxC(A,B,1.0/x)
  else
    Raise EMathError.Create(RS_ZeroVector);
end;{ of NormalizeV }

{ NormV := |B| }
function NormV(const B: IFArr1D; Normalize: boolean = false): TFloat;
begin
  result := sqrt(VxV(B, B));
  if Normalize then
    if result > SqrtMinFloat then
      B.Times(1.0/result)
    else
      Raise EMathError.Create(RS_ZeroVector);
end;{ of NormV }

{ NormV := |B| }
function NormCV(const B: ICArr1D; Normalize: boolean = false): TFloat;
begin
  result := sqrt(VxV(B.Re, B.Re) + VxV(B.Im, B.Im));
  if Normalize then
    if result > SqrtMinFloat then
    begin
      B.Re.Times(1.0/result);
      B.Im.Times(1.0/result);
    end else
      Raise EMathError.Create(RS_ZeroVector);
end;{ of NormV }

{ Norm2V := (B|B) }
function Norm2V(const B: IFArr1D): TFloat;
begin
  result := VxV(B, B);
end;{ of NormV }

{ Norm1MCol := Max(Norm1(M.Col[j])) }
function Norm1MCol(const M: IFArr2D): TFloat;
var
  i, j: TInt;
  t: TFloat;
begin
  result := 0.0;
  for j := M.Lo2 to M.Hi2 do
  begin
    t := 0.0;
    for i := M.Lo1 to M.Hi1 do
      t := t + abs(M[i,j]);
    if (t  >  result) then result := t;
  end;
end;

{ Norm1MCol := Max(Norm1(M.Row[i])) }
function Norm1MRow(const M: IFArr2D): TFloat;
var
  i, j: TInt;
  t: TFloat;
begin
  result := 0.0;
  for i := M.Lo1 to M.Hi1 do
  begin
    t := 0.0;
    for j := M.Lo2 to M.Hi2 do
      t := t + abs(M[i,j]);
    if (t  >  result) then result := t;
  end;
end;

{ A = c1*A1 + c2*A2 + . . . ; A can coincide with any Vi }
procedure SumArr1D(cc: array of TFloat; const AA: array of IFArr1D; const A: IFArr1D);
var
  i1, j, m: TInt;
  s: TFloat;
begin
  m := Length(cc);
  if m <> Length(AA)then
    Raise ERangeError.Create(RS_LimMismatch);

  for j := 1 to M-1 do
    if not SameLim(AA[0], AA[j]) then
      Raise ERangeError.Create(RS_LimMismatch);
  if not SameLim(AA[0], A) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := A.Lo1 to A.Hi1 do
  begin
    s := 0.0;
    for j := 0 to M-1 do
      s := s + cc[j]*AA[j].Value[i1];
    A[i1] := s;
  end;
end;

{ A = c1*A1 + c2*A2 + . . . ; A can coincide with any Vi }
procedure SumArr2D(cc: array of TFloat; const AA: array of IFArr2D; const A: IFArr2D);
var
  i1, i2, j, M: TInt;
  s: TFloat;
begin
  m := Length(cc);
  if m <> Length(AA)then
    Raise ERangeError.Create(RS_LimMismatch);

  for j := 1 to M-1 do
    if not SameLim(AA[0], AA[j]) then
      Raise ERangeError.Create(RS_LimMismatch);
  if not SameLim(AA[0], A) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := A.Lo1 to A.Hi1 do
    for i2 := a.Lo2 to A.Hi2 do
    begin
      s := 0.0;
      for j := 0 to M-1 do
        s := s + cc[j]*AA[j].Value[i1,i2];
      A[i1,i2] := s;
    end;
end;




{ C = rA*A + rB*B C = A or C = B is OK}
procedure CombineArr(const A, B: IFArr1D; rA, rB: TFloat; const C: IFArr1D);
var
  i: TInt;
begin
  if not SameLim(A,B) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := A.Lo1 to A.Hi1 do
    C[i] := rA*A[i] + rB*B[i];
end;

{ C = rA*A + rB*B C = A or C = B is OK}
procedure CombineArr(const A, B: IFArr2D; rA, rB: TFloat; const C: IFArr2D);
var
  i1, i2: TInt;
begin
  if not SameLim(A,B) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      C[i1,i2] := rA*A[i1,i2] + rB*B[i1, i2];
end;

{ C = cA*A + cB*B C = A or C = B is OK}
procedure CombineCArr(const A, B: ICArr1D; cA, cB: Complex; const C: ICArr1D);
var
  i1: TInt;
begin
  if not SameLim(A,B) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := A.Lo1 to A.Hi1 do
    C[i1] := Cadd( Cmul(cA, A[i1]) , Cmul(cB, B[i1]) );
end;

{ C = cA*A + cB*B C = A or C = B is OK}
procedure CombineCArr(const A, B: ICArr2D; cA, cB: Complex; const C: ICArr2D);
var
  i1, i2: TInt;
begin
  if not SameLim(A,B) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      C[i1,i2] := Cadd( Cmul(cA, A[i1,i2]) , Cmul(cB, B[i1,i2]) );
end;

{ MaxAbsV = max( |V[i]| }
function MaxAbsDiff(const V1, V2: IFArr1D): TFloat;
var
  i: TInt;
begin
  if not SameLim(V1, V2) then
    Raise ERangeError.Create(RS_LimMismatch);
  result := 0.0;
  for i := V1.Lo1 to V1.Hi1 do
    result := max(result,abs(V1[i]-V2[i]));
end;

{ MaxAbsV = max( |V[i]| }
function MaxAbsV(const V: IFArr1D): TFloat;
var
  i: TInt;
begin
  result := 0.0;
  for i := V.Lo1 to V.Hi1 do
    result := max(result,abs(V[i]));
end;

{ MaxAbsIndexV = IndexOf(MaxAbsV), val = V[MaxAbsIndexV] }
function MaxAbsIndexV(const V: IFArr1D; var val: TFloat; imin, imax: TInt): TInt;
var
  x: TFloat;
  i: TInt;
begin
  if (imin < V.Lo1) or (imax > V.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  result := imin;
  val := V[imin];
  x := abs(val);
  for i := imin to imax do
    if abs(V[i]) > x then
    begin
      val := V[i];
      x := abs(val);
      result := i;
    end;
end;

{ MaxAbsIndexV = IndexOf(MaxAbsV), val = V[MaxAbsIndexV] }
function MaxAbsIndexV(const V: IFArr1D; var val: TFloat): TInt;
var
  x: TFloat;
  i: TInt;
begin
  result := V.Lo1;
  val := V[V.Lo1];
  x := abs(val);
  for i := V.Lo1 to V.Hi1 do
    if abs(V[i]) > x then
    begin
      val := V[i];
      x := abs(val);
      result := i;
    end;
end;

{ Row index i of Maximum |Val| in column c, Val = A[i,c] }
function MaxAbsIndexCol(const A: IFArr2D; var val: TFloat; c, imin, imax: TInt):TInt;
var
  x: TFloat;
  i: TInt;
begin
  if (imin < A.Lo1) or (imax > A.Hi1) then
    Raise ERangeError.Create(RS_MinMaxError);
  result := imin;
  val := A[imin,c];
  x := abs(val);
  for i := imin to imax do
    if abs(A[i,c]) > x then
    begin
      val := A[i,c];
      x := abs(val);
      result := i;
    end;
end;

{ Row index i of Maximum |Val| in column c, Val = A[i,c] }
function MaxAbsIndexCol(const A: IFArr2D; var val: TFloat; c: TInt):TInt;
var
  x: TFloat;
  i: TInt;
begin
  result := A.Lo1;
  val := A[A.Lo1,c];
  x := abs(val);
  for i := A.Lo1 to A.Hi1 do
    if abs(A[i,c]) > x then
    begin
      val := A[i,c];
      x := abs(val);
      result := i;
    end;
end;

{ Column index i of Maximum |Val| in row r, Val = A[r,i] }
function MaxAbsIndexRow(const A: IFArr2D; var val: TFloat; r, imin, imax: TInt):TInt;
var
  x: TFloat;
  i: TInt;
begin
  if (imin < A.Lo2) or (imax > A.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);
  result := imin;
  val := A[r,imin];
  x := abs(val);
  for i := imin to imax do
    if abs(A[r,i]) > x then
    begin
      val := A[r,i];
      x := abs(val);
      result := i;
    end;
end;

{ Column index i of Maximum |Val| in row r, Val = A[r,i] }
function MaxAbsIndexRow(const A: IFArr2D; var val: TFloat; r: TInt):TInt;
var
  x: TFloat;
  i: TInt;
begin
  result := A.Lo2;
  val := A[r,A.Lo2];
  x := abs(val);
  for i := A.Lo2 to A.Hi2 do
    if abs(A[r,i]) > x then
    begin
      val := A[r,i];
      x := abs(val);
      result := i;
    end;
end;

procedure MinMaxVec(const A: IFArr1D; var MinVal, MaxVal: TFloat; First: boolean);
var
  i: TInt;
begin
  if First then
  begin
    MinVal := MaxFloat;
    MaxVal := NegMaxFloat;
  end;
  for i := A.Lo1 to A.Hi1 do
  begin
    MaxVal := max(MaxVal, A[i]);
    MinVal := min(MinVal, A[i]);
  end;
end;

procedure MinMaxVecIdx(const A: IFArr1D; var MinVal, MaxVal: TFloat;
                                       IdxMin, IdxMax: TInt; First: boolean);
var
  i, i1, i2: TInt;
begin
  if First then
  begin
    MinVal := MaxFloat;
    MaxVal := NegMaxFloat;
  end;
  i1 := max(IdxMin,A.Lo1);
  i2 := min(IdxMax,A.Hi1);
  for i := i1 to i2 do
  begin
    MaxVal := max(MaxVal, A[i]);
    MinVal := min(MinVal, A[i]);
  end;
end;

procedure MinMaxMatr(const A: IFArr2D; var MinVal, MaxVal: TFloat; First: boolean);
var
  i1, i2: TInt;
begin
  if First then
  begin
    MinVal := MaxFloat;
    MaxVal := NegMaxFloat;
  end;
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
    begin
      MaxVal := max(MaxVal, A[i1,i2]);
      MinVal := min(MinVal, A[i1,i2]);
    end;
end;

{ MaxAbsCol = A.Col[c].MaxAbs }
function MaxAbsCol(const A: IFArr2D; c: TInt):TFloat;
var
  x: TFloat;
  i: TInt;
begin
  result := 0.0;
  for i := A.Lo1 to A.Hi1 do
  begin
    x := abs(A[i,c]);
    if x > result then
      result := x;
  end;
end;

{ MaxAbsCol = A.Row[r].MaxAbs }
function MaxAbsRow(const A: IFArr2D; r: TInt):TFloat;
var
  x: TFloat;
  i: TInt;
begin
  result := 0.0;
  for i := A.Lo2 to A.Hi2 do
  begin
    x := abs(A[r,i]);
    if x > result then
      result := x;
  end;
end;

{ V1 := V1 + V2*h }
procedure VAddV(const V1, V2: IFArr1D; h: TFloat; imin, imax: TInt);
var
  i: TInt;
begin
  if not SameLim(V1,V2) or (imin > imax) or
     (imin < V1.Lo1) or (imax > V1.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := imin to imax do
    V1[i] := V1[i] + h*V2[i];
end;

{ V1 := V1 + V2*h }
procedure VAddV(const V1, V2: IFArr1D; h: TFloat);
var
  i: TInt;
begin
  if not SameLim(V1,V2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := V1.Lo1 to V1.Hi1 do
    V1[i] := V1[i] + h*V2[i];
end;

{ V1[i] := V2[i] + h }
procedure ShiftV(const V1, V2: IFArr1D; var err: TInt; h: TFloat; imin, imax: TInt);
var
  i : TInt;
begin
  if not SameLim(V1,V2) or (imin > imax) or
     (imin < V1.Lo1) or (imax > V1.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := imin to imax do
    V1[i] := V2[i] + h;
end;

{ V1[i] := V2[i] + h  V1 = V2 - OK }
procedure ShiftV(const V1, V2: IFArr1D; var err: TInt; h: TFloat);
var
  i: TInt;
begin
  if not SameLim(V1,V2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := V1.Lo1 to V1.Hi1 do
    V1[i] := V2[i] + h;
end;

{ VDotV := (V1|V2) }
function VDotV(const V1, V2: IFArr1D; imin, imax: TInt): TFloat;
var
  i: TInt;
  sum: TFloat;
begin
  if not SameLim(V1,V2) or (imin > imax) or
     (imin < V1.Lo1) or (imax > V1.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  sum := 0.0;
  for i := imin to imax do
    sum := sum + V1[i]*V2[i];
  result := sum;
end;

{ VDotV := (V1|V2) }
function VDotV(const V1, V2: IFArr1D): TFloat;
var
  i: TInt;
  sum: TFloat;
begin
  if not SameLim(V1,V2) then
    Raise ERangeError.Create(RS_LimMismatch);
  sum := 0.0;
  for i := V1.Lo1 to V1.Hi1 do
    sum := sum + V1[i]*V2[i];
  result := sum;
end;

{ InnerProd := c + Sum(i=low,high,step; A[i]*B[i]) }
function InnerProd(low, step, high: integer; const A, B: IFArr1D;
                                                       c: TFloat = 0.0): TFloat;
var
  idx: integer;
begin
  if (low < A.Lo1) or (high > A.Hi1) or not SameLim(A,B) then
    Raise ERangeError.Create(RS_LimMismatch);
  result := c;
  idx := low;
  while idx <= high do
  begin
    result := result + A[idx]*B[idx];
    inc(idx,step);
  end;
end;

{ M1 := M1 + M2*h }
procedure MAddM(const M1, M2: IFArr2D; h: TFloat);
var
  i1, i2: TInt;
begin
  if not SameLim(M1,M2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i2 := M1.Lo2 to M1.Hi2 do
    for i1 := M1.Lo1 to M1.Hi1 do
      M1[i1,i2] := M1[i1,i2] + h*M2[i1,i2];
end;

{ Mt := M + Diag(d);  Mt = M - OK }
procedure MShiftDiag(const M: IFArr2D; const d: TFloat; const Mt: IFArr2D;
                     RequireSquare: boolean = true);
var
  i1, i2: TInt;
begin
  if RequireSquare then
    if not IsSquare(M) then
      Raise ERangeError.Create(RS_NotSquare);
  if not SameLim(M,Mt) then
    Raise ERangeError.Create(RS_LimMismatch);
  i2 := M.Lo2;
  for i1 := M.Lo1 to M.Hi1 do
  begin
    Mt[i1,i2] := M[i1,i2] + d;
    inc(i2);
  end;
end;

{ Mt := M + D; Mt = M - OK }
procedure MAddDiag(const M: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
var
  i: TInt;
begin
  if not IsSquare(M) then
    Raise ERangeError.Create(RS_NotSquare);
  if (M.Lo1<>D.Lo1) or ( M.Hi1<>D.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := D.Lo1 to D.Hi1 do
    Mt[i,i] := M[i,i] + D[i];
end;

{ Norm2Col = U.Col[i].GetNorm2 }
function Norm2Col(const U: IFArr2D; i: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  for k := U.Lo1 to U.Hi1 do
    n := n + sqr(U[k,i]);
  result := n;
end;

{ dot product: ColxCol = (U.Col[i]|U.Col[i]) }
function ColxCol(const U: IFArr2D; i, j: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  for k := U.Lo1 to U.Hi1 do
    n := n + U[k,i]*U[k,j];
  result := n;
end;

{ U.Col[i] := U.Col[i] + C*U.Col[j] }
procedure ColAddCol(const U: IFArr2D; i, j: TInt; C: TFloat);
var
  k: TInt;
begin
  for k := U.Lo1 to U.Hi1 do
    U[k,i] := U[k,i] + C*U[k,j];
end;

{ U.Col[i] := U.Col[i] + f*U.Col[j] }
procedure CxCol(const U: IFArr2D; i: TInt; C: TFloat);
var
  k: TInt;
begin
  for k := U.Lo1 to U.Hi1 do
    U[k,i] :=  C*U[k,i];
end;

{ U.Col[i] := U.Col[i]/U.Col[i].norm }
procedure NormalizeCol(const U: IFArr2D; i: TInt; var err: TInt);
var
  n: TFloat;
begin
  err := 0;
  n := Norm2Col(U, i);
  if n > MinFloat then
    CxCol(U, i, 1.0/sqrt(n))
  else
    err := 1;
end;

{ |U.Col[i]| if Normalize then U.Col[i] := U.Row[i]/U.Row[i].norm }
function NormCol(const U: IFArr2D; i: TInt; Normalize: boolean = false): TFloat;
begin
  result := sqrt(Norm2Col(U, i));
  if Normalize then
    if result > SqrtMinFloat then
      CxCol(U, i, 1.0/result)
    else
      Raise EMathError.Create(RS_ZeroVector);
end;

// M.Col[j].AssignValues(A); (regardless of V.Base)
procedure ValuesToCol(const M: IFArr2D; j: TInt; const V: IFArr1D);
var
  i, iV: TInt;
begin
  iV := V.Lo1;
  Assert(M.Dim1 =  V.Dim1, RS_LimMismatch);
  for i := M.Lo1 to M.Hi1 do
  begin
    M[i, j] := V[iV];
    inc(iV);
  end;
end;

// V.AssignValues(M.Col[j]); (regardless of V.Base)
procedure ColValuesToV(const M: IFArr2D; j: TInt; const V: IFArr1D);
var
  i, iV: TInt;
begin
  iV := V.Lo1;
  for i := M.Lo1 to M.Hi1 do
  begin
    V[iV] := M[i,j];
    inc(iV);
  end;
end;

{ dot product: RowxCol = (U.Row[i]|U.Col[j]) }
function RowxCol(const U: IFArr2D; i, j: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  for k := U.Lo2 to U.Hi2 do
    n := n + U[i,k]*U[k,j];
  result := n;
end;

{ dot product: RowxCol = (U1.Row[i1]|U2.Col[j2]) }
function Row1xCol2(const U1, U2: IFArr2D; i1, j2: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  if (U1.Lo2 <> U2.Lo1) or (U1.Hi2 <> U2.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  for k := U1.Lo2 to U2.Hi2 do
    n := n + U1[i1,k]*U2[k,j2];
  result := n;
end;

{ dot product: ColxCol = (U1.Col[i1]|U2.Col[j2]) }
function Col1xCol2(const U1, U2: IFArr2D; i1, j2: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  if (U1.Lo1 <> U2.Lo1) or (U1.Hi1 <> U2.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  for k := U1.Lo1 to U1.Hi1 do
    n := n + U1[k,i1]*U2[k,j2];
  result := n;
end;

{ dot product: Row1xRow2 = (U1.Row[i1]|U2.Row[j2]) }
function Row1xRow2(const U1, U2: IFArr2D; i1, j2: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  if (U1.Lo2 <> U2.Lo2) or (U1.Hi2 <> U2.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for k := U1.Lo2 to U2.Hi2 do
    n := n + U1[i1,k]*U2[j2,k];
  result := n;
end;

{ dot product: VxCol = (V |U.Col[j]) }
function VxCol(const V: IFArr1D; const U: IFArr2D; j: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  if (V.Lo1 <> U.Lo1) or (V.Hi1 <> U.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for k := U.Lo1 to U.Hi1 do
    n := n + V[k]*U[k,j];
  result := n;
end;

{ dot product: VxCol = (V |U.Row[j]) }
function VxRow(const V: IFArr1D; const U: IFArr2D; j: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  if (V.Lo1 <> U.Lo2) or (V.Hi1 <> U.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for k := U.Lo2 to U.Hi2 do
    n := n + V[k]*U[j,k];
  result := n;
end;

{ Norm2Row = U.Row[i].GetNorm2 }
function Norm2Row(const U: IFArr2D; i: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  for k := U.Lo2 to U.Hi2 do
    n := n + sqr(U[i,k]);
  result := n;
end;

{ dot product: RowxRow = (U.Row[i]|U.Row[i]) }
function RowxRow(const U: IFArr2D; i, j: TInt):TFloat;
var
  k: TInt;
  n: TFloat;
begin
  n := 0.0;
  for k := U.Lo2 to U.Hi2 do
    n := n + U[i,k]*U[j,k];
  result := n;
end;

{ U.Row[i] := U.Row[i] + C*U.Row[j] }
procedure RowAddRow(const U: IFArr2D; i, j: TInt; C: TFloat);
var
  k: TInt;
begin
  for k := U.Lo2 to U.Hi2 do
    U[i,k] := U[i,k] + C*U[j,k];
end;

{ U.Row[i] := U.Row[i] + C*U.Row[j] }
procedure RowAddRow(const U: IFArr2D; i, j: TInt; C: TFloat; imin, imax: TInt);
var
  k: TInt;
begin
  if (imin > imax) or (imin < U.Lo2) or (imax > U.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for k := imin to imax do
    U[i,k] := U[i,k] + C*U[j,k];
end;

{ U.Row[i] := U.Row[i] + f*U.Row[j] }
procedure CxRow(const U: IFArr2D; i: TInt; C: TFloat);
var
  k: TInt;
begin
  for k := U.Lo2 to U.Hi2 do
    U[i,k] :=  C*U[i,k];
end;

{ U.Row[i] := U.Row[i] + f*U.Row[j] }
procedure CxRow(const U: IFArr2D; i: TInt; C: TFloat; imin, imax: TInt); overload;
var
  k: TInt;
begin
  if (imin > imax) or (imin < U.Lo2) or (imax > U.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);
  for k := imin to imax do
    U[i,k] := C*U[i,k];
end;

{ U.Row[i] := U.Row[i]/U.Row[i].norm }
procedure NormalizeRow(const U: IFArr2D; i: TInt; var err: TInt);
var
  n: TFloat;
begin
  err := 0;
  n := Norm2Row(U, i);
  if n > MinFloat then
    CxRow(U, i, 1.0/sqrt(n))
  else
    err := 1;
end;

{ |U.Row[i]| if Normalize then U.Row[i] := U.Row[i]/U.Row[i].norm }
function NormRow(const U: IFArr2D; i: TInt; Normalize: boolean = false): TFloat;
begin
  result := sqrt(Norm2Row(U, i));
  if Normalize then
    if result > SqrtMinFloat then
      CxRow(U, i, 1.0/result)
    else
      Raise EMathError.Create(RS_ZeroVector);
end;

// M.Row[j].AssignValues(A); (regardless of A.Base)
procedure ValuesToRow(const M: IFArr2D; j: TInt; const A: IFArr1D);
var
  i, iA: TInt;
begin
  iA := A.Lo1;
  Assert(M.Dim2 = A.Dim1, RS_LimMismatch);
  for i := M.Lo2 to M.Hi2 do
  begin
    M[j, i] := A[iA];
    inc(iA);
  end;
end;

// V.AssignValues(M.Row[j]); (regardless of V.Base)
procedure RowValuesToV(const M: IFArr2D; j: TInt; const V: IFArr1D);
var
  i, iV: TInt;
begin
  iV := V.Lo1;
  for i := M.Lo2 to M.Hi2 do
  begin
    V[i] := M[iV,i];
    inc(iV);
  end;
end;

{ VL := VR*C}
procedure VxC(const VL, VR: IFArr1D; C: TFloat);
var
  i: TInt;
begin
  if not SameLim(VL,VR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := VL.Lo1 to VL.Hi1 do
    VL[i] := VR[i]*C;
end;

{ Dot (scalar) product of two vectors without limit checks }
function VxV(const VL, VR: IFArr1D): TFloat;
var
  i1: TInt;
  s: TFloat;
begin
  if not SameLim(VL,VR) then
    Raise ERangeError.Create(RS_LimMismatch);
  s := 0.0;
  for i1 := VL.Lo1 to VL.Hi1 do
    s := s + VL[i1]*VR[i1];
  result := s;
end;

{ Vt = V*M = MT*V }
procedure VxM(const VL: IFArr1D; const MR: IFArr2D; const Vt: IFArr1D);
var
  i1,i2: TInt;
  s: TFloat;
begin
  if (Vt.Lo1<>MR.Lo2) or (Vt.Hi1<>MR.Hi2) or
     (VL.Lo1<>MR.Lo1) or (VL.Hi1<>MR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i2 := MR.Lo2 to MR.Hi2 do
  begin
    s := 0.0;
    for i1 := MR.Lo1 to MR.Hi1 do
      s := s + VL[i1]*MR[i1,i2];
    Vt[i2] := s;
  end;
end;

{ Vt = M*V = V*MT  }
procedure MxV(const ML: IFArr2D; const VR: IFArr1D; const Vt: IFArr1D);
var
  i1,i2: TInt;
  s: TFloat;
begin
  if (Vt.Lo1<>ML.Lo1) or (Vt.Hi1<>ML.Hi1) or
     (VR.Lo1<>ML.Lo2) or (VR.Hi1<>ML.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := ML.Lo1 to ML.Hi1 do
  begin
    s := 0.0;
    for i2 := ML.Lo2 to ML.Hi2 do
      s := s + ML[i1,i2]*VR[i2];
    Vt[i1] := s;
  end;
end;

{ Mt = ML*MR }
procedure MxM(const ML, MR: IFArr2D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>ML.Lo1) or (Mt.Hi1<>ML.Hi1) or
     (Mt.Lo2<>MR.Lo2) or (Mt.Hi2<>MR.Hi2) or
     (ML.Lo2<>MR.Lo1) or (ML.Hi2<>MR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := ML.Lo2 to ML.Hi2 do
        s := s + ML[i1,j]*MR[j,i2];
      Mt[i1,i2] := s;
    end;
end;

{ Mt = Transpose(ML)*MR }
procedure MTxM(const ML, MR: IFArr2D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>ML.Lo2) or (Mt.Hi1<>ML.Hi2) or
     (Mt.Lo2<>MR.Lo2) or (Mt.Hi2<>MR.Hi2) or
     (ML.Lo1<>MR.Lo1) or (ML.Hi1<>MR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := ML.Lo1 to ML.Hi1 do
        s := s + ML[j,i1]*MR[j,i2];
      Mt[i1,i2] := s;
    end;
end;

{ Mt = ML*Transpose(MR) }
procedure MxMT(const ML, MR: IFArr2D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>ML.Lo1) or (Mt.Hi1<>ML.Hi1) or
     (Mt.Lo2<>MR.Lo1) or (Mt.Hi2<>MR.Hi1) or
     (ML.Lo2<>MR.Lo2) or (ML.Hi2<>MR.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := ML.Lo2 to ML.Hi2 do
        s := s + ML[i1,j]*MR[i2,j];
      Mt[i1,i2] := s;
    end;
end;

{ matrix-matrix product: Mt = Transpose(ML)*Transpose(MR) }
procedure MT_MT(const ML, MR: IFArr2D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>ML.Lo2) or (Mt.Hi1<>ML.Hi2) or
     (Mt.Lo2<>MR.Lo1) or (Mt.Hi2<>MR.Hi1) or
     (ML.Lo1<>MR.Lo2) or (ML.Hi1<>MR.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := ML.Lo1 to ML.Hi1 do
        s := s + ML[j,i1]*MR[i2,j];
      Mt[i1,i2] := s;
    end;
end;

{ Mt = Transpose(M1)*M2*M1 }
procedure MTxMxM(const M1, M2: IFArr2D; const Mt: IFArr2D);
var
  M: IFArr2D;
begin
  if not IsSquare(M2) or  not IsSquare(Mt) or
     (Mt.Lo2<>M1.Lo2) or (Mt.Hi2<>M1.Hi2) or
     (M2.Lo2<>M1.Lo1) or (M2.Hi2<>M1.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);
  M := TFArr2D.Create(M1);
  MxM(M2, M1, M);
  MTxM(M1,M,Mt);
end;

{ Mt = DiagonalMatrix(DL)*MR  }
procedure DxM(const DL: IFArr1D; const MR:IFArr2D; const Mt: IFArr2D);
var
  i1, i2: TInt;
begin
  if (Mt.Lo1<>DL.Lo1) or (Mt.Hi1<>DL.Hi1) or
     (Mt.Lo2<>MR.Lo2) or (Mt.Hi2<>MR.Hi2) or
     (DL.Lo1<>MR.Lo1) or (DL.Hi1<>MR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
      Mt[i1,i2] := DL[i1]*MR[i1,i2];
end;

{ Mt = ML*DiagonalMatrix(DR) }
procedure MxD(const ML: IFArr2D; const DR: IFArr1D; const Mt: IFArr2D);
var
  i1, i2: TInt;
begin
  if (Mt.Lo1<>ML.Lo1) or (Mt.Hi1<>ML.Hi1) or
     (Mt.Lo2<>DR.Lo1) or (Mt.Hi2<>DR.Hi1) or
     (ML.Lo2<>DR.Lo1) or (ML.Hi2<>DR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
      Mt[i1,i2] := ML[i1,i2]*DR[i2];
end;

{ Mt = Transpose(ML)*DiagonalMatrix(DR) }
procedure MTrxD(const ML: IFArr2D; const DR: IFArr1D; const Mt: IFArr2D);
var
  i1, i2: TInt;
begin
  if (Mt.Lo1<>ML.Lo2) or (Mt.Hi1<>ML.Hi2) or
     (Mt.Lo2<>DR.Lo1) or (Mt.Hi2<>DR.Hi1) or
     (ML.Lo1<>DR.Lo1) or (ML.Hi1<>DR.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
      Mt[i1,i2] := ML[i2,i1]*DR[i2];
end;

{ Mt = M*DiagonalMatrix(DR)*MT }
procedure MxDxMT(const M: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>M.Lo1) or (Mt.Hi1<>M.Hi1) or
     (Mt.Lo2<>M.Lo1) or (Mt.Hi2<>M.Hi1) or
     (D.Lo1<>M.Lo2) or (D.Hi1<>M.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := M.Lo2 to M.Hi2 do
        s := s + M[i1,j]*M[i2,j]*D[j];
      Mt[i1,i2] := s;
    end;
end;

procedure MTxDxM(const M: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>M.Lo2) or (Mt.Hi1<>M.Hi2) or
     (Mt.Lo2<>M.Lo2) or (Mt.Hi2<>M.Hi2) or
     (D.Lo1<>M.Lo1) or (D.Hi1<>M.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := M.Lo1 to M.Hi1 do
        s := s + M[j,i1]*D[j]*M[j,i2];
      Mt[i1,i2] := s;
    end;
end;

procedure M1xDxM2T(const M1, M2: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>M1.Lo1) or (Mt.Hi1<>M1.Hi1) or
     (Mt.Lo2<>M2.Lo1) or (Mt.Hi2<>M2.Hi1) or
     (M1.Lo2<>M2.Lo2) or (M1.Hi2<>M2.Hi2) or
     (D.Lo1<>M2.Lo2) or (D.Hi1<>M2.Hi2) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := M1.Lo2 to M1.Hi2 do
        s := s + M1[i1,j]*M2[i2,j]*D[j];
      Mt[i1,i2] := s;
    end;
end;

procedure M1TxDxM2(const M1, M2: IFArr2D; const D: IFArr1D; const Mt: IFArr2D);
var
  i1, i2, j: TInt;
  s: TFloat;
begin
  if (Mt.Lo1<>M1.Lo2) or (Mt.Hi1<>M1.Hi2) or
     (Mt.Lo2<>M2.Lo2) or (Mt.Hi2<>M2.Hi2) or
     (M1.Lo1<>M2.Lo1) or (M1.Hi1<>M2.Hi1) or
     (D.Lo1<>M2.Lo1) or (D.Hi1<>M2.Hi1) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i1 := Mt.Lo1 to Mt.Hi1 do
    for i2 := Mt.Lo2 to Mt.Hi2 do
    begin
      s := 0.0;
      for j := M1.Lo1 to M1.Hi1 do
        s := s + M1[j,i1]*M2[j,i2]*D[j];
      Mt[i1,i2] := s;
    end;
end;

{ MT = Transpose(M) }
procedure MtT(const M, MT: IBArr2D);
var
  i, j: TInt;
begin
  if not SameLimT(M,MT) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := MT.Lo1 to MT.Hi1 do
    for j := MT.Lo2 to MT.Hi2 do
      MT[i,j] := M[j,i]
end;

{ MT = Transpose(M) }
procedure MtT(const M, MT: IIArr2D);
var
  i, j: TInt;
begin
  if not SameLimT(M,MT) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := MT.Lo1 to MT.Hi1 do
    for j := MT.Lo2 to MT.Hi2 do
      MT[i,j] := M[j,i]
end;

{ MT = Transpose(M) }
procedure MtT(const M, MT: IFArr2D);
var
  i, j: TInt;
begin
  if not SameLimT(M,MT) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := MT.Lo1 to MT.Hi1 do
    for j := MT.Lo2 to MT.Hi2 do
      MT[i,j] := M[j,i]
end;

{ idx is in IArr }
function IsInArr(idx: TInt; const IArr: IIArr1D): boolean;
var
  i: TInt;
begin
  for i := IArr.Lo1 to IArr.Hi1 do
    if idx = IArr[i] then
    begin
      result := true;
      exit;
    end;
  result := false;
end;

{Ascending Buble Sort}
procedure BubleSortI1D(const A: IIArr1D);
var
  i, j: TInt;
//  v: TInt;
begin
  for i := A.Lo1 to A.Hi1-1 do
    for j := i+1 to A.Hi1 do
      if A[i] > A[j] then
        A.Swap(i,j);
end;

{Ascending Buble Sort}
procedure BubleSortF1D(const A: IFArr1D);
var
  i, j: TInt;
//  v: TInt;
begin
  for i := A.Lo1 to A.Hi1-1 do
    for j := i+1 to A.Hi1 do
      if A[i] > A[j] then
        A.Swap(i,j);
end;

{ Ascending Insert Sort: Based on the Algorithm by D.F.Stermole
  http://www.macdonald.egate.net/CompSci/Pascal/hsorting.html}
procedure InsertSortI1D(const A: IIArr1D);
var
  i, j: TInt;
  v: TInt;
begin
  for i := A.Lo1+1 to A.Hi1 do
  begin
    v := A[i];
    { Move all values larger than key to the right one position }
    j := i-1;
    // it is assumed that the short-circuit evaluation is ON
    // {$B-} or "complete boolean eval" is OFF in compiler options
    while (j >= A.Lo1) and (A[j] > v)  do
    begin
      A[j+1] := A[j];
      dec(j);
//      if j < A.Lo1 then break; // uncomment if short-circuit evaluation is OFF
    end;
    { insert current value into its proper place }
    A[j+1] := v;
  end;
end;

{ Ascending NONRECURSIVE quicksort routine
  Alan R. Miller. Borland Pascal Programs for Scientists and Engineers,
  Copyright  1993, (SYBEX) ISBN 0-7821-1150-5
  Adapted from 'Software-Tools', B.Kernighan, Addison Wesley, 1976 }
procedure NRQuickSortI1D(const x: IIArr1D);
var
	left, right: IIArr1D;
	i, j, sp, mid, lo, hi: TInt;
//  pivot: TFloat;
  pivot: TInt;

begin
  lo := x.Lo1;
  hi := x.Hi1;
	left := TIArr1D.Create(lo,hi);
	right := TIArr1D.Create(lo,hi);
  left[lo] := lo;
  right[lo] := hi;
  sp := lo;
  while sp >= lo do
  begin
    if left[sp] >= right[sp] then sp := sp - 1
    else
    begin
      i := left[sp];
      j := right[sp];
      pivot := x[j];
      mid := (i+j) div 2;
      if (j-i)>5 then
        if ((x[mid]<pivot) and (x[mid]>x[i]))
          or ((x[mid]>pivot)and(x[mid]<x[i]))
        then x.Swap(mid,j)
        else
          if ((x[i]<x[mid]) and (x[i]>pivot))
            or ((x[i]>x[mid]) and (x[i]<pivot))
          then x.Swap(i,j);
      pivot := x[j];
      while i<j do
      begin
        while x[i]<pivot do
          i := i+1;
        j := j-1;
        while (i<j) and (pivot<x[j]) do
          j := j-1;
        if i<j then x.Swap(i,j)
      end;	{ while }
      j := right[sp];	{ pivot to i }
      x.Swap(i,j);
      if i-left[sp]>=right[sp]-i then
      begin		{ put shorter part first }
        left[sp+1] := left[sp];
        right[sp+1] := i-1;
        left[sp] := i+1
      end else
      begin
        left[sp+1] := i+1;
        right[sp+1] := right[sp];
        right[sp] := i-1
      end;
      sp := sp+1		{ push stack }
    end		{ if }
  end		{ while }
end;	{ NR QUICK SORT }

{ Ascending RECURSIVE quicksort routine
  Alan R. Miller. Borland Pascal Programs for Scientists and Engineers,
  Copyright  1993, (SYBEX) ISBN 0-7821-1150-5
  Adapted from 'The design of Well-Structured and Correct Programs',
	S. Alagic, Springer-Verlag, 1978 }
procedure QuickSortI1D(const x: IIArr1D);

  procedure QSort(const x: IIArr1D; m, n: TInt);
  var
  	i, j: TInt;

    procedure partit(const a: IIArr1D; var i, j: TInt; const left, right: TInt);
    var
    // pivot: TFloat;
      pivot: TInt;

    begin
      pivot := a[(left+right) div 2];
      i := left;
      j := right;
      while i<=j do
      begin
        while a[i]<pivot do i := i+1;
        while pivot<a[j] do j := j-1;
        if i<=j then
        begin
          a.Swap(i,j);
          i := i+1;
          j := j-1
        end
      end	{ while }
    end;	{ partit }

  begin	{ QSort }
    if( m < n ) then
    begin
      partit(x,i,j,m,n);	{ divide in two }
      qsort(x,m,j);		{ sort left part }
      qsort(x,i,n)		{ sort right part }
    end;
  end;  { QSort }

begin	{ QuckSort }
  QSort(x, x.Lo1, x.Hi1)
end;	{ QuckSort }

{ Based on: Sebastian Boung, Sorting Algorithms in Pascal
  http://privat.schlund.de/b/bossung/prog/delphi/sorting.html }
procedure QSortI1D(const A: IIArr1D; Ascending: boolean);

  procedure Quicksort(l, r: TInt);
  var
    i, j: TInt;
    v: TInt;
  begin
    v := A[(l + r) div 2]; { Wert des Teilungselements }
    i := l; j := r;
    repeat
      if Ascending then
      begin
        while A[i] < v do inc(i);
        while v < A[j] do dec(j);
      end else
      begin
        while A[i] > v do inc(i);
        while v > A[j] do dec(j);
      end;
      if i <= j then
      begin
        A.Swap(i,j);
        inc(i);
        dec(j); { sonst kreuzen die sich nie }
      end;
    until i > j; { bis die Zeiger sich kreuzen }
    if l < j then Quicksort(l, j);
    if i < r then Quicksort(i, r);
  end;

begin
  Quicksort(A.Lo1, A.Hi1);
end;

{ Based on: Sebastian Boung, Sorting Algorithms in Pascal
  http://privat.schlund.de/b/bossung/prog/delphi/sorting.html }
procedure QSortF1D(const A: IFArr1D; Ascending: boolean);

  procedure Quicksort(l, r: TInt);
  var
    i, j: TInt;
    v: TFloat;
  begin
    v := A[(l + r) div 2]; { Wert des Teilungselements }
    i := l; j := r;
    repeat
      if Ascending then
      begin
        while A[i] < v do inc(i);
        while v < A[j] do dec(j);
      end else
      begin
        while A[i] > v do inc(i);
        while v > A[j] do dec(j);
      end;
      if i <= j then
      begin
        A.Swap(i,j);
        inc(i);
        dec(j); { sonst kreuzen die sich nie }
      end;
    until i > j; { bis die Zeiger sich kreuzen }
    if l < j then Quicksort(l, j);
    if i < r then Quicksort(i, r);
  end;

begin
  Quicksort(A.Lo1, A.Hi1);
end;

{ 1D-array sorting with sorting of the associated 2D-array
  by columns (idx = 2) or  by rows (idx = 1)
  Based on: Sebastian Boung, Sorting Algorithms in Pascal 
  http://privat.schlund.de/b/bossung/prog/delphi/sorting.html }
procedure QSortF2D(const A: IFArr1D; const B: IFArr2D; Ascending: boolean; st: TSliceType);

  procedure Quicksort(l, r: TInt);
  var
    i, j: TInt;
    v: TFloat;
  begin
    v := A[(l + r) div 2]; { Wert des Teilungselements }
    i := l; j := r;
    repeat
      if Ascending then
      begin
        while A[i] < v do inc(i);
        while v < A[j] do dec(j);
      end else
      begin
        while A[i] > v do inc(i);
        while v > A[j] do dec(j);
      end;

      if i <= j then
      begin
        A.Swap(i,j);
        B.Swap(i,j,st);
        inc(i);
        dec(j); { sonst kreuzen die sich nie }
      end;
    until i > j; { bis die Zeiger sich kreuzen }
    if l < j then Quicksort(l, j);
    if i < r then Quicksort(i, r);
  end;

begin
  Quicksort(A.Lo1, A.Hi1);
end;

{ Old variant support }

/// VC = Conjugate(V); V = VC is OK
procedure CVC(const VR, VI, VCR, VCI: IFArr1D);
var
  i: TInt;
begin
  if not SameLim(VR,VCR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := VR.Lo1 to VR.Hi1 do
  begin
    VCR[i] :=  VR[i];
    VCI[i] := -VI[i];
  end;
end;

{ MT = Transpose(M) }
procedure CMT(const MR, MI, MTR, MTI: IFArr2D);
var
  i, j: TInt;
begin
  if not SameLimT(MR,MTR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := MTR.Lo1 to MTR.Hi1 do
    for j := MTR.Lo2 to MTR.Hi2 do
    begin
      MTR[i,j] := MR[j,i];
      MTI[i,j] := MI[j,i];
    end;
end;

{ MC = Conjugate(M); M = MC is OK }
procedure CMC(const MR, MI, MCR, MCI: IFArr2D);
var
  i, j: TInt;
begin
  if not SameLim(MR,MCR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := MCR.Lo1 to MCR.Hi1 do
    for j := MCR.Lo2 to MCR.Hi2 do
    begin
      MCR[i,j] := -MR[i,j];
      MCI[i,j] := -MI[i,j];
    end;
end;

{ MH = HTranspose(M) }
procedure CMH(const MR, MI, MHR, MHI: IFArr2D);
var
  i, j: TInt;
begin
  if not SameLimT(MR,MHR) or not SameLimT(MI,MHI) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i := MHR.Lo1 to MHR.Hi1 do
    for j := MHR.Lo2 to MHR.Hi2 do
    begin
      MHR[i,j] := MR[j,i];
      MHI[i,j] := -MI[j,i];
    end;
end;

{ Dot (scalar) product of two vectors }
procedure CVxCV(const V1R, V1I, V2R, V2I: IFArr1D; var ReResult, ImResult: TFloat);
begin
  ReResult := VxV(V1R, V2R)-VxV(V1I, V2I);
  ImResult := VxV(V1R, V2I)+VxV(V1I, V2R);
end;

{ HConjugate(V1) Dot V2 }
procedure CVHxCV(const V1R, V1I, V2R, V2I: IFArr1D; var ReResult, ImResult: TFloat);
begin
  ReResult := VxV(V1R, V2R)+VxV(V1I, V2I);
  ImResult := VxV(V1R, V2I)+VxV(V1I, V2R);
end;

{ (V3R + i*V3I) = (M1R + i*M1I)*(V2R + i*V2I)
   V3R = M1R*V2R - M1I*V2I;  M3I = M1R*V2I + M1I*V2R }
procedure CMxCV(const M1R, M1I: IFArr2D; const V2R, V2I, V3R, V3I: IFArr1D);
var
  i, k: TInt;
  rsum, isum: TFloat;
begin
  if (V3R.Lo1<>M1R.Lo1) or (V3R.Hi1<>M1R.Hi1) or
     (M1R.Lo2<>V2R.Lo1) or (M1R.Hi2<>V2R.Hi1) or
     not SameLim(M1R,M1I) or not SameLim(V2R,V2I) or
     not SameLim(V3R,V3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := V3R.Lo1 to V3R.Hi1 do
  begin
    rsum := 0.0;
    isum := 0.0;
    for k := M1R.Lo2 to M1R.Hi2 do
    begin
      rsum := rsum + M1R[i,k]*V2R[k]-M1I[i,k]*V2I[k];
      isum := isum + M1R[i,k]*V2I[k]+M1I[i,k]*V2R[k];
    end;
    V3R[i] := rsum;
    V3I[i] := isum;
  end;
end;

{ (V3R + i*V3I) = (V1R + i*V1I)*(M2R + i*M2I)
   V3R = V1R*M2R - V1I*M2I;  V3I = V1R*M2I + V1I*M2R }
procedure CVxCM(const V1R, V1I: IFArr1D; const M2R, M2I: IFArr2D; const V3R, V3I: IFArr1D);
var
  j, k: TInt;
  rsum, isum: TFloat;
begin
  if (V3R.Lo1<>M2R.Lo2) or (V3R.Hi1<>M2R.Hi2) or
     (V1R.Lo1<>M2R.Lo1) or (V1R.Hi1<>M2R.Hi1) or
     not SameLim(V1R,V1I) or not SameLim(M2R,M2I) or
     not SameLim(V3R,V3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for j := V3R.Lo1 to V3R.Hi1 do
    begin
      rsum := 0.0;
      isum := 0.0;
      for k := V1R.Lo1 to V1R.Hi1 do
      begin
        rsum := rsum + V1R[k]*M2R[k,j]-V1I[k]*M2I[k,j];
        isum := isum + V1R[k]*M2I[k,j]+V1I[k]*M2R[k,j];
      end;
      V3R[j] := rsum;
      V3I[j] := isum;
    end;
end;

{ (M3R + i*M3I) = (M1R + i*M1I)*(M2R + i*M2I)
   M3R = M1R*M2R - M1I*M2I;  M3I = M1R*M2I + M1I*M2R }
procedure CMxCM(const M1R, M1I, M2R, M2I, M3R, M3I: IFArr2D);
var
  i, j, k: TInt;
  rsum, isum: TFloat;
begin
  if (M3R.Lo1<>M1R.Lo1) or (M3R.Hi1<>M1R.Hi1) or
     (M3R.Lo2<>M2R.Lo2) or (M3R.Hi2<>M2R.Hi2) or
     (M1R.Lo2<>M2R.Lo1) or (M1R.Hi2<>M2R.Hi1) or
     not SameLim(M1R,M1I) or not SameLim(M2R,M2I) or
     not SameLim(M3R,M3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := M3R.Lo1 to M3R.Hi1 do
    for j := M3R.Lo2 to M3R.Hi2 do
    begin
      rsum := 0.0;
      isum := 0.0;
      for k := M1R.Lo2 to M1R.Hi2 do
      begin
        rsum := rsum + M1R[i,k]*M2R[k,j]-M1I[i,k]*M2I[k,j];
        isum := isum + M1R[i,k]*M2I[k,j]+M1I[i,k]*M2R[k,j];
      end;
      M3R[i,j] := rsum;
      M3I[i,j] := isum;
    end;
end;

{ (M3R + i*M3I) = (M1R + i*M1I)*(M2RT - i*M2IT)
   M3R = M1R*M2RT + M1I*M2IT;  M3I = -M1R*M2IT + M1I*M2RT }
procedure CMxCMH(const M1R, M1I, M2R, M2I, M3R, M3I: IFArr2D);
var
  i, j, k: TInt;
  rsum, isum: TFloat;
begin
  if (M3R.Lo1<>M1R.Lo1) or (M3R.Hi1<>M1R.Hi1) or
     (M3R.Lo2<>M2R.Lo1) or (M3R.Hi2<>M2R.Hi1) or
     (M1R.Lo2<>M2R.Lo2) or (M1R.Hi2<>M2R.Hi2) or
      not SameLim(M1R,M1I) or not SameLim(M2R,M2I) or
      not SameLim(M3R,M3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := M3R.Lo1 to M3R.Hi1 do
    for j := M3R.Lo2 to M3R.Hi2 do
    begin
      rsum := 0.0;
      isum := 0.0;
      for k := M1R.Lo2 to M1R.Hi2 do
      begin
        rsum := rsum + M1R[i,k]*M2R[j,k]+M1I[i,k]*M2I[j,k];
        isum := isum - M1R[i,k]*M2I[j,k]+M1I[i,k]*M2R[j,k];
      end;
      M3R[i,j] := rsum;
      M3I[i,j] := isum;
    end;
end;

{ (M3R + i*M3I) = (M1RT - i*M1IT)*(M2R + i*M2I)
   M3R = M1RT*M2R + M1IT*M2I;  M3I = M1RT*M2I - M1IT*M2R }
procedure CMHxCM(const M1R, M1I, M2R, M2I, M3R, M3I: IFArr2D);
var
  i, j, k: TInt;
  rsum, isum: TFloat;
begin
  if (M3R.Lo1<>M1R.Lo2) or (M3R.Hi1<>M1R.Hi2) or
     (M3R.Lo2<>M2R.Lo2) or (M3R.Hi2<>M2R.Hi2) or
     (M1R.Lo1<>M2R.Lo1) or (M1R.Hi1<>M2R.Hi1) or
     not SameLim(M1R,M1I) or not SameLim(M2R,M2I) or
     not SameLim(M3R,M3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := M3R.Lo1 to M3R.Hi1 do
    for j := M3R.Lo2 to M3R.Hi2 do
    begin
      rsum := 0.0;
      isum := 0.0;
      for k := M1R.Lo1 to M1R.Hi1 do
      begin
        rsum := rsum + M1R[k,i]*M2R[k,j]+M1I[k,i]*M2I[k,j];
        isum := isum + M1R[k,i]*M2I[k,j]-M1I[k,i]*M2R[k,j];
      end;
      M3R[i,j] := rsum;
      M3I[i,j] := isum;
    end;
end;

{ (M3R + i*M3I) = (M1R + i*M1I)*D*(M2RT - i*M2IT)
   M3R = M1R*D*M1RT + M1I*D*M1IT;  M3I = -M1R*D*M1IT + M1I*D*M1RT }
procedure CMxDxCMH(const M1R, M1I: IFArr2D; const D: IFArr1D; const M3R, M3I: IFArr2D);
var
  i, j, k: TInt;
  rsum, isum: TFloat;
begin
  if (M3R.Lo1<>M1R.Lo1) or (M3R.Hi1<>M1R.Hi1) or
     (M3R.Lo2<>M1R.Lo1) or (M3R.Hi2<>M1R.Hi1) or
     (D.Lo1<>M1R.Lo2) or (D.Hi1<>M1R.Hi2) or
     not SameLim(M1R,M1I) or not SameLim(M3R,M3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := M3R.Lo1 to M3R.Hi1 do
    for j := M3R.Lo2 to M3R.Hi2 do
    begin
      rsum := 0.0;
      isum := 0.0;
      for k := M1R.Lo2 to M1R.Hi2 do
      begin
        rsum := rsum + ( M1R[i,k]*M1R[j,k]+M1I[i,k]*M1I[j,k])*D[k];
        isum := isum + (-M1R[i,k]*M1I[j,k]+M1I[i,k]*M1R[j,k])*D[k];
      end;
      M3R[i,j] := rsum;
      M3I[i,j] := isum;
    end;
end;

{ (M3R + i*M3I) = (M1RT - i*M1IT)*D*(M2R + i*M2I)
   M3R = M1RT*D*M1R + M1IT*D*M1I;  M3I = M1RT*D*M2I - M1IT*D*M1R }
procedure CMHxDxCM(const M1R, M1I: IFArr2D; const D: IFArr1D; const M3R, M3I: IFArr2D);
var
  i, j, k: TInt;
  rsum, isum: TFloat;
begin
  if (M3R.Lo1<>M1R.Lo2) or (M3R.Hi1<>M1R.Hi2) or
     (M3R.Lo2<>M1R.Lo2) or (M3R.Hi2<>M1R.Hi2) or
     (M1R.Lo1<>D.Lo1) or (M1R.Hi1<>D.Hi1) or
     not SameLim(M1R,M1I) or not SameLim(M3R,M3I) then
    Raise ERangeError.Create(RS_LimMismatch);

  for i := M3R.Lo1 to M3R.Hi1 do
    for j := M3R.Lo2 to M3R.Hi2 do
    begin
      rsum := 0.0;
      isum := 0.0;
      for k := M1R.Lo1 to M1R.Hi1 do
      begin
        rsum := rsum + (M1R[k,i]*M1R[k,j]-M1I[k,i]*M1I[k,j])*D[k];
        isum := isum + (M1R[k,i]*M1I[k,j]+M1I[k,i]*M1R[k,j])*D[k];
      end;
      M3R[i,j] := rsum;
      M3I[i,j] := isum;
    end;
end;

end.
