{*
 Linear algebra extra utilities (mainly for unit testing)
 Nikolai V. Shokhirev, 2002-2007
 @author Nikolai V. Shokhirev <nikolai@shokhirev.com> http://www.shokhirev.com/nikolai.html
 @version 2002.02.02 - created
 @version 2004.06.06 - extended
 @version 2007.07.15 - Added DelphiCodeToDoc comments
}
unit uDynArrUtilsX;

interface

uses
  uMatTypes, uDynArrays;

/// helper for unit testing
function IsEqual(expected, actual: complex; var msg: string; eps: TFloat = 1.0e-12): boolean; overload;
/// helper for unit testing
function IsEqual(expected, actual: TFloat; var msg: string; eps: TFloat = 1.0e-12): boolean; overload;
/// helper for unit testing
function IsEqual(expected, actual: TInt; var msg: string): boolean; overload;
/// helper for unit testing
function IsEqual(expected, actual: boolean; var msg: string): boolean; overload;

/// result := A1 = A2
function SameArr(const A1, A2: ISArr1D; var msg: string): boolean; overload;
/// result := A1 = A2
function SameArr(const A1, A2: ISArr2D; var msg: string): boolean; overload;
/// result := A1 = A2
function SameArr(const A1, A2: IBArr1D; var msg: string): boolean; overload;
/// result := A1 = A2
function SameArr(const A1, A2: IBArr2D; var msg: string): boolean; overload;
/// result := A1 = A2
function SameArr(const A1, A2: IiArr1D; var msg: string): boolean; overload;
/// result := A1 = A2 
function SameArr(const A1, A2: IIArr2D; var msg: string): boolean; overload;
/// result := A1 = A2 
function SameArr(const A1, A2: IIArr3D; var msg: string): boolean; overload;
/// result := max |A1 = A2| < eps 
function SameArr(const A1, A2: IFArr1D; var msg: string; eps: TFloat): boolean; overload;
/// result := max |A1 = A2| < eps 
function SameArr(const A1, A2: IFArr2D; var msg: string; eps: TFloat): boolean; overload;
/// result := max |A1 = A2| < eps 
function SameArr(const A1, A2: IFArr3D; var msg: string; eps: TFloat): boolean; overload;
/// result := max |A1 = A2| < eps 
function SameArr(const A1, A2: ICArr1D; var msg: string; eps: TFloat): boolean; overload;
/// result := max |A1 = A2| < eps 
function SameArr(const A1, A2: ICArr2D; var msg: string; eps: TFloat): boolean; overload;

/// i-th vector of idx-th dimension of AR 
function GetArr1D(const A2D: IBArr2D; i: TInt; st: TSliceType): IBArr1D; overload;
/// i-th vector of idx-th dimension of AR 
function GetArr1D(const A2D: IIArr2D; i: TInt; st: TSliceType): IIArr1D; overload;
/// i-th vector of idx-th dimension of AR 
function GetArr1D(const A2D: IFArr2D; i: TInt; st: TSliceType): IFArr1D; overload;

/// Copy A1D to i-th vector of idx-th dimension of A2D 
procedure CopyArr1D(const A2D: IBArr2D; const A1D: IBArr1D; i: TInt; st: TSliceType); overload;
/// Copy A1D to i-th vector of idx-th dimension of A2D 
procedure CopyArr1D(const A2D: IIArr2D; const A1D: IIArr1D; i: TInt; st: TSliceType); overload;
/// Copy A1D to i-th vector of idx-th dimension of A2D 
procedure CopyArr1D(const A2D: IFArr2D; const A1D: IFArr1D; i: TInt; st: TSliceType); overload;

/// AL := AR boolean 1D
procedure CopyAA(const AL, AR: IBArr1D); overload;
/// AL := AR boolean 2D
procedure CopyAA(const AL, AR: IBArr2D); overload;
/// AL := AR integer 1D
procedure CopyAA(const AL, AR: IIArr1D); overload;
/// AL := AR integer 2D
procedure CopyAA(const AL, AR: IIArr2D); overload;
/// AL := AR integer 3D
procedure CopyAA(const AL, AR: IIArr3D); overload;
/// AL := AR float 1D
procedure CopyAA(const AL, AR: IFArr1D); overload;
/// AL := AR float 2D
procedure CopyAA(const AL, AR: IFArr2D); overload;
/// AL := AR float 3D
procedure CopyAA(const AL, AR: IFArr3D); overload;

/// i-th vector of idx-th dimension of AR 
function GetArr1D(const A2D: IBArr2D; i, idx: TInt): IBArr1D; overload;
/// i-th vector of idx-th dimension of AR 
function GetArr1D(const A2D: IIArr2D; i, idx: TInt): IIArr1D; overload;
/// i-th vector of idx-th dimension of AR 
function GetArr1D(const A2D: IFArr2D; i, idx: TInt): IFArr1D; overload;

/// Al := (i-th vector of idx-th dimension of A2D 
procedure CopyTo1D(const A1D: IBArr1D; const A2D: IBArr2D; i, idx: TInt); overload;
/// Al := (i-th vector of idx-th dimension of A2D 
procedure CopyTo1D(const A1D: IIArr1D; const A2D: IIArr2D; i, idx: TInt); overload;
/// Al := (i-th vector of idx-th dimension of A2D 
procedure CopyTo1D(const A1D: IFArr1D; const A2D: IFArr2D; i, idx: TInt); overload;
/// Al := (i-th matrix of idx-th dimension of A3D 
procedure CopyTo2D(const A2D: IIArr2D; const A3D: IIArr3D; i, idx: TInt); overload;
/// Al := (i-th matrix of idx-th dimension of A3D 
procedure CopyTo2D(const A2D: IFArr2D; const A3D: IFArr3D; i, idx: TInt); overload;

/// Al := (i-th vector of idx-th dimension of AR 
procedure CopyFrom1D(const A2D: IIArr2D; const A1D: IIArr1D; i, idx: TInt); overload;
/// Al := (i-th vector of idx-th dimension of AR 
procedure CopyFrom1D(const A2D: IFArr2D; const A1D: IFArr1D; i, idx: TInt); overload;

/// A := CxA 
procedure CxArr(C: TInt; const A: IIArr1D); overload;
/// A := CxA 
procedure CxArr(C: TInt; const A: IIArr2D); overload;
/// A := CxA 
procedure CxArr(C: TInt; const A: IIArr3D); overload;
/// A := CxA 
procedure CxArr(C: TFloat; const A: IFArr1D); overload;
/// A := CxA 
procedure CxArr(C: TFloat; const A: IFArr2D); overload;
/// A := CxA 
procedure CxArr(C: TFloat; const A: IFArr3D); overload;
/// A := CxA 
procedure CxArr(C: Complex; const A: ICArr1D); overload;
/// A := CxA 
procedure CxArr(C: Complex; const A: ICArr2D); overload;

/// A[ ] := val 
procedure Fill(const A: IBArr1D; val: boolean); overload;
/// A[ ] := val 
procedure Fill(const A: IBArr2D; val: boolean); overload;
/// A[ ] := val 
procedure Fill(const A: IIArr1D; val: TInt); overload;
/// A[ ] := val 
procedure Fill(const A: IIArr2D; val: TInt); overload;
/// A[ ] := val 
procedure Fill(const A: IIArr3D; val: TInt); overload;
/// A[ ] := val 
procedure Fill(const A: IFArr1D; val: TFloat); overload;
/// A[ ] := val 
procedure Fill(const A: IFArr2D; val: TFloat); overload;
/// A[ ] := val 
procedure Fill(const A: IFArr3D; val: TFloat); overload;
/// A[ ] := val 
procedure Fill(const A: ICArr1D; val: Complex); overload;
/// A[ ] := val 
procedure Fill(const A: ICArr2D; val: Complex); overload;

implementation

uses
  SysUtils, uDynArrUtils;

ResourceString
//  RS_MinMax = 'imin = %d > imax = %d';
  /// 'Index No= %d : out of dimension %d';
  RS_DimError = 'Index No= %d : out of dimension %d';
//  RS_NotSquare = 'Matrix is not square';
//  RS_LimMismatch = 'Lim Mismatch';
//  RS_ZeroVector = 'Cannot Normaloze: Zero Vector';

function IsEqual(expected, actual: complex; var msg: string; eps: TFloat): boolean;
begin
  result := Cabs(Csub(expected,actual)) < eps;
  if result then msg := 'OK'
  else msg := 'Expected:'+ComplexToStr(expected)+' Actual:'+ComplexToStr(actual);
end;

function IsEqual(expected, actual: TInt; var msg: string): boolean;
begin
  result := expected = actual;
  if result then msg := 'OK'
  else msg := 'Expected:'+IntToStr(expected)+' Actual:'+IntToStr(actual);
end;

function IsEqual(expected, actual: boolean; var msg: string): boolean;
begin
  result := expected = actual;
  if result then msg := 'OK'
  else msg := 'Expected:'+BoolToStr(expected)+' Actual:'+BoolToStr(actual);
end;

function IsEqual(expected, actual: TFloat; var msg: string; eps: TFloat): boolean;
begin
  result := abs(expected-actual) < eps;
  if result then msg := 'OK'
  else msg := 'Expected:'+FloatToStr(expected)+' Actual:'+FloatToStr(actual);
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: ISArr1D; var msg: string): boolean;
var
  i1: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      if (A1[i1] <> A2[i1]) then
        begin
          msg := 'Not Equal';
          result := false;
          exit;
        end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: ISArr2D; var msg: string): boolean;
var
  i1, i2: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
        if (A1[i1,i2] <> A2[i1,i2]) then
          begin
            msg := 'Not Equal';
            result := false;
            exit;
          end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: IBArr1D; var msg: string): boolean;
var
  i1: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      if (A1[i1] <> A2[i1]) then
        begin
          msg := 'Not Equal';
          result := false;
          exit;
        end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: IBArr2D; var msg: string): boolean;
var
  i1, i2: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
        if (A1[i1,i2] <> A2[i1,i2]) then
          begin
            msg := 'Not Equal';
            result := false;
            exit;
          end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: IIArr1D; var msg: string): boolean;
var
  i1: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      if (A1[i1] <> A2[i1]) then
        begin
          msg := 'Not Equal';
          result := false;
          exit;
        end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: IIArr2D; var msg: string): boolean;
var
  i1, i2: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
        if (A1[i1,i2] <> A2[i1,i2]) then
          begin
            msg := 'Not Equal';
            result := false;
            exit;
          end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := A1 = A2 }
function SameArr(const A1, A2: IIArr3D; var msg: string): boolean;
var
  i1, i2, i3: TInt;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
        for i3 := A1.Lo3 to A1.Hi3 do
          if (A1[i1,i2,i3] <> A2[i1,i2,i3]) then
            begin
              msg := 'Not Equal';
              result := false;
              exit;
            end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := max |A1 = A2| < eps }
function SameArr(const A1, A2: IFArr1D; var msg: string; eps: TFloat): boolean;
var
  i1: TInt;
  del: TFloat;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
    begin
      del := abs (A1[i1] - A2[i1]);
      if del > eps then
        begin
          msg := 'del = '+FloatToStr(del)+' > eps = '+FloatToStr(eps);
          result := false;
          exit;
        end;
    end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := max |A1 = A2| < eps }
function SameArr(const A1, A2: IFArr2D; var msg: string; eps: TFloat): boolean;
var
  i1, i2: TInt;
  del: TFloat;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
      begin
        del := abs (A1[i1,i2] - A2[i1,i2]);
        if del > eps then
          begin
            msg := 'del = '+FloatToStr(del)+' > eps = '+FloatToStr(eps);
            result := false;
            exit;
          end;
      end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := max |A1 = A2| < eps }
function SameArr(const A1, A2: IFArr3D; var msg: string; eps: TFloat): boolean;
var
  i1, i2, i3: TInt;
  del: TFloat;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
        for i3 := A1.Lo3 to A1.Hi3 do
        begin
          del := abs (A1[i1,i2,i3] - A2[i1,i2,i3]);
          if del > eps then
          begin
            msg := 'del = '+FloatToStr(del)+' > eps = '+FloatToStr(eps);
            result := false;
            exit;
          end;
        end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := max |A1 = A2| < eps }
function SameArr(const A1, A2: ICArr1D; var msg: string; eps: TFloat): boolean;
var
  i1: TInt;
  del: TFloat;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      begin
      del := Diff0(A1[i1],A2[i1]);
      if (del > eps) then
      begin
        msg := 'del = '+FloatToStr(del)+' > eps = '+FloatToStr(eps);
        result := false;
        exit;
      end;
    end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ result := max |A1 = A2| < eps }
function SameArr(const A1, A2: ICArr2D; var msg: string; eps: TFloat): boolean;
var
  i1, i2: TInt;
  del: TFloat;
begin
  msg := 'OK';
  result := true;
  if SameLim(A1,A2) then
  begin
    for i1 := A1.Lo1 to A1.Hi1 do
      for i2 := A1.Lo2 to A1.Hi2 do
      begin
        del := Diff0(A1[i1,i2],A2[i1,i2]);
        if (del > eps) then
        begin
          msg := 'del= '+FloatToStr(del)+' > eps = '+FloatToStr(eps);
          result := false;
          exit;
        end;
      end;
  end else
  begin
    msg := 'Lim mismatch';
    result := false;
    exit;
  end;
end;

{ i-th vector of idx-th dimension of AR }
function GetArr1D(const A2D: IBArr2D; i: TInt; st: TSliceType): IBArr1D;
var
  k: TInt;
  A1D: IBArr1D;
begin
  case st of
  _Row:begin
          A1D := TBArr1D.Create(A2D.Lo2,A2D.Hi2);
          for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
        end;
  _Col:begin
          A1D := TBArr1D.Create(A2D.Lo1,A2D.Hi1);
          for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
        end;
  else
//    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
  result := A1D;
end;

{ i-th vector of idx-th dimension of AR }
function GetArr1D(const A2D: IIArr2D; i: TInt; st: TSliceType): IIArr1D;
var
  k: TInt;
  A1D: IIArr1D;
begin
  case st of
  _Row:begin
          A1D := TIArr1D.Create(A2D.Lo2,A2D.Hi2);
          for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
        end;
  _Col:begin
          A1D := TIArr1D.Create(A2D.Lo1,A2D.Hi1);
          for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
        end;
  else
//    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
  result := A1D;
end;

{ i-th vector of idx-th dimension of AR }
function GetArr1D(const A2D: IFArr2D; i: TInt; st: TSliceType): IFArr1D;
var
  k: TInt;
  A1D: IFArr1D;
begin
  case st of
   _Row:begin
          A1D := TFArr1D.Create(A2D.Lo2,A2D.Hi2);
          for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
        end;
   _Col:begin
          A1D := TFArr1D.Create(A2D.Lo1,A2D.Hi1);
          for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
        end;
  else
//    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
  result := A1D;
end;

{ Copy A1D to i-th vector of idx-th dimension of A2D }
procedure CopyArr1D(const A2D: IBArr2D; const A1D: IBArr1D; i: TInt; st: TSliceType);
var
  k: TInt;
begin
  case st of
  _Row: for k := A2D.Lo2 to A2D.Hi2 do A2D[i,k] := A1D[k];
  _Col: for k := A2D.Lo1 to A2D.Hi1 do A2D[k,i] := A1D[k];
  else
//    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Copy A1D to i-th vector of idx-th dimension of A2D }
procedure CopyArr1D(const A2D: IIArr2D; const A1D: IIArr1D; i: TInt; st: TSliceType);
var
  k: TInt;
begin
  case st of
  _Row: for k := A2D.Lo2 to A2D.Hi2 do A2D[i,k] := A1D[k];
  _Col: for k := A2D.Lo1 to A2D.Hi1 do A2D[k,i] := A1D[k];
  else
//    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Copy A1D to i-th vector of idx-th dimension of A2D }
procedure CopyArr1D(const A2D: IFArr2D; const A1D: IFArr1D; i: TInt; st: TSliceType);
var
  k: TInt;
begin
  case st of
  _Row: for k := A2D.Lo2 to A2D.Hi2 do A2D[i,k] := A1D[k];
  _Col: for k := A2D.Lo1 to A2D.Hi1 do A2D[k,i] := A1D[k];
  else
//    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ AL := AR boolean 1D}
procedure CopyAA(const AL, AR: IBArr1D);
var
  i1: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    AL[i1] := AR[i1];
end;

{ AL := AR boolean 2D}
procedure CopyAA(const AL, AR: IBArr2D);
var
  i1, i2: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    for i2 := AR.Lo2 to AR.Hi2 do
      AL[i1,i2] := AR[i1,i2];
end;

{ AL := AR integer 1D}
procedure CopyAA(const AL, AR: IIArr1D);
var
  i1: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    AL[i1] := AR[i1];
end;

{ AL := AR integer 2D}
procedure CopyAA(const AL, AR: IIArr2D);
var
  i1, i2: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    for i2 := AR.Lo2 to AR.Hi2 do
      AL[i1,i2] := AR[i1,i2];
end;

{ AL := AR integer 3D}
procedure CopyAA(const AL, AR: IIArr3D);
var
  i1, i2, i3: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    for i2 := AR.Lo2 to AR.Hi2 do
      for i3 := AR.Lo3 to AR.Hi3 do
        AL[i1,i2,i3] := AR[i1,i2,i3];
end;

{ AL := AR float 1D}
procedure CopyAA(const AL, AR: IFArr1D);
var
  i1: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    AL[i1] := AR[i1];
end;

{ AL := AR float 2D}
procedure CopyAA(const AL, AR: IFArr2D);
var
  i1, i2: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    for i2 := AR.Lo2 to AR.Hi2 do
      AL[i1,i2] := AR[i1,i2];
end;

{ AL := AR float 3D}
procedure CopyAA(const AL, AR: IFArr3D);
var
  i1, i2, i3: TInt;
begin
  if not SameLim(AL,AR) then
    Raise ERangeError.Create(RS_LimMismatch);
  for i1 := AR.Lo1 to AR.Hi1 do
    for i2 := AR.Lo2 to AR.Hi2 do
      for i3 := AR.Lo3 to AR.Hi3 do
        AL[i1,i2,i3] := AR[i1,i2,i3];
end;

{ i-th vector of idx-th dimension of AR }
function GetArr1D(const A2D: IBArr2D; i, idx: TInt): IBArr1D;
var
  k: TInt;
  A1D: IBArr1D;
begin
  case idx of
  1:begin
      A1D := TBArr1D.Create(A2D.Lo2,A2D.Hi2);
      for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
    end;
  2:begin
      A1D := TBArr1D.Create(A2D.Lo1,A2D.Hi1);
      for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
    end;
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
  result := A1D;
end;

{ i-th vector of idx-th dimension of AR }
function GetArr1D(const A2D: IIArr2D; i, idx: TInt): IIArr1D;
var
  k: TInt;
  A1D: IIArr1D;
begin
  case idx of
  1:begin
      A1D := TIArr1D.Create(A2D.Lo2,A2D.Hi2);
      for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
    end;
  2:begin
      A1D := TIArr1D.Create(A2D.Lo1,A2D.Hi1);
      for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
    end;
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
  result := A1D;
end;

{ i-th vector of idx-th dimension of AR }
function GetArr1D(const A2D: IFArr2D; i, idx: TInt): IFArr1D;
var
  k: TInt;
  A1D: IFArr1D;
begin
  case idx of
  1:begin
      A1D := TFArr1D.Create(A2D.Lo2,A2D.Hi2);
      for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
    end;
  2:begin
      A1D := TFArr1D.Create(A2D.Lo1,A2D.Hi1);
      for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
    end;
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
  result := A1D;
end;

{ Al := (i-th matrix of idx-th dimension of A3D) }
procedure CopyTo2D(const A2D: IIArr2D; const A3D: IIArr3D; i, idx: TInt);
var
  k, l: TInt;
begin
  case idx of
  1:for k := A3D.Lo2 to A3D.Hi2 do
      for l := A3D.Lo3 to A3D.Hi3 do A2D[k,l] := A3D[i,k,l];
  2:for k := A3D.Lo1 to A3D.Hi1 do
      for l := A3D.Lo3 to A3D.Hi3 do A2D[k,l] := A3D[k,i,l];
  3:for k := A3D.Lo1 to A3D.Hi1 do
      for l := A3D.Lo2 to A3D.Hi2 do A2D[k,l] := A3D[k,l,i];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Al := (i-th vector of idx-th dimension of A1D }
procedure CopyFrom1D(const A2D: IIArr2D; const A1D: IIArr1D; i, idx: TInt); overload;
var
  k: TInt;
begin
  case idx of
  1:for k := A2D.Lo2 to A2D.Hi2 do A2D[i,k] := A1D[k];
  2:for k := A2D.Lo1 to A2D.Hi1 do A2D[k,i] := A1D[k];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Al := (i-th vector of idx-th dimension of A2D }
procedure CopyFrom1D(const A2D: IFArr2D; const A1D: IFArr1D; i, idx: TInt); overload;
var
  k: TInt;
begin
  case idx of
  1:for k := A2D.Lo2 to A2D.Hi2 do A2D[i,k] := A1D[k];
  2:for k := A2D.Lo1 to A2D.Hi1 do A2D[k,i] := A1D[k];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Al := (i-th matrix of idx-th dimension of A2D }
procedure CopyTo2D(const A2D: IFArr2D; const A3D: IFArr3D; i, idx: TInt);
var
  k, l: TInt;
begin
  case idx of
  1:for k := A3D.Lo2 to A3D.Hi2 do
      for l := A3D.Lo3 to A3D.Hi3 do A2D[k,l] := A3D[i,k,l];
  2:for k := A3D.Lo1 to A3D.Hi1 do
      for l := A3D.Lo3 to A3D.Hi3 do A2D[k,l] := A3D[k,i,l];
  3:for k := A3D.Lo1 to A3D.Hi1 do
      for l := A3D.Lo2 to A3D.Hi2 do A2D[k,l] := A3D[k,l,i];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ AlD := (i-th vector of idx-th dimension of A2D) }
procedure CopyTo1D(const A1D: IBArr1D; const A2D: IBArr2D; i, idx: TInt);
var
  k: TInt;
begin
  case idx of
  1:for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
  2:for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Al := (i-th vector of idx-th dimension of A2D }
procedure CopyTo1D(const A1D: IIArr1D; const A2D: IIArr2D; i, idx: TInt);
var
  k: TInt;
begin
  case idx of
  1:for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
  2:for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ Al := (i-th vector of idx-th dimension of A2D }
procedure CopyTo1D(const A1D: IFArr1D; const A2D: IFArr2D; i, idx: TInt);
var
  k: TInt;
begin
  case idx of
  1:for k := A2D.Lo2 to A2D.Hi2 do A1D[k] := A2D[i,k];
  2:for k := A2D.Lo1 to A2D.Hi1 do A1D[k] := A2D[k,i];
  else
    Raise ERangeError.CreateFmt(RS_DimError,[idx, 2]);
  end;
end;

{ A := CxA }
procedure CxArr(C: TInt; const A: IIArr1D);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := C*A[i1];
end;

{ A := CxA }
procedure CxArr(C: TInt; const A: IIArr2D);
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,i2] := C*A[i1,i2];
end;

{ A := CxA }
procedure CxArr(C: TInt; const A: IIArr3D);
var
  i1, i2, i3: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      for i3 := A.Lo3 to A.Hi3 do
        A[i1,i2,i3] := C*A[i1,i2,i3];
end;

{ A := CxA }
procedure CxArr(C: TFloat; const A: IFArr1D);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := C*A[i1];
end;

{ A := CxA }
procedure CxArr(C: TFloat; const A: IFArr2D);
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,i2] := C*A[i1,i2];
end;

{ A := CxA }
procedure CxArr(C: TFloat; const A: IFArr3D);
var
  i1, i2, i3: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      for i3 := A.Lo3 to A.Hi3 do
        A[i1,i2,i3] := C*A[i1,i2,i3];
end;

{ A := CxA }
procedure CxArr(C: Complex; const A: ICArr1D);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := cmplx(C.Re*A[i1].Re-C.Im*A[i1].Im,
                   C.Im*A[i1].Re+C.Re*A[i1].Im);
end;

{ A := CxA }
procedure CxArr(C: Complex; const A: ICArr2D);
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,I2] := Cmplx(C.Re*A[i1,I2].Re-C.Im*A[i1,I2].Im,
                        C.Im*A[i1,I2].Re + C.Re*A[i1,I2].Im);
end;

{ A[ ] := val }
procedure Fill(const A: IBArr1D; val: boolean);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IBArr2D; val: boolean);
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,i2] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IIArr1D; val: TInt);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IIArr2D; val: TInt);
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,i2] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IIArr3D; val: TInt);
var
  i1, i2, i3: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      for i3 := A.Lo3 to A.Hi3 do
        A[i1,i2,i3] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IFArr1D; val: TFloat);
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IFArr2D; val: TFloat);
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,i2] := val;
end;

{ A[ ] := val }
procedure Fill(const A: IFArr3D; val: TFloat);
var
  i1, i2, i3: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      for i3 := A.Lo3 to A.Hi3 do
        A[i1,i2,i3] := val;
end;

{ A[ ] := val }
procedure Fill(const A: ICArr1D; val: Complex); overload;
var
  i1: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    A[i1] := val;
end;

{ A[ ] := val }
procedure Fill(const A: ICArr2D; val: Complex); overload;
var
  i1, i2: TInt;
begin
  for i1 := A.Lo1 to A.Hi1 do
    for i2 := A.Lo2 to A.Hi2 do
      A[i1,i2] := val;
end;

end.
