{*
 Signal and Data objects
 Nikolai V. Shokhirev, 2002-2007
 @author(Nikolai V. Shokhirev <nikolai@shokhirev.com> <nikolai@u.arizona.edu>)
 @version 2002.02.02 - created
 @version 2005.09.24 - added FArrData2D
 @version 2007.08.06: Panel data moved/merged into uPanelData
 }
unit uDynData;

interface

uses
  SysUtils, uMatTypes, uDynArrays, Contnrs, classes;

type

{ Interface for 1D Data Limits }
  IDataLim1D = interface
  ['{5C9ADA14-DC97-466E-987F-5FCFA766A499}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{ Interface for 2D Data Limits }
  IDataLim2D = interface
  ['{4715C380-36D8-403D-934F-C563D6FF796F}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

{ Interface for 1D integer Signal }
  IISignal1D = interface(IIArr1D)
  ['{E2EE0268-82AA-4DD8-B9A6-B5C6BD0CC258}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{ Interface for 2D integer Signal }
  IISignal2D = interface(IIArr2D)
  ['{D811D555-57DA-49EE-B195-FA3A696C33A0}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

{ Interface for 1D float Signal }
  IFSignal1D = interface(IFArr1D)
  ['{D3C20007-A7DC-4E60-853C-AE3128FE1063}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{ Interface for 2D float Signal }
  IFSignal2D = interface(IFArr2D)
  ['{D430CEA1-3FB0-4DA6-8FD9-AE25EE125234}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

{ Interface for 2D float Signal }
  IFSignal3D = interface(IFArr3D)
  ['{F18F1CFB-616A-4517-A330-41D2A8F15A8C}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
    function GetStep3: TFloat;
    procedure SetStep3(const Value: TFloat);
    function GetStart3: TFloat;
    procedure SetStart3(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
    property Step3: TFloat read GetStep2 write SetStep2;
    property Start3: TFloat read GetStart2 write SetStart2;
  end;

{<pre> Interface for Dynamic Complex 1D Signal:<br>
  CVector = ReVector + i*ImVector </pre>}
  ICSignal1D = interface(ICArr1D)
  ['{F3128A2D-088C-4686-B5E5-6FF7C65EB584}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{<pre> Interface for Dynamic Complex 2D Signal:<br>
  CMatrix = ReMatrix + i*ImMatrix </pre>}
  ICSignal2D = interface(ICArr2D)
  ['{4FC57A45-5455-438A-BEAA-BCC2E0AF1C5F}']
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

{<pre> Interface for Float 1D Data  </pre>}
  IFData1D = interface(ILim1D)
  ['{74979ECF-2911-4BDF-BA07-354DAA564F4C}']
    procedure SetX1(const Value: IFArr1D);
    procedure SetX2(const Value: IFArr1D);
    function  GetX1: IFArr1D;
    function  GetX2: IFArr1D;
    property x1: IFArr1D read GetX1 write SetX1;
    property x2: IFArr1D read GetX2 write SetX2;
  end;

{<pre> Interface for Float 2D Data  </pre>}
  IFData2D = interface(ILim2D)
  ['{D4F723C0-4755-48B2-B40C-C1D7741E746F}']
    procedure SetX1(const Value: IFArr1D);
    procedure SetX2(const Value: IFArr1D);
    procedure SetZ(const Value: IFArr2D);
    function  GetX1: IFArr1D;
    function  GetX2: IFArr1D;
    function  GetZ: IFArr2D;
    property x1: IFArr1D read GetX1 write SetX1;
    property x2: IFArr1D read GetX2 write SetX2;
    property z: IFArr2D read GetZ write SetZ;
  end;

{<pre> Interface for Float 2D Data  </pre>}
{   inherited Create(aLo1, aHi1, aLo2, aHi2);
  fColInfo := TSArr1D.Create(aLo2,aHi2);
  fColNames := TSArr1D.Create(aLo2,aHi2);
  fRowNames := TSArr1D.Create(aLo1,aHi1);

}
  IFArrData2D = interface(IFArr2D)
  ['{DA803439-5ED3-4BB6-8A3A-336E445DB20A}']
    function GetColInfo: ISArr1D;
    function GetColNames: ISArr1D;
    function GetRowNames: ISArr1D;
    procedure SetColInfo(const Value: ISArr1D);
    procedure SetColNames(const Value: ISArr1D);
    procedure SetRowNames(const Value: ISArr1D);
    property ColInfo: ISArr1D read GetColInfo write SetColInfo;
    property ColNames: ISArr1D read GetColNames write SetColNames;
    property RowNames: ISArr1D read GetRowNames write SetRowNames;
  end;

{ Object for 1D integer Signal }

  TISignal1D = class(TIArr1D, IISignal1D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{ Object for 2D integer Signal }
  TISignal2D = class(TIArr2D, IISignal2D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
    fStep2: TFloat;
    fStart2: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

{ Object for 1D float Signal }
  TFSignal1D = class(TFArr1D, IFSignal1D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{ Object for 2D float Signal }
  TFSignal2D = class(TFArr2D, IFSignal2D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
    fStep2: TFloat;
    fStart2: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

{ Object for 3D float Signal }
  TFSignal3D = class(TFArr3D, IFSignal3D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
    fStep2: TFloat;
    fStart2: TFloat;
    fStep3: TFloat;
    fStart3: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
    function GetStep3: TFloat;
    procedure SetStep3(const Value: TFloat);
    function GetStart3: TFloat;
    procedure SetStart3(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
    property Step3: TFloat read GetStep2 write SetStep2;
    property Start3: TFloat read GetStart2 write SetStart2;
  end;

    { Object for 1D complex Signal }
  TCSignal1D = class(TCArr1D, ICSignal1D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
  end;

{ Object for 2D complex Signal }
  TCSignal2D = class(TCArr2D, ICSignal2D)
  private
    fStep1: TFloat;
    fStart1: TFloat;
    fStep2: TFloat;
    fStart2: TFloat;
  protected
    function GetStep1: TFloat;
    procedure SetStep1(const Value: TFloat);
    function GetStart1: TFloat;
    procedure SetStart1(const Value: TFloat);
    function GetStep2: TFloat;
    procedure SetStep2(const Value: TFloat);
    function GetStart2: TFloat;
    procedure SetStart2(const Value: TFloat);
  public
    property Step1: TFloat read GetStep1 write SetStep1;
    property Start1: TFloat read GetStart1 write SetStart1;
    property Step2: TFloat read GetStep2 write SetStep2;
    property Start2: TFloat read GetStart2 write SetStart2;
  end;

  TFData1D = class(TComment, IFData1D)
  private
    fx1: IFArr1D;
    fx2: IFArr1D;
  protected
    function GetHi1: TInt;
    function GetLo1: TInt;
    function GetDim1: TInt;
    procedure SetLo1(const Value: TInt);
    procedure SetX1(const Value: IFArr1D);
    procedure SetX2(const Value: IFArr1D);
    function GetX1: IFArr1D;
    function GetX2: IFArr1D;
  public
//    constructor Create(A: IFData1D; CopyData: boolean = false); overload;
    constructor Create(aHi1: TInt); overload;
    constructor Create(aLo1, aHi1: TInt); overload;
    destructor Destroy; override;
    procedure Swap(i, j: TInt);
//    procedure dx2dx1(i, j: TInt);
    property Hi1: TInt read GetHi1;
    property Lo1: TInt read GetLo1 write SetLo1;
    property Dim1: TInt read GetDim1;
    property x1: IFArr1D read GetX1 write SetX1;
    property x2: IFArr1D read GetX2 write SetX2;
  end;

{<pre> Object for Float 2D Data  </pre>}
  TFData2D = class(TComment, IFData2D)
  private
    fx1: IFArr1D;
    fx2: IFArr1D;
    fz: IFArr2D;
  protected
    function GetDim1: TInt;
    function GetDim2: TInt;
    function GetHi1: TInt;
    function GetHi2: TInt;
    function GetLo1: TInt;
    function GetLo2: TInt;
    procedure SetLo1(const Value: TInt);
    procedure SetLo2(const Value: TInt);
    procedure SetX1(const Value: IFArr1D);
    procedure SetX2(const Value: IFArr1D);
    procedure SetZ(const Value: IFArr2D);
    function  GetX1: IFArr1D;
    function  GetX2: IFArr1D;
    function  GetZ: IFArr2D;
  public
    constructor Create(aDIm1, aDim2: TInt); overload;
    constructor Create(aLo1, aHi1, aLo2, aHi2: TInt); overload;
    property Hi1: TInt read GetHi1;
    property Lo1: TInt read GetLo1 write SetLo1;
    property Dim1: TInt read GetDim1;
    property Hi2: TInt read GetHi2;
    property Lo2: TInt read GetLo2 write SetLo2;
    property Dim2: TInt read GetDim2;
    property x1: IFArr1D read GetX1 write SetX1;
    property x2: IFArr1D read GetX2 write SetX2;
    property z: IFArr2D read GetZ write SetZ;
  end;

{<pre> Object for Float 2D Data  </pre>}
  TFArrData2D = class(TFArr2D, IFArrData2D)
  private
    fColInfo: ISArr1D;
    fColNames: ISArr1D;
    fRowNames: ISArr1D;
  protected
    function GetColInfo: ISArr1D;
    function GetColNames: ISArr1D;
    function GetRowNames: ISArr1D;
    procedure SetColInfo(const Value: ISArr1D);
    procedure SetColNames(const Value: ISArr1D);
    procedure SetRowNames(const Value: ISArr1D);
  public
    constructor Create(A: IFArr2D; CopyData: boolean = false); overload;
    constructor Create(aHi1, aHi2: TInt); overload;
    constructor Create(aLo1, aHi1, aLo2, aHi2: TInt); overload;
    destructor Destroy; override;
    property ColInfo: ISArr1D read GetColInfo write SetColInfo;
    property ColNames: ISArr1D read GetColNames write SetColNames;
    property RowNames: ISArr1D read GetRowNames write SetRowNames;
  end;

implementation

ResourceString
  RS_StrIndexError = 'Incorrect index = %s';
  RS_ArrIndexError = 'Index = %d : out of range [1, %d]';
  RS_StrIndexExistsError = 'Index "%s" already exists';
  RS_ColNumberError = 'Index %d: Nomber of Col %d <> ColCount %d';

{ TISignal1D }

function TISignal1D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TISignal1D.GetStep1: TFloat;
begin
  result := fStep1;
end;

procedure TISignal1D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TISignal1D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

{ TISignal2D }

function TISignal2D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TISignal2D.GetStart2: TFloat;
begin
  result := fStart2;
end;

function TISignal2D.GetStep1: TFloat;
begin
  result := fStep1;
end;

function TISignal2D.GetStep2: TFloat;
begin
  result := fStep2;
end;

procedure TISignal2D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TISignal2D.SetStart2(const Value: TFloat);
begin
  fStart2 := Value;
end;

procedure TISignal2D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

procedure TISignal2D.SetStep2(const Value: TFloat);
begin
  fStep2 := Value;
end;

{ TFSignal1D }

function TFSignal1D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TFSignal1D.GetStep1: TFloat;
begin
  result := fStep1;
end;

procedure TFSignal1D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TFSignal1D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

{ TFSignal2D }

function TFSignal2D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TFSignal2D.GetStart2: TFloat;
begin
  result := fStart2;
end;

function TFSignal2D.GetStep1: TFloat;
begin
  result := fStep1;
end;

function TFSignal2D.GetStep2: TFloat;
begin
  result := fStep2;
end;

procedure TFSignal2D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TFSignal2D.SetStart2(const Value: TFloat);
begin
  fStart2 := Value;
end;

procedure TFSignal2D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

procedure TFSignal2D.SetStep2(const Value: TFloat);
begin
  fStep2 := Value;
end;

{ TCSignal1D }

function TCSignal1D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TCSignal1D.GetStep1: TFloat;
begin
  result := fStep1;
end;

procedure TCSignal1D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TCSignal1D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

{ TCSignal2D }

function TCSignal2D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TCSignal2D.GetStart2: TFloat;
begin
  result := fStart2;
end;

function TCSignal2D.GetStep1: TFloat;
begin
  result := fStep1;
end;

function TCSignal2D.GetStep2: TFloat;
begin
  result := fStep2;
end;

procedure TCSignal2D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TCSignal2D.SetStart2(const Value: TFloat);
begin
  fStart2 := Value;
end;

procedure TCSignal2D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

procedure TCSignal2D.SetStep2(const Value: TFloat);
begin
  fStep2 := Value;
end;

{ TFSignal3D }

function TFSignal3D.GetStart1: TFloat;
begin
  result := fStart1;
end;

function TFSignal3D.GetStart2: TFloat;
begin
  result := fStart2;
end;

function TFSignal3D.GetStart3: TFloat;
begin
  result := fStart3;
end;

function TFSignal3D.GetStep1: TFloat;
begin
  result := fStep1;
end;

function TFSignal3D.GetStep2: TFloat;
begin
  result := fStep2;
end;

function TFSignal3D.GetStep3: TFloat;
begin
  result := fStep3;
end;

procedure TFSignal3D.SetStart1(const Value: TFloat);
begin
  fStart1 := Value;
end;

procedure TFSignal3D.SetStart2(const Value: TFloat);
begin
  fStart2 := Value;
end;

procedure TFSignal3D.SetStart3(const Value: TFloat);
begin
  fStart3 := Value;
end;

procedure TFSignal3D.SetStep1(const Value: TFloat);
begin
  fStep1 := Value;
end;

procedure TFSignal3D.SetStep2(const Value: TFloat);
begin
  fStep2 := Value;
end;

procedure TFSignal3D.SetStep3(const Value: TFloat);
begin
  fStep3 := Value;
end;

{ TFData1D }

constructor TFData1D.Create(aLo1, aHi1: TInt);
begin
  inherited Create;
  fx1 := TFArr1D.Create(aLo1, aHi1);
  fx2 := TFArr1D.Create(aLo1, aHi1);
end;

constructor TFData1D.Create(aHi1: TInt);
begin
  Create(1, aHi1);
end;

destructor TFData1D.Destroy;
begin
  fx1 := nil;
  fx2 := nil;
  inherited;
end;
{
constructor TFData1D.Create(A: IFData1D; CopyData: boolean);
begin
  Create(A.Lo1, A.Hi1);
  if CopyData then
  begin
    x1.Copy(A.x1);
    x2.Copy(A.x2);
  end;
end;

procedure TFData1D.Copy(const A: ICArr1D);
begin
  fx1.Copy(A.Re);
  fx2.Copy(A.Im);
end;
}
function TFData1D.GetX2: IFArr1D;
begin
  result := fx2;
end;

function TFData1D.GetX1: IFArr1D;
begin
  result := fx1;
end;

procedure TFData1D.SetX2(const Value: IFArr1D);
begin
  fx2 := Value;
end;

procedure TFData1D.SetX1(const Value: IFArr1D);
begin
  fx1 := Value;
end;

procedure TFData1D.Swap(i, j: TInt);
begin
  fx1.Swap(i, j);
  fx2.Swap(i, j);
end;

function TFData1D.GetDim1: TInt;
begin
  result := fx1.Dim1;
end;

function TFData1D.GetHi1: TInt;
begin
  result := fx1.Hi1;
end;

function TFData1D.GetLo1: TInt;
begin
  result := fx1.Lo1;
end;

procedure TFData1D.SetLo1(const Value: TInt);
begin
  fx1.Lo1 := Value;
  fx2.Lo1 := Value;
end;

{ TFData2D }

constructor TFData2D.Create(aLo1, aHi1, aLo2, aHi2: TInt);
begin
  inherited Create;
  fx1 := TFArr1D.Create(aLo1, aHi1);
  fx2 := TFArr1D.Create(aLo2, aHi2);
  fz := TFArr2D.Create(aLo1, aHi1, aLo2, aHi2);
end;

constructor TFData2D.Create(aDIm1, aDim2: TInt);
begin
  Create(1, aDim1, 1, aDim2);
end;

function TFData2D.GetDim1: TInt;
begin
  result := fz.Dim1;
end;

function TFData2D.GetDim2: TInt;
begin
  result := fz.Dim2;
end;

function TFData2D.GetHi1: TInt;
begin
  result := fz.Hi1;
end;

function TFData2D.GetHi2: TInt;
begin
  result := fz.Hi2;
end;

function TFData2D.GetLo1: TInt;
begin
  result := fz.Lo1;
end;

function TFData2D.GetLo2: TInt;
begin
  result := fz.Lo2;
end;

function TFData2D.GetX1: IFArr1D;
begin
  result := fx1;
end;

function TFData2D.GetX2: IFArr1D;
begin
  result := fx2;
end;

function TFData2D.GetZ: IFArr2D;
begin
  result := fz;
end;

procedure TFData2D.SetLo1(const Value: TInt);
begin
  fx1.Lo1 := Value;
  fz.Lo1 := Value;
end;

procedure TFData2D.SetLo2(const Value: TInt);
begin
  fx2.Lo1 := Value;
  fz.Lo2 := Value;
end;

procedure TFData2D.SetX1(const Value: IFArr1D);
begin
  fx1 := Value;
end;

procedure TFData2D.SetX2(const Value: IFArr1D);
begin
  fx2 := Value;
end;

procedure TFData2D.SetZ(const Value: IFArr2D);
begin
  fz := Value;
end;

{ TFArrData2D }

constructor TFArrData2D.Create(A: IFArr2D; CopyData: boolean);
begin
  inherited Create(A, CopyData);
  fColInfo := TSArr1D.Create(A.Lo2,A.Hi2);
  fColNames := TSArr1D.Create(A.Lo2,A.Hi2);
  fRowNames := TSArr1D.Create(A.Lo1,A.Hi1);
end;

constructor TFArrData2D.Create(aHi1, aHi2: TInt);
begin
  inherited Create(aHi1, aHi2);
  fColInfo := TSArr1D.Create(aHi2);
  fColNames := TSArr1D.Create(aHi2);
  fRowNames := TSArr1D.Create(aHi1);
end;

constructor TFArrData2D.Create(aLo1, aHi1, aLo2, aHi2: TInt);
begin
  inherited Create(aLo1, aHi1, aLo2, aHi2);
  fColInfo := TSArr1D.Create(aLo2,aHi2);
  fColNames := TSArr1D.Create(aLo2,aHi2);
  fRowNames := TSArr1D.Create(aLo1,aHi1);
end;

destructor TFArrData2D.Destroy;
begin

  inherited;
end;

function TFArrData2D.GetColInfo: ISArr1D;
begin
  result := fColInfo;
end;

function TFArrData2D.GetColNames: ISArr1D;
begin
  result := fColNames;
end;

function TFArrData2D.GetRowNames: ISArr1D;
begin
  result := fRowNames;
end;

procedure TFArrData2D.SetColInfo(const Value: ISArr1D);
begin
  fColInfo := Value;
end;

procedure TFArrData2D.SetColNames(const Value: ISArr1D);
begin
  fColNames := Value;
end;

procedure TFArrData2D.SetRowNames(const Value: ISArr1D);
begin
  fRowNames := Value;
end;

end.
