{@abstract(I/O routines for Signal and data)
@author(Nikolai Shokhirev <nikolai@shokhirev.com>)
@created(2005.04.04)
@lastmod(2005.05.05)
Nikolai V. Shokhirev, 2005-2005
}
unit  uDynDataIO;

interface

uses
  SysUtils, uMatTypes, uDynArrays, uDynArrUtils, uDynData, uDynArrIO;

const
  sIOVersion = '1.2';
  sISignal1D = 'ISignal1D';
  sISignal2D = 'ISignal2D';
  sFSignal1D = 'FSignal1D';
  sFSignal2D = 'FSignal2D';
  sFSignal3D = 'FSignal3D';
  sCSignal1D = 'CSignal1D';
  sCSignal2D = 'CSignal2D';
  sFData1D = 'FData1D';
  sFData2D = 'FData2D';
  sFArrData2D = 'FArrData2D';

{ Create from File }
function ISignal1DCreate(FileName: string): IISignal1D;
{ Save to File }
procedure ISignal1DSave(const A: IISignal1D; FileName: string);
{ Load data from File }
procedure ISignal1DRead(const A: IISignal1D; FileName: string);

{ Create from File }
function ISignal2DCreate(FileName: string): IISignal2D;
{ Save to File }
procedure ISignal2DSave(const A: IISignal2D; FileName: string);
{ Load data from File }
procedure ISignal2DRead(const A: IISignal2D; FileName: string);

{ Create from File }
function FSignal1DCreate(FileName: string): IFSignal1D;
{ Save to File }
procedure FSignal1DSave(const A: IFSignal1D; FileName: string);
{ Load data from File }
procedure FSignal1DRead(const A: IFSignal1D; FileName: string);

{ Create from File }
function FSignal2DCreate(FileName: string): IFSignal2D;
{ Save to File }
procedure FSignal2DSave(const A: IFSignal2D; FileName: string;
                        AsTable: boolean = false);
{ Load data from File }
procedure FSignal2DRead(const A: IFSignal2D; FileName: string);

{ Create from File }
function FSignal3DCreate(FileName: string): IFSignal3D;
{ Save to File }
procedure FSignal3DSave(const A: IFSignal3D; FileName: string);
{ Load data from File }
procedure FSignal3DRead(const A: IFSignal3D; FileName: string);

{ Create from File }
function CSignal1DCreate(FileName: string): ICSignal1D;
{ Save to File }
procedure CSignal1DSave(const A: ICSignal1D; FileName: string);
{ Load data from File }
procedure CSignal1DRead(const A: ICSignal1D; FileName: string);

{ Create from File }
function CSignal2DCreate(FileName: string): ICSignal2D;
{ Save to File }
//procedure CSignal2DSave(const A: ICSignal2D; FileName: string);
procedure CSignal2DSave(const A: ICSignal2D; FileName: string;
                        AsTable: boolean = false);
{ Load data from File }
procedure CSignal2DRead(const A: ICSignal2D; FileName: string);

{ Create from File }
function FArrData2DCreate(FileName: string): IFArrData2D;
{ Save to File }
procedure FArrData2DSave(const A: IFArrData2D; FileName: string);
{ Load data from File }
procedure FArrData2DRead(const A: IFArrData2D; FileName: string);

implementation

uses
  Math, classes, Dialogs;

{ Dynamic array I/O routines }

function ISignal1DCreate(FileName: string): IISignal1D;
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: IISignal1D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sISignal1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TISignal1D.Create(L1, H1);
    t.Comment := aComment;
    ReadI1DValues(t, TF, s);
    ReadLn(TF, step, start);
    t.Step1 := step;
    t.Start1 := start;
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure ISignal1DRead(const A: IISignal1D; FileName: string);
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: ILim1d;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sISignal1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TLim1d.Create(L1, H1);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadI1DValues(A, TF, s);
    ReadLn(TF, step, start);
    A.Step1 := step;
    A.Start1 := start;
  finally
    CloseFile(TF);
  end;
end;

procedure ISignal1DSave(const A: IISignal1D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sISignal1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,  ' ',A.Hi1);
    SaveI1DValues(A, TF, s);
    WriteLn(TF, A.Step1, ' ', A.Start1);
  finally
    CloseFile(TF);
  end;
end;

function ISignal2DCreate(FileName: string): IISignal2D;
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: IISignal2D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sISignal2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TISignal2D.Create(L1, H1, L2, H2);
    t.Comment := aComment;
    ReadI2DValues(t, TF, s);
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure ISignal2DRead(const A: IISignal2D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: ILim2D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sISignal2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TLim2D.Create(L1, H1, L2, H2);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadI2DValues(A, TF, s);
  finally
    CloseFile(TF);
  end;
end;

procedure ISignal2DSave(const A: IISignal2D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sISignal2D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2);
    SaveI2DValues(A, TF, s);
    WriteLn(TF, A.Step1, ' ', A.Start1);
    WriteLn(TF, A.Step2, ' ', A.Start2);
  finally
    CloseFile(TF);
  end;
end;

function FSignal1DCreate(FileName: string): IFSignal1D;
var
  TF: TextFile;
  L1, H1, i1: TInt;
  aVersion, aComment, s: string;
  x: TFloat;
  t: IFSignal1D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFSignal1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TFSignal1D.Create(L1, H1);
    t.Comment := aComment;
    ReadF1DValues(t, TF, s);
    ReadLn(TF, step, start);
    t.Step1 := step;
    t.Start1 := start;
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure FSignal1DRead(const A: IFSignal1D; FileName: string);
var
  TF: TextFile;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  t: ILim1d;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFSignal1d then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TLim1d.Create(L1, H1);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadF1DValues(A, TF, s);
    ReadLn(TF, step, start);
    A.Step1 := step;
    A.Start1 := start;
  finally
    CloseFile(TF);
  end;
end;

procedure FSignal1DSave(const A: IFSignal1D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFSignal1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,  ' ',A.Hi1);
    SaveF1DValues(A, TF, s);
    WriteLn(TF, A.Step1, ' ', A.Start1);
  finally
    CloseFile(TF);
  end;
end;

function FSignal2DCreate(FileName: string): IFSignal2D;
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: IFSignal2D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFSignal2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TFSignal2D.Create(L1, H1, L2, H2);
    t.Comment := aComment;
    ReadF2DValues(t, TF, s);
    ReadLn(TF, step, start);
    t.Step1 := step;
    t.Start1 := start;
    ReadLn(TF, step, start);
    t.Step2 := step;
    t.Start2 := start;
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure FSignal2DRead(const A: IFSignal2D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: ILim2D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFSignal2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TLim2D.Create(L1, H1, L2, H2);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadF2DValues(A, TF, s);
    ReadLn(TF, step, start);
    A.Step1 := step;
    A.Start1 := start;
    ReadLn(TF, step, start);
    A.Step2 := step;
    A.Start2 := start;
  finally
    CloseFile(TF);
  end;
end;

procedure FSignal2DSave(const A: IFSignal2D; FileName: string;
                        AsTable: boolean = false);
//const
//  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFSignal2D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2);
    if AsTable then
      SaveF2DTable(A, TF, 'Table')
    else
      SaveF2DValues(A, TF, 'Values');
    WriteLn(TF, A.Step1, ' ', A.Start1);
    WriteLn(TF, A.Step2, ' ', A.Start2);
  finally
    CloseFile(TF);
  end;
end;

function FSignal3DCreate(FileName: string): IFSignal3D;
var
  TF: TextFile;
  L1, H1, L2, H2, L3, H3: TInt;
  aVersion, aComment, s: string;
  t: IFSignal3D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFSignal3D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2, L3, H3);
    t := TFSignal3D.Create(L1, H1, L2, H2, L3, H3);
    t.Comment := aComment;
    REadF3DValues(t, TF, s);
    ReadLn(TF, step, start);
    t.Step1 := step;
    t.Start1 := start;
    ReadLn(TF, step, start);
    t.Step2 := step;
    t.Start2 := start;
    ReadLn(TF, step, start);
    t.Step3 := step;
    t.Start3 := start;
  finally
    CloseFile(TF);
  end;
  result := t;
end;

procedure FSignal3DRead(const A: IFSignal3D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2, L3, H3: TInt;
  aVersion, aComment, s: string;
  t: ILim3D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFSignal3D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2, L3, H3);
    t := TLim3D.Create(L1, H1, L2, H2, L3, H3);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadF3DValues(A, TF, s);
    ReadLn(TF, step, start);
    A.Step1 := step;
    A.Start1 := start;
    ReadLn(TF, step, start);
    A.Step2 := step;
    A.Start2 := start;
    ReadLn(TF, step, start);
    A.Step3 := step;
    A.Start3 := start;
  finally
    CloseFile(TF);
  end;
end;

procedure FSignal3DSave(const A: IFSignal3D; FileName: string);
const
  s = 'Values';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFSignal3D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2,' ',A.Lo3,' ',A.Hi3);
    SaveF3DValues(A, TF, s);
    WriteLn(TF, A.Step1, ' ', A.Start1);
    WriteLn(TF, A.Step2, ' ', A.Start2);
    WriteLn(TF, A.Step3, ' ', A.Start3);
  finally
    CloseFile(TF);
  end;
end;

{ Create from File }
function CSignal1DCreate(FileName: string): ICSignal1D;
var
  TF: TextFile;
  t: ICSignal1D;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sCSignal1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TCSignal1D.Create(L1, H1);
    t.Comment := aComment;
    REadF1DValues(t.Re, TF, s);
    REadF1DValues(t.Im, TF, s);
    ReadLn(TF, step, start);
    t.Step1 := step;
    t.Start1 := start;
  finally
    CloseFile(TF);
  end;
  result := t;
end;

{ Save to File }
procedure CSignal1DSave(const A: ICSignal1D; FileName: string);
const
  sr = 'ReValues';
  si = 'ImValues';
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sCSignal1D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1);
    SaveF1DValues(A.Re, TF, sr);
    SaveF1DValues(A.Im, TF, si);
    WriteLn(TF, A.Step1, ' ', A.Start1);
  finally
    CloseFile(TF);
  end;
end;

{ Load data from File }
procedure CSignal1DRead(const A: ICSignal1D; FileName: string);
var
  TF: TextFile;
  t: ILim1D;
  L1, H1: TInt;
  aVersion, aComment, s: string;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sCSignal1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1);
    t := TLim1D.Create(L1, H1);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    REadF1DValues(A.Re, TF, s);
    REadF1DValues(A.Im, TF, s);
    ReadLn(TF, step, start);
    A.Step1 := step;
    A.Start1 := start;
  finally
    CloseFile(TF);
  end;
end;

{ Create from File }
function CSignal2DCreate(FileName: string): ICSignal2D;
var
  TF: TextFile;
  t: ICSignal2D;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sCSignal1D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TCSignal2D.Create(L1, H1, L2, H2);
    t.Comment := aComment;
    REadF2DValues(t.Re, TF, s);
    REadF2DValues(t.Im, TF, s);
    ReadLn(TF, step, start);
    t.Step1 := step;
    t.Start1 := start;
    ReadLn(TF, step, start);
    t.Step2 := step;
    t.Start2 := start;
  finally
    CloseFile(TF);
  end;
  result := t;
end;

{ Save to File }
procedure CSignal2DSave(const A: ICSignal2D; FileName: string;
                        AsTable: boolean = false);
var
  TF: TextFile;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sCSignal2D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2);
    if AsTable then
      SaveF2DTable(A.Re, TF, 'ReTable')
    else
      SaveF2DValues(A.Re, TF, 'ReValues');

    if AsTable then
      SaveF2DTable(A.Im, TF, 'ImTable')
    else
      SaveF2DValues(A.Im, TF, 'ImValues');
    WriteLn(TF, A.Step1, ' ', A.Start1);
    WriteLn(TF, A.Step2, ' ', A.Start2);
  finally
    CloseFile(TF);
  end;
end;

{ Load data from File }
procedure CSignal2DRead(const A: ICSignal2D; FileName: string);
var
  TF: TextFile;
  t: ILim2D;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sCSignal2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TLim2D.Create(L1, H1, L2, H2);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    REadF2DValues(A.Re, TF, s);
    REadF2DValues(A.Im, TF, s);
    ReadLn(TF, step, start);
    A.Step1 := step;
    A.Start1 := start;
    ReadLn(TF, step, start);
    A.Step2 := step;
    A.Start2 := start;
  finally
    CloseFile(TF);
  end;
end;

{ Create from File }
function FArrData2DCreate(FileName: string): IFArrData2D;
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  t: IFArrData2D;
  step, start: TFloat;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArrData2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TFArrData2D.Create(L1, H1, L2, H2);
    t.Comment := aComment;
    ReadLn(TF,s);
    t.ColNames.CommaText := s;

    ReadF2DTable(t, TF, s);

    ReadLn(TF,s);
    t.RowNames.CommaText := s;
    ReadLn(TF,s);
    t.ColInfo.CommaText := s;

  finally
    CloseFile(TF);
  end;
  result := t;
end;

{ Save to File }
procedure FArrData2DSave(const A: IFArrData2D; FileName: string);
//const
//  s = 'Values';
var
  TF: TextFile;
  ss: string;
begin
  AssignFile(TF,FileName);
  try
    rewrite(TF);
    WriteLn(TF, sIOVersion);
    WriteLn(TF, sFArrData2D);
    WriteLn(TF, A.Comment);
    WriteLn(TF, A.Lo1,' ',A.Hi1,' ',A.Lo2,' ',A.Hi2);
    ss := A.ColNames.CommaText;
    WriteLn(TF, ss);
    SaveF2DTable(A, TF, 'Table');
//    SaveF2DValues(A, TF, s);
    ss := A.RowNames.CommaText;
    WriteLn(TF, ss);
    ss := A.ColInfo.CommaText;
    WriteLn(TF, ss);
  finally
    CloseFile(TF);
  end;
end;

{ Load data from File }
procedure FArrData2DRead(const A: IFArrData2D; FileName: string);
var
  TF: TextFile;
  L1, H1, L2, H2: TInt;
  aVersion, aComment, s: string;
  step, start: TFloat;
  t: ILim2D;
begin
  AssignFile(TF,FileName);
  try
    reset(TF);
    ReadLn(TF, aVersion); // No check so far ...
    ReadLn(TF, s); // ArrType
    if s <> sFArrData2D then
      Raise EInvalidType.CreateFmt(RS_IncorrectType,[s]);
    ReadLn(TF, aComment);
    ReadLn(TF, L1, H1, L2, H2);
    t := TLim2D.Create(L1, H1, L2, H2);
    if not SameLim(t,A) then
      Raise ERangeError.Create(RS_LimMismatch);
    A.Comment := aComment;
    ReadLn(TF,s);
    A.ColNames.CommaText := s;

    ReadF2DTable(A, TF, s);

    ReadLn(TF,s);
    A.RowNames.CommaText := s;
    ReadLn(TF,s);
    A.ColInfo.CommaText := s;

  finally
    CloseFile(TF);
  end;
end;

end.
